@@ -7,6 +7,7 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef RD_FPBREADER_H_DEC2015
 #define RD_FPBREADER_H_DEC2015
 /*! \file FPBReader.h
@@ -54,7 +55,7 @@ struct FPBReader_impl;
   \c lazyRead mode.
 
 */
-class FPBReader {
+class RDKIT_DATASTRUCTS_EXPORT FPBReader {
  public:
   FPBReader()
       : dp_istrm(NULL),
@@ -90,6 +91,7 @@ class FPBReader {
   FPBReader(std::istream *inStream, bool takeOwnership = true,
             bool lazyRead = false)
       : dp_istrm(inStream),
+        dp_impl(NULL),
         df_owner(takeOwnership),
         df_init(false),
         df_lazyRead(lazyRead){};
@@ -112,6 +114,15 @@ class FPBReader {
   and delete inStream after calling \c init()
   */
   void init();
+  //! cleanup
+  /*!
+  Cleans up whatever memory was allocated during init()
+  */
+  void cleanup() {
+    if (!df_init) return;
+    destroy();
+    df_init = false;
+  };
   //! returns the requested fingerprint as an \c ExplicitBitVect
   boost::shared_ptr<ExplicitBitVect> getFP(unsigned int idx) const;
   //! returns the requested fingerprint as an array of bytes
@@ -259,6 +270,7 @@ class FPBReader {
       throw BadFileException(errout.str());
     }
     dp_istrm = tmpStream;
+    dp_impl = NULL;
     df_owner = true;
     df_init = false;
     df_lazyRead = lazyRead;
