@@ -8,6 +8,7 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/test.h>
 #include <cstdlib>
 #include <iostream>
 #include <sstream>
@@ -115,7 +116,7 @@ void Test(T arg) {
 
   try {
     t3.getBit(4000);
-  } catch (IndexErrorException) {
+  } catch (IndexErrorException &) {
     std::cout << " except " << endl;
   } catch (...) {
     std::cout << " ERROR EXCEPT " << endl;
@@ -553,8 +554,7 @@ void test6SparseIntVect() {
     iV1.setVal(4, 4);
     iV1.setVal(0, 2);
     iV1.setVal(3, 1);
-    SparseIntVect<int>::StorageType::const_iterator iter =
-        iV1.getNonzeroElements().begin();
+    auto iter = iV1.getNonzeroElements().begin();
     TEST_ASSERT(iter->first == 0);
     TEST_ASSERT(iter->second == 2);
     ++iter;
@@ -881,7 +881,7 @@ void test6SparseIntVect() {
   }
 
   {  // operator== and operator!=
-    SparseIntVect<int> iV1(5), iV2(5), iV3(3);
+    SparseIntVect<int> iV1(5), iV2(5), iV3(3), iV4(5);
     iV1.setVal(0, 2);
     iV1.setVal(2, 1);
     iV1.setVal(3, 4);
@@ -892,6 +892,11 @@ void test6SparseIntVect() {
     iV2.setVal(3, 4);
     iV2.setVal(4, 6);
 
+    iV4.setVal(1, 2);
+    iV4.setVal(2, 3);
+    iV4.setVal(3, 4);
+    iV4.setVal(4, 6);
+
     TEST_ASSERT(iV1 == iV1);
     TEST_ASSERT(iV2 == iV2);
     TEST_ASSERT(iV3 == iV3);
@@ -900,6 +905,8 @@ void test6SparseIntVect() {
     TEST_ASSERT(iV2 != iV1);
     TEST_ASSERT(iV3 != iV1);
     TEST_ASSERT(iV1 != iV3);
+    TEST_ASSERT(iV1 != iV4);
+    TEST_ASSERT(iV2 == iV4);
   }
 
   {  // test negative values (was sf.net Issue 3295215)
@@ -999,7 +1006,7 @@ void test8BitVectPickles() {
     inS.open(pklName.c_str(), std::ios_base::binary);
     unsigned int length;
     inS >> length;
-    char *buff = new char[length];
+    auto *buff = new char[length];
     length = inS.readsome(buff, length);
     inS.close();
     std::string pkl(buff, length);
@@ -1092,8 +1099,8 @@ void test10BitVectBinaryText() {
 
     fps = BitVectToBinaryText(bv);
     TEST_ASSERT(fps.size() == 4);
-    for (unsigned int i = 0; i < fps.size(); ++i) {
-      TEST_ASSERT(fps[i] == 0);
+    for (char fp : fps) {
+      TEST_ASSERT(fp == 0);
     }
 
     bv.setBit(0);
@@ -1103,8 +1110,8 @@ void test10BitVectBinaryText() {
 
     fps = BitVectToBinaryText(bv);
     TEST_ASSERT(fps.size() == 4);
-    for (unsigned int i = 0; i < fps.size(); ++i) {
-      TEST_ASSERT(fps[i] != 0);
+    for (char fp : fps) {
+      TEST_ASSERT(fp != 0);
     }
   }
   {
@@ -1113,8 +1120,8 @@ void test10BitVectBinaryText() {
 
     fps = BitVectToBinaryText(bv);
     TEST_ASSERT(fps.size() == 4);
-    for (unsigned int i = 0; i < fps.size(); ++i) {
-      TEST_ASSERT(fps[i] == 0);
+    for (char fp : fps) {
+      TEST_ASSERT(fp == 0);
     }
     UpdateBitVectFromBinaryText(bv2, fps);
     TEST_ASSERT(bv == bv2);
@@ -1136,8 +1143,8 @@ void test10BitVectBinaryText() {
 
     fps = BitVectToBinaryText(bv);
     TEST_ASSERT(fps.size() == 5);
-    for (unsigned int i = 0; i < fps.size(); ++i) {
-      TEST_ASSERT(fps[i] == 0);
+    for (char fp : fps) {
+      TEST_ASSERT(fp == 0);
     }
 
     bv.setBit(0);
@@ -1396,7 +1403,7 @@ int main() {
   RDLog::InitLogs();
   try {
     throw IndexErrorException(3);
-  } catch (IndexErrorException) {
+  } catch (IndexErrorException &) {
     BOOST_LOG(rdInfoLog) << "pass" << endl;
   }
 
