@@ -74,14 +74,14 @@ ForceField::~ForceField() {
   d_positions.clear();
   d_contribs.clear();
   delete[] dp_distMat;
-  dp_distMat = 0;
+  dp_distMat = nullptr;
 }
 
 ForceField::ForceField(const ForceField &other)
     : d_dimension(other.d_dimension),
       df_init(false),
       d_numPoints(other.d_numPoints),
-      dp_distMat(0) {
+      dp_distMat(nullptr) {
   d_contribs.clear();
   BOOST_FOREACH (const ContribPtr &contrib, other.d_contribs) {
     ForceFieldContrib *ncontrib = contrib->copy();
@@ -92,8 +92,8 @@ ForceField::ForceField(const ForceField &other)
 
 double ForceField::distance(unsigned int i, unsigned int j, double *pos) {
   PRECONDITION(df_init, "not initialized");
-  URANGE_CHECK(i, d_numPoints - 1);
-  URANGE_CHECK(j, d_numPoints - 1);
+  URANGE_CHECK(i, d_numPoints);
+  URANGE_CHECK(j, d_numPoints);
   if (j < i) {
     int tmp = j;
     j = i;
@@ -133,8 +133,8 @@ double ForceField::distance(unsigned int i, unsigned int j, double *pos) {
 
 double ForceField::distance(unsigned int i, unsigned int j, double *pos) const {
   PRECONDITION(df_init, "not initialized");
-  URANGE_CHECK(i, d_numPoints - 1);
-  URANGE_CHECK(j, d_numPoints - 1);
+  URANGE_CHECK(i, d_numPoints);
+  URANGE_CHECK(j, d_numPoints);
   if (j < i) {
     int tmp = j;
     j = i;
@@ -171,7 +171,7 @@ void ForceField::initialize() {
   // clean up if we have used this already:
   df_init = false;
   delete[] dp_distMat;
-  dp_distMat = 0;
+  dp_distMat = nullptr;
 
   d_numPoints = d_positions.size();
   d_matSize = d_numPoints * (d_numPoints + 1) / 2;
@@ -182,6 +182,12 @@ void ForceField::initialize() {
 
 int ForceField::minimize(unsigned int maxIts, double forceTol,
                          double energyTol) {
+  return minimize(0, nullptr, maxIts, forceTol, energyTol);
+}
+
+int ForceField::minimize(unsigned int snapshotFreq,
+                         RDKit::SnapshotVect *snapshotVect, unsigned int maxIts,
+                         double forceTol, double energyTol) {
   PRECONDITION(df_init, "not initialized");
   PRECONDITION(static_cast<unsigned int>(d_numPoints) == d_positions.size(),
                "size mismatch");
@@ -190,32 +196,40 @@ int ForceField::minimize(unsigned int maxIts, double forceTol,
   unsigned int numIters = 0;
   unsigned int dim = this->d_numPoints * d_dimension;
   double finalForce;
-  double *points = new double[dim];
+  auto *points = new double[dim];
 
   this->scatter(points);
   ForceFieldsHelper::calcEnergy eCalc(this);
   ForceFieldsHelper::calcGradient gCalc(this);
 
-  int res = BFGSOpt::minimize(dim, points, forceTol, numIters, finalForce,
-                              eCalc, gCalc, energyTol, maxIts);
+  int res =
+      BFGSOpt::minimize(dim, points, forceTol, numIters, finalForce, eCalc,
+                        gCalc, snapshotFreq, snapshotVect, energyTol, maxIts);
   this->gather(points);
 
   delete[] points;
   return res;
 }
 
-double ForceField::calcEnergy() const {
+double ForceField::calcEnergy(std::vector<double> *contribs) const {
   PRECONDITION(df_init, "not initialized");
   double res = 0.0;
   if (d_contribs.empty()) return res;
+  if (contribs) {
+    contribs->clear();
+    contribs->reserve(d_contribs.size());
+  }
 
   unsigned int N = d_positions.size();
-  double *pos = new double[d_dimension * N];
+  auto *pos = new double[d_dimension * N];
   this->scatter(pos);
   // now loop over the contribs
-  for (ContribPtrVect::const_iterator contrib = d_contribs.begin();
-       contrib != d_contribs.end(); contrib++) {
-    res += (*contrib)->getEnergy(pos);
+  for (const auto &d_contrib : d_contribs) {
+    double e = d_contrib->getEnergy(pos);
+    res += e;
+    if (contribs) {
+      contribs->push_back(e);
+    }
   }
   delete[] pos;
   return res;
@@ -244,18 +258,16 @@ void ForceField::calcGrad(double *grad) const {
   if (d_contribs.empty()) return;
 
   unsigned int N = d_positions.size();
-  double *pos = new double[d_dimension * N];
+  auto *pos = new double[d_dimension * N];
   this->scatter(pos);
-  for (ContribPtrVect::const_iterator contrib = d_contribs.begin();
-       contrib != d_contribs.end(); contrib++) {
-    (*contrib)->getGrad(pos, grad);
+  for (const auto &d_contrib : d_contribs) {
+    d_contrib->getGrad(pos, grad);
   }
   // zero out gradient values for any fixed points:
-  for (INT_VECT::const_iterator it = d_fixedPoints.begin();
-       it != d_fixedPoints.end(); it++) {
-    CHECK_INVARIANT(static_cast<unsigned int>(*it) < d_numPoints,
+  for (int d_fixedPoint : d_fixedPoints) {
+    CHECK_INVARIANT(static_cast<unsigned int>(d_fixedPoint) < d_numPoints,
                     "bad fixed point index");
-    unsigned int idx = d_dimension * (*it);
+    unsigned int idx = d_dimension * d_fixedPoint;
     for (unsigned int di = 0; di < this->dimension(); ++di) {
       grad[idx + di] = 0.0;
     }
@@ -289,9 +301,9 @@ void ForceField::scatter(double *pos) const {
   PRECONDITION(pos, "bad position vector");
 
   unsigned int tab = 0;
-  for (unsigned int i = 0; i < d_positions.size(); i++) {
+  for (auto d_position : d_positions) {
     for (unsigned int di = 0; di < this->dimension(); ++di) {
-      pos[tab + di] = (*d_positions[i])[di];  //->x;
+      pos[tab + di] = (*d_position)[di];  //->x;
     }
     tab += this->dimension();
   }
@@ -303,9 +315,9 @@ void ForceField::gather(double *pos) {
   PRECONDITION(pos, "bad position vector");
 
   unsigned int tab = 0;
-  for (unsigned int i = 0; i < d_positions.size(); i++) {
+  for (auto &d_position : d_positions) {
     for (unsigned int di = 0; di < this->dimension(); ++di) {
-      (*d_positions[i])[di] = pos[tab + di];
+      (*d_position)[di] = pos[tab + di];
     }
     tab += this->dimension();
   }
