@@ -7,6 +7,7 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef __RD_FORCEFIELD_H__
 #define __RD_FORCEFIELD_H__
 
@@ -14,6 +15,7 @@
 #include <boost/smart_ptr.hpp>
 #include <boost/foreach.hpp>
 #include <Geometry/point.h>
+#include <GraphMol/Trajectory/Snapshot.h>
 
 namespace ForceFields {
 class ForceFieldContrib;
@@ -53,7 +55,7 @@ typedef std::vector<ContribPtr> ContribPtrVect;
        this is almost certainly inefficient.
 
 */
-class ForceField {
+class RDKIT_FORCEFIELD_EXPORT ForceField {
  public:
   //! construct with a dimension
   ForceField(unsigned int dimension = 3)
@@ -68,7 +70,7 @@ class ForceField {
   void initialize();
 
   //! calculates and returns the energy (in kcal/mol) based on existing
-  //positions in the forcefield
+  // positions in the forcefield
   /*!
 
   \return the current energy
@@ -78,7 +80,7 @@ class ForceField {
   double *
       the positions need to be converted to double * here
   */
-  double calcEnergy() const;
+  double calcEnergy(std::vector<double> *contribs = NULL) const;
 
   // these next two aren't const because they may update our
   // distance matrix
@@ -128,6 +130,27 @@ class ForceField {
       - 0: indicates success
       - 1: the minimization did not converge in \c maxIts iterations.
   */
+  int minimize(unsigned int snapshotFreq, RDKit::SnapshotVect *snapshotVect,
+               unsigned int maxIts = 200, double forceTol = 1e-4,
+               double energyTol = 1e-6);
+
+  //! minimizes the energy of the system by following gradients
+  /*!
+    \param maxIts            the maximum number of iterations to try
+    \param forceTol          the convergence criterion for forces
+    \param energyTol         the convergence criterion for energies
+    \param snapshotFreq      a snapshot of the minimization trajectory
+                             will be stored after as many steps as indicated
+                             through this parameter; defaults to 0 (no
+                             trajectory stored)
+    \param snapshotVect      a pointer to a std::vector<Snapshot> where
+                             coordinates and energies will be stored
+
+    \return an integer value indicating whether or not the convergence
+            criteria were achieved:
+      - 0: indicates success
+      - 1: the minimization did not converge in \c maxIts iterations.
+  */
   int minimize(unsigned int maxIts = 200, double forceTol = 1e-4,
                double energyTol = 1e-6);
 
