@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (c) 2007-2014, Novartis Institutes for BioMedical Research Inc.
+//  Copyright (c) 2007-2018, Novartis Institutes for BioMedical Research Inc.
 //  All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,14 +29,20 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
+#include <GraphMol/MolPickler.h>
+#include <GraphMol/Wrap/props.hpp>
 #include <RDBoost/python.h>
 #include <GraphMol/ChemReactions/Reaction.h>
 #include <GraphMol/ChemReactions/ReactionPickler.h>
 #include <GraphMol/ChemReactions/ReactionParser.h>
 #include <GraphMol/ChemReactions/ReactionRunner.h>
+#include <GraphMol/ChemReactions/PreprocessRxn.h>
+#include <GraphMol/ChemReactions/SanitizeRxn.h>
 #include <GraphMol/Depictor/DepictUtils.h>
+#include <GraphMol/FilterCatalog/FunctionalGroupHierarchy.h>
 
 #include <RDBoost/Wrap.h>
+
 #include <RDGeneral/Exceptions.h>
 #include <GraphMol/SanitException.h>
 #include <RDGeneral/FileParseException.h>
@@ -61,13 +66,18 @@ void rdChemicalReactionExceptionTranslator(
 }
 
 namespace RDKit {
-python::object ReactionToBinary(const ChemicalReaction &self) {
+python::object ReactionToBinaryWithProps(const ChemicalReaction &self,
+                                         unsigned int props) {
   std::string res;
-  ReactionPickler::pickleReaction(self, res);
+  ReactionPickler::pickleReaction(self, res, props);
   python::object retval = python::object(
       python::handle<>(PyBytes_FromStringAndSize(res.c_str(), res.length())));
   return retval;
 }
+python::object ReactionToBinary(const ChemicalReaction &self) {
+  return ReactionToBinaryWithProps(self,
+                                   MolPickler::getDefaultPickleProperties());
+}
 //
 // allows reactions to be pickled.
 //
@@ -78,7 +88,7 @@ struct reaction_pickle_suite : python::pickle_suite {
 };
 
 template <typename T>
-PyObject *RunReactants(ChemicalReaction *self, T reactants) {
+PyObject *RunReactants(ChemicalReaction *self, T reactants, unsigned int maxProducts) {
   if (!self->isInitialized()) {
     NOGIL gil;
     self->initReactantMatchers();
@@ -94,7 +104,7 @@ PyObject *RunReactants(ChemicalReaction *self, T reactants) {
   std::vector<MOL_SPTR_VECT> mols;
   {
     NOGIL gil;
-    mols = self->runReactants(reacts);
+    mols = self->runReactants(reacts, maxProducts);
   }
   PyObject *res = PyTuple_New(mols.size());
 
@@ -147,7 +157,7 @@ ROMol *GetProductTemplate(const ChemicalReaction *self, unsigned int which) {
   if (which >= self->getNumProductTemplates()) {
     throw_value_error("requested template index too high");
   }
-  MOL_SPTR_VECT::const_iterator iter = self->beginProductTemplates();
+  auto iter = self->beginProductTemplates();
   iter += which;
   ROMol *res = const_cast<ROMol *>(iter->get());
   return res;
@@ -156,7 +166,7 @@ ROMol *GetReactantTemplate(const ChemicalReaction *self, unsigned int which) {
   if (which >= self->getNumReactantTemplates()) {
     throw_value_error("requested template index too high");
   }
-  MOL_SPTR_VECT::const_iterator iter = self->beginReactantTemplates();
+  auto iter = self->beginReactantTemplates();
   iter += which;
   ROMol *res = const_cast<ROMol *>(iter->get());
   return res;
@@ -165,7 +175,7 @@ ROMol *GetAgentTemplate(const ChemicalReaction *self, unsigned int which) {
   if (which >= self->getNumAgentTemplates()) {
     throw_value_error("requested template index too high");
   }
-  MOL_SPTR_VECT::const_iterator iter = self->beginAgentTemplates();
+  auto iter = self->beginAgentTemplates();
   iter += which;
   ROMol *res = const_cast<ROMol *>(iter->get());
   return res;
@@ -184,8 +194,8 @@ void RemoveUnmappedReactantTemplates(ChemicalReaction *self,
                                           moveToAgentTemplates, &tmp);
     python::list molList = python::extract<python::list>(targetList);
     if (tmp.size() > 0) {
-      for (unsigned int i = 0; i < tmp.size(); i++) {
-        molList.append(tmp.at(i));
+      for (auto &i : tmp) {
+        molList.append(i);
       }
     }
   }
@@ -204,8 +214,8 @@ void RemoveUnmappedProductTemplates(ChemicalReaction *self,
                                          moveToAgentTemplates, &tmp);
     python::list molList = python::extract<python::list>(targetList);
     if (tmp.size() > 0) {
-      for (unsigned int i = 0; i < tmp.size(); i++) {
-        molList.append(tmp.at(i));
+      for (auto &i : tmp) {
+        molList.append(i);
       }
     }
   }
@@ -219,8 +229,8 @@ void RemoveAgentTemplates(ChemicalReaction &self, python::object targetList) {
     self.removeAgentTemplates(&tmp);
     python::list molList = python::extract<python::list>(targetList);
     if (tmp.size() > 0) {
-      for (unsigned int i = 0; i < tmp.size(); i++) {
-        molList.append(tmp[i]);
+      for (auto &i : tmp) {
+        molList.append(i);
       }
     }
   }
@@ -279,8 +289,8 @@ python::object GetReactingAtoms(const ChemicalReaction &self,
                                 bool mappedAtomsOnly) {
   python::list res;
   VECT_INT_VECT rAs = getReactingAtoms(self, mappedAtomsOnly);
-  for (VECT_INT_VECT_I rIt = rAs.begin(); rIt != rAs.end(); ++rIt) {
-    res.append(python::tuple(*rIt));
+  for (auto &rA : rAs) {
+    res.append(python::tuple(rA));
   }
   return python::tuple(res);
 }
@@ -301,17 +311,17 @@ python::object AddRecursiveQueriesToReaction(ChemicalReaction &self,
   }
 
   if (getLabels) {
-    std::vector<std::vector<std::pair<unsigned int, std::string> > > labels;
+    std::vector<std::vector<std::pair<unsigned int, std::string>>> labels;
     addRecursiveQueriesToReaction(self, queries, propName, &labels);
 
     // transform labels into python::tuple(python::tuple(python::tuple))
     python::list reactantLabels;
-    for (unsigned int i = 0; i < labels.size(); ++i) {
+    for (auto &label : labels) {
       python::list tmpLabels;
-      for (unsigned int j = 0; j < labels[i].size(); ++j) {
+      for (auto &j : label) {
         python::list tmpPair;
-        tmpPair.append(labels[i][j].first);
-        tmpPair.append(labels[i][j].second);
+        tmpPair.append(j.first);
+        tmpPair.append(j.second);
         tmpLabels.append(python::tuple(tmpPair));
       }
       reactantLabels.append(python::tuple(tmpLabels));
@@ -322,8 +332,69 @@ python::object AddRecursiveQueriesToReaction(ChemicalReaction &self,
     return python::object();  // this is None
   }
 }
+
+python::object PreprocessReaction(ChemicalReaction &reaction,
+                                  python::dict queryDict,
+                                  std::string propName) {
+  // transform dictionary into map
+  std::map<std::string, ROMOL_SPTR> queries;
+  unsigned int size =
+      python::extract<unsigned int>(queryDict.keys().attr("__len__")());
+  if (!size) {
+    const bool normalized = true;
+    queries = GetFlattenedFunctionalGroupHierarchy(normalized);
+  } else {
+    for (unsigned int i = 0; i < size; ++i) {
+      ROMol *m = python::extract<ROMol *>(queryDict.values()[i]);
+      ROMOL_SPTR nm(new ROMol(*m));
+      std::string k = python::extract<std::string>(queryDict.keys()[i]);
+      queries[k] = nm;
+    }
+  }
+
+  unsigned int nReactants = reaction.getNumReactantTemplates();
+  unsigned int nProducts = reaction.getNumProductTemplates();
+  unsigned int nWarn, nError;
+  reaction.validate(nWarn, nError);
+  std::vector<std::vector<std::pair<unsigned int, std::string>>> labels;
+
+  if (!nError) {
+    preprocessReaction(reaction, nWarn, nError, labels, queries, propName);
+  }
+
+  // transform labels into python::tuple(python::tuple(python::tuple))
+  python::list reactantLabels;
+  for (auto &label : labels) {
+    python::list tmpLabels;
+    for (auto &j : label) {
+      python::list tmpPair;
+      tmpPair.append(j.first);
+      tmpPair.append(j.second);
+      tmpLabels.append(python::tuple(tmpPair));
+    }
+    reactantLabels.append(python::tuple(tmpLabels));
+  }
+  return python::make_tuple(nWarn, nError, nReactants, nProducts,
+                            python::tuple(reactantLabels));
 }
 
+typedef boost::uint64_t sanitize_ops;
+
+RxnOps::SanitizeRxnFlags sanitizeReaction(
+    ChemicalReaction &rxn, sanitize_ops sanitizeOps,
+    const MolOps::AdjustQueryParameters &params, bool catchErrors) {
+  unsigned int operationsThatFailed = 0;
+  try {
+    RxnOps::sanitizeRxn(rxn, operationsThatFailed, sanitizeOps, params);
+  } catch (...) {
+    if (!catchErrors) throw;
+  }
+  return static_cast<RxnOps::SanitizeRxnFlags>(operationsThatFailed);
+}
+}
+
+void wrap_enumeration();
+
 BOOST_PYTHON_MODULE(rdChemReactions) {
   python::scope().attr("__doc__") =
       "Module containing classes and functions for working with chemical "
@@ -364,6 +435,8 @@ BOOST_PYTHON_MODULE(rdChemReactions) {
       "A class for storing and applying chemical reactions.\n\
 \n\
 Sample Usage:\n\
+>>> from rdkit import Chem\n\
+>>> from rdkit.Chem import rdChemReactions\n\
 >>> rxn = rdChemReactions.ReactionFromSmarts('[C:1](=[O:2])O.[N:3]>>[C:1](=[O:2])[N:3]')\n\
 >>> reacts = (Chem.MolFromSmiles('C(=O)O'),Chem.MolFromSmiles('CNC'))\n\
 >>> products = rxn.RunReactants(reacts)\n\
@@ -371,13 +444,20 @@ Sample Usage:\n\
 1\n\
 >>> len(products[0])\n\
 1\n\
->>> Chem.MolToSmiles(products[0])\n\
+>>> Chem.MolToSmiles(products[0][0])\n\
 'CN(C)C=O'\n\
 \n\
 ";
 
-  python::class_<RDKit::MOL_SPTR_VECT>("ROMolList")
-      .def(python::vector_indexing_suite<RDKit::MOL_SPTR_VECT, true>());
+  // logic from https://stackoverflow.com/a/13017303
+  boost::python::type_info info =
+      boost::python::type_id<RDKit::MOL_SPTR_VECT>();
+  const boost::python::converter::registration *reg =
+      boost::python::converter::registry::query(info);
+  if (reg == NULL || (*reg).m_to_python == NULL) {
+    python::class_<RDKit::MOL_SPTR_VECT>("MOL_SPTR_VECT")
+        .def(python::vector_indexing_suite<RDKit::MOL_SPTR_VECT, true>());
+  }
 
   python::class_<RDKit::ChemicalReaction>(
       "ChemicalReaction", docString.c_str(),
@@ -420,13 +500,18 @@ Sample Usage:\n\
            "Removes agents from reaction. If targetList is provide the agents "
            "will be transfered to that list.")
       .def("RunReactants", (PyObject * (*)(RDKit::ChemicalReaction *,
-                                           python::tuple))RDKit::RunReactants,
+                                           python::tuple,
+                                           unsigned int maxProducts))RDKit::RunReactants,
+           (python::arg("self"), python::arg("reactants"), python::arg("maxProducts")=1000),
            "apply the reaction to a sequence of reactant molecules and return "
-           "the products as a tuple of tuples")
+           "the products as a tuple of tuples.  If maxProducts is not zero,"
+           " stop the reaction when maxProducts have been generated [default=1000]")
       .def("RunReactants", (PyObject * (*)(RDKit::ChemicalReaction *,
-                                           python::list))RDKit::RunReactants,
+                                           python::list, unsigned int maxProducts))RDKit::RunReactants,
+           (python::arg("self"), python::arg("reactants"), python::arg("maxProducts")=1000),
            "apply the reaction to a sequence of reactant molecules and return "
-           "the products as a tuple of tuples")
+           "the products as a tuple of tuples.  If maxProducts is not zero,"
+           " stop the reaction when maxProducts have been generated [default=1000]")
       .def("RunReactant",
            (PyObject * (*)(RDKit::ChemicalReaction *, python::object,
                            unsigned))RDKit::RunReactant,
@@ -461,7 +546,10 @@ Sample Usage:\n\
            (python::arg("self")),
            "EXPERT USER: returns whether or not the reaction can have implicit "
            "properties")
-      .def("ToBinary", RDKit::ReactionToBinary,
+      .def("ToBinary", RDKit::ReactionToBinary, (python::arg("self")),
+           "Returns a binary string representation of the reaction.")
+      .def("ToBinary", RDKit::ReactionToBinaryWithProps,
+           (python::arg("self"), python::arg("propertyFlags")),
            "Returns a binary string representation of the reaction.")
       .def("IsMoleculeReactant", RDKit::IsMoleculeReactantOfReaction,
            "returns whether or not the molecule has a substructure match to "
@@ -493,6 +581,142 @@ Sample Usage:\n\
            python::return_value_policy<python::reference_existing_object>(),
            "get the agent templates")
 
+      // properties
+      .def("SetProp", RDKit::MolSetProp<RDKit::ChemicalReaction, std::string>,
+           (python::arg("self"), python::arg("key"), python::arg("val"),
+            python::arg("computed") = false),
+           "Sets a molecular property\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to be set (a string).\n"
+           "    - value: the property value (a string).\n"
+           "    - computed: (optional) marks the property as being "
+           "computed.\n"
+           "                Defaults to False.\n\n")
+      .def("SetDoubleProp", RDKit::MolSetProp<RDKit::ChemicalReaction, double>,
+           (python::arg("self"), python::arg("key"), python::arg("val"),
+            python::arg("computed") = false),
+           "Sets a double valued molecular property\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to be set (a string).\n"
+           "    - value: the property value as a double.\n"
+           "    - computed: (optional) marks the property as being "
+           "computed.\n"
+           "                Defaults to 0.\n\n")
+      .def("SetIntProp", RDKit::MolSetProp<RDKit::ChemicalReaction, int>,
+           (python::arg("self"), python::arg("key"), python::arg("val"),
+            python::arg("computed") = false),
+           "Sets an integer valued molecular property\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to be set (an unsigned "
+           "number).\n"
+           "    - value: the property value as an integer.\n"
+           "    - computed: (optional) marks the property as being "
+           "computed.\n"
+           "                Defaults to False.\n\n")
+      .def("SetUnsignedProp",
+           RDKit::MolSetProp<RDKit::ChemicalReaction, unsigned int>,
+           (python::arg("self"), python::arg("key"), python::arg("val"),
+            python::arg("computed") = false),
+           "Sets an unsigned integer valued molecular property\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to be set (a string).\n"
+           "    - value: the property value as an unsigned integer.\n"
+           "    - computed: (optional) marks the property as being "
+           "computed.\n"
+           "                Defaults to False.\n\n")
+      .def("SetBoolProp", RDKit::MolSetProp<RDKit::ChemicalReaction, bool>,
+           (python::arg("self"), python::arg("key"), python::arg("val"),
+            python::arg("computed") = false),
+           "Sets a boolean valued molecular property\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to be set (a string).\n"
+           "    - value: the property value as a bool.\n"
+           "    - computed: (optional) marks the property as being "
+           "computed.\n"
+           "                Defaults to False.\n\n")
+      .def("HasProp", RDKit::MolHasProp<RDKit::ChemicalReaction>,
+           "Queries a molecule to see if a particular property has been "
+           "assigned.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to check for (a string).\n")
+      .def("GetProp", RDKit::GetProp<RDKit::ChemicalReaction, std::string>,
+           "Returns the value of the property.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to return (a string).\n\n"
+           "  RETURNS: a string\n\n"
+           "  NOTE:\n"
+           "    - If the property has not been set, a KeyError exception "
+           "will be raised.\n")
+      .def("GetDoubleProp", RDKit::GetProp<RDKit::ChemicalReaction, double>,
+           "Returns the double value of the property if possible.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to return (a string).\n\n"
+           "  RETURNS: a double\n\n"
+           "  NOTE:\n"
+           "    - If the property has not been set, a KeyError exception "
+           "will be raised.\n")
+      .def("GetIntProp", RDKit::GetProp<RDKit::ChemicalReaction, int>,
+           "Returns the integer value of the property if possible.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to return (a string).\n\n"
+           "  RETURNS: an integer\n\n"
+           "  NOTE:\n"
+           "    - If the property has not been set, a KeyError exception "
+           "will be raised.\n")
+      .def("GetUnsignedProp",
+           RDKit::GetProp<RDKit::ChemicalReaction, unsigned int>,
+           "Returns the unsigned int value of the property if possible.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to return (a string).\n\n"
+           "  RETURNS: an unsigned integer\n\n"
+           "  NOTE:\n"
+           "    - If the property has not been set, a KeyError exception "
+           "will be raised.\n")
+      .def("GetBoolProp", RDKit::GetProp<RDKit::ChemicalReaction, bool>,
+           "Returns the Bool value of the property if possible.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to return (a string).\n\n"
+           "  RETURNS: a bool\n\n"
+           "  NOTE:\n"
+           "    - If the property has not been set, a KeyError exception "
+           "will be raised.\n")
+      .def("ClearProp", RDKit::MolClearProp<RDKit::ChemicalReaction>,
+           "Removes a property from the reaction.\n\n"
+           "  ARGUMENTS:\n"
+           "    - key: the name of the property to clear (a string).\n")
+
+      .def("ClearComputedProps",
+           RDKit::MolClearComputedProps<RDKit::ChemicalReaction>,
+           "Removes all computed properties from the reaction.\n\n")
+
+      .def("GetPropNames", &RDKit::ChemicalReaction::getPropList,
+           (python::arg("self"), python::arg("includePrivate") = false,
+            python::arg("includeComputed") = false),
+           "Returns a tuple with all property names for this reaction.\n\n"
+           "  ARGUMENTS:\n"
+           "    - includePrivate: (optional) toggles inclusion of private "
+           "properties in the result set.\n"
+           "                      Defaults to 0.\n"
+           "    - includeComputed: (optional) toggles inclusion of computed "
+           "properties in the result set.\n"
+           "                      Defaults to 0.\n\n"
+           "  RETURNS: a tuple of strings\n")
+
+      .def("GetPropsAsDict", RDKit::GetPropsAsDict<RDKit::ChemicalReaction>,
+           (python::arg("self"), python::arg("includePrivate") = false,
+            python::arg("includeComputed") = false),
+           "Returns a dictionary populated with the reaction's properties.\n"
+           " n.b. Some properties are not able to be converted to python "
+           "types.\n\n"
+           "  ARGUMENTS:\n"
+           "    - includePrivate: (optional) toggles inclusion of private "
+           "properties in the result set.\n"
+           "                      Defaults to False.\n"
+           "    - includeComputed: (optional) toggles inclusion of computed "
+           "properties in the result set.\n"
+           "                      Defaults to False.\n\n"
+           "  RETURNS: a dictionary\n")
+
       // enable pickle support
       .def_pickle(RDKit::reaction_pickle_suite());
 
@@ -608,6 +832,7 @@ of the replacements argument.",
       "Caution: This is an expert-user function which will change a property (molInversionFlag) of your products.\
           This function is called by default using the RXN or SMARTS parser for reactions and should really only be called if reactions have been constructed some other way.\
           The function updates the stereochemistry of the product by considering 4 different cases: inversion, retention, removal, and introduction");
+
   python::def(
       "ReduceProductToSideChains", RDKit::reduceProductToSideChains,
       (python::arg("product"), python::arg("addDummyAtoms") = true),
@@ -615,8 +840,150 @@ of the replacements argument.",
               The output is a molecule with attached wildcards indicating where the product was attached.\
               The dummy atom has the same reaction-map number as the product atom (if available).",
       python::return_value_policy<python::manage_new_object>());
+
   python::def("RemoveMappingNumbersFromReactions",
               RDKit::removeMappingNumbersFromReactions,
               (python::arg("reaction")),
               "Removes the mapping numbers from the molecules of a reaction");
+
+  docString =
+      "A function for preprocessing reactions with more specific queries.\n\
+Queries are indicated by labels on atoms (molFileAlias property by default)\n\
+When these labels are found, more specific queries are placed on the atoms.\n\
+By default, the available quieries come from \n\
+  FilterCatalog.GetFlattenedFunctionalGroupHierarchy(True)\n\n\n\
+Sample Usage:\n\
+  >>> from rdkit import Chem, RDConfig\n\
+  >>> from rdkit.Chem import MolFromSmiles, AllChem\n\
+  >>> from rdkit.Chem.rdChemReactions import PreprocessReaction\n\
+  >>> import os\n\
+  >>> testFile = os.path.join(RDConfig.RDCodeDir,'Chem','SimpleEnum','test_data','boronic1.rxn')\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> nWarn,nError,nReacts,nProds,reactantLabels = PreprocessReaction(rxn)\n\
+  >>> nWarn\n\
+  0\n\
+  >>> nError\n\
+  0\n\
+  >>> nReacts\n\
+  2\n\
+  >>> nProds\n\
+  1\n\
+  >>> reactantLabels\n\
+  (((0, 'halogen.bromine.aromatic'),), ((1, 'boronicacid'),))\n\
+\n\
+  If there are functional group labels in the input reaction (via atoms with molFileValue properties),\n\
+  the corresponding atoms will have queries added to them so that they only match such things. We can\n\
+  see this here:\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> r1 = rxn.GetReactantTemplate(0)\n\
+  >>> m1 = Chem.MolFromSmiles('CCBr')\n\
+  >>> m2 = Chem.MolFromSmiles('c1ccccc1Br')\n\
+  \n\
+  These both match because the reaction file itself just has R1-Br:\n\
+  >>> m1.HasSubstructMatch(r1)\n\
+  True\n\
+  >>> m2.HasSubstructMatch(r1)\n\
+  True\n\
+\n\
+  After preprocessing, we only match the aromatic Br:\n\
+  >>> d = PreprocessReaction(rxn)\n\
+  >>> m1.HasSubstructMatch(r1)\n\
+  False\n\
+  >>> m2.HasSubstructMatch(r1)\n\
+  True\n\
+\n\
+  We also support or queries in the values field (separated by commas):\n\
+  >>> testFile = os.path.join(RDConfig.RDCodeDir,'Chem','SimpleEnum','test_data','azide_reaction.rxn')\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> reactantLabels = PreprocessReaction(rxn)[-1]\n\
+  >>> reactantLabels\n\
+  (((1, 'azide'),), ((1, 'carboxylicacid,acidchloride'),))\n\
+  >>> m1 = Chem.MolFromSmiles('CC(=O)O')\n\
+  >>> m2 = Chem.MolFromSmiles('CC(=O)Cl')\n\
+  >>> m3 = Chem.MolFromSmiles('CC(=O)N')\n\
+  >>> r2 = rxn.GetReactantTemplate(1)\n\
+  >>> m1.HasSubstructMatch(r2)\n\
+  True\n\
+  >>> m2.HasSubstructMatch(r2)\n\
+  True\n\
+  >>> m3.HasSubstructMatch(r2)\n\
+  False\n\
+\n\
+  unrecognized final group types are returned as None:\n\
+  >>> testFile = os.path.join(RDConfig.RDCodeDir,'Chem','SimpleEnum','test_data','bad_value1.rxn')\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> nWarn,nError,nReacts,nProds,reactantLabels = PreprocessReaction(rxn)\n\
+  Traceback (most recent call last):\n\
+    ...\n\
+  RuntimeError: KeyErrorException\n\
+\n\
+  One unrecognized group type in a comma-separated list makes the whole thing fail:\n\
+  >>> testFile = os.path.join(RDConfig.RDCodeDir,'Chem','SimpleEnum','test_data','bad_value2.rxn')\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> nWarn,nError,nReacts,nProds,reactantLabels = PreprocessReaction(rxn)\n\
+  Traceback (most recent call last):\n\
+    ...\n\
+  RuntimeError: KeyErrorException\n\
+  >>> testFile = os.path.join(RDConfig.RDCodeDir,'Chem','SimpleEnum','test_data','bad_value3.rxn')\n\
+  >>> rxn = AllChem.ReactionFromRxnFile(testFile)\n\
+  >>> rxn.Initialize()\n\
+  >>> nWarn,nError,nReacts,nProds,reactantLabels = PreprocessReaction(rxn)\n\
+  Traceback (most recent call last):\n\
+    ...\n\
+  RuntimeError: KeyErrorException\n\
+  >>> rxn = rdChemReactions.ChemicalReaction()\n\
+  >>> rxn.Initialize()\n\
+  >>> nWarn,nError,nReacts,nProds,reactantLabels = PreprocessReaction(rxn)\n\
+  >>> reactantLabels\n\
+  ()\n\
+  >>> reactantLabels == ()\n\
+  True\n\
+";
+
+  python::def(
+      "PreprocessReaction", RDKit::PreprocessReaction,
+      (python::arg("reaction"), python::arg("queries") = python::dict(),
+       python::arg("propName") = RDKit::common_properties::molFileValue),
+      docString.c_str());
+
+  python::enum_<RDKit::RxnOps::SanitizeRxnFlags>("SanitizeFlags")
+      .value("SANITIZE_NONE", RDKit::RxnOps::SANITIZE_NONE)
+      .value("SANITIZE_ATOM_MAPS", RDKit::RxnOps::SANITIZE_ATOM_MAPS)
+      .value("SANITIZE_RGROUP_NAMES", RDKit::RxnOps::SANITIZE_RGROUP_NAMES)
+      .value("SANITIZE_ADJUST_REACTANTS",
+             RDKit::RxnOps::SANITIZE_ADJUST_REACTANTS)
+      .value("SANITIZE_MERGEHS", RDKit::RxnOps::SANITIZE_MERGEHS)
+      .value("SANITIZE_ALL", RDKit::RxnOps::SANITIZE_ALL)
+      .export_values();
+  ;
+
+  python::def(
+      "GetDefaultAdjustParams", RDKit::RxnOps::DefaultRxnAdjustParams,
+      "Returns the default adjustment parameters for reactant templates");
+
+  python::def("GetChemDrawRxnAdjustParams",
+              RDKit::RxnOps::ChemDrawRxnAdjustParams,
+              "(deprecated, see MatchOnlyAtRgroupsAdjustParams)\n\tReturns the "
+              "chemdraw style adjustment parameters for reactant templates");
+
+  python::def(
+      "MatchOnlyAtRgroupsAdjustParams",
+      RDKit::RxnOps::MatchOnlyAtRgroupsAdjustParams,
+      "Only match at the specified rgroup locations in the reactant templates");
+
+  std::string docstring = "feed me";
+  python::def("SanitizeRxn", RDKit::sanitizeReaction,
+              (python::arg("rxn"),
+               python::arg("sanitizeOps") =
+                   rdcast<unsigned int>(RDKit::RxnOps::SANITIZE_ALL),
+               python::arg("params") = RDKit::RxnOps::DefaultRxnAdjustParams(),
+               python::arg("catchErrors") = false),
+              docString.c_str());
+
+  wrap_enumeration();
 }
