@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2004-2008 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2004-2017 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -28,16 +27,10 @@ const double NEIGH_RADIUS = 2.5;
 
 namespace RDDepict {
 namespace {
-unsigned int getHeavyDegree(const RDKit::Atom *atom) {
+// returns the atomic degree to be used for coordinate generation
+unsigned int getDepictDegree(const RDKit::Atom *atom) {
   PRECONDITION(atom, "no atom");
-  unsigned int res = 0;
-  RDKit::ROMol::ADJ_ITER nbrIdx, endNbrs;
-  boost::tie(nbrIdx, endNbrs) = atom->getOwningMol().getAtomNeighbors(atom);
-  while (nbrIdx != endNbrs) {
-    if (atom->getOwningMol()[*nbrIdx]->getAtomicNum() != 1) ++res;
-    ++nbrIdx;
-  }
-  return res;
+  return atom->getDegree();
 }
 }  // end of anonymous namespace
 
@@ -100,6 +93,7 @@ EmbeddedFrag::EmbeddedFrag(const RDKit::ROMol *mol,
     CHECK_INVARIANT(aid < na, "");
     EmbeddedAtom eatom(aid, cri->second);
     eatom.neighs.clear();
+    eatom.df_fixed = true;
     d_eatoms[aid] = eatom;
     d_done = false;
   }
@@ -283,7 +277,7 @@ EmbeddedFrag::EmbeddedFrag(const RDKit::Bond *dblBond) {
   eeatm.loc = RDGeom::Point2D(BOND_LEN, 0.0);
   eeatm.nbr1 = begAtm;
   eeatm.CisTransNbr = nbrAtms[1];
-  if (stype == RDKit::Bond::STEREOZ) {
+  if (stype == RDKit::Bond::STEREOZ || stype == RDKit::Bond::STEREOCIS) {
     eeatm.normal = RDGeom::Point2D(0.0, -1.0);
     eeatm.ccw = true;
   } else {
@@ -331,7 +325,7 @@ void EmbeddedFrag::updateNewNeighs(
   d_eatoms[aid].neighs.insert(d_eatoms[aid].neighs.end(), hIndices.begin(),
                               hIndices.end());
 
-  int deg = getHeavyDegree(dp_mol->getAtomWithIdx(aid));
+  int deg = getDepictDegree(dp_mol->getAtomWithIdx(aid));
   // order the neigbors by their CIPranks, if the number is between > 0 but less
   // than 3
   if ((d_eatoms[aid].neighs.size() > 0) &&
@@ -808,7 +802,7 @@ void EmbeddedFrag::addAtomToAtomWithNoAng(unsigned int aid,
 
   // find out what angle we want to add bond at
   const RDKit::Atom *atm = dp_mol->getAtomWithIdx(toAid);
-  int deg = getHeavyDegree(atm);
+  int deg = getDepictDegree(atm);
 
   double angle = computeSubAngle(deg, atm->getHybridization());
 
@@ -1169,9 +1163,8 @@ void EmbeddedFrag::canonicalizeOrientation() {
   if (d_eatoms.size() <= 1) return;
 
   RDGeom::Point2D cent(0.0, 0.0);
-  INT_EATOM_MAP_I eri;
-  for (eri = d_eatoms.begin(); eri != d_eatoms.end(); eri++) {
-    cent += eri->second.loc;
+  for (const auto &elem : d_eatoms) {
+    cent += elem.second.loc;
   }
   cent *= (1.0 / d_eatoms.size());
 
@@ -1182,11 +1175,11 @@ void EmbeddedFrag::canonicalizeOrientation() {
 
   // shift the center of the fragment to the origin and compute the covariance
   // matrix
-  for (eri = d_eatoms.begin(); eri != d_eatoms.end(); eri++) {
-    eri->second.loc -= cent;
-    xx += (eri->second.loc.x) * (eri->second.loc.x);
-    xy += (eri->second.loc.x) * (eri->second.loc.y);
-    yy += (eri->second.loc.y) * (eri->second.loc.y);
+  for (auto &elem : d_eatoms ){
+    elem.second.loc -= cent;
+    xx += (elem.second.loc.x) * (elem.second.loc.x);
+    xy += (elem.second.loc.x) * (elem.second.loc.y);
+    yy += (elem.second.loc.y) * (elem.second.loc.y);
   }
 
   RDGeom::Point2D eig1, eig2;
@@ -1204,15 +1197,15 @@ void EmbeddedFrag::canonicalizeOrientation() {
 
   eig2.x = 2 * xy;
   eig2.y = (yy - xx) - d;
-  if (eig2.length() <= 1e-4) return;
   double eVal2 = (xx + yy - d) / 2;
-  eig2.normalize();
 
-  // make sure eig1 corresponds to the larger eigenvalue:
-  if (eVal2 > eVal1) {
-    RDGeom::Point2D tmp = eig1;
-    eig1 = eig2;
-    eig2 = tmp;
+  if (eig2.length() > 1e-4) {
+    eig2.normalize();
+
+    // make sure eig1 corresponds to the larger eigenvalue:
+    if (eVal2 > eVal1) {
+      std::swap(eig1, eig2);
+    }
   }
   // now rotate eig1 onto the X axis:
   trans.setVal(0, 0, eig1.x);
@@ -1271,7 +1264,7 @@ void EmbeddedFrag::computeDistMat(DOUBLE_SMART_PTR &dmat) {
   INT_EATOM_MAP_I efi, efj;
   RDGeom::Point2D pti, ptj;
 
-  INT_EATOM_MAP_I tempi = d_eatoms.begin();
+  auto tempi = d_eatoms.begin();
   tempi++;
   double *dmatPtr = dmat.get();
   for (efi = tempi; efi != d_eatoms.end(); efi++) {
@@ -1296,11 +1289,11 @@ double EmbeddedFrag::mimicDistMatAndDensityCostFunc(
   if (dmat) {
     ddata = dmat->get();
   } else {
-    ddata = 0;
+    ddata = nullptr;
   }
   unsigned int na = dp_mol->getNumAtoms();
   unsigned int dsize = na * (na - 1) / 2;
-  double *ddata2D = new double[dsize];
+  auto *ddata2D = new double[dsize];
   DOUBLE_SMART_PTR dmat2D(ddata2D);
   this->computeDistMat(dmat2D);
   double res1 = 0.0;
@@ -1400,16 +1393,19 @@ void EmbeddedFrag::randomSampleFlipsAndPermutations(
     for (RDKit::ROMol::ConstAtomIterator ai = dp_mol->beginAtoms();
          ai != dp_mol->endAtoms(); ai++) {
       unsigned int caid = (*ai)->getIdx();
-      if ((getHeavyDegree(*ai) == 4) &&
+      if ((getDepictDegree(*ai) == 4) &&
           (!(dp_mol->getRingInfo()->numAtomRings(caid)))) {
         RDKit::INT_VECT aids, bids;
         getNbrAtomAndBondIds(caid, dp_mol, aids, bids);
-        // make sure all the atoms in aids are in this embeddedfrag
+        // make sure all the atoms in aids are in this embeddedfrag and can be
+        // perturbed
         bool allin = true;
         for (RDKit::INT_VECT_CI ivci = aids.begin(); ivci != aids.end();
              ivci++) {
-          if (d_eatoms.find(*ivci) == d_eatoms.end()) {
+          auto nbrIter = d_eatoms.find(*ivci);
+          if (nbrIter == d_eatoms.end() || nbrIter->second.df_fixed) {
             allin = false;
+            break;
           }
         }
         if (allin) {
@@ -1608,7 +1604,7 @@ void _recurseDegTwoRingAtoms(unsigned int aid, const RDKit::ROMol *mol,
   int bondId;
   RDKit::INT_VECT nbrs;
   while (atomBonds.first != atomBonds.second) {
-    const RDKit::BOND_SPTR bnd = (*mol)[*atomBonds.first];
+    const RDKit::Bond *bnd = (*mol)[*atomBonds.first];
     bondId = bnd->getIdx();
     if (mol->getRingInfo()->numBondRings(bondId)) {
       nbrs.push_back(bnd->getOtherAtomIdx(aid));
@@ -1675,28 +1671,48 @@ void EmbeddedFrag::flipAboutBond(unsigned int bondId, bool flipEnd) {
   endSideAids.clear();
   _recurseAtomOneSide(endAid, begAid, dp_mol, endSideAids);
 
-  // noew we have the molecule split into two groups of atoms
-  // atom on the side of endAid and the rest.
-  // we will flip the side that is smaller
-  int nats = d_eatoms.size();
-  int nEndSide = endSideAids.size();
-  bool endSideFlip = true;
-  if ((nats - nEndSide) < nEndSide) {
-    endSideFlip = false;
+  // look for fixed atoms in the fragment:
+  unsigned int nEndAtomsFixed = 0;
+  unsigned int nAtomsFixed = 0;
+  for (auto &d_eatom : d_eatoms) {
+    if (d_eatom.second.df_fixed) ++nAtomsFixed;
   }
-
-  for (INT_EATOM_MAP_I efi = d_eatoms.begin(); efi != d_eatoms.end(); efi++) {
+  // if there are fixed atoms, look at the atoms on the "end side"
+  if (nAtomsFixed) {
+    BOOST_FOREACH (int endAtomId, endSideAids) {
+      if (d_eatoms[endAtomId].df_fixed) ++nEndAtomsFixed;
+    }
+  }
+  // std::cerr << "  FLIP: " << nAtomsFixed << " " << nEndAtomsFixed <<
+  // std::endl;
+  // now we have the molecule split into two groups of atoms
+  // atom on the side of endAid and the rest.
+  // we will flip the side that is smaller, assuming that there
+  // are no fixed atoms there
+  bool endSideFlip = true;
+  if (nEndAtomsFixed) {
+    endSideFlip = false;
+    // there are fixed atoms on both sides, just return
+    return;
+  } else {
+    size_t nats = d_eatoms.size();
+    size_t nEndSide = endSideAids.size();
+    if ((nats - nEndSide) < nEndSide) {
+      endSideFlip = false;
+    }
+  }
+  for (auto &d_eatom : d_eatoms) {
     RDKit::INT_VECT_CI fii = std::find(endSideAids.begin(), endSideAids.end(),
-                                       static_cast<int>(efi->first));
+                                       static_cast<int>(d_eatom.first));
     if (endSideFlip ^ (fii == endSideAids.end())) {
-      efi->second.Reflect(begLoc, endLoc);
+      d_eatom.second.Reflect(begLoc, endLoc);
     }
   }
 }
 
 unsigned int _findDeg1Neighbor(const RDKit::ROMol *mol, unsigned int aid) {
   PRECONDITION(mol, "");
-  unsigned int deg = getHeavyDegree(mol->getAtomWithIdx(aid));
+  unsigned int deg = getDepictDegree(mol->getAtomWithIdx(aid));
   CHECK_INVARIANT(deg == 1, "");
   unsigned int res = 0;
   RDKit::ROMol::ADJ_ITER nbrIdx, endNbrs;
@@ -1755,20 +1771,22 @@ void EmbeddedFrag::openAngles(const double *dmat, unsigned int aid1,
 
   PRECONDITION(dp_mol, "");
   PRECONDITION(dmat, "");
-  unsigned int deg1 = getHeavyDegree(dp_mol->getAtomWithIdx(aid1));
-  unsigned int deg2 = getHeavyDegree(dp_mol->getAtomWithIdx(aid2));
-  if ((deg1 > 1) && (deg2 > 1)) {
+  unsigned int deg1 = getDepictDegree(dp_mol->getAtomWithIdx(aid1));
+  unsigned int deg2 = getDepictDegree(dp_mol->getAtomWithIdx(aid2));
+  bool fixed1 = d_eatoms[aid1].df_fixed;
+  bool fixed2 = d_eatoms[aid2].df_fixed;
+  if ((deg1 > 1 || fixed1) && (deg2 > 1 || fixed2)) {
     return;
   }
   unsigned int aidA;
   unsigned int aidB;
   RDKit::ROMol::ADJ_ITER nbrIdx, endNbrs;
   int type = 0;
-  if ((deg1 == 1) && (deg2 == 1)) {
+  if ((deg1 == 1 && !fixed1) && (deg2 == 1 && !fixed2)) {
     aidA = _findDeg1Neighbor(dp_mol, aid1);
     aidB = _findDeg1Neighbor(dp_mol, aid2);
     type = 1;
-  } else if ((deg1 == 1) && (deg2 > 1)) {
+  } else if ((deg1 == 1 && !fixed1) && (deg2 > 1 || fixed2)) {
     aidA = _findDeg1Neighbor(dp_mol, aid1);
     aidB = _findClosestNeighbor(dp_mol, dmat, aidA, aid2);
     type = 2;
@@ -1778,10 +1796,8 @@ void EmbeddedFrag::openAngles(const double *dmat, unsigned int aid1,
     type = 3;
   }
 
-  // std::cerr<<" openAngles: "<<aid1<<"-"<<aidA<<"-"<<aidB<<"-"<<aid2<<"  type:
-  // "<<type<<std::endl;
-  // std::cerr<<"             len: "<<(d_eatoms[aid1].loc -
-  // d_eatoms[aid2].loc).length()<<std::endl;
+  // std::cerr << " openAngles: " << aid1 << "-" << aidA << "-" << aidB << "-"
+  //           << aid2 << "  type:" << type << std::endl;
 
   RDGeom::Point2D v2 = d_eatoms[aid1].loc - d_eatoms[aidA].loc;
   RDGeom::Point2D v1 = d_eatoms[aidB].loc - d_eatoms[aidA].loc;
@@ -1926,20 +1942,28 @@ void EmbeddedFrag::removeCollisionsShortenBonds() {
     // we will use the one with the smallest degree
     int aid1 = cAids.first;
     int aid2 = cAids.second;
+    bool fixed1 = d_eatoms[aid1].df_fixed;
+    bool fixed2 = d_eatoms[aid2].df_fixed;
+    if (fixed1 && fixed2) {
+      // both atoms are fixed, so there's nothing
+      // we can do about this collision.
+      colls.erase(colls.begin());
+      ncols = colls.size();
+      ++iter;
+      continue;
+    }
     int deg1 = dp_mol->getAtomWithIdx(aid1)->getDegree();
     int deg2 = dp_mol->getAtomWithIdx(aid2)->getDegree();
-    if (deg2 > deg1) {
+    if (fixed1 || (deg2 > deg1 && !fixed2)) {
       // reverse the order
-      int temp = aid1;
-      aid1 = aid2;
-      aid2 = temp;
-      temp = deg1;
-      deg1 = deg2;
-      deg2 = temp;
+      std::swap(deg1, deg2);
+      std::swap(aid1, aid2);
+      std::swap(fixed1, fixed2);
     }
     // now find the path between the two ends
     RDKit::INT_LIST path = RDKit::MolOps::getShortestPath(*dp_mol, aid1, aid2);
-    // std::cerr<<" collide! "<<aid1<<" "<<aid2<<" "<<path.size()<<std::endl;
+    // std::cerr << " collide! " << aid1 << " " << aid2 << " " << path.size()
+    //           << std::endl;
     if (!path.size()) {
       // there's no path between the ends, so there's nothing
       // we can really do about this collision.
@@ -1957,18 +1981,18 @@ void EmbeddedFrag::removeCollisionsShortenBonds() {
           int aidA = _findDeg1Neighbor(dp_mol, aid1);
           loc -= d_eatoms[aidA].loc;
           loc *= .9;
-          // std::cerr<<"  >>> "<<aid1<<" "<<loc.length()<<std::endl;
+          // std::cerr << "  >>> " << aid1 << " " << loc.length() << std::endl;
           if (loc.length() > .75) {
             loc += d_eatoms[aidA].loc;
             d_eatoms[aid1].loc = loc;
           }
         }
-        if (deg2 == 1) {
+        if (deg2 == 1 && !fixed2) {
           RDGeom::Point2D loc = d_eatoms[aid2].loc;
           int aidA = _findDeg1Neighbor(dp_mol, aid2);
           loc -= d_eatoms[aidA].loc;
           loc *= .9;
-          // std::cerr<<"  >>> "<<aid2<<" "<<loc.length()<<std::endl;
+          // std::cerr << "  >>> " << aid2 << " " << loc.length() << std::endl;
           if (loc.length() > .75) {
             loc += d_eatoms[aidA].loc;
             d_eatoms[aid2].loc = loc;
@@ -1992,6 +2016,7 @@ void EmbeddedFrag::removeCollisionsShortenBonds() {
         RDKit::INT_VECT_CI rpi;
         RDGeom::INT_POINT2D_MAP moveMap;
         for (rpi = rPath.begin(); rpi != rPath.end(); rpi++) {
+          if (d_eatoms[*rpi].df_fixed) continue;
           RDGeom::Point2D move;
           move = d_eatoms[nbrMap[*rpi][0]].loc;
           move += d_eatoms[nbrMap[*rpi][1]].loc;
