@@ -7,13 +7,14 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #pragma once
 #include <stdio.h>
 #include <string.h>
 #include <stddef.h>
 #include <time.h>
 #include <iostream>
-#ifdef WIN32
+#ifdef _MSC_VER
 #define _CRT_SECURE_NO_WARNINGS
 #include <Windows.h>  // for Winmm.lib timeGetTime()
 #ifdef _DEBUG         // check memory leaks
@@ -27,8 +28,10 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/time.h>
+#ifndef _WIN32
 #include <sys/resource.h>
 #endif
+#endif
 
 // SELECT ALGORITHM OPTIONS by comment some lines to exclude additional or
 // experimental optimisations:
@@ -51,7 +54,7 @@
 
 #define VERBOSE_STATISTICS_ON
 
-#ifdef WIN32
+#ifdef _MSC_VER
 #define DELTA_EPOCH_IN_MICROSECS 11644473600000000ULL
 
 struct timezone {
