@@ -20,6 +20,7 @@
 #include <sstream>
 #include "SubstructMatchCustom.h"
 #include "MaximumCommonSubgraph.h"
+#include <GraphMol/QueryOps.h>
 
 namespace RDKit {
 
@@ -38,6 +39,10 @@ void parseMCSParametersJSON(const char* json, MCSParameters* params) {
         pt.get<bool>("MatchValences", p.AtomCompareParameters.MatchValences);
     p.AtomCompareParameters.MatchChiralTag =
         pt.get<bool>("MatchChiralTag", p.AtomCompareParameters.MatchChiralTag);
+    p.AtomCompareParameters.MatchFormalCharge = pt.get<bool>(
+        "MatchFormalCharge", p.AtomCompareParameters.MatchFormalCharge);
+    p.AtomCompareParameters.RingMatchesRingOnly = pt.get<bool>(
+        "RingMatchesRingOnly", p.AtomCompareParameters.RingMatchesRingOnly);
     p.BondCompareParameters.RingMatchesRingOnly = pt.get<bool>(
         "RingMatchesRingOnly", p.BondCompareParameters.RingMatchesRingOnly);
     p.BondCompareParameters.CompleteRingsOnly = pt.get<bool>(
@@ -68,7 +73,7 @@ void parseMCSParametersJSON(const char* json, MCSParameters* params) {
 MCSResult findMCS(const std::vector<ROMOL_SPTR>& mols,
                   const MCSParameters* params) {
   MCSParameters p;
-  if (0 == params) params = &p;
+  if (nullptr == params) params = &p;
   RDKit::FMCS::MaximumCommonSubgraph fmcs(params);
   return fmcs.find(mols);
 }
@@ -87,7 +92,7 @@ MCSResult findMCS(const std::vector<ROMOL_SPTR>& mols, bool maximizeBonds,
                   AtomComparator atomComp, BondComparator bondComp) {
   // AtomComparator atomComp=AtomCompareElements;
   // BondComparator bondComp=BondCompareOrder;
-  MCSParameters* ps = new MCSParameters();
+  auto* ps = new MCSParameters();
   ps->MaximizeBonds = maximizeBonds;
   ps->Threshold = threshold;
   ps->Timeout = timeout;
@@ -105,6 +110,7 @@ MCSResult findMCS(const std::vector<ROMOL_SPTR>& mols, bool maximizeBonds,
       ps->AtomTyper = MCSAtomCompareIsotopes;
       break;
   }
+  ps->AtomCompareParameters.RingMatchesRingOnly = ringMatchesRingOnly;
   switch (bondComp) {
     case BondCompareAny:
       ps->BondTyper = MCSBondCompareAny;
@@ -134,6 +140,26 @@ bool MCSProgressCallbackTimeout(const MCSProgressData& stat,
 // PREDEFINED FUNCTORS:
 
 //=== ATOM COMPARE ========================================================
+static bool checkRingMatch(const MCSAtomCompareParameters& p, const ROMol& mol1,
+                           unsigned int atom1, const ROMol& mol2,
+                           unsigned int atom2) {
+  if (p.RingMatchesRingOnly) {
+    bool atom1inRing = queryIsAtomInRing(mol1.getAtomWithIdx(atom1));
+    bool atom2inRing = queryIsAtomInRing(mol2.getAtomWithIdx(atom2));
+    return atom1inRing == atom2inRing;
+  } else {
+    return true;
+  }
+}
+
+static bool checkAtomCharge(const MCSAtomCompareParameters& p,
+                            const ROMol& mol1, unsigned int atom1,
+                            const ROMol& mol2, unsigned int atom2) {
+  RDUNUSED_PARAM(p);
+  const Atom& a1 = *mol1.getAtomWithIdx(atom1);
+  const Atom& a2 = *mol2.getAtomWithIdx(atom2);
+  return a1.getFormalCharge() == a2.getFormalCharge();
+}
 static bool checkAtomChirality(const MCSAtomCompareParameters& p,
                                const ROMol& mol1, unsigned int atom1,
                                const ROMol& mol2, unsigned int atom2) {
@@ -152,7 +178,12 @@ static bool checkAtomChirality(const MCSAtomCompareParameters& p,
 bool MCSAtomCompareAny(const MCSAtomCompareParameters& p, const ROMol& mol1,
                        unsigned int atom1, const ROMol& mol2,
                        unsigned int atom2, void*) {
-  if (p.MatchChiralTag) return checkAtomChirality(p, mol1, atom1, mol2, atom2);
+  if (p.MatchChiralTag && !checkAtomChirality(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.MatchFormalCharge && !checkAtomCharge(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.RingMatchesRingOnly) return checkRingMatch(p, mol1, atom1, mol2, atom2);
+
   return true;
 }
 
@@ -164,7 +195,11 @@ bool MCSAtomCompareElements(const MCSAtomCompareParameters& p,
   if (a1.getAtomicNum() != a2.getAtomicNum()) return false;
   if (p.MatchValences && a1.getTotalValence() != a2.getTotalValence())
     return false;
-  if (p.MatchChiralTag) return checkAtomChirality(p, mol1, atom1, mol2, atom2);
+  if (p.MatchChiralTag && !checkAtomChirality(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.MatchFormalCharge && !checkAtomCharge(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.RingMatchesRingOnly) return checkRingMatch(p, mol1, atom1, mol2, atom2);
   return true;
 }
 
@@ -178,7 +213,11 @@ bool MCSAtomCompareIsotopes(const MCSAtomCompareParameters& p,
   const Atom& a1 = *mol1.getAtomWithIdx(atom1);
   const Atom& a2 = *mol2.getAtomWithIdx(atom2);
   if (a1.getIsotope() != a2.getIsotope()) return false;
-  if (p.MatchChiralTag) return checkAtomChirality(p, mol1, atom1, mol2, atom2);
+  if (p.MatchChiralTag && !checkAtomChirality(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.MatchFormalCharge && !checkAtomCharge(p, mol1, atom1, mol2, atom2))
+    return false;
+  if (p.RingMatchesRingOnly) return checkRingMatch(p, mol1, atom1, mol2, atom2);
   return true;
 }
 
@@ -226,9 +265,7 @@ static bool checkBondStereo(const MCSBondCompareParameters& p,
   Bond::BondStereo bs1 = b1->getStereo();
   Bond::BondStereo bs2 = b2->getStereo();
   if (b1->getBondType() == Bond::DOUBLE && b2->getBondType() == Bond::DOUBLE) {
-    if ((bs1 == Bond::STEREOZ || bs1 == Bond::STEREOE) &&
-        !(bs2 == Bond::STEREOZ || bs2 == Bond::STEREOE))
-      return false;
+    if (bs1 > Bond::STEREOANY && !(bs2 > Bond::STEREOANY)) return false;
   }
   return true;
 }
@@ -241,9 +278,9 @@ static bool checkRingMatch(const MCSBondCompareParameters& p, const ROMol& mol1,
       static_cast<FMCS::RingMatchTableSet*>(v_ringMatchMatrixSet);
 
   const std::vector<size_t>& ringsIdx1 =
-      ringMatchMatrixSet->getQueryBondRings(bond1);  // indeces of rings
+      ringMatchMatrixSet->getQueryBondRings(bond1);  // indices of rings
   const std::vector<size_t>& ringsIdx2 =
-      ringMatchMatrixSet->getTargetBondRings(&mol2, bond2);  // indeces of rings
+      ringMatchMatrixSet->getTargetBondRings(&mol2, bond2);  // indices of rings
   bool bond1inRing = !ringsIdx1.empty();
   bool bond2inRing = !ringsIdx2.empty();
 
@@ -256,14 +293,12 @@ static bool checkRingMatch(const MCSBondCompareParameters& p, const ROMol& mol1,
     const RingInfo::VECT_INT_VECT& r1 = mol1.getRingInfo()->bondRings();
     const RingInfo::VECT_INT_VECT& r2 = mol2.getRingInfo()->bondRings();
     // for each query ring contains bond1
-    for (std::vector<size_t>::const_iterator r1i = ringsIdx1.begin();
-         r1i != ringsIdx1.end(); r1i++) {
-      const INT_VECT& br1 = r1[*r1i];  // ring contains bond1
+    for (unsigned long r1i : ringsIdx1) {
+      const INT_VECT& br1 = r1[r1i];  // ring contains bond1
       // check all target rings contained bond2
-      for (std::vector<size_t>::const_iterator r2i = ringsIdx2.begin();
-           r2i != ringsIdx2.end(); r2i++) {
-        const INT_VECT& br2 = r2[*r2i];  // ring contains bond2
-        if (br1.size() != br2.size())    // rings are different
+      for (unsigned long r2i : ringsIdx2) {
+        const INT_VECT& br2 = r2[r2i];  // ring contains bond2
+        if (br1.size() != br2.size())   // rings are different
           continue;
         // compare rings as substructures
         if (ringMatchMatrixSet->isEqual(&br1, &br2,
@@ -340,7 +375,7 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
     ///*------------------ OLD Code :
     // ???: non chiral query atoms ARE ALLOWED TO MATCH to Chiral target atoms
     // (see test for issue 481)
-    if (a1.getDegree() < 3 ||//#688: doesn't deal with "explicit" Hs properly
+    if (a1.getDegree() < 3 ||  //#688: doesn't deal with "explicit" Hs properly
         !(ac1 == Atom::CHI_TETRAHEDRAL_CW || ac1 == Atom::CHI_TETRAHEDRAL_CCW))
       continue;  // skip non chiral center QUERY atoms
     if (!(ac2 == Atom::CHI_TETRAHEDRAL_CW || ac2 == Atom::CHI_TETRAHEDRAL_CCW))
@@ -363,7 +398,7 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
     */
     const unsigned a1Degree =
         boost::out_degree(c1[i], query);  // a1.getDegree();
-        //number of all connected atoms in a seed
+    // number of all connected atoms in a seed
     if (a1Degree > a2.getDegree()) {  //#688 was != . // FIX issue 631
       // printf("atoms Degree (%u, %u) %u [%u], %u\n", query[c1[i]],
       // target[c2[i]], a1Degree, a1.getDegree(), a2.getDegree());
@@ -382,17 +417,17 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
     //#688
     INT_LIST qmoOrder;
     {
-        ROMol::OEDGE_ITER dbeg, dend;
-        boost::tie(dbeg, dend) = mol1.getAtomBonds(&a1);
-        for (; dbeg != dend; dbeg++) {
-            int dbidx = mol1[*dbeg]->getIdx();
-            if (std::find(qOrder.begin(), qOrder.end(), dbidx) != qOrder.end())
-                qmoOrder.push_back(dbidx);
-//            else
-//                qmoOrder.push_back(-1);
-        }
+      ROMol::OEDGE_ITER dbeg, dend;
+      boost::tie(dbeg, dend) = mol1.getAtomBonds(&a1);
+      for (; dbeg != dend; dbeg++) {
+        int dbidx = mol1[*dbeg]->getIdx();
+        if (std::find(qOrder.begin(), qOrder.end(), dbidx) != qOrder.end())
+          qmoOrder.push_back(dbidx);
+        //            else
+        //                qmoOrder.push_back(-1);
+      }
     }
-    int qPermCount = //was: a1.getPerturbationOrder(qOrder);
+    int qPermCount =  // was: a1.getPerturbationOrder(qOrder);
         static_cast<int>(countSwapsToInterconvert(qmoOrder, qOrder));
 
     INT_LIST mOrder;
@@ -400,24 +435,24 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
       const Bond* mB = mol2.getBondBetweenAtoms(target[c2[i]], target[c2[j]]);
       if (mB) mOrder.push_back(mB->getIdx());
     }
-    
+
     //#688
     while (mOrder.size() < a2.getDegree()) {
-        mOrder.push_back(-1);
+      mOrder.push_back(-1);
     }
     INT_LIST moOrder;
     ROMol::OEDGE_ITER dbeg, dend;
     boost::tie(dbeg, dend) = mol2.getAtomBonds(&a2);
     for (; dbeg != dend; dbeg++) {
-        int dbidx = mol2[*dbeg]->getIdx();
-        if (std::find(mOrder.begin(), mOrder.end(), dbidx) != mOrder.end())
-            moOrder.push_back(dbidx);
-        else
-            moOrder.push_back(-1);
+      int dbidx = mol2[*dbeg]->getIdx();
+      if (std::find(mOrder.begin(), mOrder.end(), dbidx) != mOrder.end())
+        moOrder.push_back(dbidx);
+      else
+        moOrder.push_back(-1);
     }
 
-    int mPermCount = //was: a2.getPerturbationOrder(mOrder);
-            static_cast<int>(countSwapsToInterconvert(moOrder, mOrder));
+    int mPermCount =  // was: a2.getPerturbationOrder(mOrder);
+        static_cast<int>(countSwapsToInterconvert(moOrder, mOrder));
     //----
 
     if ((qPermCount % 2 == mPermCount % 2 &&
@@ -436,8 +471,7 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
   for (unsigned int i = 0; i < qnb; i++, ++bIter) {
     const Bond* qBnd = mol1.getBondWithIdx(query[*bIter]);
     if (qBnd->getBondType() != Bond::DOUBLE ||
-        (qBnd->getStereo() != Bond::STEREOZ &&
-         qBnd->getStereo() != Bond::STEREOE))
+        qBnd->getStereo() <= Bond::STEREOANY)
       continue;
     // don't think this can actually happen, but check to be sure:
     if (qBnd->getStereoAtoms().size() != 2)  // MUST check it in the seed, not
@@ -450,8 +484,7 @@ bool FinalChiralityCheckFunction(const short unsigned c1[],
                                  target[c2[qMap[qBnd->getEndAtomIdx()]]]);
     CHECK_INVARIANT(mBnd, "Matching bond not found");
     if (mBnd->getBondType() != Bond::DOUBLE ||
-        (mBnd->getStereo() != Bond::STEREOZ &&
-         mBnd->getStereo() != Bond::STEREOE))
+        mBnd->getStereo() <= Bond::STEREOANY)
       continue;
     // don't think this can actually happen, but check to be sure:
     if (mBnd->getStereoAtoms().size() != 2) continue;
@@ -543,8 +576,7 @@ bool FinalChiralityCheckFunction_1(const short unsigned c1[],
   for (unsigned int i = 0; i < qnb; i++, ++bIter) {
     const Bond* qBnd = mol1.getBondWithIdx(query[*bIter]);
     if (qBnd->getBondType() != Bond::DOUBLE ||
-        (qBnd->getStereo() != Bond::STEREOZ &&
-         qBnd->getStereo() != Bond::STEREOE))
+        qBnd->getStereo() <= Bond::STEREOANY)
       continue;
     // don't think this can actually happen, but check to be sure:
     if (qBnd->getStereoAtoms().size() != 2)  // MUST check it in the seed, not
@@ -557,8 +589,7 @@ bool FinalChiralityCheckFunction_1(const short unsigned c1[],
                                  target[c2[qMap[qBnd->getEndAtomIdx()]]]);
     CHECK_INVARIANT(mBnd, "Matching bond not found");
     if (mBnd->getBondType() != Bond::DOUBLE ||
-        (mBnd->getStereo() != Bond::STEREOZ &&
-         mBnd->getStereo() != Bond::STEREOE))
+        mBnd->getStereo() <= Bond::STEREOANY)
       continue;
     // don't think this can actually happen, but check to be sure:
     if (mBnd->getStereoAtoms().size() != 2) continue;
