@@ -35,7 +35,6 @@
 //
 
 #include "FileParsers.h"
-#include "MolFileStereochem.h"
 #include <RDGeneral/Invariant.h>
 #include <GraphMol/RDKitQueries.h>
 #include <RDGeneral/StreamOps.h>
@@ -79,7 +78,7 @@ void readFormalChargesFromAttr(std::istream *inStream, RWMol *res) {
   PRECONDITION(inStream, "inStream not valid");
   PRECONDITION(!inStream->eof(), "inStream is at eof");
   PRECONDITION(res, "RWMol not valid");
-  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+  typedef boost::tokenizer<boost::char_separator<char>> tokenizer;
   boost::char_separator<char> sep(" \t\n");
   bool readNextAtomAttribs = true;
   unsigned int atomIdx = 0, noAtomAttr = 0;
@@ -204,8 +203,10 @@ void guessFormalCharges(RWMol *res) {
         std::string nm;
         res->getProp(common_properties::_Name, nm);
         BOOST_LOG(rdWarningLog)
-            << nm << ": warning - aromatic N with 3 aromatic bonds - "
-                     "skipping charge guess for this atom" << std::endl;
+            << nm
+            << ": warning - aromatic N with 3 aromatic bonds - "
+               "skipping charge guess for this atom"
+            << std::endl;
         continue;
       }
 
@@ -307,8 +308,8 @@ bool cleanUpMol2Substructures(RWMol *res) {
       // phosphates,
       // FIX: do it also for phsopahtes and sulphates ...
       if (at->getDegree() != 1) {
-        BOOST_LOG(rdWarningLog) << "Warning - O.co2 with degree >1."
-                                << std::endl;
+        BOOST_LOG(rdWarningLog)
+            << "Warning - O.co2 with degree >1." << std::endl;
         return false;
       }
       ROMol::ADJ_ITER nbrIdxIt, endNbrsIdxIt;
@@ -509,7 +510,7 @@ bool cleanUpMol2Substructures(RWMol *res) {
 }
 
 Atom *ParseMol2FileAtomLine(const std::string atomLine, RDGeom::Point3D &pos) {
-  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+  typedef boost::tokenizer<boost::char_separator<char>> tokenizer;
   boost::char_separator<char> sep(" \t\n");
   std::string tAN, tAT;
   tokenizer tokens(atomLine, sep);
@@ -518,7 +519,7 @@ Atom *ParseMol2FileAtomLine(const std::string atomLine, RDGeom::Point3D &pos) {
     throw FileParseException("no info in mol2 atom line");
   }
 
-  Atom *res = new Atom();
+  auto *res = new Atom();
 
   // skip TriposAtomId
   ++itemIt;
@@ -560,22 +561,22 @@ Atom *ParseMol2FileAtomLine(const std::string atomLine, RDGeom::Point3D &pos) {
   // LP is not an atom so remove it ...
   if (symb == "LP") {
     delete res;
-    return NULL;
+    return nullptr;
   } else if (symb == "ANY" || symb == "Du") {
     // queryAtoms
     // according to the SYBYL spec, these match anything
-    QueryAtom *query = new QueryAtom(0);
+    auto *query = new QueryAtom(0);
     query->setQuery(makeAtomNullQuery());
     delete res;
     res = query;
   } else if (symb == "HEV") {
-    QueryAtom *query = new QueryAtom(1);
+    auto *query = new QueryAtom(1);
     query->getQuery()->setNegation(true);
     delete res;
     res = query;
   } else if (symb == "HET") {
     // Tripos: N,O,P,S
-    QueryAtom *query = new QueryAtom(7);
+    auto *query = new QueryAtom(7);
     query->expandQuery(makeAtomNumQuery(8), Queries::COMPOSITE_OR, true);
     query->expandQuery(makeAtomNumQuery(15), Queries::COMPOSITE_OR, true);
     query->expandQuery(makeAtomNumQuery(16), Queries::COMPOSITE_OR, true);
@@ -583,7 +584,7 @@ Atom *ParseMol2FileAtomLine(const std::string atomLine, RDGeom::Point3D &pos) {
     res = query;
   } else if (symb == "HAL") {
     // Tripos: F,Cl,Br,I
-    QueryAtom *query = new QueryAtom(9);
+    auto *query = new QueryAtom(9);
     query->expandQuery(makeAtomNumQuery(17), Queries::COMPOSITE_OR, true);
     query->expandQuery(makeAtomNumQuery(35), Queries::COMPOSITE_OR, true);
     query->expandQuery(makeAtomNumQuery(53), Queries::COMPOSITE_OR, true);
@@ -624,7 +625,7 @@ Bond *ParseMol2FileBondLine(const std::string bondLine,
                             const INT_VECT &idxCorresp) {
   unsigned int idx1, idx2;
 
-  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+  typedef boost::tokenizer<boost::char_separator<char>> tokenizer;
   boost::char_separator<char> sep(" \t\n");
 
   tokenizer tokens(bondLine, sep);
@@ -663,7 +664,7 @@ Bond *ParseMol2FileBondLine(const std::string bondLine,
   }
 
   if (idxCorresp[idx1] < 0 || idxCorresp[idx2] < 0) {
-    return NULL;
+    return nullptr;
   }
 
   // lexical casts of bond types is not really useful in this case since we
@@ -689,9 +690,9 @@ Bond *ParseMol2FileBondLine(const std::string bondLine,
     // but why would anyone specify a bond which is not connected?
     BOOST_LOG(rdWarningLog) << "Warning - unsupported bond type: " << tBType
                             << " ignored!" << std::endl;
-    return NULL;
+    return nullptr;
   }
-  Bond *res = new Bond(type);
+  auto *res = new Bond(type);
   res->setBeginAtomIdx(idxCorresp[idx1]);
   res->setEndAtomIdx(idxCorresp[idx2]);
   return res;
@@ -734,12 +735,14 @@ void ParseMol2AtomBlock(std::istream *inStream, RWMol *res, unsigned int nAtoms,
   if (!hasHAtoms) {
     std::string nm;
     res->getProp(common_properties::_Name, nm);
-    BOOST_LOG(rdWarningLog) << nm << ": Warning - no explicit hydrogens in "
-                                     "mol2 file but needed for formal charge "
-                                     "estimation." << std::endl;
+    BOOST_LOG(rdWarningLog) << nm
+                            << ": Warning - no explicit hydrogens in "
+                               "mol2 file but needed for formal charge "
+                               "estimation."
+                            << std::endl;
   }
   // create conformer based on 3DPoints and add to RWMol
-  Conformer *conf = new Conformer(nAtoms - nLP);
+  auto *conf = new Conformer(nAtoms - nLP);
   std::vector<RDGeom::Point3D>::const_iterator threeDPsIt = threeDPs.begin();
   for (unsigned int i = 0; i < threeDPs.size(); ++i) {
     conf->setAtomPos(i, *threeDPsIt);
@@ -790,11 +793,11 @@ void ParseMol2BondBlock(std::istream *inStream, RWMol *res, unsigned int nBonds,
 //
 //------------------------------------------------
 RWMol *Mol2DataStreamToMol(std::istream *inStream, bool sanitize, bool removeHs,
-                           Mol2Type variant) {
+                           Mol2Type variant, bool cleanupSubstructures) {
   RDUNUSED_PARAM(variant);
   PRECONDITION(inStream, "no stream");
   std::string tempStr, lineBeg;
-  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+  typedef boost::tokenizer<boost::char_separator<char>> tokenizer;
   boost::char_separator<char> sep(" \t\n");
   Utils::LocaleSwitcher ls;
 
@@ -852,7 +855,7 @@ RWMol *Mol2DataStreamToMol(std::istream *inStream, bool sanitize, bool removeHs,
 
   inStream->seekg(molStart, std::ios::beg);
   tempStr = getLine(inStream);
-  RWMol *res = new RWMol();
+  auto *res = new RWMol();
   boost::trim_right(tempStr);
   res->setProp(common_properties::_Name, tempStr);
 
@@ -919,11 +922,16 @@ RWMol *Mol2DataStreamToMol(std::istream *inStream, bool sanitize, bool removeHs,
   }
 
   if (!chargeStart) {
-    bool molFixed = cleanUpMol2Substructures(res);
+    bool molFixed;
+    if (cleanupSubstructures) {
+      molFixed = cleanUpMol2Substructures(res);
+    } else {
+      molFixed = true;
+    }
 
     if (!molFixed) {
       delete res;
-      return NULL;
+      return nullptr;
     }
 
     // mol2 format does not support formal charge information, hence we need to
@@ -959,11 +967,10 @@ RWMol *Mol2DataStreamToMol(std::istream *inStream, bool sanitize, bool removeHs,
         MolOps::sanitizeMol(*res);
       }
 
-      // call DetectBondStereoChemistry after sanitization "because we need
+      // call detectBondStereochemistry after sanitization "because we need
       // the ring information".  Also this will set the E/Z labels on the bond.
       // Similar in spirit to what happens in MolFileParser
-      const Conformer &conf = res->getConformer();
-      DetectBondStereoChemistry(*res, &conf);
+      MolOps::detectBondStereochemistry(*res);
 
     } catch (MolSanitizeException &se) {
       BOOST_LOG(rdWarningLog) << "sanitise ";
@@ -982,9 +989,10 @@ RWMol *Mol2DataStreamToMol(std::istream *inStream, bool sanitize, bool removeHs,
 };
 
 RWMol *Mol2DataStreamToMol(std::istream &inStream, bool sanitize, bool removeHs,
-                           Mol2Type variant) {
+                           Mol2Type variant, bool cleanupSubstructures) {
   RDUNUSED_PARAM(variant);
-  return Mol2DataStreamToMol(&inStream, sanitize, removeHs);
+  return Mol2DataStreamToMol(&inStream, sanitize, removeHs, variant,
+                             cleanupSubstructures);
 };
 //------------------------------------------------
 //
@@ -992,10 +1000,11 @@ RWMol *Mol2DataStreamToMol(std::istream &inStream, bool sanitize, bool removeHs,
 //
 //------------------------------------------------
 RWMol *Mol2BlockToMol(const std::string &molBlock, bool sanitize, bool removeHs,
-                      Mol2Type variant) {
+                      Mol2Type variant, bool cleanupSubstructures) {
   RDUNUSED_PARAM(variant);
   std::istringstream inStream(molBlock);
-  return Mol2DataStreamToMol(inStream, sanitize, removeHs);
+  return Mol2DataStreamToMol(inStream, sanitize, removeHs, variant,
+                             cleanupSubstructures);
 }
 
 //------------------------------------------------
@@ -1004,7 +1013,7 @@ RWMol *Mol2BlockToMol(const std::string &molBlock, bool sanitize, bool removeHs,
 //
 //------------------------------------------------
 RWMol *Mol2FileToMol(const std::string &fName, bool sanitize, bool removeHs,
-                     Mol2Type variant) {
+                     Mol2Type variant, bool cleanupSubstructures) {
   // FIX: this binary mode of opening file is here because of a bug in VC++ 6.0
   // the function "tellg" does not work correctly if we do not open it this way
   //   Jan 2009: Confirmed that this is still the case in visual studio 2008
@@ -1015,10 +1024,11 @@ RWMol *Mol2FileToMol(const std::string &fName, bool sanitize, bool removeHs,
     errout << "Bad input file " << fName;
     throw BadFileException(errout.str());
   }
-  RWMol *res = NULL;
+  RWMol *res = nullptr;
   if (!inStream.eof()) {
-    res = Mol2DataStreamToMol(inStream, sanitize, removeHs);
+    res = Mol2DataStreamToMol(inStream, sanitize, removeHs, variant,
+                              cleanupSubstructures);
   }
   return res;
 }
-}
+}  // namespace RDKit
