@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2002-2014 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2002-2017 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -8,27 +7,44 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
-
-#include "FileParsers.h"
-#include "FileParserUtils.h"
-#include "MolFileStereochem.h"
-#include <GraphMol/RDKitQueries.h>
-#include <RDGeneral/StreamOps.h>
-#include <RDGeneral/RDLog.h>
-
-#include <fstream>
+#include <RDGeneral/BoostStartInclude.h>
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/tokenizer.hpp>
 #include <boost/algorithm/string/trim.hpp>
+#include <RDGeneral/BoostEndInclude.h>
+
+#include "FileParsers.h"
+#include "FileParserUtils.h"
+#include "MolFileStereochem.h"
+
+#include <GraphMol/SmilesParse/SmilesParse.h>
+#include <GraphMol/RDKitQueries.h>
+#include <GraphMol/StereoGroup.h>
+#include <RDGeneral/StreamOps.h>
+#include <RDGeneral/RDLog.h>
+
+#include <fstream>
 #include <RDGeneral/FileParseException.h>
 #include <RDGeneral/BadFileException.h>
 #include <RDGeneral/LocaleSwitcher.h>
 #include <typeinfo>
 #include <exception>
+#ifdef RDKIT_USE_BOOST_REGEX
+#include <boost/regex.hpp>
+using boost::regex;
+using boost::regex_match;
+using boost::smatch;
+#else
+#include <regex>
+using std::regex;
+using std::regex_match;
+using std::smatch;
+#endif
 #include <sstream>
 #include <locale>
 #include <stdlib.h>
+#include <cstdio>
 
 namespace RDKit {
 class MolFileUnhandledFeatureException : public std::exception {
@@ -40,7 +56,7 @@ class MolFileUnhandledFeatureException : public std::exception {
       : _msg(msg){};
   //! get the error message
   const char *message() const { return _msg.c_str(); };
-  ~MolFileUnhandledFeatureException() throw(){};
+  ~MolFileUnhandledFeatureException() throw() override{};
 
  private:
   std::string _msg;
@@ -51,7 +67,7 @@ int toInt(const std::string &input, bool acceptSpaces) {
   int res = 0;
   // don't need to worry about locale stuff here because
   // we're not going to have delimiters
-  res = strtol(input.c_str(), NULL, 10);
+  res = strtol(input.c_str(), nullptr, 10);
   if (!res && !acceptSpaces && input[0] == ' ') {
     std::string trimmed = boost::trim_copy(input);
     if (trimmed.length() == 0) throw boost::bad_lexical_cast();
@@ -117,7 +133,7 @@ Atom *replaceAtomWithQueryAtom(RWMol *mol, Atom *atom) {
   mol->replaceAtom(idx, &qa);
   return mol->getAtomWithIdx(idx);
 }
-}
+}  // namespace FileParserUtils
 using RDKit::FileParserUtils::getV3000Line;
 
 namespace {
@@ -136,7 +152,7 @@ void completeQueryAndChildren(ATOM_EQUALS_QUERY *query, Atom *tgt,
                              magicVal);
   }
 }
-void CompleteMolQueries(RWMol *mol, int magicVal = -0xDEADBEEF) {
+void completeMolQueries(RWMol *mol, int magicVal = 0xDEADBEEF) {
   for (ROMol::AtomIterator ai = mol->beginAtoms(); ai != mol->endAtoms();
        ++ai) {
     if ((*ai)->hasQuery()) {
@@ -147,6 +163,71 @@ void CompleteMolQueries(RWMol *mol, int magicVal = -0xDEADBEEF) {
   }
 }
 
+bool startsWith(const std::string &haystack, const char *needle, size_t size) {
+  return haystack.compare(0u, size, needle, size) == 0;
+}
+
+//! parse a collection block to find enhanced stereo groups
+std::string parseEnhancedStereo(std::istream *inStream, unsigned int &line,
+                                RWMol *mol) {
+  // Lines like (absolute, relative, racemic):
+  // M  V30 MDLV30/STEABS ATOMS=(2 2 3)
+  // M  V30 MDLV30/STEREL1 ATOMS=(1 12)
+  // M  V30 MDLV30/STERAC1 ATOMS=(1 12)
+  const regex stereo_label(
+      R"regex(MDLV30/STE(...)[0-9]* +ATOMS=\(([0-9]+) +(.*)\))regex");
+
+  smatch match;
+  std::vector<StereoGroup> groups;
+
+  // Read the collection until the end
+  auto tempStr = getV3000Line(inStream, line);
+  boost::to_upper(tempStr);
+  while (!startsWith(tempStr, "END", 3)) {
+    // If this line in the collection is part of a stereo group
+    if (regex_match(tempStr, match, stereo_label)) {
+      StereoGroupType grouptype = RDKit::StereoGroupType::STEREO_ABSOLUTE;
+
+      if (match[1] == "ABS") {
+        grouptype = RDKit::StereoGroupType::STEREO_ABSOLUTE;
+      } else if (match[1] == "REL") {
+        grouptype = RDKit::StereoGroupType::STEREO_OR;
+      } else if (match[1] == "RAC") {
+        grouptype = RDKit::StereoGroupType::STEREO_AND;
+      } else {
+        std::ostringstream errout;
+        errout << "Unrecognized stereogroup type : '" << tempStr << "' on line"
+               << line;
+        throw FileParseException(errout.str());
+      }
+
+      const unsigned int count = FileParserUtils::toInt(match[2], true);
+      std::vector<Atom *> atoms;
+      std::stringstream ss(match[3]);
+      unsigned int index;
+      for (size_t i = 0; i < count; ++i) {
+        ss >> index;
+        // atoms are 1 indexed in molfiles
+        atoms.push_back(mol->getAtomWithIdx(index - 1));
+      }
+      groups.emplace_back(grouptype, std::move(atoms));
+    } else {
+      // skip collection types we don't know how to read. Only one documented
+      // is MDLV30/HILITE
+      BOOST_LOG(rdWarningLog) << "Skipping unrecognized collection type at "
+                                 "line "
+                              << line << ": " << tempStr << std::endl;
+    }
+    tempStr = getV3000Line(inStream, line);
+  }
+
+  if (!groups.empty()) {
+    mol->setStereoGroups(std::move(groups));
+  }
+  tempStr = getV3000Line(inStream, line);
+  return tempStr;
+}
+
 //*************************************
 //
 // Every effort has been made to adhere to MDL's standard
@@ -167,10 +248,10 @@ void ParseOldAtomList(RWMol *mol, const std::string &text, unsigned int line) {
     throw FileParseException(errout.str());
   }
 
-  URANGE_CHECK(idx, mol->getNumAtoms() - 1);
+  URANGE_CHECK(idx, mol->getNumAtoms());
   QueryAtom a(*(mol->getAtomWithIdx(idx)));
 
-  ATOM_OR_QUERY *q = new ATOM_OR_QUERY;
+  auto *q = new ATOM_OR_QUERY;
   q->setDescription("AtomOr");
 
   switch (text[4]) {
@@ -216,6 +297,8 @@ void ParseOldAtomList(RWMol *mol, const std::string &text, unsigned int line) {
   }
 
   a.setQuery(q);
+  a.setProp(common_properties::_MolFileAtomQuery, 1);
+
   mol->replaceAtom(idx, &a);
 };
 
@@ -292,11 +375,11 @@ void ParseSGroup2000STYLine(RWMol *mol, const std::string &text,
       errout << "SGroup line too short: '" << text << "' on line " << line;
       throw FileParseException(errout.str());
     }
-#if 0        
+#if 0
         int nbr;
         try {
           nbr = FileParserUtils::toInt(text.substr(spos,4));
-        } 
+        }
         catch (boost::bad_lexical_cast &) {
           std::ostringstream errout;
           errout << "Cannot convert " << text.substr(spos,3) << " to int on line "<<line;
@@ -310,8 +393,8 @@ void ParseSGroup2000STYLine(RWMol *mol, const std::string &text,
       errout << "S group " << typ;
       throw MolFileUnhandledFeatureException(errout.str());
     } else {
-      BOOST_LOG(rdWarningLog) << " S group " << typ << " ignored on line "
-                              << line << std::endl;
+      BOOST_LOG(rdWarningLog)
+          << " S group " << typ << " ignored on line " << line << std::endl;
     }
     spos += 4;
   }
@@ -460,7 +543,8 @@ void ParseSubstitutionCountLine(RWMol *mol, const std::string &text,
           case 6:
             BOOST_LOG(rdWarningLog) << " atom degree query with value 6 found. "
                                        "This will not match degree >6. The MDL "
-                                       "spec says it should.  line: " << line;
+                                       "spec says it should.  line: "
+                                    << line;
             q->setVal(6);
             break;
           default:
@@ -519,9 +603,11 @@ void ParseUnsaturationLine(RWMol *mol, const std::string &text,
           atom->expandQuery(q, Queries::COMPOSITE_AND);
         } else {
           std::ostringstream errout;
-          errout << "Value " << count << " is not supported as an unsaturation "
-                                         "query (only 0 and 1 are allowed). "
-                                         "line: " << line;
+          errout << "Value " << count
+                 << " is not supported as an unsaturation "
+                    "query (only 0 and 1 are allowed). "
+                    "line: "
+                 << line;
           throw FileParseException(errout.str());
         }
       }
@@ -566,7 +652,7 @@ void ParseRingBondCountLine(RWMol *mol, const std::string &text,
             q->setVal(0);
             break;
           case -2:
-            q->setVal(-0xDEADBEEF);
+            q->setVal(0xDEADBEEF);
             mol->setProp(common_properties::_NeedsQueryScan, 1);
             break;
           case 1:
@@ -762,6 +848,60 @@ void ParseZBOLine(RWMol *mol, const std::string &text, unsigned int line) {
   }
 }
 
+void ParseMarvinSmartsLine(RWMol *mol, const std::string &text,
+                           unsigned int line) {
+  const unsigned int atomNumStart = 10;
+  const unsigned int smartsStart = 15;
+  const unsigned int SMA = 7;
+  // M  MRV SMA   1 [*;A]
+  // 01234567890123456789
+  //           1111111111
+  if (text.substr(SMA, 3) != "SMA") {
+    return;
+  }
+
+  unsigned int idx;
+  std::string idxTxt = text.substr(atomNumStart, smartsStart - atomNumStart);
+  try {
+    idx = FileParserUtils::stripSpacesAndCast<unsigned int>(idxTxt) - 1;
+  } catch (boost::bad_lexical_cast &) {
+    std::ostringstream errout;
+    errout << "Cannot convert " << idxTxt << " to an atom index on line "
+           << line;
+    throw FileParseException(errout.str());
+  }
+
+  URANGE_CHECK(idx, mol->getNumAtoms());
+  // Should we check the validity of the marvin line here?  Should we
+  // automatically
+  //   Add these as recursive smarts?  I tend to think so...
+  std::string sma = text.substr(smartsStart);
+  Atom *at = mol->getAtomWithIdx(idx);
+  at->setProp(common_properties::MRV_SMA, sma);
+  RWMol *m = 0;
+  try {
+    m = SmartsToMol(sma);
+  } catch (...) {
+    // Is this every used?
+  }
+
+  if (m) {
+    QueryAtom::QUERYATOM_QUERY *query = new RecursiveStructureQuery(m);
+    if (!at->hasQuery()) {
+      QueryAtom qAt(*at);
+      int oidx = at->getIdx();
+      mol->replaceAtom(oidx, &qAt);
+      at = mol->getAtomWithIdx(oidx);
+    }
+    at->expandQuery(query, Queries::COMPOSITE_AND);
+    at->setProp(common_properties::_MolFileAtomQuery, 1);
+  } else {
+    std::ostringstream errout;
+    errout << "Cannot parse smarts: '" << sma << "' on line " << line;
+    throw FileParseException(errout.str());
+  }
+}
+
 void ParseNewAtomList(RWMol *mol, const std::string &text, unsigned int line) {
   if (text.size() < 15) {
     std::ostringstream errout;
@@ -782,8 +922,8 @@ void ParseNewAtomList(RWMol *mol, const std::string &text, unsigned int line) {
            << line;
     throw FileParseException(errout.str());
   }
-  URANGE_CHECK(idx, mol->getNumAtoms() - 1);
-  QueryAtom *a = 0;
+  URANGE_CHECK(idx, mol->getNumAtoms());
+  QueryAtom *a = nullptr;
 
   int nQueries;
   try {
@@ -795,7 +935,18 @@ void ParseNewAtomList(RWMol *mol, const std::string &text, unsigned int line) {
     throw FileParseException(errout.str());
   }
 
-  ASSERT_INVARIANT(nQueries > 0, "no queries provided");
+  if (!nQueries) {
+    BOOST_LOG(rdWarningLog) << "Empty atom list: '" << text << "' on line "
+                            << line << "." << std::endl;
+    return;
+  }
+
+  if (nQueries < 0) {
+    std::ostringstream errout;
+    errout << "negative length atom list: '" << text << "' on line " << line
+           << "." << std::endl;
+    throw FileParseException(errout.str());
+  }
   for (unsigned int i = 0; i < static_cast<unsigned int>(nQueries); i++) {
     unsigned int pos = 16 + i * 4;
     if (text.size() < pos + 4) {
@@ -810,16 +961,14 @@ void ParseNewAtomList(RWMol *mol, const std::string &text, unsigned int line) {
     if (!i) {
       a = new QueryAtom(*(mol->getAtomWithIdx(idx)));
       // replace the query:
-      Atom::QUERYATOM_QUERY *oq = a->getQuery();
       a->setAtomicNum(atNum);
       a->setQuery(makeAtomNumQuery(atNum));
-      delete oq;
     } else {
       a->expandQuery(makeAtomNumQuery(atNum), Queries::COMPOSITE_OR, true);
     }
   }
   ASSERT_INVARIANT(a, "no atom built");
-
+  a->setProp(common_properties::_MolFileAtomQuery, 1);
   switch (text[14]) {
     case 'T':
       a->getQuery()->setNegation(true);
@@ -849,7 +998,7 @@ void ParseV3000RGroups(RWMol *mol, Atom *&atom, const std::string &text,
   }
   std::vector<std::string> splitToken;
   std::string resid = text.substr(1, text.size() - 2);
-  boost::split(splitToken, resid, boost::is_any_of(" "));
+  boost::split(splitToken, resid, boost::is_any_of(std::string(" ")));
   if (splitToken.size() < 1) {
     std::ostringstream errout;
     errout << "Bad RGROUPS specification " << text << " on line " << line
@@ -883,7 +1032,7 @@ void ParseV3000RGroups(RWMol *mol, Atom *&atom, const std::string &text,
     }
     atom = FileParserUtils::replaceAtomWithQueryAtom(mol, atom);
     atom->setProp(common_properties::_MolFileRLabel, rLabel);
-    std::string dLabel = "R" + boost::lexical_cast<std::string>(rLabel);
+    std::string dLabel = "R" + std::to_string(rLabel);
     atom->setProp(common_properties::dummyLabel, dLabel);
     atom->setIsotope(rLabel);
     atom->setQuery(makeAtomNullQuery());
@@ -940,7 +1089,7 @@ void ParseRGroupLabels(RWMol *mol, const std::string &text, unsigned int line) {
     // set the dummy label so that this is shown correctly
     // in other pieces of the code :
     // (this was sf.net issue 3316600)
-    std::string dLabel = "R" + boost::lexical_cast<std::string>(rLabel);
+    std::string dLabel = "R" + std::to_string(rLabel);
     qatom.setProp(common_properties::dummyLabel, dLabel);
 
     // the CTFile spec (June 2005 version) technically only allows
@@ -969,7 +1118,7 @@ void ParseAtomAlias(RWMol *mol, std::string text, const std::string &nextLine,
            << line;
     throw FileParseException(errout.str());
   }
-  URANGE_CHECK(idx, mol->getNumAtoms() - 1);
+  URANGE_CHECK(idx, mol->getNumAtoms());
   Atom *at = mol->getAtomWithIdx(idx);
   at->setProp(common_properties::molFileAlias, nextLine);
 };
@@ -988,7 +1137,7 @@ void ParseAtomValue(RWMol *mol, std::string text, unsigned int line) {
            << line;
     throw FileParseException(errout.str());
   }
-  URANGE_CHECK(idx, mol->getNumAtoms() - 1);
+  URANGE_CHECK(idx, mol->getNumAtoms());
   Atom *at = mol->getAtomWithIdx(idx);
   at->setProp(common_properties::molFileValue,
               text.substr(7, text.length() - 7));
@@ -996,7 +1145,7 @@ void ParseAtomValue(RWMol *mol, std::string text, unsigned int line) {
 
 Atom *ParseMolFileAtomLine(const std::string text, RDGeom::Point3D &pos,
                            unsigned int line) {
-  Atom *res = new Atom;
+  auto *res = new Atom;
   std::string symb;
   int massDiff, chg, hCount;
 
@@ -1056,22 +1205,14 @@ Atom *ParseMolFileAtomLine(const std::string text, RDGeom::Point3D &pos,
       symb == "LP" || symb == "R" || symb == "R#" ||
       (symb[0] == 'R' && symb >= "R0" && symb <= "R99")) {
     if (symb == "A" || symb == "Q" || symb == "*") {
-      QueryAtom *query = new QueryAtom(0);
+      auto *query = new QueryAtom(0);
       if (symb == "*") {
         // according to the MDL spec, these match anything
         query->setQuery(makeAtomNullQuery());
       } else if (symb == "Q") {
-        ATOM_OR_QUERY *q = new ATOM_OR_QUERY;
-        q->setDescription("AtomOr");
-        q->setNegation(true);
-        q->addChild(
-            QueryAtom::QUERYATOM_QUERY::CHILD_TYPE(makeAtomNumQuery(6)));
-        q->addChild(
-            QueryAtom::QUERYATOM_QUERY::CHILD_TYPE(makeAtomNumQuery(1)));
-        query->setQuery(q);
+        query->setQuery(makeQAtomQuery());
       } else if (symb == "A") {
-        query->setQuery(makeAtomNumQuery(1));
-        query->getQuery()->setNegation(true);
+        query->setQuery(makeAAtomQuery());
       }
       delete res;
       res = query;
@@ -1265,7 +1406,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
   idx2--;
 
   Bond::BondType type;
-  Bond *res = 0;
+  Bond *res = nullptr;
   switch (bType) {
     case 1:
       type = Bond::SINGLE;
@@ -1286,9 +1427,9 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
     case 0:
       type = Bond::UNSPECIFIED;
       res = new Bond;
-      BOOST_LOG(rdWarningLog) << "bond with order 0 found on line " << line
-                              << ". This is not part of the MDL specification."
-                              << std::endl;
+      BOOST_LOG(rdWarningLog)
+          << "bond with order 0 found on line " << line
+          << ". This is not part of the MDL specification." << std::endl;
       break;
     default:
       type = Bond::UNSPECIFIED;
@@ -1298,7 +1439,10 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
         BOND_NULL_QUERY *q;
         q = makeBondNullQuery();
         res->setQuery(q);
-      } else if (bType == 5 || bType == 6 || bType == 7) {
+      } else if (bType == 6) {
+        res->setQuery(makeSingleOrAromaticBondQuery());
+        res->setProp(common_properties::_MolFileBondQuery, 1);
+      } else if (bType == 5 || bType == 7) {
         BOND_OR_QUERY *q;
         q = new BOND_OR_QUERY;
         if (bType == 5) {
@@ -1308,13 +1452,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
           q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
               makeBondOrderEqualsQuery(Bond::DOUBLE)));
           q->setDescription("BondOr");
-        } else if (bType == 6) {
-          // single or aromatic
-          q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
-              makeBondOrderEqualsQuery(Bond::SINGLE)));
-          q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
-              makeBondOrderEqualsQuery(Bond::AROMATIC)));
-          q->setDescription("BondOr");
+          res->setProp(common_properties::_MolFileBondQuery, 1);
         } else if (bType == 7) {
           // double or aromatic
           q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
@@ -1324,13 +1462,14 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
           q->setDescription("BondOr");
         }
         res->setQuery(q);
+        res->setProp(common_properties::_MolFileBondQuery, 1);
       } else {
         BOND_NULL_QUERY *q;
         q = makeBondNullQuery();
         res->setQuery(q);
-        BOOST_LOG(rdWarningLog) << "unrecognized query bond type, " << bType
-                                << ", found on line " << line
-                                << ". Using an \"any\" query." << std::endl;
+        BOOST_LOG(rdWarningLog)
+            << "unrecognized query bond type, " << bType << ", found on line "
+            << line << ". Using an \"any\" query." << std::endl;
       }
       break;
   }
@@ -1359,7 +1498,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
           res->setBondDir(Bond::UNKNOWN);
           break;
       }
-    } catch (boost::bad_lexical_cast) {
+    } catch (boost::bad_lexical_cast &) {
       ;
     }
   }
@@ -1368,7 +1507,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
       int topology = FileParserUtils::toInt(text.substr(15, 3));
       if (topology) {
         if (!res->hasQuery()) {
-          QueryBond *qBond = new QueryBond(*res);
+          auto *qBond = new QueryBond(*res);
           delete res;
           res = qBond;
         }
@@ -1387,7 +1526,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
         }
         res->expandQuery(q);
       }
-    } catch (boost::bad_lexical_cast) {
+    } catch (boost::bad_lexical_cast &) {
       ;
     }
   }
@@ -1395,7 +1534,7 @@ Bond *ParseMolFileBondLine(const std::string &text, unsigned int line) {
     try {
       int reactStatus = FileParserUtils::toInt(text.substr(18, 3));
       res->setProp("molReactStatus", reactStatus);
-    } catch (boost::bad_lexical_cast) {
+    } catch (boost::bad_lexical_cast &) {
       ;
     }
   }
@@ -1450,15 +1589,31 @@ void ParseMolBlockBonds(std::istream *inStream, unsigned int &line,
 }
 
 bool ParseMolBlockProperties(std::istream *inStream, unsigned int &line,
-                             RWMol *mol) {
+                             RWMol *mol, bool strictParsing) {
   PRECONDITION(inStream, "bad stream");
   PRECONDITION(mol, "bad molecule");
   // older mol files can have an atom list block here
   std::string tempStr = getLine(inStream);
   ++line;
-  if (tempStr[0] != 'M' && tempStr[0] != 'A' && tempStr[0] != 'V' &&
-      tempStr[0] != 'G' && tempStr[0] != 'S') {
-    ParseOldAtomList(mol, tempStr, line);
+  // there is apparently some software out there that puts a
+  // blank line in mol blocks before the "M  END". If we aren't
+  // doing strict parsing, deal with that here.
+  if (!tempStr.size()) {
+    if (!strictParsing) {
+      tempStr = getLine(inStream);
+      ++line;
+    } else {
+      std::ostringstream errout;
+      errout << "Problems encountered parsing Mol data, unexpected blank line "
+                "found at line "
+             << line;
+      throw FileParseException(errout.str());
+    }
+  } else {
+    if (tempStr[0] != 'M' && tempStr[0] != 'A' && tempStr[0] != 'V' &&
+        tempStr[0] != 'G' && tempStr[0] != 'S') {
+      ParseOldAtomList(mol, tempStr, line);
+    }
   }
 
   bool fileComplete = false;
@@ -1514,10 +1669,13 @@ bool ParseMolBlockProperties(std::istream *inStream, unsigned int &line,
       ParseSGroup2000STYLine(mol, tempStr, line);
     } else if (lineBeg == "M  ZBO")
       ParseZBOLine(mol, tempStr, line);
-    else if (lineBeg == "M  ZCH")
+    else if (lineBeg == "M  ZCH") {
       ParseZCHLine(mol, tempStr, line);
-    else if (lineBeg == "M  HYD")
+    } else if (lineBeg == "M  HYD") {
       ParseHYDLine(mol, tempStr, line);
+    } else if (lineBeg == "M  MRV") {
+      ParseMarvinSmartsLine(mol, tempStr, line);
+    }
     line++;
     tempStr = getLine(inStream);
     lineBeg = tempStr.substr(0, 6);
@@ -1539,7 +1697,7 @@ Atom *ParseV3000AtomSymbol(std::string token, unsigned int &line) {
     boost::trim(token);
   }
 
-  Atom *res = 0;
+  Atom *res = nullptr;
   if (token[0] == '[') {
     // atom list:
     if (token[token.length() - 1] != ']') {
@@ -1584,7 +1742,7 @@ Atom *ParseV3000AtomSymbol(std::string token, unsigned int &line) {
           // according to the MDL spec, these match anything
           res->setQuery(makeAtomNullQuery());
         } else if (token == "Q") {
-          ATOM_OR_QUERY *q = new ATOM_OR_QUERY;
+          auto *q = new ATOM_OR_QUERY;
           q->setDescription("AtomOr");
           q->setNegation(true);
           q->addChild(
@@ -1769,6 +1927,7 @@ void ParseV3000AtomProps(RWMol *mol, Atom *&atom, typename T::iterator &token,
 }
 
 void tokenizeV3000Line(std::string line, std::vector<std::string> &tokens) {
+  tokens.clear();
   bool inQuotes = false, inParens = false;
   unsigned int start = 0;
   unsigned int pos = 0;
@@ -1912,12 +2071,11 @@ void ParseV3000AtomBlock(std::istream *inStream, unsigned int &line,
     throw FileParseException(errout.str());
   }
 
-  if (mol->hasProp(common_properties::_2DConf)) {
-    conf->set3D(false);
-    mol->clearProp(common_properties::_2DConf);
-  } else if (mol->hasProp(common_properties::_3DConf)) {
+  if (mol->hasProp(common_properties::_3DConf)) {
     conf->set3D(true);
     mol->clearProp(common_properties::_3DConf);
+  } else {
+    conf->set3D(hasNonZeroZCoords(*conf));
   }
 }
 void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
@@ -1927,17 +2085,14 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
   PRECONDITION(nBonds > 0, "bad bond count");
   PRECONDITION(mol, "bad molecule");
 
-  std::string tempStr;
-  std::vector<std::string> splitLine;
-
-  tempStr = getV3000Line(inStream, line);
+  auto tempStr = getV3000Line(inStream, line);
   if (tempStr.length() < 10 || tempStr.substr(0, 10) != "BEGIN BOND") {
     throw FileParseException("BEGIN BOND line not found");
   }
   for (unsigned int i = 0; i < nBonds; ++i) {
     tempStr = boost::trim_copy(getV3000Line(inStream, line));
-    boost::split(splitLine, tempStr, boost::is_any_of(" \t"),
-                 boost::token_compress_on);
+    std::vector<std::string> splitLine;
+    tokenizeV3000Line(tempStr, splitLine);
     if (splitLine.size() < 4) {
       std::ostringstream errout;
       errout << "bond line " << line << " is too short";
@@ -1963,6 +2118,9 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
         bond = new Bond(Bond::AROMATIC);
         bond->setIsAromatic(true);
         break;
+      case 9:
+        bond = new Bond(Bond::DATIVE);
+        break;
       case 0:
         bond = new Bond(Bond::UNSPECIFIED);
         BOOST_LOG(rdWarningLog)
@@ -1976,7 +2134,10 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
           BOND_NULL_QUERY *q;
           q = makeBondNullQuery();
           bond->setQuery(q);
-        } else if (bType == 5 || bType == 6 || bType == 7) {
+        } else if (bType == 6) {
+          bond->setQuery(makeSingleOrAromaticBondQuery());
+          bond->setProp(common_properties::_MolFileBondQuery, 1);
+        } else if (bType == 5 || bType == 7) {
           BOND_OR_QUERY *q;
           q = new BOND_OR_QUERY;
           if (bType == 5) {
@@ -1986,13 +2147,6 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
             q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
                 makeBondOrderEqualsQuery(Bond::DOUBLE)));
             q->setDescription("BondOr");
-          } else if (bType == 6) {
-            // single or aromatic
-            q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
-                makeBondOrderEqualsQuery(Bond::SINGLE)));
-            q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
-                makeBondOrderEqualsQuery(Bond::AROMATIC)));
-            q->setDescription("BondOr");
           } else if (bType == 7) {
             // double or aromatic
             q->addChild(QueryBond::QUERYBOND_QUERY::CHILD_TYPE(
@@ -2001,14 +2155,15 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
                 makeBondOrderEqualsQuery(Bond::AROMATIC)));
             q->setDescription("BondOr");
           }
+          bond->setProp(common_properties::_MolFileBondQuery, 1);
           bond->setQuery(q);
         } else {
           BOND_NULL_QUERY *q;
           q = makeBondNullQuery();
           bond->setQuery(q);
-          BOOST_LOG(rdWarningLog) << "unrecognized query bond type, " << bType
-                                  << ", found on line" << line
-                                  << ". Using an \"any\" query." << std::endl;
+          BOOST_LOG(rdWarningLog)
+              << "unrecognized query bond type, " << bType << ", found on line "
+              << line << ". Using an \"any\" query." << std::endl;
         }
         break;
     }
@@ -2051,7 +2206,7 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
       } else if (prop == "TOPO") {
         if (val != "0") {
           if (!bond->hasQuery()) {
-            QueryBond *qBond = new QueryBond(*bond);
+            auto *qBond = new QueryBond(*bond);
             delete bond;
             bond = qBond;
           }
@@ -2070,6 +2225,10 @@ void ParseV3000BondBlock(std::istream *inStream, unsigned int &line,
         int reactStatus = FileParserUtils::toInt(val);
         bond->setProp("molReactStatus", reactStatus);
       } else if (prop == "STBOX") {
+      } else if (prop == "ENDPTS") {
+        bond->setProp(common_properties::_MolFileBondEndPts, val);
+      } else if (prop == "ATTACH") {
+        bond->setProp(common_properties::_MolFileBondAttach, val);
       }
       ++lPos;
     }
@@ -2103,7 +2262,7 @@ void ProcessMolProps(RWMol *mol) {
       atom->setNoImplicit(true);
       if (totV == 15     // V2000
           || totV == -1  // v3000
-          ) {
+      ) {
         atom->setNumExplicitHs(0);
       } else {
         if (atom->getExplicitValence() > totV) {
@@ -2120,7 +2279,7 @@ void ProcessMolProps(RWMol *mol) {
   }
 }
 
-}  // end of local namespace
+}  // namespace
 namespace FileParserUtils {
 bool ParseV3000CTAB(std::istream *inStream, unsigned int &line, RWMol *mol,
                     Conformer *&conf, bool &chiralityPossible,
@@ -2241,15 +2400,17 @@ bool ParseV3000CTAB(std::istream *inStream, unsigned int &line, RWMol *mol,
   }
 
   while (tempStr.length() > 5 && tempStr.substr(0, 5) == "BEGIN") {
-    // skip blocks we don't know how to read
-    BOOST_LOG(rdWarningLog) << "skipping block at line " << line << ": "
-                            << tempStr << std::endl;
-    tempStr = getV3000Line(inStream, line);
-
-    while (tempStr.length() < 3 || tempStr.substr(0, 3) != "END") {
+    if (tempStr.length() > 15 && tempStr.substr(6, 10) == "COLLECTION") {
+      tempStr = parseEnhancedStereo(inStream, line, mol);
+    } else {
+      // skip blocks we don't know how to read
+      BOOST_LOG(rdWarningLog)
+          << "skipping block at line " << line << ": " << tempStr << std::endl;
+      while (tempStr.length() < 3 || tempStr.substr(0, 3) != "END") {
+        tempStr = getV3000Line(inStream, line);
+      }
       tempStr = getV3000Line(inStream, line);
     }
-    tempStr = getV3000Line(inStream, line);
   }
 
   boost::to_upper(tempStr);
@@ -2268,7 +2429,7 @@ bool ParseV3000CTAB(std::istream *inStream, unsigned int &line, RWMol *mol,
   }
 
   mol->addConformer(conf, true);
-  conf = 0;
+  conf = nullptr;
 
   return fileComplete;
 }
@@ -2284,24 +2445,24 @@ bool ParseV2000CTAB(std::istream *inStream, unsigned int &line, RWMol *mol,
   } else {
     ParseMolBlockAtoms(inStream, line, nAtoms, mol, conf);
 
-    if (mol->hasProp(common_properties::_2DConf)) {
-      conf->set3D(false);
-      mol->clearProp(common_properties::_2DConf);
-    } else if (mol->hasProp(common_properties::_3DConf)) {
+    if (mol->hasProp(common_properties::_3DConf)) {
       conf->set3D(true);
       mol->clearProp(common_properties::_3DConf);
+    } else { // default is 2D
+      conf->set3D(hasNonZeroZCoords(*conf));
     }
   }
   mol->addConformer(conf, true);
-  conf = 0;
+  conf = nullptr;
 
   ParseMolBlockBonds(inStream, line, nBonds, mol, chiralityPossible);
 
-  bool fileComplete = ParseMolBlockProperties(inStream, line, mol);
+  bool fileComplete =
+      ParseMolBlockProperties(inStream, line, mol, strictParsing);
   return fileComplete;
 }
 
-}  // end of FileParserUtils namespace
+}  // namespace FileParserUtils
 
 //------------------------------------------------
 //
@@ -2319,9 +2480,9 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
   line++;
   tempStr = getLine(inStream);
   if (inStream->eof()) {
-    return NULL;
+    return nullptr;
   }
-  RWMol *res = new RWMol();
+  auto *res = new RWMol();
   res->setProp(common_properties::_Name, tempStr);
 
   // info
@@ -2330,9 +2491,8 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
   res->setProp("_MolFileInfo", tempStr);
   if (tempStr.length() >= 22) {
     std::string dimLabel = tempStr.substr(20, 2);
-    if (dimLabel == "2d" || dimLabel == common_properties::TWOD) {
-      res->setProp(common_properties::_2DConf, 1);
-    } else if (dimLabel == "3d" || dimLabel == "3D") {
+    // Unless labelled as 3D we assume 2D
+    if (dimLabel == "3d" || dimLabel == "3D") {
       res->setProp(common_properties::_3DConf, 1);
     }
   }
@@ -2357,7 +2517,7 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
   if (tempStr.size() < 6) {
     if (res) {
       delete res;
-      res = NULL;
+      res = nullptr;
     }
     std::ostringstream errout;
     errout << "Counts line too short: '" << tempStr << "' on line" << line;
@@ -2375,7 +2535,7 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
   } catch (boost::bad_lexical_cast &) {
     if (res) {
       delete res;
-      res = NULL;
+      res = nullptr;
     }
     std::ostringstream errout;
     errout << "Cannot convert " << tempStr.substr(spos, 3) << " to int on line "
@@ -2423,7 +2583,7 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
       errout << "CTAB version string invalid at line " << line;
       if (strictParsing) {
         delete res;
-        res = NULL;
+        res = nullptr;
         throw FileParseException(errout.str());
       } else {
         BOOST_LOG(rdWarningLog) << errout.str() << std::endl;
@@ -2436,7 +2596,7 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
              << "' at line " << line;
       if (strictParsing) {
         delete res;
-        res = NULL;
+        res = nullptr;
         throw FileParseException(errout.str());
       } else {
         BOOST_LOG(rdWarningLog) << errout.str() << std::endl;
@@ -2448,7 +2608,7 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
     res->setProp(common_properties::_MolFileChiralFlag, chiralFlag);
   }
 
-  Conformer *conf = 0;
+  Conformer *conf = nullptr;
   try {
     if (ctabVersion == 2000) {
       fileComplete = FileParserUtils::ParseV2000CTAB(inStream, line, res, conf,
@@ -2458,10 +2618,11 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
       if (nAtoms != 0 || nBonds != 0) {
         std::ostringstream errout;
         errout << "V3000 mol blocks should have 0s in the initial counts line. "
-                  "(line: " << line << ")";
+                  "(line: "
+               << line << ")";
         if (strictParsing) {
           delete res;
-          res = NULL;
+          res = nullptr;
           throw FileParseException(errout.str());
         } else {
           BOOST_LOG(rdWarningLog) << errout.str() << std::endl;
@@ -2475,8 +2636,8 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
     // unhandled mol file feature, just delete the result
     delete res;
     delete conf;
-    res = NULL;
-    conf = NULL;
+    res = nullptr;
+    conf = nullptr;
     BOOST_LOG(rdErrorLog) << " Unhandled CTAB feature: " << e.message()
                           << " on line: " << line << ". Molecule skipped."
                           << std::endl;
@@ -2497,16 +2658,16 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
     // catch our exceptions and throw them back after cleanup
     delete res;
     delete conf;
-    res = NULL;
-    conf = NULL;
+    res = nullptr;
+    conf = nullptr;
     throw e;
   }
 
   if (!fileComplete) {
     delete res;
     delete conf;
-    res = NULL;
-    conf = NULL;
+    res = nullptr;
+    conf = nullptr;
     std::ostringstream errout;
     errout
         << "Problems encountered parsing Mol data, M  END missing around line "
@@ -2532,20 +2693,14 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
     // sign that chirality ever existed and makes us sad... so first
     // perceive chirality, then remove the Hs and sanitize.
     //
-    // One exception to this (of course, there's always an exception):
-    // DetectAtomStereoChemistry() needs to check the number of
-    // implicit hydrogens on atoms to detect if things can be
-    // chiral. However, if we ask for the number of implicit Hs before
-    // we've called MolOps::cleanUp() on the molecule, we'll get
-    // exceptions for common "weird" cases like a nitro group
-    // mis-represented as -N(=O)=O.  *SO*... we need to call
-    // cleanUp(), then detect the stereochemistry.
-    // (this was Issue 148)
-    //
     const Conformer &conf = res->getConformer();
     if (chiralityPossible) {
-      MolOps::cleanUp(*res);
-      DetectAtomStereoChemistry(*res, &conf);
+      if (!conf.is3D()) {
+        DetectAtomStereoChemistry(*res, &conf);
+      } else {
+        res->updatePropertyCache(false);
+        MolOps::assignChiralTypesFrom3D(*res, conf.getId(), true);
+      }
     }
 
     if (sanitize) {
@@ -2561,24 +2716,28 @@ RWMol *MolDataStreamToMol(std::istream *inStream, unsigned int &line,
         // single bond dir flags:
         ClearSingleBondDirFlags(*res);
 
-        // unlike DetectAtomStereoChemistry we call DetectBondStereoChemistry
+        // unlike DetectAtomStereoChemistry we call detectBondStereochemistry
         // here after sanitization because we need the ring information:
-        DetectBondStereoChemistry(*res, &conf);
+        MolOps::detectBondStereochemistry(*res);
       } catch (...) {
         delete res;
-        res = NULL;
+        res = nullptr;
         throw;
       }
       MolOps::assignStereochemistry(*res, true, true, true);
     } else {
       // we still need to do something about double bond stereochemistry
       // (was github issue 337)
-      DetectBondStereoChemistry(*res, &conf);
+      // now that atom stereochem has been perceived, the wedging
+      // information is no longer needed, so we clear
+      // single bond dir flags:
+      ClearSingleBondDirFlags(*res);
+      MolOps::detectBondStereochemistry(*res);
     }
 
     if (res->hasProp(common_properties::_NeedsQueryScan)) {
       res->clearProp(common_properties::_NeedsQueryScan);
-      CompleteMolQueries(res);
+      completeMolQueries(res);
     }
   }
   return res;
@@ -2613,11 +2772,11 @@ RWMol *MolFileToMol(const std::string &fName, bool sanitize, bool removeHs,
     errout << "Bad input file " << fName;
     throw BadFileException(errout.str());
   }
-  RWMol *res = NULL;
+  RWMol *res = nullptr;
   if (!inStream.eof()) {
     unsigned int line = 0;
     res = MolDataStreamToMol(inStream, line, sanitize, removeHs, strictParsing);
   }
   return res;
 }
-}
+}  // namespace RDKit
