@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2003-2014 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2003-2017 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -27,6 +26,7 @@
 #include <boost/dynamic_bitset.hpp>
 #include <RDGeneral/BadFileException.h>
 #include <GraphMol/SmilesParse/SmartsWrite.h>
+#include <GraphMol/Depictor/RDDepictor.h>
 
 namespace RDKit {
 
@@ -47,8 +47,8 @@ int getQueryBondTopology(const Bond *bond) {
 
   if (qry->getDescription() == "BondAnd" && !qry->getNegation() &&
       qry->endChildren() - qry->beginChildren() == 2) {
-    Bond::QUERYBOND_QUERY::CHILD_VECT_CI child1 = qry->beginChildren();
-    Bond::QUERYBOND_QUERY::CHILD_VECT_CI child2 = child1 + 1;
+    auto child1 = qry->beginChildren();
+    auto child2 = child1 + 1;
     if ((*child1)->getDescription() == "BondOr" &&
         (*child2)->getDescription() == "BondInRing") {
       qry = child2->get();
@@ -80,8 +80,8 @@ int getQueryBondSymbol(const Bond *bond) {
     // start by catching combined bond order + bond topology queries
     if (qry->getDescription() == "BondAnd" && !qry->getNegation() &&
         qry->endChildren() - qry->beginChildren() == 2) {
-      Bond::QUERYBOND_QUERY::CHILD_VECT_CI child1 = qry->beginChildren();
-      Bond::QUERYBOND_QUERY::CHILD_VECT_CI child2 = child1 + 1;
+      auto child1 = qry->beginChildren();
+      auto child2 = child1 + 1;
       if ((*child1)->getDescription() == "BondOr" &&
           (*child2)->getDescription() == "BondInRing") {
         qry = child1->get();
@@ -92,8 +92,8 @@ int getQueryBondSymbol(const Bond *bond) {
     }
     if (qry->getDescription() == "BondOr" && !qry->getNegation()) {
       if (qry->endChildren() - qry->beginChildren() == 2) {
-        Bond::QUERYBOND_QUERY::CHILD_VECT_CI child1 = qry->beginChildren();
-        Bond::QUERYBOND_QUERY::CHILD_VECT_CI child2 = child1 + 1;
+        auto child1 = qry->beginChildren();
+        auto child2 = child1 + 1;
         if ((*child1)->getDescription() == "BondOrder" &&
             !(*child1)->getNegation() &&
             (*child2)->getDescription() == "BondOrder" &&
@@ -111,11 +111,14 @@ int getQueryBondSymbol(const Bond *bond) {
           }
         }
       }
+    } else if (qry->getDescription() == "SingleOrAromaticBond" &&
+               !qry->getNegation()) {
+      res = 6;
     }
   }
   return res;
 }
-}
+}  // namespace
 
 const std::string GetMolFileChargeInfo(const RWMol &mol) {
   std::stringstream res;
@@ -211,8 +214,7 @@ bool isListQuery(const Atom::QUERYATOM_QUERY *q) {
   std::string descr = q->getDescription();
   if (descr == "AtomOr") {
     res = true;
-    for (Atom::QUERYATOM_QUERY::CHILD_VECT_CI cIt = q->beginChildren();
-         cIt != q->endChildren() && res; ++cIt) {
+    for (auto cIt = q->beginChildren(); cIt != q->endChildren() && res; ++cIt) {
       std::string descr = (*cIt)->getDescription();
       // we don't allow negation of any children of the query:
       if ((*cIt)->getNegation()) {
@@ -233,8 +235,7 @@ void getListQueryVals(const Atom::QUERYATOM_QUERY *q, INT_VECT &vals) {
   std::string descr = q->getDescription();
   PRECONDITION(descr == "AtomOr", "bad query");
   if (descr == "AtomOr") {
-    for (Atom::QUERYATOM_QUERY::CHILD_VECT_CI cIt = q->beginChildren();
-         cIt != q->endChildren(); ++cIt) {
+    for (auto cIt = q->beginChildren(); cIt != q->endChildren(); ++cIt) {
       std::string descr = (*cIt)->getDescription();
       CHECK_INVARIANT((descr == "AtomOr" || descr == "AtomAtomicNum"),
                       "bad query");
@@ -276,9 +277,8 @@ const std::string GetMolFileQueryInfo(const RWMol &mol) {
       wrote_query = true;
     }
     std::string molFileValue;
-    if (!wrote_query &&
-        (*atomIt)
-            ->getPropIfPresent(common_properties::molFileValue, molFileValue))
+    if (!wrote_query && (*atomIt)->getPropIfPresent(
+                            common_properties::molFileValue, molFileValue))
       ss << "V  " << std::setw(3) << (*atomIt)->getIdx() + 1 << " "
          << molFileValue << std::endl;
   }
@@ -329,8 +329,8 @@ const std::string GetMolFileAliasInfo(const RWMol &mol) {
     std::string lbl;
     if ((*atomIt)->getPropIfPresent(common_properties::molFileAlias, lbl)) {
       if (!lbl.empty())
-        ss << "A  " << std::setw(3) << (*atomIt)->getIdx() + 1 << "\n" << lbl
-           << "\n";
+        ss << "A  " << std::setw(3) << (*atomIt)->getIdx() + 1 << "\n"
+           << lbl << "\n";
     }
   }
   return ss.str();
@@ -481,7 +481,7 @@ unsigned int getAtomParityFlag(const Atom *atom, const Conformer *conf) {
 
   const ROMol &mol = atom->getOwningMol();
   RDGeom::Point3D pos = conf->getAtomPos(atom->getIdx());
-  std::vector<std::pair<unsigned int, RDGeom::Point3D> > vs;
+  std::vector<std::pair<unsigned int, RDGeom::Point3D>> vs;
   ROMol::ADJ_ITER nbrIdx, endNbrs;
   boost::tie(nbrIdx, endNbrs) = mol.getAtomNeighbors(atom);
   while (nbrIdx != endNbrs) {
@@ -510,7 +510,7 @@ unsigned int getAtomParityFlag(const Atom *atom, const Conformer *conf) {
   }
   return 0;
 }
-}
+}  // namespace
 
 bool hasNonDefaultValence(const Atom *atom) {
   if (atom->getNumRadicalElectrons() != 0) return true;
@@ -574,7 +574,7 @@ void GetMolFileAtomProperties(const Atom *atom, const Conformer *conf,
 }
 
 const std::string GetMolFileAtomLine(const Atom *atom,
-                                     const Conformer *conf = 0) {
+                                     const Conformer *conf = nullptr) {
   PRECONDITION(atom, "");
   std::string res;
   int totValence, atomMapNumber;
@@ -613,7 +613,7 @@ const std::string GetMolFileAtomLine(const Atom *atom,
   // time of this writing (with boost 1.55), the snprintf version runs in 20% of
   // the time.
   char dest[128];
-#ifndef WIN32
+#ifndef _MSC_VER
   snprintf(dest, 128,
            "%10.4f%10.4f%10.4f %3s%2d%3d%3d%3d%3d%3d  0%3d%3d%3d%3d%3d", x, y,
            z, symbol.c_str(), massDiff, chg, parityFlag, hCount, stereoCare,
@@ -624,10 +624,11 @@ const std::string GetMolFileAtomLine(const Atom *atom,
   // the format string makes it impossible for this to overflow, I think we're
   // safe. I just used the snprintf above to prevent linters from complaining
   // about use of sprintf
-  sprintf(dest, "%10.4f%10.4f%10.4f %3s%2d%3d%3d%3d%3d%3d  0%3d%3d%3d%3d%3d", x,
-          y, z, symbol.c_str(), massDiff, chg, parityFlag, hCount, stereoCare,
-          totValence, rxnComponentType, rxnComponentNumber, atomMapNumber,
-          inversionFlag, exactChangeFlag);
+  sprintf_s(dest, 128,
+            "%10.4f%10.4f%10.4f %3s%2d%3d%3d%3d%3d%3d  0%3d%3d%3d%3d%3d", x, y,
+            z, symbol.c_str(), massDiff, chg, parityFlag, hCount, stereoCare,
+            totValence, rxnComponentType, rxnComponentNumber, atomMapNumber,
+            inversionFlag, exactChangeFlag);
 
 #endif
   res += dest;
@@ -635,6 +636,28 @@ const std::string GetMolFileAtomLine(const Atom *atom,
   return res;
 };
 
+namespace {
+/*
+  If a molecule contains dative bonds the V2000 format should not
+  be used as it doesn't support dative bonds. If a dative bond is
+  detected while writing a V2000 molfile the RequiresV3000Exception
+  is thrown and the V2000 writer will redo the export in V3000 format.
+
+  This is arguably a rather brute-force way of detecting the proper output
+  format, but the only alternatives I (Jan Holst Jensen) had in mind were:
+
+    1) Check all bond types before output. Slow and would affect all
+       V2000 exports.
+    2) Maintain a reference count of dative bonds in molecule. Complex
+       and error-prone.
+*/
+class RequiresV3000Exception : public std::runtime_error {
+ public:
+  explicit RequiresV3000Exception()
+      : std::runtime_error("RequiresV3000Exception"){};
+};
+}  // namespace
+
 int BondGetMolFileSymbol(const Bond *bond) {
   PRECONDITION(bond, "");
   // FIX: should eventually recognize queries
@@ -667,6 +690,10 @@ int BondGetMolFileSymbol(const Bond *bond) {
       case Bond::ZERO:
         res = 1;
         break;
+      case Bond::DATIVE:
+        // Dative bonds requires V3000 format. Throw special exception to
+        // force output to be re-done in V3000.
+        throw RequiresV3000Exception();
       default:
         break;
     }
@@ -714,7 +741,7 @@ void GetMolFileBondStereoInfo(const Bond *bond, const INT_MAP_INT &wedgeBonds,
     // reverse the begin and end atoms for the bond when we write
     // the mol file
     if ((dirCode == 1) || (dirCode == 6)) {
-      INT_MAP_INT_CI wbi = wedgeBonds.find(bond->getIdx());
+      auto wbi = wedgeBonds.find(bond->getIdx());
       if (wbi != wedgeBonds.end() &&
           static_cast<unsigned int>(wbi->second) != bond->getBeginAtomIdx()) {
         reverse = true;
@@ -751,7 +778,7 @@ void GetMolFileBondStereoInfo(const Bond *bond, const INT_MAP_INT &wedgeBonds,
           boost::tie(beg, end) =
               bond->getOwningMol().getAtomBonds(bond->getBeginAtom());
           while (beg != end && !nbrHasDir) {
-            const BOND_SPTR nbrBond = bond->getOwningMol()[*beg];
+            const Bond *nbrBond = bond->getOwningMol()[*beg];
             if (nbrBond->getBondType() == Bond::SINGLE &&
                 (nbrBond->getBondDir() == Bond::ENDUPRIGHT ||
                  nbrBond->getBondDir() == Bond::ENDDOWNRIGHT)) {
@@ -762,7 +789,7 @@ void GetMolFileBondStereoInfo(const Bond *bond, const INT_MAP_INT &wedgeBonds,
           boost::tie(beg, end) =
               bond->getOwningMol().getAtomBonds(bond->getEndAtom());
           while (beg != end && !nbrHasDir) {
-            const BOND_SPTR nbrBond = bond->getOwningMol()[*beg];
+            const Bond *nbrBond = bond->getOwningMol()[*beg];
             if (nbrBond->getBondType() == Bond::SINGLE &&
                 (nbrBond->getBondDir() == Bond::ENDUPRIGHT ||
                  nbrBond->getBondDir() == Bond::ENDDOWNRIGHT)) {
@@ -812,7 +839,7 @@ const std::string GetMolFileBondLine(const Bond *bond,
 }
 
 const std::string GetV3000MolFileAtomLine(const Atom *atom,
-                                          const Conformer *conf = 0) {
+                                          const Conformer *conf = nullptr) {
   PRECONDITION(atom, "");
   int totValence, atomMapNumber;
   unsigned int parityFlag;
@@ -894,6 +921,7 @@ const std::string GetV3000MolFileAtomLine(const Atom *atom,
 
 int GetV3000BondCode(const Bond *bond) {
   // JHJ: As far as I can tell, the V3000 bond codes are the same as for V2000.
+  //      Except: The dative bond type is only supported in V3000.
   PRECONDITION(bond, "");
   int res = 0;
   // FIX: should eventually recognize queries
@@ -920,6 +948,9 @@ int GetV3000BondCode(const Bond *bond) {
       case Bond::AROMATIC:
         res = 4;
         break;
+      case Bond::DATIVE:
+        res = 9;
+        break;
       default:
         res = 0;
         break;
@@ -978,37 +1009,49 @@ const std::string GetV3000MolFileBondLine(const Bond *bond,
   return ss.str();
 }
 
+void appendEnhancedStereoGroups(std::string &res, const RWMol &tmol) {
+  unsigned or_count = 1u, and_count = 1u;
+  auto &stereo_groups = tmol.getStereoGroups();
+  if (!stereo_groups.empty()) {
+    res += "M  V30 BEGIN COLLECTION\n";
+    for (auto &&group : stereo_groups) {
+      res += "M  V30 MDLV30/";
+      switch (group.getGroupType()) {
+        case RDKit::StereoGroupType::STEREO_ABSOLUTE:
+          res += "STEABS";
+          break;
+        case RDKit::StereoGroupType::STEREO_OR:
+          res += "STEREL";
+          res += boost::lexical_cast<std::string>(or_count);
+          ++or_count;
+          break;
+        case RDKit::StereoGroupType::STEREO_AND:
+          res += "STERAC";
+          res += boost::lexical_cast<std::string>(and_count);
+          ++and_count;
+          break;
+      }
+      res += " ATOMS=(";
+      auto& atoms = group.getAtoms();
+      res += boost::lexical_cast<std::string>(atoms.size());
+      for (auto &&atom : atoms) {
+        res += ' ';
+        // atoms are 1 indexed in molfiles
+        res += boost::lexical_cast<std::string>(atom->getIdx() + 1);
+      }
+      res += ")\n";
+    }
+    res += "M  V30 END COLLECTION\n";
+  }
+}
+
 //------------------------------------------------
 //
 //  gets a mol block as a string
 //
 //------------------------------------------------
-std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
-                          bool kekulize, bool forceV3000) {
-  RDUNUSED_PARAM(includeStereo);
-  RDKit::Utils::LocaleSwitcher switcher;
-  ROMol tromol(mol);
-  RWMol &trwmol = static_cast<RWMol &>(tromol);
-  // NOTE: kekulize the molecule before writing it out
-  // because of the way mol files handle aromaticity
-  if (trwmol.needsUpdatePropertyCache()) {
-    trwmol.updatePropertyCache(false);
-  }
-  if (kekulize) MolOps::Kekulize(trwmol);
-
-#if 0
-    if(includeStereo){
-      // assign "any" status to any stereo bonds that are not 
-      // marked with "E" or "Z" code - these bonds need to be explictly written
-      // out to the mol file
-      MolOps::findPotentialStereoBonds(trwmol);
-      // now assign stereo code if any have been specified by the directions on
-      // single bonds
-      MolOps::assignStereochemistry(trwmol);
-    }
-#endif
-  const RWMol &tmol = const_cast<RWMol &>(trwmol);
-
+std::string outputMolToMolBlock(const RWMol &tmol, int confId,
+                                bool forceV3000) {
   std::string res;
 
   bool isV3000;
@@ -1025,11 +1068,11 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
   nProducts = 0;
   nIntermediates = 0;
 
-  mol.getPropIfPresent(common_properties::_MolFileChiralFlag, chiralFlag);
+  tmol.getPropIfPresent(common_properties::_MolFileChiralFlag, chiralFlag);
 
   const Conformer *conf;
   if (confId < 0 && tmol.getNumConformers() == 0) {
-    conf = 0;
+    conf = nullptr;
   } else {
     conf = &(tmol.getConformer(confId));
   }
@@ -1066,7 +1109,8 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
   if (forceV3000)
     isV3000 = true;
   else
-    isV3000 = (nAtoms > 999) || (nBonds > 999);
+    isV3000 =
+        (nAtoms > 999) || (nBonds > 999) || !tmol.getStereoGroups().empty();
 
   // the counts line:
   std::stringstream ss;
@@ -1126,8 +1170,8 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
     std::stringstream ss;
     //                                           numSgroups (not implemented)
     //                                           | num3DConstraints (not
-    //                                           implemented)
-    //                                           | |
+    //                                           +---------+ |   implemented)
+    //                                                     | |
     ss << "M  V30 COUNTS " << nAtoms << " " << nBonds << " 0 0 " << chiralFlag
        << "\n";
     res += ss.str();
@@ -1150,12 +1194,51 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
       }
       res += "M  V30 END BOND\n";
     }
+    appendEnhancedStereoGroups(res, tmol);
+
     res += "M  V30 END CTAB\n";
   }
   res += "M  END\n";
   return res;
 }
 
+std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
+                          bool kekulize, bool forceV3000) {
+  RDUNUSED_PARAM(includeStereo);
+  RDKit::Utils::LocaleSwitcher switcher;
+  ROMol tromol(mol);
+  RWMol &trwmol = static_cast<RWMol &>(tromol);
+  // NOTE: kekulize the molecule before writing it out
+  // because of the way mol files handle aromaticity
+  if (trwmol.needsUpdatePropertyCache()) {
+    trwmol.updatePropertyCache(false);
+  }
+  if (kekulize) MolOps::Kekulize(trwmol);
+
+  if (includeStereo && !trwmol.getNumConformers()) {
+    // generate coordinates so that the stereo we generate makes sense
+    RDDepict::compute2DCoords(trwmol);
+  }
+#if 0
+    if(includeStereo){
+      // assign "any" status to any stereo bonds that are not
+      // marked with "E" or "Z" code - these bonds need to be explictly written
+      // out to the mol file
+      MolOps::findPotentialStereoBonds(trwmol);
+      // now assign stereo code if any have been specified by the directions on
+      // single bonds
+      MolOps::assignStereochemistry(trwmol);
+    }
+#endif
+  const RWMol &tmol = const_cast<RWMol &>(trwmol);
+
+  try {
+    return outputMolToMolBlock(tmol, confId, forceV3000);
+  } catch (RequiresV3000Exception &) {
+    return outputMolToMolBlock(tmol, confId, true);
+  }
+}
+
 //------------------------------------------------
 //
 //  Dump a molecule to a file
@@ -1164,7 +1247,7 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
 void MolToMolFile(const ROMol &mol, const std::string &fName,
                   bool includeStereo, int confId, bool kekulize,
                   bool forceV3000) {
-  std::ofstream *outStream = new std::ofstream(fName.c_str());
+  auto *outStream = new std::ofstream(fName.c_str());
   if (!outStream || !(*outStream) || outStream->bad()) {
     std::ostringstream errout;
     errout << "Bad output file " << fName;
@@ -1175,4 +1258,4 @@ void MolToMolFile(const ROMol &mol, const std::string &fName,
   *outStream << outString;
   delete outStream;
 }
-}
+}  // namespace RDKit
