@@ -31,7 +31,7 @@ SDMolSupplier::SDMolSupplier(const std::string &fileName, bool sanitize,
   // FIX: this binary mode of opening file is here because of a bug in VC++ 6.0
   // the function "tellg" does not work correctly if we do not open it this way
   //   Jan 2009: Confirmed that this is still the case in visual studio 2008
-  std::istream *tmpStream = 0;
+  std::istream *tmpStream = nullptr;
   tmpStream = static_cast<std::istream *>(
       new std::ifstream(fileName.c_str(), std::ios_base::binary));
   if (!tmpStream || (!(*tmpStream)) || (tmpStream->bad())) {
@@ -85,7 +85,7 @@ void SDMolSupplier::setDataCommon(const std::string &text, bool sanitize,
                                   bool removeHs) {
   if (dp_inStream && df_owner) delete dp_inStream;
   init();
-  std::istream *tmpStream = 0;
+  std::istream *tmpStream = nullptr;
   tmpStream = static_cast<std::istream *>(
       new std::istringstream(text, std::ios_base::binary));
   dp_inStream = tmpStream;
@@ -120,7 +120,7 @@ void SDMolSupplier::checkForEnd() {
   // or we reach end of file in the meantime
   if (dp_inStream->eof()) {
     df_end = true;
-    d_len = d_molpos.size();
+    d_len = rdcast<int>(d_molpos.size());
     return;
   }
   // we are not at the end of file, check for blank lines
@@ -130,7 +130,7 @@ void SDMolSupplier::checkForEnd() {
     tempStr = getLine(dp_inStream);
     if (dp_inStream->eof()) {
       df_end = true;
-      d_len = d_molpos.size();
+      d_len = rdcast<int>(d_molpos.size());
       return;
     }
     if (tempStr.find_first_not_of(" \t\r\n") == std::string::npos) {
@@ -139,7 +139,7 @@ void SDMolSupplier::checkForEnd() {
   }
   if (nempty == 4) {
     df_end = true;
-    d_len = d_molpos.size();
+    d_len = rdcast<int>(d_molpos.size());
   }
 }
 
@@ -162,12 +162,12 @@ ROMol *SDMolSupplier::next() {
   dp_inStream->seekg(d_molpos[d_last]);
 
   std::string tempStr;
-  ROMol *res = NULL;
+  ROMol *res = nullptr;
   // finally if we reached the end of the file set end to be true
   if (dp_inStream->eof()) {
     // FIX: we should probably be throwing an exception here
     df_end = true;
-    d_len = d_molpos.size();
+    d_len = rdcast<int>(d_molpos.size());
     return res;
   }
 
@@ -198,7 +198,7 @@ std::string SDMolSupplier::getItemText(unsigned int idx) {
     endP = dp_inStream->tellg();
   }
   d_last = holder;
-  char *buff = new char[endP - begP];
+  auto *buff = new char[endP - begP];
   dp_inStream->seekg(begP);
   dp_inStream->read(buff, endP - begP);
   std::string res(buff, endP - begP);
@@ -220,7 +220,7 @@ void SDMolSupplier::moveTo(unsigned int idx) {
   } else {
     std::string tempStr;
     dp_inStream->seekg(d_molpos.back());
-    d_last = d_molpos.size() - 1;
+    d_last = rdcast<int>(d_molpos.size()) - 1;
     while ((d_last < static_cast<int>(idx)) && (!dp_inStream->eof())) {
       d_line++;
       tempStr = getLine(dp_inStream);
@@ -236,7 +236,7 @@ void SDMolSupplier::moveTo(unsigned int idx) {
     }
     // if we reached end of file without reaching "idx" we have an index error
     if (dp_inStream->eof()) {
-      d_len = d_molpos.size();
+      d_len = rdcast<int>(d_molpos.size());
       std::ostringstream errout;
       errout << "ERROR: Index error (idx = " << idx << ") : "
              << " we do no have enough mol blocks";
@@ -259,7 +259,7 @@ unsigned int SDMolSupplier::length() {
     return d_len;
   } else {
     std::string tempStr;
-    d_len = d_molpos.size();
+    d_len = rdcast<int>(d_molpos.size());
     dp_inStream->seekg(d_molpos.back());
     while (!dp_inStream->eof()) {
       std::getline(*dp_inStream, tempStr);
@@ -291,6 +291,6 @@ void SDMolSupplier::setStreamIndices(const std::vector<std::streampos> &locs) {
   d_molpos.resize(locs.size());
   std::copy(locs.begin(), locs.end(), d_molpos.begin());
   this->reset();
-  d_len = d_molpos.size();
+  d_len = rdcast<int>(d_molpos.size());
 }
 }
