@@ -24,7 +24,7 @@
 namespace RDKit {
 SDWriter::SDWriter(const std::string &fileName) {
   if (fileName != "-") {
-    std::ofstream *tmpStream = new std::ofstream(fileName.c_str());
+    auto *tmpStream = new std::ofstream(fileName.c_str());
     df_owner = true;
     if (!tmpStream || !(*tmpStream) || (tmpStream->bad())) {
       std::ostringstream errout;
@@ -55,7 +55,7 @@ SDWriter::SDWriter(std::ostream *outStream, bool takeOwnership) {
 
 SDWriter::~SDWriter() {
   // close the writer if it's still open:
-  if (dp_ostream != NULL) close();
+  if (dp_ostream != nullptr) close();
 }
 
 void SDWriter::setProps(const STR_VECT &propNames) {
@@ -115,12 +115,12 @@ void _MolToSDStream(std::ostream *dp_ostream, const ROMol &mol, int confId,
     // out to the file
     STR_VECT properties = mol.getPropList();
     STR_VECT compLst;
-    mol.getPropIfPresent(detail::computedPropName, compLst);
+    mol.getPropIfPresent(RDKit::detail::computedPropName, compLst);
 
     STR_VECT_CI pi;
     for (pi = properties.begin(); pi != properties.end(); pi++) {
       // ignore any of the following properties
-      if (((*pi) == detail::computedPropName) ||
+      if (((*pi) == RDKit::detail::computedPropName) ||
           ((*pi) == common_properties::_Name) || ((*pi) == "_MolFileInfo") ||
           ((*pi) == "_MolFileComments") ||
           ((*pi) == common_properties::_MolFileChiralFlag)) {
