@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2015 Sereina Riniker
+//  Copyright (C) 2017 Sereina Riniker
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -7,8 +7,10 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef _RD_TORSIONPREFERENCES_H_
 #define _RD_TORSIONPREFERENCES_H_
+#include <vector>
 
 namespace RDKit {
 class ROMol;
@@ -17,12 +19,12 @@ class ROMol;
 namespace ForceFields {
 namespace CrystalFF {
 //! Get the experimental torsional angles in a molecule
-void getExperimentalTorsions(
+RDKIT_FORCEFIELDHELPERS_EXPORT void getExperimentalTorsions(
     const RDKit::ROMol &mol, std::vector<std::vector<int> > &expTorsionAtoms,
     std::vector<std::pair<std::vector<int>, std::vector<double> > > &
         expTorsionAngles,
     std::vector<std::vector<int> > &improperAtoms, bool useExpTorsions = false,
-    bool useBasicKnowledge = false, bool verbose = false);
+    bool useBasicKnowledge = false, unsigned int version = 1, bool verbose = false);
 }
 }
 
