@@ -9,6 +9,7 @@
 //  of the RDKit source tree.
 //
 
+#include <RDGeneral/test.h>
 #include <RDGeneral/utils.h>
 #include <Geometry/point.h>
 #include <GraphMol/RDKitBase.h>
@@ -129,7 +130,7 @@ void testTorsionPrefs() {
 
   ForceFields::CrystalFF::getExperimentalTorsions(
       *mol, expTorsionAtoms, expTorsionAngles, improperAtoms, true, false,
-      false);
+      1, false);
   TEST_ASSERT(expTorsionAtoms.size() == 1);
   TEST_ASSERT(expTorsionAngles.size() == 1);
   TEST_ASSERT(expTorsionAtoms[0][0] == 0);
@@ -143,7 +144,7 @@ void testTorsionPrefs() {
 
   ForceFields::CrystalFF::getExperimentalTorsions(
       *mol, expTorsionAtoms, expTorsionAngles, improperAtoms, true, false,
-      false);
+      1, false);
   TEST_ASSERT(expTorsionAtoms.size() == 2);
   TEST_ASSERT(expTorsionAngles.size() == 2);
 }
