@@ -1,6 +1,4 @@
-// $Id$
-//
-//  Copyright (C) 2013 Paolo Tosco
+//  Copyright (C) 2013-2018 Paolo Tosco
 //
 //  Copyright (C) 2004-2010 Greg Landrum and Rational Discovery LLC
 //
@@ -12,17 +10,16 @@
 //
 #include <iostream>
 #include <cmath>
+#include <cctype>
 
 #include <RDGeneral/Invariant.h>
 #include <GraphMol/RDKitBase.h>
 #include <GraphMol/SmilesParse/SmilesParse.h>
 #include <GraphMol/Substruct/SubstructMatch.h>
-#include <ForceField/ForceField.h>
 #include <ForceField/MMFF/Params.h>
 #include <ForceField/MMFF/Contribs.h>
 #include "AtomTyper.h"
 #include "Builder.h"
-#include <stdio.h>
 
 namespace RDKit {
 namespace MMFF {
@@ -52,20 +49,20 @@ void addBonds(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  "  I        J          I    J   CLASS   LENGTH   LENGTH    "
                  "DIFF.    ENERGY   CONSTANT\n"
                  "-------------------------------------------------------------"
-                 "------------------------" << std::endl;
+                 "------------------------"
+              << std::endl;
     }
-    field->initialize();
   }
   for (ROMol::ConstBondIterator bi = mol.beginBonds(); bi != mol.endBonds();
        ++bi) {
     unsigned int idx1 = (*bi)->getBeginAtomIdx();
     unsigned int idx2 = (*bi)->getEndAtomIdx();
     unsigned int bondType;
-    BondStretchContrib *contrib;
     MMFFBond mmffBondParams;
     if (mmffMolProperties->getMMFFBondStretchParams(mol, idx1, idx2, bondType,
                                                     mmffBondParams)) {
-      contrib = new BondStretchContrib(field, idx1, idx2, &mmffBondParams);
+      auto *contrib =
+          new BondStretchContrib(field, idx1, idx2, &mmffBondParams);
       field->contribs().push_back(ForceFields::ContribPtr(contrib));
       if (mmffMolProperties->getMMFFVerbosity()) {
         unsigned int iAtomType = mmffMolProperties->getMMFFAtomType(idx1);
@@ -73,7 +70,7 @@ void addBonds(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
         const Atom *iAtom = (*bi)->getBeginAtom();
         const Atom *jAtom = (*bi)->getEndAtom();
         const double dist = field->distance(idx1, idx2);
-        const double bondStretchEnergy = Utils::calcBondStretchEnergy(
+        const double bondStretchEnergy = MMFF::Utils::calcBondStretchEnergy(
             mmffBondParams.r0, mmffBondParams.kb, dist);
         if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
           oStream << std::left << std::setw(2) << iAtom->getSymbol() << " #"
@@ -101,6 +98,12 @@ void addBonds(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
   }
 }
 
+unsigned int twoBitCellPos(unsigned int nAtoms, int i, int j) {
+  if (j < i) std::swap(i, j);
+
+  return i * (nAtoms - 1) + i * (1 - i) / 2 + j;
+}
+
 void setTwoBitCell(boost::shared_array<boost::uint8_t> &res, unsigned int pos,
                    boost::uint8_t value) {
   unsigned int twoBitPos = pos / 4;
@@ -131,95 +134,74 @@ boost::uint8_t getTwoBitCell(boost::shared_array<boost::uint8_t> &res,
 //
 // ------------------------------------------------------------------------
 boost::shared_array<boost::uint8_t> buildNeighborMatrix(const ROMol &mol) {
+  const boost::uint8_t RELATION_1_X_INIT = RELATION_1_X | (RELATION_1_X << 2) |
+                                           (RELATION_1_X << 4) |
+                                           (RELATION_1_X << 6);
   unsigned int nAtoms = mol.getNumAtoms();
-  unsigned nTwoBitCells = (nAtoms * nAtoms - 1) / 4 + 1;
+  unsigned nTwoBitCells = (nAtoms * (nAtoms + 1) - 1) / 8 + 1;
   boost::shared_array<boost::uint8_t> res(new boost::uint8_t[nTwoBitCells]);
-  for (unsigned int i = 0; i < nTwoBitCells; ++i) {
-    res[i] = 0;
-  }
-  for (unsigned int i = 0; i < nAtoms; ++i) {
-    unsigned int iTab = i * nAtoms;
-    for (unsigned int j = i; j < nAtoms; ++j) {
-      setTwoBitCell(res, iTab + j, RELATION_1_X);
-      setTwoBitCell(res, i + j * nAtoms, RELATION_1_X);
-    }
-  }
-  for (unsigned int i = 0; i < mol.getNumBonds(); ++i) {
-    const Bond *bondi = mol.getBondWithIdx(i);
-
-    setTwoBitCell(res,
-                  bondi->getBeginAtomIdx() * nAtoms + bondi->getEndAtomIdx(),
+  std::memset(res.get(), RELATION_1_X_INIT, nTwoBitCells);
+  for (ROMol::ConstBondIterator bondi = mol.beginBonds();
+       bondi != mol.endBonds(); ++bondi) {
+    setTwoBitCell(res, twoBitCellPos(nAtoms, (*bondi)->getBeginAtomIdx(),
+                                     (*bondi)->getEndAtomIdx()),
                   RELATION_1_2);
-    setTwoBitCell(res,
-                  bondi->getEndAtomIdx() * nAtoms + bondi->getBeginAtomIdx(),
-                  RELATION_1_2);
-
-    for (unsigned int j = i + 1; j < mol.getNumBonds(); ++j) {
-      const Bond *bondj = mol.getBondWithIdx(j);
+    unsigned int bondiBeginAtomIdx = (*bondi)->getBeginAtomIdx();
+    unsigned int bondiEndAtomIdx = (*bondi)->getEndAtomIdx();
+    for (ROMol::ConstBondIterator bondj = bondi; ++bondj != mol.endBonds();) {
       int idx1 = -1;
       int idx3 = -1;
-      if (bondi->getBeginAtomIdx() == bondj->getBeginAtomIdx()) {
-        idx1 = bondi->getEndAtomIdx();
-        idx3 = bondj->getEndAtomIdx();
-      } else if (bondi->getBeginAtomIdx() == bondj->getEndAtomIdx()) {
-        idx1 = bondi->getEndAtomIdx();
-        idx3 = bondj->getBeginAtomIdx();
-      } else if (bondi->getEndAtomIdx() == bondj->getBeginAtomIdx()) {
-        idx1 = bondi->getBeginAtomIdx();
-        idx3 = bondj->getEndAtomIdx();
-      } else if (bondi->getEndAtomIdx() == bondj->getEndAtomIdx()) {
-        idx1 = bondi->getBeginAtomIdx();
-        idx3 = bondj->getBeginAtomIdx();
+      unsigned int bondjBeginAtomIdx = (*bondj)->getBeginAtomIdx();
+      unsigned int bondjEndAtomIdx = (*bondj)->getEndAtomIdx();
+      if (bondiBeginAtomIdx == bondjBeginAtomIdx) {
+        idx1 = bondiEndAtomIdx;
+        idx3 = bondjEndAtomIdx;
+      } else if (bondiBeginAtomIdx == bondjEndAtomIdx) {
+        idx1 = bondiEndAtomIdx;
+        idx3 = bondjBeginAtomIdx;
+      } else if (bondiEndAtomIdx == bondjBeginAtomIdx) {
+        idx1 = bondiBeginAtomIdx;
+        idx3 = bondjEndAtomIdx;
+      } else if (bondiEndAtomIdx == bondjEndAtomIdx) {
+        idx1 = bondiBeginAtomIdx;
+        idx3 = bondjBeginAtomIdx;
       } else {
         // check if atoms i and j are in a 1,4-relationship
-        if ((mol.getBondBetweenAtoms(bondi->getBeginAtomIdx(),
-                                     bondj->getBeginAtomIdx())) &&
-            (getTwoBitCell(res, bondi->getEndAtomIdx() * nAtoms +
-                                    bondj->getEndAtomIdx()) == RELATION_1_X)) {
-          setTwoBitCell(
-              res, bondi->getEndAtomIdx() * nAtoms + bondj->getEndAtomIdx(),
-              RELATION_1_4);
-          setTwoBitCell(
-              res, bondj->getEndAtomIdx() * nAtoms + bondi->getEndAtomIdx(),
-              RELATION_1_4);
-        } else if ((mol.getBondBetweenAtoms(bondi->getBeginAtomIdx(),
-                                            bondj->getEndAtomIdx())) &&
-                   (getTwoBitCell(res, bondi->getEndAtomIdx() * nAtoms +
-                                           bondj->getBeginAtomIdx()) ==
+        if ((mol.getBondBetweenAtoms(bondiBeginAtomIdx, bondjBeginAtomIdx)) &&
+            (getTwoBitCell(res, twoBitCellPos(nAtoms, bondiEndAtomIdx,
+                                              bondjEndAtomIdx)) ==
+             RELATION_1_X)) {
+          setTwoBitCell(res,
+                        twoBitCellPos(nAtoms, bondiEndAtomIdx, bondjEndAtomIdx),
+                        RELATION_1_4);
+        } else if ((mol.getBondBetweenAtoms(bondiBeginAtomIdx,
+                                            bondjEndAtomIdx)) &&
+                   (getTwoBitCell(res, twoBitCellPos(nAtoms, bondiEndAtomIdx,
+                                                     bondjBeginAtomIdx)) ==
                     RELATION_1_X)) {
           setTwoBitCell(
-              res, bondi->getEndAtomIdx() * nAtoms + bondj->getBeginAtomIdx(),
+              res, twoBitCellPos(nAtoms, bondiEndAtomIdx, bondjBeginAtomIdx),
               RELATION_1_4);
-          setTwoBitCell(
-              res, bondj->getBeginAtomIdx() * nAtoms + bondi->getEndAtomIdx(),
-              RELATION_1_4);
-        } else if ((mol.getBondBetweenAtoms(bondi->getEndAtomIdx(),
-                                            bondj->getBeginAtomIdx())) &&
-                   (getTwoBitCell(res, bondi->getBeginAtomIdx() * nAtoms +
-                                           bondj->getEndAtomIdx()) ==
+        } else if ((mol.getBondBetweenAtoms(bondiEndAtomIdx,
+                                            bondjBeginAtomIdx)) &&
+                   (getTwoBitCell(res, twoBitCellPos(nAtoms, bondiBeginAtomIdx,
+                                                     bondjEndAtomIdx)) ==
                     RELATION_1_X)) {
           setTwoBitCell(
-              res, bondi->getBeginAtomIdx() * nAtoms + bondj->getEndAtomIdx(),
+              res, twoBitCellPos(nAtoms, bondiBeginAtomIdx, bondjEndAtomIdx),
               RELATION_1_4);
-          setTwoBitCell(
-              res, bondj->getEndAtomIdx() * nAtoms + bondi->getBeginAtomIdx(),
-              RELATION_1_4);
-        } else if ((mol.getBondBetweenAtoms(bondi->getEndAtomIdx(),
-                                            bondj->getEndAtomIdx())) &&
-                   (getTwoBitCell(res, bondi->getBeginAtomIdx() * nAtoms +
-                                           bondj->getBeginAtomIdx()) ==
+        } else if ((mol.getBondBetweenAtoms(bondiEndAtomIdx,
+                                            bondjEndAtomIdx)) &&
+                   (getTwoBitCell(res, twoBitCellPos(nAtoms, bondiBeginAtomIdx,
+                                                     bondjBeginAtomIdx)) ==
                     RELATION_1_X)) {
           setTwoBitCell(
-              res, bondi->getBeginAtomIdx() * nAtoms + bondj->getBeginAtomIdx(),
-              RELATION_1_4);
-          setTwoBitCell(
-              res, bondj->getBeginAtomIdx() * nAtoms + bondi->getBeginAtomIdx(),
+              res, twoBitCellPos(nAtoms, bondiBeginAtomIdx, bondjBeginAtomIdx),
               RELATION_1_4);
         }
       }
       if (idx1 > -1) {
-        setTwoBitCell(res, idx1 * nAtoms + idx3, RELATION_1_3);
-        setTwoBitCell(res, idx3 * nAtoms + idx1, RELATION_1_3);
+        setTwoBitCell(res, twoBitCellPos(nAtoms, idx1, idx3), RELATION_1_3);
       }
     }
   }
@@ -241,7 +223,6 @@ void addAngles(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
   std::ostream &oStream = mmffMolProperties->getMMFFOStream();
   unsigned int idx[3];
   MMFFPropCollection *mmffProp = MMFFPropCollection::getMMFFProp();
-  AngleBendContrib *contrib;
   ROMol::ADJ_ITER nbr1Idx;
   ROMol::ADJ_ITER end1Nbrs;
   ROMol::ADJ_ITER nbr2Idx;
@@ -259,9 +240,9 @@ void addAngles(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  "  I        J        K          I    J    K   CLASS    ANGLE  "
                  "   ANGLE      DIFF.    ENERGY   CONSTANT\n"
                  "-------------------------------------------------------------"
-                 "-----------------------------------------" << std::endl;
+                 "-----------------------------------------"
+              << std::endl;
     }
-    field->initialize();
     points = field->positions();
   }
   for (idx[1] = 0; idx[1] < nAtoms; ++idx[1]) {
@@ -273,20 +254,20 @@ void addAngles(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
     const MMFFProp *mmffPropParamsCentralAtom = (*mmffProp)(jAtomType);
     boost::tie(nbr1Idx, end1Nbrs) = mol.getAtomNeighbors(jAtom);
     for (; nbr1Idx != end1Nbrs; ++nbr1Idx) {
-      const Atom *iAtom = mol[*nbr1Idx].get();
+      const Atom *iAtom = mol[*nbr1Idx];
       idx[0] = iAtom->getIdx();
       boost::tie(nbr2Idx, end2Nbrs) = mol.getAtomNeighbors(jAtom);
       for (; nbr2Idx != end2Nbrs; ++nbr2Idx) {
         if (nbr2Idx < (nbr1Idx + 1)) {
           continue;
         }
-        const Atom *kAtom = mol[*nbr2Idx].get();
+        const Atom *kAtom = mol[*nbr2Idx];
         idx[2] = kAtom->getIdx();
         unsigned int angleType;
         MMFFAngle mmffAngleParams;
         if (mmffMolProperties->getMMFFAngleBendParams(
                 mol, idx[0], idx[1], idx[2], angleType, mmffAngleParams)) {
-          contrib =
+          auto *contrib =
               new AngleBendContrib(field, idx[0], idx[1], idx[2],
                                    &mmffAngleParams, mmffPropParamsCentralAtom);
           field->contribs().push_back(ForceFields::ContribPtr(contrib));
@@ -302,11 +283,11 @@ void addAngles(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
             const RDGeom::Point3D p3((*(points[idx[2]]))[0],
                                      (*(points[idx[2]]))[1],
                                      (*(points[idx[2]]))[2]);
-            const double cosTheta =
-                Utils::calcCosTheta(p1, p2, p3, field->distance(idx[0], idx[1]),
-                                    field->distance(idx[1], idx[2]));
+            const double cosTheta = MMFF::Utils::calcCosTheta(
+                p1, p2, p3, field->distance(idx[0], idx[1]),
+                field->distance(idx[1], idx[2]));
             const double theta = RAD2DEG * acos(cosTheta);
-            const double angleBendEnergy = Utils::calcAngleBendEnergy(
+            const double angleBendEnergy = MMFF::Utils::calcAngleBendEnergy(
                 mmffAngleParams.theta0, mmffAngleParams.ka,
                 mmffPropParamsCentralAtom->linh, cosTheta);
             if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
@@ -356,7 +337,6 @@ void addStretchBend(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
   unsigned int idx[3];
   MMFFPropCollection *mmffProp = MMFFPropCollection::getMMFFProp();
   std::pair<bool, const MMFFStbn *> mmffStbnParams;
-  StretchBendContrib *contrib;
   ROMol::ADJ_ITER nbr1Idx;
   ROMol::ADJ_ITER end1Nbrs;
   ROMol::ADJ_ITER nbr2Idx;
@@ -377,7 +357,6 @@ void addStretchBend(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  "-----------------------------------------------------"
               << std::endl;
     }
-    field->initialize();
     points = field->positions();
   }
   for (idx[1] = 0; idx[1] < nAtoms; ++idx[1]) {
@@ -393,11 +372,11 @@ void addStretchBend(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
     boost::tie(nbr1Idx, end1Nbrs) = mol.getAtomNeighbors(jAtom);
     unsigned int i = 0;
     for (; nbr1Idx != end1Nbrs; ++nbr1Idx) {
-      const Atom *iAtom = mol[*nbr1Idx].get();
+      const Atom *iAtom = mol[*nbr1Idx];
       boost::tie(nbr2Idx, end2Nbrs) = mol.getAtomNeighbors(jAtom);
       unsigned int j = 0;
       for (; nbr2Idx != end2Nbrs; ++nbr2Idx) {
-        const Atom *kAtom = mol[*nbr2Idx].get();
+        const Atom *kAtom = mol[*nbr2Idx];
         if (j < (i + 1)) {
           ++j;
           continue;
@@ -411,7 +390,7 @@ void addStretchBend(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
         if (mmffMolProperties->getMMFFStretchBendParams(
                 mol, idx[0], idx[1], idx[2], stretchBendType, mmffStbnParams,
                 mmffBondParams, mmffAngleParams)) {
-          contrib = new StretchBendContrib(
+          auto *contrib = new StretchBendContrib(
               field, idx[0], idx[1], idx[2], &mmffStbnParams, &mmffAngleParams,
               &mmffBondParams[0], &mmffBondParams[1]);
           field->contribs().push_back(ForceFields::ContribPtr(contrib));
@@ -431,12 +410,12 @@ void addStretchBend(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                                      (*(points[idx[2]]))[1],
                                      (*(points[idx[2]]))[2]);
             const double cosTheta =
-                Utils::calcCosTheta(p1, p2, p3, dist1, dist2);
+                MMFF::Utils::calcCosTheta(p1, p2, p3, dist1, dist2);
             const double theta = RAD2DEG * acos(cosTheta);
             const std::pair<double, double> forceConstants =
-                Utils::calcStbnForceConstants(&mmffStbnParams);
+                MMFF::Utils::calcStbnForceConstants(&mmffStbnParams);
             const std::pair<double, double> stretchBendEnergies =
-                Utils::calcStretchBendEnergy(
+                MMFF::Utils::calcStretchBendEnergy(
                     dist1 - mmffBondParams[0].r0, dist2 - mmffBondParams[1].r0,
                     theta - mmffAngleParams.theta0, forceConstants);
             if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
@@ -507,7 +486,6 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
   ROMol::ADJ_ITER nbrIdx;
   ROMol::ADJ_ITER endNbrs;
 
-  OopBendContrib *contrib;
   double totalOopBendEnergy = 0.0;
   RDGeom::PointPtrVect points;
   if (mmffMolProperties->getMMFFVerbosity()) {
@@ -519,9 +497,9 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  "  I        J        K        L          I    J    K    L     "
                  "ANGLE    ENERGY   CONSTANT\n"
                  "-------------------------------------------------------------"
-                 "-----------------------------" << std::endl;
+                 "-----------------------------"
+              << std::endl;
     }
-    field->initialize();
     points = field->positions();
   }
   for (idx[1] = 0; idx[1] < mol.getNumAtoms(); ++idx[1]) {
@@ -533,7 +511,7 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
     boost::tie(nbrIdx, endNbrs) = mol.getAtomNeighbors(atom[1]);
     unsigned int i = 0;
     for (; nbrIdx != endNbrs; ++nbrIdx) {
-      atom[i] = mol[*nbrIdx].get();
+      atom[i] = mol[*nbrIdx];
       idx[i] = atom[i]->getIdx();
       atomType[i] = mmffMolProperties->getMMFFAtomType(idx[i]);
       if (!i) {
@@ -568,8 +546,8 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
           n[3] = 0;
           break;
       }
-      contrib = new OopBendContrib(field, idx[n[0]], idx[n[1]], idx[n[2]],
-                                   idx[n[3]], &mmffOopParams);
+      auto *contrib = new OopBendContrib(field, idx[n[0]], idx[n[1]], idx[n[2]],
+                                         idx[n[3]], &mmffOopParams);
       field->contribs().push_back(ForceFields::ContribPtr(contrib));
       if (mmffMolProperties->getMMFFVerbosity()) {
         const RDGeom::Point3D p1((*(points[idx[n[0]]]))[0],
@@ -584,9 +562,9 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
         const RDGeom::Point3D p4((*(points[idx[n[3]]]))[0],
                                  (*(points[idx[n[3]]]))[1],
                                  (*(points[idx[n[3]]]))[2]);
-        const double chi = Utils::calcOopChi(p1, p2, p3, p4);
+        const double chi = MMFF::Utils::calcOopChi(p1, p2, p3, p4);
         const double oopBendEnergy =
-            Utils::calcOopBendEnergy(chi, mmffOopParams.koop);
+            MMFF::Utils::calcOopBendEnergy(chi, mmffOopParams.koop);
         if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
           oStream << std::left << std::setw(2) << atom[0]->getSymbol() << " #"
                   << std::setw(5) << idx[n[0]] + 1 << std::setw(2)
@@ -620,9 +598,33 @@ void addOop(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
 //
 //
 // ------------------------------------------------------------------------
+const std::string DefaultTorsionBondSmarts::ds_string =
+    "[!$(*#*)&!D1]~[!$(*#*)&!D1]";
+boost::scoped_ptr<const ROMol> DefaultTorsionBondSmarts::ds_instance;
+
+void DefaultTorsionBondSmarts::create() {
+  ds_instance.reset(SmartsToMol(ds_string));
+}
+
+#ifdef RDK_THREADSAFE_SSS
+std::once_flag DefaultTorsionBondSmarts::ds_flag;
+#endif
+const ROMol *DefaultTorsionBondSmarts::query() {
+#ifdef RDK_THREADSAFE_SSS
+  std::call_once(ds_flag, create);
+#else
+  static bool created = false;
+  if (!created) {
+    created = true;
+    create();
+  }
+#endif
+  return ds_instance.get();
+}
+
 void addTorsions(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  ForceFields::ForceField *field,
-                 std::string torsionBondSmarts) {
+                 const std::string &torsionBondSmarts) {
   PRECONDITION(field, "bad ForceField");
   PRECONDITION(mmffMolProperties, "bad MMFFMolProperties");
   PRECONDITION(mmffMolProperties->isValid(),
@@ -647,14 +649,16 @@ void addTorsions(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                  "-----------------------------------------------------"
               << std::endl;
     }
-    field->initialize();
     points = field->positions();
   }
   std::vector<MatchVectType> matchVect;
-  ROMol *query = SmartsToMol(torsionBondSmarts);
+  const ROMol *defaultQuery = DefaultTorsionBondSmarts::query();
+  const ROMol *query = (torsionBondSmarts == DefaultTorsionBondSmarts::string())
+                           ? defaultQuery
+                           : SmartsToMol(torsionBondSmarts);
   TEST_ASSERT(query);
   unsigned int nHits = SubstructMatch(mol, *query, matchVect);
-  delete query;
+  if (query != defaultQuery) delete query;
 
   for (unsigned int i = 0; i < nHits; ++i) {
     MatchVectType match = matchVect[i];
@@ -672,13 +676,13 @@ void addTorsions(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
       ROMol::OEDGE_ITER beg1, end1;
       boost::tie(beg1, end1) = mol.getAtomBonds(jAtom);
       while (beg1 != end1) {
-        const Bond *tBond1 = mol[*beg1].get();
+        const Bond *tBond1 = mol[*beg1];
         if (tBond1 != bond) {
           int idx1 = tBond1->getOtherAtomIdx(idx2);
           ROMol::OEDGE_ITER beg2, end2;
           boost::tie(beg2, end2) = mol.getAtomBonds(kAtom);
           while (beg2 != end2) {
-            const Bond *tBond2 = mol[*beg2].get();
+            const Bond *tBond2 = mol[*beg2];
             if ((tBond2 != bond) && (tBond2 != tBond1)) {
               int idx4 = tBond2->getOtherAtomIdx(idx3);
               // make sure this isn't a three-membered ring:
@@ -689,7 +693,7 @@ void addTorsions(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                 MMFFTor mmffTorParams;
                 if (mmffMolProperties->getMMFFTorsionParams(
                         mol, idx1, idx2, idx3, idx4, torType, mmffTorParams)) {
-                  TorsionAngleContrib *contrib = new TorsionAngleContrib(
+                  auto *contrib = new TorsionAngleContrib(
                       field, idx1, idx2, idx3, idx4, &mmffTorParams);
                   field->contribs().push_back(ForceFields::ContribPtr(contrib));
                   if (mmffMolProperties->getMMFFVerbosity()) {
@@ -716,8 +720,8 @@ void addTorsions(const ROMol &mol, MMFFMolProperties *mmffMolProperties,
                                              (*(points[idx4]))[1],
                                              (*(points[idx4]))[2]);
                     const double cosPhi =
-                        Utils::calcTorsionCosPhi(p1, p2, p3, p4);
-                    const double torsionEnergy = Utils::calcTorsionEnergy(
+                        MMFF::Utils::calcTorsionCosPhi(p1, p2, p3, p4);
+                    const double torsionEnergy = MMFF::Utils::calcTorsionEnergy(
                         mmffTorParams.V1, mmffTorParams.V2, mmffTorParams.V3,
                         cosPhi);
                     if (mmffMolProperties->getMMFFVerbosity() ==
@@ -793,7 +797,8 @@ void addVdW(const ROMol &mol, int confId, MMFFMolProperties *mmffMolProperties,
                  "  I        J          I    J    DISTANCE   ENERGY     R*     "
                  " DEPTH\n"
                  "-------------------------------------------------------------"
-                 "-------" << std::endl;
+                 "-------"
+              << std::endl;
     }
   }
   const Conformer &conf = mol.getConformer(confId);
@@ -802,20 +807,21 @@ void addVdW(const ROMol &mol, int confId, MMFFMolProperties *mmffMolProperties,
       if (ignoreInterfragInteractions && (fragMapping[i] != fragMapping[j])) {
         continue;
       }
-      if (getTwoBitCell(neighborMatrix, i * nAtoms + j) >= RELATION_1_4) {
+      if (getTwoBitCell(neighborMatrix, twoBitCellPos(nAtoms, i, j)) >=
+          RELATION_1_4) {
         double dist = (conf.getAtomPos(i) - conf.getAtomPos(j)).length();
         if (dist > nonBondedThresh) {
           continue;
         }
-        MMFFVdWRijstarEps mmffVdWParams;
-        if (mmffMolProperties->getMMFFVdWParams(i, j, mmffVdWParams)) {
-          VdWContrib *contrib = new VdWContrib(field, i, j, &mmffVdWParams);
+        MMFFVdWRijstarEps mmffVdWConstants;
+        if (mmffMolProperties->getMMFFVdWParams(i, j, mmffVdWConstants)) {
+          auto *contrib = new VdWContrib(field, i, j, &mmffVdWConstants);
           field->contribs().push_back(ForceFields::ContribPtr(contrib));
           if (mmffMolProperties->getMMFFVerbosity()) {
             const Atom *iAtom = mol.getAtomWithIdx(i);
             const Atom *jAtom = mol.getAtomWithIdx(j);
-            const double vdWEnergy = Utils::calcVdWEnergy(
-                dist, mmffVdWParams.R_ij_star, mmffVdWParams.epsilon);
+            const double vdWEnergy = MMFF::Utils::calcVdWEnergy(
+                dist, mmffVdWConstants.R_ij_star, mmffVdWConstants.epsilon);
             if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
               unsigned int iAtomType = mmffMolProperties->getMMFFAtomType(i);
               unsigned int jAtomType = mmffMolProperties->getMMFFAtomType(j);
@@ -825,8 +831,8 @@ void addVdW(const ROMol &mol, int confId, MMFFMolProperties *mmffMolProperties,
                       << std::right << std::setw(5) << iAtomType << std::setw(5)
                       << jAtomType << "  " << std::fixed << std::setprecision(3)
                       << std::setw(9) << dist << std::setw(10) << vdWEnergy
-                      << std::setw(9) << mmffVdWParams.R_ij_star << std::setw(9)
-                      << mmffVdWParams.epsilon << std::endl;
+                      << std::setw(9) << mmffVdWConstants.R_ij_star
+                      << std::setw(9) << mmffVdWConstants.epsilon << std::endl;
             }
             totalVdWEnergy += vdWEnergy;
           }
@@ -884,29 +890,30 @@ void addEle(const ROMol &mol, int confId, MMFFMolProperties *mmffMolProperties,
       if (ignoreInterfragInteractions && (fragMapping[i] != fragMapping[j])) {
         continue;
       }
-      if ((getTwoBitCell(neighborMatrix, i * nAtoms + j) >= RELATION_1_4) &&
-          (!isDoubleZero(mmffMolProperties->getMMFFPartialCharge(i))) &&
-          (!isDoubleZero(mmffMolProperties->getMMFFPartialCharge(j)))) {
+      boost::uint8_t cell =
+          getTwoBitCell(neighborMatrix, twoBitCellPos(nAtoms, i, j));
+      bool is1_4 = (cell == RELATION_1_4);
+      if (cell >= RELATION_1_4) {
+        if (isDoubleZero(mmffMolProperties->getMMFFPartialCharge(i)) ||
+            isDoubleZero(mmffMolProperties->getMMFFPartialCharge(j)))
+          continue;
         double dist = (conf.getAtomPos(i) - conf.getAtomPos(j)).length();
         if (dist > nonBondedThresh) {
           continue;
         }
-        EleContrib *contrib;
         double chargeTerm = mmffMolProperties->getMMFFPartialCharge(i) *
                             mmffMolProperties->getMMFFPartialCharge(j) /
                             dielConst;
-        contrib = new EleContrib(
-            field, i, j, chargeTerm, dielModel,
-            getTwoBitCell(neighborMatrix, i * nAtoms + j) == RELATION_1_4);
+        auto *contrib =
+            new EleContrib(field, i, j, chargeTerm, dielModel, is1_4);
         field->contribs().push_back(ForceFields::ContribPtr(contrib));
         if (mmffMolProperties->getMMFFVerbosity()) {
           const unsigned int iAtomType = mmffMolProperties->getMMFFAtomType(i);
           const unsigned int jAtomType = mmffMolProperties->getMMFFAtomType(j);
           const Atom *iAtom = mol.getAtomWithIdx(i);
           const Atom *jAtom = mol.getAtomWithIdx(j);
-          const double eleEnergy = Utils::calcEleEnergy(
-              i, j, dist, chargeTerm, dielModel,
-              getTwoBitCell(neighborMatrix, i * nAtoms + j) == RELATION_1_4);
+          const double eleEnergy = MMFF::Utils::calcEleEnergy(
+              i, j, dist, chargeTerm, dielModel, is1_4);
           if (mmffMolProperties->getMMFFVerbosity() == MMFF_VERBOSITY_HIGH) {
             oStream << std::left << std::setw(2) << iAtom->getSymbol() << " #"
                     << std::setw(5) << i + 1 << std::setw(2)
@@ -963,13 +970,14 @@ ForceFields::ForceField *constructForceField(
   PRECONDITION(mmffMolProperties->isValid(),
                "missing atom types - invalid force-field");
 
-  ForceFields::ForceField *res = new ForceFields::ForceField();
+  auto *res = new ForceFields::ForceField();
   // add the atomic positions:
   Conformer &conf = mol.getConformer(confId);
   for (unsigned int i = 0; i < mol.getNumAtoms(); ++i) {
     res->positions().push_back(&(conf.getAtomPos(i)));
   }
 
+  res->initialize();
   if (mmffMolProperties->getMMFFBondTerm()) {
     Tools::addBonds(mol, mmffMolProperties, res);
   }
