@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2004-2010 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2004-2018 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -9,6 +8,7 @@
 //  of the RDKit source tree.
 //
 
+#include <RDGeneral/test.h>
 #include <iostream>
 #include <RDGeneral/Invariant.h>
 #include <RDGeneral/RDLog.h>
@@ -213,18 +213,32 @@ void testUFFBuilder1() {
   boost::shared_array<boost::uint8_t> nbrMat;
 
   mol = SmilesToMol("CC(O)C");
-  Conformer *conf = new Conformer(mol->getNumAtoms());
+  auto *conf = new Conformer(mol->getNumAtoms());
   int cid = static_cast<int>(mol->addConformer(conf, true));
   TEST_ASSERT(mol);
   boost::tie(types, foundAll) = UFF::getAtomTypes(*mol);
   TEST_ASSERT(foundAll);
   TEST_ASSERT(types.size() == mol->getNumAtoms());
   field = new ForceFields::ForceField();
+  // add the atomic positions:
+  for (unsigned int i = 0; i < mol->getNumAtoms(); ++i) {
+    field->positions().push_back(&((conf->getAtomPos(i))));
+  }
+
   UFF::Tools::addBonds(*mol, types, field);
 
   TEST_ASSERT(field->contribs().size() == 3);
 
   nbrMat = UFF::Tools::buildNeighborMatrix(*mol);
+  // the neighbor matrix is an upper triangular matrix
+  // position indices are as follows:
+  //  0  1  2  3
+  //     4  5  6
+  //        7  8
+  //           9
+  TEST_ASSERT(UFF::Tools::twoBitCellPos(mol->getNumAtoms(), 1, 1) == 4);
+  TEST_ASSERT(UFF::Tools::twoBitCellPos(mol->getNumAtoms(), 2, 1) == 5);
+  TEST_ASSERT(UFF::Tools::twoBitCellPos(mol->getNumAtoms(), 1, 2) == 5);
   TEST_ASSERT(UFF::Tools::getTwoBitCell(nbrMat, 0) == UFF::Tools::RELATION_1_X);
   TEST_ASSERT(UFF::Tools::getTwoBitCell(nbrMat, 1) == UFF::Tools::RELATION_1_2);
   TEST_ASSERT(UFF::Tools::getTwoBitCell(nbrMat, 2) == UFF::Tools::RELATION_1_3);
@@ -244,13 +258,18 @@ void testUFFBuilder1() {
   delete mol;
   delete field;
   mol = SmilesToMol("CCOC");
-  Conformer *conf2 = new Conformer(mol->getNumAtoms());
+  auto *conf2 = new Conformer(mol->getNumAtoms());
   cid = static_cast<int>(mol->addConformer(conf2, true));
   TEST_ASSERT(mol);
   boost::tie(types, foundAll) = UFF::getAtomTypes(*mol);
   TEST_ASSERT(foundAll);
   TEST_ASSERT(types.size() == mol->getNumAtoms());
   field = new ForceFields::ForceField();
+  // add the atomic positions:
+  for (unsigned int i = 0; i < mol->getNumAtoms(); ++i) {
+    field->positions().push_back(&(conf2->getAtomPos(i)));
+  }
+
   UFF::Tools::addBonds(*mol, types, field);
 
   TEST_ASSERT(field->contribs().size() == 3);
@@ -271,7 +290,7 @@ void testUFFBuilder1() {
   delete mol;
   delete field;
   mol = SmilesToMol("CO");
-  Conformer *conf3 = new Conformer(mol->getNumAtoms());
+  auto *conf3 = new Conformer(mol->getNumAtoms());
   cid = static_cast<int>(mol->addConformer(conf3, true));
   TEST_ASSERT(mol);
   boost::tie(types, foundAll) = UFF::getAtomTypes(*mol);
@@ -279,6 +298,11 @@ void testUFFBuilder1() {
   TEST_ASSERT(types.size() == mol->getNumAtoms());
 
   field = new ForceFields::ForceField();
+  // add the atomic positions:
+  for (unsigned int i = 0; i < mol->getNumAtoms(); ++i) {
+    field->positions().push_back(&(conf3->getAtomPos(i)));
+  }
+
   UFF::Tools::addBonds(*mol, types, field);
 
   TEST_ASSERT(field->contribs().size() == 1);
@@ -301,8 +325,15 @@ void testUFFBuilder1() {
   boost::tie(types, foundAll) = UFF::getAtomTypes(*mol2);
   TEST_ASSERT(foundAll);
   TEST_ASSERT(types.size() == mol2->getNumAtoms());
+  auto *conf4 = new Conformer(mol2->getNumAtoms());
+  cid = static_cast<int>(mol2->addConformer(conf4, true));
 
   field = new ForceFields::ForceField();
+  // add the atomic positions:
+  for (unsigned int i = 0; i < mol2->getNumAtoms(); ++i) {
+    field->positions().push_back(&(conf4->getAtomPos(i)));
+  }
+
   UFF::Tools::addBonds(*mol2, types, field);
   TEST_ASSERT(field->contribs().size() == 5);
 
@@ -347,7 +378,7 @@ void testUFFBuilder2() {
     RWMol *mol = MolFileToMol(pathName + "/small1.mol", false);
     TEST_ASSERT(mol);
     MolOps::sanitizeMol(*mol);
-    Conformer *newConf = new Conformer(mol->getConformer());
+    auto *newConf = new Conformer(mol->getConformer());
     newConf->setId(111);
     mol->addConformer(newConf, false);
     RDGeom::Point3D p0 = mol->getConformer().getAtomPos(0);
@@ -439,10 +470,36 @@ void testUFFBuilder2() {
     RWMol *mol = MolFileToMol(pathName + "/small1.mol", false);
     TEST_ASSERT(mol);
     MolOps::sanitizeMol(*mol);
-    RWMol *mol2 = new RWMol(*mol);
+
+    UFF::AtomicParamVect types;
+    bool foundAll;
+    boost::shared_array<boost::uint8_t> nbrMat;
+    boost::tie(types, foundAll) = UFF::getAtomTypes(*mol);
 
     ForceFields::ForceField *field;
+    field = new ForceFields::ForceField();
+
+    // add the atomic positions:
+    for (unsigned int i = 0; i < mol->getNumAtoms(); ++i) {
+      field->positions().push_back(&((mol->getConformer().getAtomPos(i))));
+    }
+
+    UFF::Tools::addBonds(*mol, types, field);
+    nbrMat = UFF::Tools::buildNeighborMatrix(*mol);
+    UFF::Tools::addAngles(*mol, types, field);
+    UFF::Tools::addTorsions(*mol, types, field);
+    // std::cout << field->contribs().size() << std::endl;
+    UFF::Tools::addNonbonded(*mol, 0, types, field, nbrMat);
+    delete field;
+
     field = UFF::constructForceField(*mol);
+    field->initialize();
+    field->minimize();
+    delete field;
+
+    auto *mol2 = new RWMol(*mol);
+
+    field = UFF::constructForceField(*mol2);
     TEST_ASSERT(field);
     field->initialize();
     int needMore = field->minimize();
@@ -467,8 +524,8 @@ void testUFFBuilder2() {
     RWMol *mol = MolFileToMol(pathName + "/complex1.mol", false);
     TEST_ASSERT(mol);
     MolOps::sanitizeMol(*mol);
-    RWMol *mol2 = new RWMol(*mol);
-    Conformer *newConf = new Conformer(mol->getConformer());
+    auto *mol2 = new RWMol(*mol);
+    auto *newConf = new Conformer(mol->getConformer());
     newConf->setId(111);
     mol->addConformer(newConf, false);
 
@@ -486,7 +543,7 @@ void testUFFBuilder2() {
     newConf->setId(112);
     mol2->addConformer(newConf, false);
 
-    std::vector<std::pair<int, double> > res;
+    std::vector<std::pair<int, double>> res;
     UFF::UFFOptimizeMoleculeConfs(*mol2, res);
     TEST_ASSERT(res.size() == 2);
     TEST_ASSERT(!res[0].first);
@@ -535,14 +592,14 @@ void testUFFBatch() {
     try {
       field = UFF::constructForceField(*mol);
     } catch (...) {
-      field = 0;
+      field = nullptr;
     }
     if (field) {
       field->initialize();
       int failed = field->minimize(500);
       if (failed) {
-        BOOST_LOG(rdErrorLog) << " not converged (code=" << failed << ")"
-                              << std::endl;
+        BOOST_LOG(rdErrorLog)
+            << " not converged (code=" << failed << ")" << std::endl;
         std::cout << origMolBlock << "$$$$" << std::endl;
         std::cout << MolToMolBlock(*mol) << "$$$$" << std::endl;
       }
@@ -646,6 +703,91 @@ void testIssue239() {
 }
 #endif
 
+void testCalcEnergyPassedCoords() {
+  BOOST_LOG(rdErrorLog) << "-------------------------------------" << std::endl;
+  BOOST_LOG(rdErrorLog) << "    Testing calcEnergy with passed coords."
+                        << std::endl;
+
+  RWMol *mol;
+  ForceFields::ForceField *field;
+  double e1, e2, e3;
+
+  std::string pathName = getenv("RDBASE");
+  pathName += "/Code/GraphMol/ForceFieldHelpers/MMFF/test_data";
+  mol = MolFileToMol(pathName + "/Issue239.mol", false);
+  TEST_ASSERT(mol);
+  MolOps::sanitizeMol(*mol);
+
+  field = UFF::constructForceField(*mol);
+  TEST_ASSERT(field);
+  field->initialize();
+  double *savedPos = new double[3 * field->numPoints()];
+  size_t i = 0;
+  for (const auto pptr : field->positions()) {
+    for (size_t j = 0; j < 3; ++j) savedPos[i++] = (*pptr)[j];
+  }
+  e1 = field->calcEnergy();
+  field->minimize(10000, 1.0e-6, 1.0e-3);
+  e2 = field->calcEnergy();
+  TEST_ASSERT(e2 < e1);
+  e3 = field->calcEnergy(savedPos);
+  TEST_ASSERT(feq(e3, e1, 0.01));
+
+  delete[] savedPos;
+  delete mol;
+  delete field;
+
+  BOOST_LOG(rdErrorLog) << "  done" << std::endl;
+}
+
+void testCalcGrad() {
+  BOOST_LOG(rdErrorLog) << "-------------------------------------" << std::endl;
+  BOOST_LOG(rdErrorLog) << "    Testing calcGrad." << std::endl;
+
+  RWMol *mol;
+  ForceFields::ForceField *field;
+
+  std::string pathName = getenv("RDBASE");
+  pathName += "/Code/GraphMol/ForceFieldHelpers/MMFF/test_data";
+  mol = MolFileToMol(pathName + "/Issue239.mol", false);
+  TEST_ASSERT(mol);
+  MolOps::sanitizeMol(*mol);
+
+  field = UFF::constructForceField(*mol);
+  TEST_ASSERT(field);
+  field->initialize();
+  size_t l = 3 * field->numPoints();
+  double *savedPos = new double[l];
+  double *grad1 = new double[l];
+  double *grad2 = new double[l];
+  size_t i = 0;
+  for (const auto pptr : field->positions()) {
+    for (size_t j = 0; j < 3; ++j) savedPos[i++] = (*pptr)[j];
+  }
+  TEST_ASSERT(i == l);
+
+  std::memset(grad1, 0, l * sizeof(double));
+  field->calcGrad(grad1);
+  for (i = 0; i < l; ++i) TEST_ASSERT(!feq(grad1[i], 0.0, 0.001));
+
+  field->minimize(10000, 1.0e-6, 1.0e-3);
+  std::memset(grad2, 0, l * sizeof(double));
+  field->calcGrad(grad2);
+  for (i = 0; i < l; ++i) TEST_ASSERT(feq(grad2[i], 0.0, 0.001));
+
+  field->initialize();
+  std::memset(grad2, 0, l * sizeof(double));
+  field->calcGrad(savedPos, grad2);
+  for (i = 0; i < l; ++i) TEST_ASSERT(feq(grad1[i], grad2[i], 0.001));
+
+  delete[] savedPos;
+  delete[] grad1;
+  delete[] grad2;
+  delete mol;
+  delete field;
+
+  BOOST_LOG(rdErrorLog) << "  done" << std::endl;
+}
 void testIssue242() {
 #if 0
 // FIX: Changes to the forcefield (connected to Issue 408) have
@@ -676,7 +818,7 @@ void testIssue242() {
   //BOOST_LOG(rdInfoLog) << "\nMol1\n" << mb1 << std::endl;
   //BOOST_LOG(rdInfoLog) << "\nMol2\n" << mb2 << std::endl;
 
-  
+
   field=UFF::constructForceField(*mol);
   TEST_ASSERT(field);
   field->initialize();
@@ -698,7 +840,7 @@ void testIssue242() {
   BOOST_LOG(rdInfoLog) << "E1: " << e1 << std::endl;
   BOOST_LOG(rdInfoLog) << "E2: " << e2 << std::endl;
   TEST_ASSERT(feq(e2,e1,1.0));
-  
+
   needMore = field->minimize(600,1e-4);
   TEST_ASSERT(!needMore)
   needMore = field2->minimize(600,1e-4);
@@ -708,7 +850,7 @@ void testIssue242() {
   BOOST_LOG(rdInfoLog) << "rE1: " << e1 << std::endl;
   BOOST_LOG(rdInfoLog) << "rE2: " << e2 << std::endl;
   TEST_ASSERT(feq(e2,e1,1.0));
-  
+
   delete mol;
   delete mol2;
   delete field;
@@ -751,7 +893,7 @@ void testIssue242() {
   delete mol2;
   delete field;
   delete field2;
-  
+
 
   BOOST_LOG(rdErrorLog) << "  done" << std::endl;
 #endif
@@ -779,6 +921,11 @@ void testSFIssue1653802() {
   TEST_ASSERT(foundAll);
   TEST_ASSERT(types.size() == mol->getNumAtoms());
   field = new ForceFields::ForceField();
+  // add the atomic positions:
+  for (unsigned int i = 0; i < mol->getNumAtoms(); ++i) {
+    field->positions().push_back(&((mol->getConformer().getAtomPos(i))));
+  }
+
   UFF::Tools::addBonds(*mol, types, field);
   TEST_ASSERT(field->contribs().size() == 8);
 
@@ -1026,14 +1173,14 @@ void runblock_uff(const std::vector<ROMol *> &mols,
     for (unsigned int i = 0; i < mols.size(); ++i) {
       if (i % count != idx) continue;
       ROMol *mol = mols[i];
-      ForceFields::ForceField *field = 0;
+      ForceFields::ForceField *field = nullptr;
       if (!(rep % 100)) {
         BOOST_LOG(rdErrorLog) << "Rep: " << rep << " Mol:" << i << std::endl;
       }
       try {
         field = UFF::constructForceField(*mol);
       } catch (...) {
-        field = 0;
+        field = nullptr;
       }
       TEST_ASSERT(field);
       field->initialize();
@@ -1046,7 +1193,8 @@ void runblock_uff(const std::vector<ROMol *> &mols,
   }
 }
 }
-#include <boost/thread.hpp>
+#include <thread>
+#include <future>
 void testUFFMultiThread() {
   BOOST_LOG(rdErrorLog) << "-------------------------------------" << std::endl;
   BOOST_LOG(rdErrorLog) << "    Test UFF multithreading" << std::endl;
@@ -1058,7 +1206,7 @@ void testUFFMultiThread() {
   SDMolSupplier suppl(pathName + "/bulk.sdf");
   std::vector<ROMol *> mols;
   while (!suppl.atEnd() && mols.size() < 100) {
-    ROMol *mol = 0;
+    ROMol *mol = nullptr;
     try {
       mol = suppl.next();
     } catch (...) {
@@ -1072,11 +1220,11 @@ void testUFFMultiThread() {
   std::vector<double> energies(mols.size(), 0.0);
   for (unsigned int i = 0; i < mols.size(); ++i) {
     ROMol mol(*mols[i]);
-    ForceFields::ForceField *field = 0;
+    ForceFields::ForceField *field = nullptr;
     try {
       field = UFF::constructForceField(mol);
     } catch (...) {
-      field = 0;
+      field = nullptr;
     }
     TEST_ASSERT(field);
     field->initialize();
@@ -1086,16 +1234,19 @@ void testUFFMultiThread() {
     delete field;
   }
 
-  boost::thread_group tg;
+  std::vector<std::future<void>> tg;
 
   std::cerr << "processing" << std::endl;
   unsigned int count = 4;
   for (unsigned int i = 0; i < count; ++i) {
     std::cerr << " launch :" << i << std::endl;
     std::cerr.flush();
-    tg.add_thread(new boost::thread(runblock_uff, mols, energies, count, i));
+    tg.emplace_back(
+        std::async(std::launch::async, runblock_uff, mols, energies, count, i));
+  }
+  for (auto &fut : tg) {
+    fut.get();
   }
-  tg.join_all();
 
   BOOST_FOREACH (ROMol *mol, mols) { delete mol; }
   BOOST_LOG(rdErrorLog) << "  done" << std::endl;
@@ -1110,11 +1261,11 @@ void testUFFMultiThread2() {
   SDMolSupplier suppl(pathName + "/bulk.sdf");
   ROMol *m = suppl[4];
   TEST_ASSERT(m);
-  ROMol *om = new ROMol(*m);
+  auto *om = new ROMol(*m);
   for (unsigned int i = 0; i < 1000; ++i) {
     m->addConformer(new Conformer(m->getConformer()), true);
   }
-  std::vector<std::pair<int, double> > res;
+  std::vector<std::pair<int, double>> res;
 
   UFF::UFFOptimizeMolecule(*om);
   UFF::UFFOptimizeMoleculeConfs(*m, res, 0);
@@ -1143,7 +1294,8 @@ void testUFFMultiThread2() {
 void testGitHubIssue613() {
   BOOST_LOG(rdErrorLog) << "-------------------------------------" << std::endl;
   BOOST_LOG(rdErrorLog) << "    Test Github Issue 613: UFF Atom type not "
-                           "properly assigned to lanthanides." << std::endl;
+                           "properly assigned to lanthanides."
+                        << std::endl;
   {
     ROMol *mol =
         SmilesToMol("[Eu+3]123456.[Cl]1.[Cl]2.[Cl]3.[Cl]4.[Cl]5.[Cl]6");
@@ -1179,6 +1331,8 @@ int main() {
   testUFFBatch();
   testUFFBuilderSpecialCases();
   testIssue239();
+  testCalcEnergyPassedCoords();
+  testCalcGrad();
   testIssue242();
   testSFIssue1653802();
   testSFIssue2378119();
