@@ -29,21 +29,22 @@ int UFFHelper(ROMol &mol, int maxIters, double vdwThresh, int confId,
               bool ignoreInterfragInteractions) {
   NOGIL gil;
   return UFF::UFFOptimizeMolecule(mol, maxIters, vdwThresh, confId,
-                                  ignoreInterfragInteractions).first;
+                                  ignoreInterfragInteractions)
+      .first;
 }
 python::object UFFConfsHelper(ROMol &mol, int numThreads, int maxIters,
                               double vdwThresh, int confId,
                               bool ignoreInterfragInteractions) {
   RDUNUSED_PARAM(confId);  // XXX FIX ME?
-  std::vector<std::pair<int, double> > res;
+  std::vector<std::pair<int, double>> res;
   {
     NOGIL gil;
     UFF::UFFOptimizeMoleculeConfs(mol, res, numThreads, maxIters, vdwThresh,
                                   ignoreInterfragInteractions);
   }
   python::list pyres;
-  for (unsigned int i = 0; i < res.size(); ++i) {
-    pyres.append(python::make_tuple(res[i].first, res[i].second));
+  for (auto &itm : res) {
+    pyres.append(python::make_tuple(itm.first, itm.second));
   }
   return pyres;
 }
@@ -52,7 +53,7 @@ python::object MMFFConfsHelper(ROMol &mol, int numThreads, int maxIters,
                                std::string mmffVariant, double nonBondedThresh,
                                int confId, bool ignoreInterfragInteractions) {
   RDUNUSED_PARAM(confId);  // Fix me?
-  std::vector<std::pair<int, double> > res;
+  std::vector<std::pair<int, double>> res;
   {
     NOGIL gil;
     MMFF::MMFFOptimizeMoleculeConfs(mol, res, numThreads, maxIters, mmffVariant,
@@ -60,8 +61,8 @@ python::object MMFFConfsHelper(ROMol &mol, int numThreads, int maxIters,
                                     ignoreInterfragInteractions);
   }
   python::list pyres;
-  for (unsigned int i = 0; i < res.size(); ++i) {
-    pyres.append(python::make_tuple(res[i].first, res[i].second));
+  for (auto &itm : res) {
+    pyres.append(python::make_tuple(itm.first, itm.second));
   }
   return pyres;
 }
@@ -71,7 +72,7 @@ ForceFields::PyForceField *UFFGetMoleculeForceField(
     bool ignoreInterfragInteractions = true) {
   ForceFields::ForceField *ff = UFF::constructForceField(
       mol, vdwThresh, confId, ignoreInterfragInteractions);
-  ForceFields::PyForceField *res = new ForceFields::PyForceField(ff);
+  auto *res = new ForceFields::PyForceField(ff);
   res->initialize();
   return res;
 }
@@ -110,9 +111,9 @@ unsigned int SanitizeMMFFMol(ROMol &mol) {
 ForceFields::PyMMFFMolProperties *GetMMFFMolProperties(
     ROMol &mol, std::string mmffVariant = "MMFF94",
     unsigned int mmffVerbosity = MMFF::MMFF_VERBOSITY_NONE) {
-  MMFF::MMFFMolProperties *mmffMolProperties =
+  auto *mmffMolProperties =
       new MMFF::MMFFMolProperties(mol, mmffVariant, mmffVerbosity);
-  ForceFields::PyMMFFMolProperties *pyMP = NULL;
+  ForceFields::PyMMFFMolProperties *pyMP = nullptr;
 
   if (mmffMolProperties && mmffMolProperties->isValid()) {
     pyMP = new ForceFields::PyMMFFMolProperties(mmffMolProperties);
@@ -125,7 +126,7 @@ ForceFields::PyForceField *MMFFGetMoleculeForceField(
     ROMol &mol, ForceFields::PyMMFFMolProperties *pyMMFFMolProperties,
     double nonBondedThresh = 100.0, int confId = -1,
     bool ignoreInterfragInteractions = true) {
-  ForceFields::PyForceField *pyFF = NULL;
+  ForceFields::PyForceField *pyFF = nullptr;
   boost::python::list res;
 
   if (pyMMFFMolProperties) {
@@ -155,7 +156,7 @@ namespace ForceFields {
 PyObject *getUFFBondStretchParams(const RDKit::ROMol &mol,
                                   const unsigned int idx1,
                                   const unsigned int idx2) {
-  PyObject *res = NULL;
+  PyObject *res = nullptr;
   ForceFields::UFF::UFFBond uffBondStretchParams;
   if (RDKit::UFF::getUFFBondStretchParams(mol, idx1, idx2,
                                           uffBondStretchParams)) {
@@ -170,7 +171,7 @@ PyObject *getUFFAngleBendParams(const RDKit::ROMol &mol,
                                 const unsigned int idx1,
                                 const unsigned int idx2,
                                 const unsigned int idx3) {
-  PyObject *res = NULL;
+  PyObject *res = nullptr;
   ForceFields::UFF::UFFAngle uffAngleBendParams;
   if (RDKit::UFF::getUFFAngleBendParams(mol, idx1, idx2, idx3,
                                         uffAngleBendParams)) {
@@ -184,7 +185,7 @@ PyObject *getUFFAngleBendParams(const RDKit::ROMol &mol,
 PyObject *getUFFTorsionParams(const RDKit::ROMol &mol, const unsigned int idx1,
                               const unsigned int idx2, const unsigned int idx3,
                               const unsigned int idx4) {
-  PyObject *res = NULL;
+  PyObject *res = nullptr;
   ForceFields::UFF::UFFTor uffTorsionParams;
   if (RDKit::UFF::getUFFTorsionParams(mol, idx1, idx2, idx3, idx4,
                                       uffTorsionParams)) {
@@ -198,7 +199,7 @@ PyObject *getUFFInversionParams(const RDKit::ROMol &mol,
                                 const unsigned int idx2,
                                 const unsigned int idx3,
                                 const unsigned int idx4) {
-  PyObject *res = NULL;
+  PyObject *res = nullptr;
   ForceFields::UFF::UFFInv uffInversionParams;
   if (RDKit::UFF::getUFFInversionParams(mol, idx1, idx2, idx3, idx4,
                                         uffInversionParams)) {
@@ -209,7 +210,7 @@ PyObject *getUFFInversionParams(const RDKit::ROMol &mol,
 
 PyObject *getUFFVdWParams(const RDKit::ROMol &mol, const unsigned int idx1,
                           const unsigned int idx2) {
-  PyObject *res = NULL;
+  PyObject *res = nullptr;
   ForceFields::UFF::UFFVdW uffVdWParams;
   if (RDKit::UFF::getUFFVdWParams(mol, idx1, idx2, uffVdWParams)) {
     res = PyTuple_New(2);
@@ -259,7 +260,8 @@ BOOST_PYTHON_MODULE(rdForceFieldHelpers) {
     - ignoreInterfragInteractions : if true, nonbonded terms between\n\
                   fragments will not be added to the forcefield.\n\
 \n\
- RETURNS: 0 if the optimization converged, 1 if more iterations are required.\n\
+ RETURNS: a list of (not_converged, energy) 2-tuples. \n\
+     If not_converged is 0 the optimization converged for that conformer.\n\
 \n";
   python::def("UFFOptimizeMoleculeConfs", RDKit::UFFConfsHelper,
               (python::arg("self"), python::arg("numThreads") = 1,
@@ -389,7 +391,8 @@ BOOST_PYTHON_MODULE(rdForceFieldHelpers) {
     - ignoreInterfragInteractions : if true, nonbonded terms between\n\
                   fragments will not be added to the forcefield.\n\
 \n\
- RETURNS: 0 if the optimization converged, 1 if more iterations are required.\n\
+RETURNS: a list of (not_converged, energy) 2-tuples. \n\
+    If not_converged is 0 the optimization converged for that conformer.\n\
 \n";
   python::def(
       "MMFFOptimizeMoleculeConfs", RDKit::MMFFConfsHelper,
