@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2001-2006 Rational Discovery LLC
+//  Copyright (C) 2001-2018 Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -8,6 +7,7 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/test.h>
 #include "AlignMolecules.h"
 #include "O3AAlignMolecules.h"
 #include <GraphMol/FileParsers/MolSupplier.h>
@@ -66,17 +66,35 @@ void test1MolAlign() {
   delete m3;
 }
 
+void test1GetBestRMS() {
+  std::string rdbase = getenv("RDBASE");
+  std::string fname =
+      rdbase + "/Code/GraphMol/MolAlign/test_data/probe_mol.sdf";
+  SDMolSupplier supplier(fname, true, false);
+  ROMol *m1 = supplier[1];
+  ROMol *m2 = supplier[2];
+
+  // alignMol() would return this for the rms: 2.50561
+  // But the best rms is: 2.43449
+  double rmsd = MolAlign::getBestRMS(*m1, *m2);
+
+  TEST_ASSERT(RDKit::feq(rmsd, 2.43449));
+  delete m1;
+  delete m2;
+}
+
 void test1MolWithQueryAlign() {
-  // identical to test1MolAlign except we replace one atom with a QueryAtom instead
+  // identical to test1MolAlign except we replace one atom with a QueryAtom
+  // instead
 
   std::string rdbase = getenv("RDBASE");
   std::string fname1 = rdbase + "/Code/GraphMol/MolAlign/test_data/1oir.mol";
-  RWMol *m1 = new RWMol(*MolFileToMol(fname1));
-  QueryAtom *a1 = new QueryAtom(6);
+  auto *m1 = new RWMol(*MolFileToMol(fname1));
+  auto *a1 = new QueryAtom(6);
   std::string fname2 =
       rdbase + "/Code/GraphMol/MolAlign/test_data/1oir_conf.mol";
-  RWMol *m2 = new RWMol(*MolFileToMol(fname2));
-  QueryAtom *a2 = new QueryAtom(6);
+  auto *m2 = new RWMol(*MolFileToMol(fname2));
+  auto *a2 = new QueryAtom(6);
 
   // we replace the same nitrogen instead with a null
   // query  28 and 19 are the "same" atoms
@@ -88,8 +106,8 @@ void test1MolWithQueryAlign() {
 
   std::string fname3 =
       rdbase + "/Code/GraphMol/MolAlign/test_data/1oir_trans.mol";
-  
-  RWMol *m3 = new RWMol(*MolFileToMol(fname3));
+
+  auto *m3 = new RWMol(*MolFileToMol(fname3));
   m3->replaceAtom(0, new QueryAtom(5));
 
   const Conformer &conf1 = m2->getConformer(0);
@@ -114,8 +132,7 @@ void test1MolWithQueryAlign() {
   // provide an atom mapping
   delete m1;
   delete m2;
-  delete m3;  
-
+  delete m3;
 }
 
 void test2AtomMap() {
@@ -173,7 +190,7 @@ void testIssue241() {
   ROMol *m1 = MolFileToMol(fname1);
   std::string res;
   MolPickler::pickleMol(*m1, res);
-  ROMol *ref = new ROMol(res);
+  auto *ref = new ROMol(res);
   DGeomHelpers::EmbedMolecule(*ref, 30, 239 * 10);
   ForceFields::ForceField *ff1 = UFF::constructForceField(*ref);
   ff1->initialize();
@@ -181,7 +198,7 @@ void testIssue241() {
 
   std::string pkl2;
   MolPickler::pickleMol(*m1, pkl2);
-  ROMol *probe = new ROMol(pkl2);
+  auto *probe = new ROMol(pkl2);
   DGeomHelpers::EmbedMolecule(*probe, 30, 239 * 10);
   ForceFields::ForceField *ff2 = UFF::constructForceField(*probe);
   ff2->initialize();
@@ -289,7 +306,7 @@ void testMMFFO3AMolHist() {
     MolAlign::MolHistogram prbHist(*prbMol, prbDmat);
 
     MolAlign::O3A o3a(*prbMol, *refMol, &prbMP, &refMP, MolAlign::O3A::MMFF94,
-                      -1, -1, false, 50, 0, NULL, NULL, NULL, &prbHist,
+                      -1, -1, false, 50, 0, nullptr, nullptr, nullptr, &prbHist,
                       &refHist);
     double rmsd = o3a.align();
     cumScore += o3a.score();
@@ -340,8 +357,8 @@ void testCrippenO3AMolHist() {
     MolAlign::MolHistogram prbHist(*prbMol, prbDmat);
 
     MolAlign::O3A o3a(*prbMol, *refMol, &prbLogpContribs, &refLogpContribs,
-                      MolAlign::O3A::CRIPPEN, -1, -1, false, 50, 0, NULL, NULL,
-                      NULL, &prbHist, &refHist);
+                      MolAlign::O3A::CRIPPEN, -1, -1, false, 50, 0, nullptr,
+                      nullptr, nullptr, &prbHist, &refHist);
     double rmsd = o3a.align();
     cumScore += o3a.score();
     cumMsd += rmsd * rmsd;
@@ -380,12 +397,13 @@ void testMMFFO3AConstraints() {
   ROMol m2(*m1);
   MolAlign::randomTransform(m2);
   ROMol m3(m2);
-  MolAlign::O3A *o3a = new MolAlign::O3A(m2, *m1, &mp, &mp);
+  auto *o3a = new MolAlign::O3A(m2, *m1, &mp, &mp);
   TEST_ASSERT(o3a);
   o3a->align();
   delete o3a;
-  double d = (m2.getConformer().getAtomPos(cIdx) -
-              m1->getConformer().getAtomPos(cIdx)).length();
+  double d =
+      (m2.getConformer().getAtomPos(cIdx) - m1->getConformer().getAtomPos(cIdx))
+          .length();
   TEST_ASSERT(feq(d, 0.0, 1));
   MatchVectType constraintMap;
   constraintMap.push_back(std::make_pair(cIdx, nIdx));
@@ -431,13 +449,14 @@ void testCrippenO3AConstraints() {
   std::vector<std::string> prbAtomTypeLabels(prbNAtoms);
   Descriptors::getCrippenAtomContribs(m2, prbLogpContribs, prbMRContribs, true,
                                       &prbAtomTypes, &prbAtomTypeLabels);
-  MolAlign::O3A *o3a = new MolAlign::O3A(
-      m2, *m1, &prbLogpContribs, &prbLogpContribs, MolAlign::O3A::CRIPPEN);
+  auto *o3a = new MolAlign::O3A(m2, *m1, &prbLogpContribs, &prbLogpContribs,
+                                MolAlign::O3A::CRIPPEN);
   TEST_ASSERT(o3a);
   o3a->align();
   delete o3a;
-  double d = (m2.getConformer().getAtomPos(cIdx) -
-              m1->getConformer().getAtomPos(cIdx)).length();
+  double d =
+      (m2.getConformer().getAtomPos(cIdx) - m1->getConformer().getAtomPos(cIdx))
+          .length();
   TEST_ASSERT(feq(d, 0.0, 1));
   MatchVectType constraintMap;
   constraintMap.push_back(std::make_pair(cIdx, nIdx));
@@ -496,7 +515,7 @@ void testMMFFO3AConstraintsAndLocalOnly() {
     constraintMap.push_back(std::make_pair(prbOIdx, refSIdx));
     RDNumeric::DoubleVector constraintWeights(1);
     constraintWeights[0] = weights[i];
-    MolAlign::O3A *o3a =
+    auto *o3a =
         new MolAlign::O3A(*prbMol, *refMol, &prbLogpContribs, &refLogpContribs,
                           MolAlign::O3A::CRIPPEN, -1, -1, false, 50, 0,
                           &constraintMap, &constraintWeights);
@@ -510,7 +529,8 @@ void testMMFFO3AConstraintsAndLocalOnly() {
     o3a->align();
     delete o3a;
     double d = (prbMol->getConformer().getAtomPos(prbOIdx) -
-                refMol->getConformer().getAtomPos(refSIdx)).length();
+                refMol->getConformer().getAtomPos(refSIdx))
+                   .length();
     TEST_ASSERT(feq(d, distOS[i], 0.1));
   }
 }
@@ -548,9 +568,9 @@ void testCrippenO3AConstraintsAndLocalOnly() {
     constraintMap.push_back(std::make_pair(prbOIdx, refSIdx));
     RDNumeric::DoubleVector constraintWeights(1);
     constraintWeights[0] = weights[i];
-    MolAlign::O3A *o3a = new MolAlign::O3A(
-        *prbMol, *refMol, &prbMP, &refMP, MolAlign::O3A::MMFF94, -1, -1, false,
-        50, 0, &constraintMap, &constraintWeights);
+    auto *o3a = new MolAlign::O3A(*prbMol, *refMol, &prbMP, &refMP,
+                                  MolAlign::O3A::MMFF94, -1, -1, false, 50, 0,
+                                  &constraintMap, &constraintWeights);
     TEST_ASSERT(o3a);
     o3a->align();
     delete o3a;
@@ -561,7 +581,8 @@ void testCrippenO3AConstraintsAndLocalOnly() {
     o3a->align();
     delete o3a;
     double d = (prbMol->getConformer().getAtomPos(prbOIdx) -
-                refMol->getConformer().getAtomPos(refSIdx)).length();
+                refMol->getConformer().getAtomPos(refSIdx))
+                   .length();
     TEST_ASSERT(feq(d, distOS[i], 0.1));
   }
 }
@@ -627,10 +648,8 @@ void runblock_o3a_crippen(ROMol *refMol, const std::vector<ROMol *> &mols,
   }
 }
 }
-#include <RDGeneral/BoostStartInclude.h>
-#include <boost/thread.hpp>
-#include <RDGeneral/BoostEndInclude.h>
-
+#include <thread>
+#include <future>
 void testMMFFO3AMultiThread() {
   std::string rdbase = getenv("RDBASE");
   std::string sdf = rdbase + "/Code/GraphMol/MolAlign/test_data/ref_e2.sdf";
@@ -639,7 +658,7 @@ void testMMFFO3AMultiThread() {
 
   std::vector<ROMol *> mols;
   while (!suppl.atEnd() && mols.size() < 100) {
-    ROMol *mol = 0;
+    ROMol *mol = nullptr;
     try {
       mol = suppl.next();
     } catch (...) {
@@ -664,17 +683,19 @@ void testMMFFO3AMultiThread() {
     scores[i] = o3a.score();
   }
 
-  boost::thread_group tg;
+  std::vector<std::future<void>> tg;
 
   std::cerr << "processing" << std::endl;
   unsigned int count = 4;
   for (unsigned int i = 0; i < count; ++i) {
     std::cerr << " launch :" << i << std::endl;
     std::cerr.flush();
-    tg.add_thread(new boost::thread(runblock_o3a_mmff, refMol, mols, rmsds,
-                                    scores, count, i));
+    tg.emplace_back(std::async(std::launch::async, runblock_o3a_mmff, refMol,
+                               mols, rmsds, scores, count, i));
+  }
+  for (auto &fut : tg) {
+    fut.get();
   }
-  tg.join_all();
 
   BOOST_FOREACH (ROMol *mol, mols) { delete mol; }
   BOOST_LOG(rdErrorLog) << "  done" << std::endl;
@@ -688,7 +709,7 @@ void testCrippenO3AMultiThread() {
 
   std::vector<ROMol *> mols;
   while (!suppl.atEnd() && mols.size() < 100) {
-    ROMol *mol = 0;
+    ROMol *mol = nullptr;
     try {
       mol = suppl.next();
     } catch (...) {
@@ -727,17 +748,19 @@ void testCrippenO3AMultiThread() {
     scores[i] = o3a.score();
   }
 
-  boost::thread_group tg;
+  std::vector<std::future<void>> tg;
 
   std::cerr << "processing" << std::endl;
   unsigned int count = 4;
   for (unsigned int i = 0; i < count; ++i) {
     std::cerr << " launch :" << i << std::endl;
     std::cerr.flush();
-    tg.add_thread(new boost::thread(runblock_o3a_crippen, refMol, mols, rmsds,
-                                    scores, count, i));
+    tg.emplace_back(std::async(std::launch::async, runblock_o3a_crippen, refMol,
+                               mols, rmsds, scores, count, i));
+  }
+  for (auto &fut : tg) {
+    fut.get();
   }
-  tg.join_all();
 
   BOOST_FOREACH (ROMol *mol, mols) { delete mol; }
   BOOST_LOG(rdErrorLog) << "  done" << std::endl;
@@ -759,7 +782,7 @@ void testGetO3AForProbeConfs() {
   while (!psuppl.atEnd()) {
     ROMol *mol = psuppl.next();
     if (!mol) continue;
-    Conformer *conf = new Conformer(mol->getConformer());
+    auto *conf = new Conformer(mol->getConformer());
     prbMol->addConformer(conf, true);
     delete mol;
   }
@@ -768,7 +791,7 @@ void testGetO3AForProbeConfs() {
   MMFF::MMFFMolProperties refMP(*refMol);
   MMFF::MMFFMolProperties prbMP(*prbMol);
 
-  std::vector<std::pair<double, double> > oscores;
+  std::vector<std::pair<double, double>> oscores;
   for (unsigned int i = 0; i < prbMol->getNumConformers(); ++i) {
     MolAlign::O3A o3a(*prbMol, *refMol, &prbMP, &refMP, MolAlign::O3A::MMFF94,
                       i);
@@ -778,7 +801,7 @@ void testGetO3AForProbeConfs() {
   }
 
   {
-    std::vector<boost::shared_ptr<MolAlign::O3A> > o3s;
+    std::vector<boost::shared_ptr<MolAlign::O3A>> o3s;
     MolAlign::getO3AForProbeConfs(*prbMol, *refMol, &prbMP, &refMP, o3s);
     TEST_ASSERT(o3s.size() == prbMol->getNumConformers());
     for (unsigned int i = 0; i < prbMol->getNumConformers(); ++i) {
@@ -796,7 +819,7 @@ void testGetO3AForProbeConfs() {
       }
     }
 
-    std::vector<boost::shared_ptr<MolAlign::O3A> > o3s;
+    std::vector<boost::shared_ptr<MolAlign::O3A>> o3s;
     MolAlign::getO3AForProbeConfs(prbMol2, *refMol, &prbMP, &refMP, o3s, 4);
     TEST_ASSERT(o3s.size() == prbMol2.getNumConformers());
     for (unsigned int i = 0; i < prbMol2.getNumConformers(); ++i) {
@@ -826,27 +849,27 @@ void testO3AMultiThreadBug() {
     if (!mol) continue;
 
     while (mol->getNumConformers() < 50) {
-      Conformer *conf = new Conformer(mol->getConformer(0));
+      auto *conf = new Conformer(mol->getConformer(0));
       mol->addConformer(conf, true);
     }
     mols.push_back(mol);
   }
   TEST_ASSERT(mols.size() == 10);
 
-  ROMol *refMol = new ROMol(*mols[0]);
+  auto *refMol = new ROMol(*mols[0]);
   TEST_ASSERT(refMol);
 
   MMFF::MMFFMolProperties refMP(*refMol);
 
 #ifdef RDK_TEST_MULTITHREADED
   {
-    for (unsigned int j = 0; j < mols.size(); ++j) {
-      ROMol prbMol = *(mols[j]);
+    for (auto &mol : mols) {
+      ROMol prbMol = *mol;
       TEST_ASSERT(prbMol.getNumConformers() == 50);
 
       MMFF::MMFFMolProperties prbMP(prbMol);
 
-      std::vector<std::pair<double, double> > oscores;
+      std::vector<std::pair<double, double>> oscores;
       for (unsigned int i = 0; i < prbMol.getNumConformers(); ++i) {
         MolAlign::O3A o3a(prbMol, *refMol, &prbMP, &refMP,
                           MolAlign::O3A::MMFF94, i);
@@ -855,8 +878,8 @@ void testO3AMultiThreadBug() {
         oscores.push_back(std::make_pair(rmsd, score));
       }
 
-      ROMol prbMol2 = *(mols[j]);
-      std::vector<boost::shared_ptr<MolAlign::O3A> > o3s;
+      ROMol prbMol2 = *mol;
+      std::vector<boost::shared_ptr<MolAlign::O3A>> o3s;
       MolAlign::getO3AForProbeConfs(prbMol2, *refMol, &prbMP, &refMP, o3s, 0);
       TEST_ASSERT(o3s.size() == prbMol2.getNumConformers());
       for (unsigned int i = 0; i < prbMol2.getNumConformers(); ++i) {
@@ -870,7 +893,7 @@ void testO3AMultiThreadBug() {
 
 #endif
   delete refMol;
-  for (unsigned int j = 0; j < mols.size(); ++j) delete mols[j];
+  for (auto &mol : mols) delete mol;
 
   BOOST_LOG(rdErrorLog) << "  done" << std::endl;
 }
@@ -884,6 +907,10 @@ int main() {
   std::cout << "\t test1MolAlign \n\n";
   test1MolAlign();
 
+  std::cout << "\t---------------------------------\n";
+  std::cout << "\t test1GetBestRMS \n\n";
+  test1GetBestRMS();
+
   std::cout << "\t---------------------------------\n";
   std::cout << "\t test1MolWithQueryAlign \n\n";
   test1MolWithQueryAlign();
