@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2001-2016 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2001-2017 Greg Landrum and Rational Discovery LLC
 //  Copyright (c) 2014, Novartis Institutes for BioMedical Research Inc.
 //
 //   @@ All Rights Reserved @@
@@ -120,8 +120,8 @@ void phosphorusCleanup(RWMol &mol, Atom *atom) {
   // the sanitization process):
   if (atom->calcExplicitValence(false) == 5 && atom->getDegree() == 3) {
     unsigned int aid = atom->getIdx();
-    Bond *dbl_to_O = NULL;
-    Atom *O_atom = NULL;
+    Bond *dbl_to_O = nullptr;
+    Atom *O_atom = nullptr;
     bool hasDoubleToCorN = false;
     RWMol::ADJ_ITER nid1, end1;
     boost::tie(nid1, end1) = mol.getAtomNeighbors(atom);
@@ -142,8 +142,8 @@ void phosphorusCleanup(RWMol &mol, Atom *atom) {
       }
       ++nid1;
     }  // end of loop over the first neigh
-    if (hasDoubleToCorN && dbl_to_O != NULL) {
-      TEST_ASSERT(O_atom != NULL);
+    if (hasDoubleToCorN && dbl_to_O != nullptr) {
+      TEST_ASSERT(O_atom != nullptr);
       O_atom->setFormalCharge(-1);
       dbl_to_O->setBondType(Bond::SINGLE);
       atom->setFormalCharge(1);
@@ -173,23 +173,25 @@ void halogenCleanup(RWMol &mol, Atom *atom) {
       ++nid1;
     }
     if (neighborsAllO) {
-      atom->setFormalCharge(ev / 2);
+      int formalCharge = 0;
       boost::tie(nid1, end1) = mol.getAtomNeighbors(atom);
       while (nid1 != end1) {
         Bond *b = mol.getBondBetweenAtoms(aid, *nid1);
         if (b->getBondType() == Bond::DOUBLE) {
           b->setBondType(Bond::SINGLE);
           Atom *otherAtom = mol.getAtomWithIdx(*nid1);
+          formalCharge++;
           otherAtom->setFormalCharge(-1);
           otherAtom->calcExplicitValence(false);
         }
         ++nid1;
       }
+      atom->setFormalCharge(formalCharge);
       atom->calcExplicitValence(false);
     }
   }
 }
-}
+}  // namespace
 
 void cleanUp(RWMol &mol) {
   ROMol::AtomIterator ai;
@@ -223,10 +225,10 @@ void adjustHs(RWMol &mol) {
   //
   for (ROMol::AtomIterator ai = mol.beginAtoms(); ai != mol.endAtoms(); ++ai) {
     int origImplicitV = (*ai)->getImplicitValence();
-    (*ai)->calcExplicitValence();
+    (*ai)->calcExplicitValence(false);
     int origExplicitV = (*ai)->getNumExplicitHs();
 
-    int newImplicitV = (*ai)->calcImplicitValence();
+    int newImplicitV = (*ai)->calcImplicitValence(false);
     //
     //  Case 1: The disappearing Hydrogen
     //    Smiles:  O=C1NC=CC2=C1C=CC=C2
@@ -245,7 +247,7 @@ void adjustHs(RWMol &mol) {
     //    write:
     if (newImplicitV < origImplicitV) {
       (*ai)->setNumExplicitHs(origExplicitV + (origImplicitV - newImplicitV));
-      (*ai)->calcExplicitValence();
+      (*ai)->calcExplicitValence(false);
     }
   }
 }
@@ -396,7 +398,7 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
   unsigned int nFrags = getMolFrags(mol, *mapping);
   std::vector<ROMOL_SPTR> res;
   if (nFrags == 1) {
-    ROMol *tmp = new ROMol(mol);
+    auto *tmp = new ROMol(mol);
     ROMOL_SPTR sptr(tmp);
     res.push_back(sptr);
     if (fragsMolAtomMapping) {
@@ -412,7 +414,7 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
     boost::dynamic_bitset<> copiedBonds(mol.getNumBonds(), 0);
     res.reserve(nFrags);
     for (unsigned int frag = 0; frag < nFrags; ++frag) {
-      ROMol *tmp = new ROMol();
+      auto *tmp = new ROMol();
       ROMOL_SPTR sptr(tmp);
       res.push_back(sptr);
     }
@@ -449,10 +451,10 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
       if (oAtm->getPropIfPresent(common_properties::_ringStereoAtoms,
                                  ringStereoAtomsMol)) {
         INT_VECT ringStereoAtomsCopied;
-        for (unsigned rnbr = 0; rnbr < ringStereoAtomsMol.size(); ++rnbr) {
-          int ori_ridx = abs(ringStereoAtomsMol[rnbr]) - 1;
+        for (int rnbr : ringStereoAtomsMol) {
+          int ori_ridx = abs(rnbr) - 1;
           int ridx = ids[ori_ridx] + 1;
-          if (ringStereoAtomsMol[rnbr] < 0) {
+          if (rnbr < 0) {
             ridx *= (-1);
           }
           ringStereoAtomsCopied.push_back(ridx);
@@ -467,7 +469,7 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
     ROMol::EDGE_ITER beg, end;
     boost::tie(beg, end) = mol.getEdges();
     while (beg != end) {
-      BOND_SPTR bond = (mol)[*beg];
+      const Bond *bond = (mol)[*beg];
       ++beg;
       if (!copiedBonds[bond->getIdx()]) {
         continue;
@@ -480,24 +482,22 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
       nBond->setEndAtomIdx(ids[nBond->getEndAtomIdx()]);
       nBond->getStereoAtoms().clear();
       INT_VECT stereoAtoms = bond->getStereoAtoms();
-      for (unsigned i = 0; i < stereoAtoms.size(); ++i) {
-        nBond->getStereoAtoms().push_back(ids[stereoAtoms[i]]);
+      for (int stereoAtom : stereoAtoms) {
+        nBond->getStereoAtoms().push_back(ids[stereoAtom]);
       }
       tmp->addBond(nBond, true);
     }
 
     // copy RingInfo
     if (mol.getRingInfo()->isInitialized()) {
-      for (unsigned i = 0; i < mol.getRingInfo()->atomRings().size(); ++i) {
+      for (const auto &i : mol.getRingInfo()->atomRings()) {
         INT_VECT aids;
-        RWMol *tmp = static_cast<RWMol *>(
-            res[(*mapping)[mol.getRingInfo()->atomRings()[i][0]]].get());
+        RWMol *tmp = static_cast<RWMol *>(res[(*mapping)[i[0]]].get());
         if (!tmp->getRingInfo()->isInitialized()) {
           tmp->getRingInfo()->initialize();
         }
-        for (unsigned j = 0; j < mol.getRingInfo()->atomRings()[i].size();
-             ++j) {
-          aids.push_back(ids[mol.getRingInfo()->atomRings()[i][j]]);
+        for (int j : i) {
+          aids.push_back(ids[j]);
         }
         INT_VECT bids;
         INT_VECT_CI lastRai;
@@ -518,12 +518,11 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
 
     if (copyConformers) {
       // copy conformers
-      for (ROMol::ConstConformerIterator cit = mol.beginConformers();
-           cit != mol.endConformers(); ++cit) {
-        for (std::vector<ROMOL_SPTR>::iterator iter = res.begin();
-             iter != res.end(); ++iter) {
-          ROMol *newM = iter->get();
-          Conformer *conf = new Conformer(newM->getNumAtoms());
+      for (auto cit = mol.beginConformers(); cit != mol.endConformers();
+           ++cit) {
+        for (auto &re : res) {
+          ROMol *newM = re.get();
+          auto *conf = new Conformer(newM->getNumAtoms());
           conf->setId((*cit)->getId());
           conf->set3D((*cit)->is3D());
           newM->addConformer(conf);
@@ -546,9 +545,8 @@ std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
   }
 
   if (sanitizeFrags) {
-    for (std::vector<ROMOL_SPTR>::iterator iter = res.begin();
-         iter != res.end(); ++iter) {
-      sanitizeMol(*static_cast<RWMol *>(iter->get()));
+    for (auto &re : res) {
+      sanitizeMol(*static_cast<RWMol *>(re.get()));
     }
   }
 
@@ -587,14 +585,14 @@ unsigned int getMolFrags(const ROMol &mol, VECT_INT_VECT &frags) {
 }
 
 template <typename T>
-std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
+std::map<T, boost::shared_ptr<ROMol>> getMolFragsWithQuery(
     const ROMol &mol, T (*query)(const ROMol &, const Atom *),
     bool sanitizeFrags, const std::vector<T> *whiteList, bool negateList) {
   PRECONDITION(query, "no query");
 
   std::vector<T> assignments(mol.getNumAtoms());
   std::vector<int> ids(mol.getNumAtoms(), -1);
-  std::map<T, boost::shared_ptr<ROMol> > res;
+  std::map<T, boost::shared_ptr<ROMol>> res;
   for (unsigned int i = 0; i < mol.getNumAtoms(); ++i) {
     T where = query(mol, mol.getAtomWithIdx(i));
     if (whiteList) {
@@ -627,13 +625,10 @@ std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
     }
   }
   // update conformers
-  for (ROMol::ConstConformerIterator cit = mol.beginConformers();
-       cit != mol.endConformers(); ++cit) {
-    for (typename std::map<T, boost::shared_ptr<ROMol> >::iterator iter =
-             res.begin();
-         iter != res.end(); ++iter) {
+  for (auto cit = mol.beginConformers(); cit != mol.endConformers(); ++cit) {
+    for (auto iter = res.begin(); iter != res.end(); ++iter) {
       ROMol *newM = iter->second.get();
-      Conformer *conf = new Conformer(newM->getNumAtoms());
+      auto *conf = new Conformer(newM->getNumAtoms());
       conf->setId((*cit)->getId());
       conf->set3D((*cit)->is3D());
       newM->addConformer(conf);
@@ -646,21 +641,19 @@ std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
     }
   }
   if (sanitizeFrags) {
-    for (typename std::map<T, boost::shared_ptr<ROMol> >::iterator iter =
-             res.begin();
-         iter != res.end(); ++iter) {
+    for (auto iter = res.begin(); iter != res.end(); ++iter) {
       sanitizeMol(*static_cast<RWMol *>(iter->second.get()));
     }
   }
   return res;
 }
-template std::map<std::string, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
+template RDKIT_GRAPHMOL_EXPORT std::map<std::string, boost::shared_ptr<ROMol>> getMolFragsWithQuery(
     const ROMol &mol, std::string (*query)(const ROMol &, const Atom *),
     bool sanitizeFrags, const std::vector<std::string> *, bool);
-template std::map<int, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
+template RDKIT_GRAPHMOL_EXPORT std::map<int, boost::shared_ptr<ROMol>> getMolFragsWithQuery(
     const ROMol &mol, int (*query)(const ROMol &, const Atom *),
     bool sanitizeFrags, const std::vector<int> *, bool);
-template std::map<unsigned int, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
+template RDKIT_GRAPHMOL_EXPORT std::map<unsigned int, boost::shared_ptr<ROMol>> getMolFragsWithQuery(
     const ROMol &mol, unsigned int (*query)(const ROMol &, const Atom *),
     bool sanitizeFrags, const std::vector<unsigned int> *, bool);
 
