@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2001-2012 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2001-2017 Greg Landrum and Rational Discovery LLC
 //  Copyright (c) 2014, Novartis Institutes for BioMedical Research Inc.
 //
 //   @@ All Rights Reserved @@
@@ -8,22 +8,26 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef _RD_MOL_OPS_H_
 #define _RD_MOL_OPS_H_
 
 #include <vector>
 #include <map>
 #include <list>
+#include <RDGeneral/BoostStartInclude.h>
 #include <boost/smart_ptr.hpp>
 #include <boost/dynamic_bitset.hpp>
+#include <RDGeneral/BoostEndInclude.h>
 #include <RDGeneral/types.h>
 
-extern const int ci_LOCAL_INF;
+RDKIT_GRAPHMOL_EXPORT extern const int ci_LOCAL_INF;
 namespace RDKit {
 class ROMol;
 class RWMol;
 class Atom;
 class Bond;
+class Conformer;
 typedef std::vector<double> INVAR_VECT;
 typedef INVAR_VECT::iterator INVAR_VECT_I;
 typedef INVAR_VECT::const_iterator INVAR_VECT_CI;
@@ -43,13 +47,13 @@ namespace MolOps {
 
    \return the number of electrons
 */
-int countAtomElec(const Atom *at);
+RDKIT_GRAPHMOL_EXPORT int countAtomElec(const Atom *at);
 
 //! sums up all atomic formal charges and returns the result
-int getFormalCharge(const ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT int getFormalCharge(const ROMol &mol);
 
 //! returns whether or not the given Atom is involved in a conjugated bond
-bool atomHasConjugatedBond(const Atom *at);
+RDKIT_GRAPHMOL_EXPORT bool atomHasConjugatedBond(const Atom *at);
 
 //! find fragments (disconnected components of the molecular graph)
 /*!
@@ -62,7 +66,8 @@ bool atomHasConjugatedBond(const Atom *at);
   \return the number of fragments found.
 
 */
-unsigned int getMolFrags(const ROMol &mol, std::vector<int> &mapping);
+RDKIT_GRAPHMOL_EXPORT unsigned int getMolFrags(const ROMol &mol,
+                                               std::vector<int> &mapping);
 //! find fragments (disconnected components of the molecular graph)
 /*!
 
@@ -74,8 +79,8 @@ unsigned int getMolFrags(const ROMol &mol, std::vector<int> &mapping);
   \return the number of fragments found.
 
 */
-unsigned int getMolFrags(const ROMol &mol,
-                         std::vector<std::vector<int> > &frags);
+RDKIT_GRAPHMOL_EXPORT unsigned int getMolFrags(
+    const ROMol &mol, std::vector<std::vector<int>> &frags);
 
 //! splits a molecule into its component fragments
 //  (disconnected components of the molecular graph)
@@ -95,9 +100,9 @@ unsigned int getMolFrags(const ROMol &mol,
   \return a vector of the fragments as smart pointers to ROMols
 
 */
-std::vector<boost::shared_ptr<ROMol> > getMolFrags(
+RDKIT_GRAPHMOL_EXPORT std::vector<boost::shared_ptr<ROMol>> getMolFrags(
     const ROMol &mol, bool sanitizeFrags = true, std::vector<int> *frags = 0,
-    std::vector<std::vector<int> > *fragsMolAtomMapping = 0,
+    std::vector<std::vector<int>> *fragsMolAtomMapping = 0,
     bool copyConformers = true);
 
 //! splits a molecule into pieces based on labels assigned using a query
@@ -115,10 +120,11 @@ std::vector<boost::shared_ptr<ROMol> > getMolFrags(
 
 */
 template <typename T>
-std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
-    const ROMol &mol, T (*query)(const ROMol &, const Atom *),
-    bool sanitizeFrags = true, const std::vector<T> *whiteList = 0,
-    bool negateList = false);
+RDKIT_GRAPHMOL_EXPORT std::map<T, boost::shared_ptr<ROMol>>
+getMolFragsWithQuery(const ROMol &mol, T (*query)(const ROMol &, const Atom *),
+                     bool sanitizeFrags = true,
+                     const std::vector<T> *whiteList = 0,
+                     bool negateList = false);
 
 #if 0
     //! finds a molecule's minimium spanning tree (MST)
@@ -126,7 +132,7 @@ std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
       \param mol  the molecule of interest
       \param mst  used to return the MST as a vector of bond indices
     */
-    void findSpanningTree(const ROMol &mol,std::vector<int> &mst);
+    RDKIT_GRAPHMOL_EXPORT void findSpanningTree(const ROMol &mol,std::vector<int> &mst);
 #endif
 
 //! calculates Balaban's J index for the molecule
@@ -142,11 +148,11 @@ std::map<T, boost::shared_ptr<ROMol> > getMolFragsWithQuery(
   \return the J index
 
 */
-double computeBalabanJ(const ROMol &mol, bool useBO = true, bool force = false,
-                       const std::vector<int> *bondPath = 0,
-                       bool cacheIt = true);
+RDKIT_GRAPHMOL_EXPORT double computeBalabanJ(
+    const ROMol &mol, bool useBO = true, bool force = false,
+    const std::vector<int> *bondPath = 0, bool cacheIt = true);
 //! \overload
-double computeBalabanJ(double *distMat, int nb, int nAts);
+RDKIT_GRAPHMOL_EXPORT double computeBalabanJ(double *distMat, int nb, int nAts);
 
 //! \name Dealing with hydrogens
 //{@
@@ -161,6 +167,8 @@ double computeBalabanJ(double *distMat, int nb, int nAts);
                 of the added Hs will be used.
     \param onlyOnAtoms   (optional) if provided, this should be a vector of
                 IDs of the atoms that will be considered for H addition.
+    \param addResidueInfo   (optional) if this is true, add residue info to
+                hydrogen atoms (useful for PDB files).
 
     \return the new molecule
 
@@ -171,12 +179,16 @@ double computeBalabanJ(double *distMat, int nb, int nAts);
        - the caller is responsible for <tt>delete</tt>ing the pointer this
    returns.
  */
-ROMol *addHs(const ROMol &mol, bool explicitOnly = false,
-             bool addCoords = false, const UINT_VECT *onlyOnAtoms = NULL);
+RDKIT_GRAPHMOL_EXPORT ROMol *addHs(const ROMol &mol, bool explicitOnly = false,
+                                   bool addCoords = false,
+                                   const UINT_VECT *onlyOnAtoms = NULL,
+                                   bool addResidueInfo = false);
 //! \overload
 // modifies the molecule in place
-void addHs(RWMol &mol, bool explicitOnly = false, bool addCoords = false,
-           const UINT_VECT *onlyOnAtoms = NULL);
+RDKIT_GRAPHMOL_EXPORT void addHs(RWMol &mol, bool explicitOnly = false,
+                                 bool addCoords = false,
+                                 const UINT_VECT *onlyOnAtoms = NULL,
+                                 bool addResidueInfo = false);
 
 //! returns a copy of a molecule with hydrogens removed
 /*!
@@ -201,16 +213,22 @@ void addHs(RWMol &mol, bool explicitOnly = false, bool addCoords = false,
          will not be removed.
        - two coordinate Hs, like the central H in C[H-]C, will not be removed
        - Hs connected to dummy atoms will not be removed
+       - Hs that are part of the definition of double bond Stereochemistry
+         will not be removed
+       - Hs that are not connected to anything else will not be removed
 
        - the caller is responsible for <tt>delete</tt>ing the pointer this
    returns.
 */
-ROMol *removeHs(const ROMol &mol, bool implicitOnly = false,
-                bool updateExplicitCount = false, bool sanitize = true);
+RDKIT_GRAPHMOL_EXPORT ROMol *removeHs(const ROMol &mol,
+                                      bool implicitOnly = false,
+                                      bool updateExplicitCount = false,
+                                      bool sanitize = true);
 //! \overload
 // modifies the molecule in place
-void removeHs(RWMol &mol, bool implicitOnly = false,
-              bool updateExplicitCount = false, bool sanitize = true);
+RDKIT_GRAPHMOL_EXPORT void removeHs(RWMol &mol, bool implicitOnly = false,
+                                    bool updateExplicitCount = false,
+                                    bool sanitize = true);
 
 //! returns a copy of a molecule with hydrogens removed and added as queries
 //!  to the heavy atoms to which they are bound.
@@ -236,34 +254,57 @@ void removeHs(RWMol &mol, bool implicitOnly = false,
       in an atom map will be retained
 
 */
-ROMol *mergeQueryHs(const ROMol &mol, bool mergeUnmappedOnly = false);
+RDKIT_GRAPHMOL_EXPORT ROMol *mergeQueryHs(const ROMol &mol,
+                                          bool mergeUnmappedOnly = false);
 //! \overload
 // modifies the molecule in place
-void mergeQueryHs(RWMol &mol, bool mergeUnmappedOnly = false);
+RDKIT_GRAPHMOL_EXPORT void mergeQueryHs(RWMol &mol,
+                                        bool mergeUnmappedOnly = false);
 
 typedef enum {
-  ADJUST_EMPTY = 0x0,
-  ADJUST_RINGSONLY = 0x1,
+  ADJUST_IGNORENONE = 0x0,
+  ADJUST_IGNORECHAINS = 0x1,
+  ADJUST_IGNORERINGS = 0x4,
   ADJUST_IGNOREDUMMIES = 0x2,
-  ADJUST_SETALL = 0xFFFFFFF
+  ADJUST_IGNORENONDUMMIES = 0x8,
+  ADJUST_IGNOREMAPPED = 0x10,
+  ADJUST_IGNOREALL = 0xFFFFFFF
 } AdjustQueryWhichFlags;
-struct AdjustQueryParameters {
+
+struct RDKIT_GRAPHMOL_EXPORT AdjustQueryParameters {
   bool adjustDegree; /**< add degree queries */
-  AdjustQueryWhichFlags adjustDegreeFlags;
+  boost::uint32_t adjustDegreeFlags;
   bool adjustRingCount; /**< add ring-count queries */
-  AdjustQueryWhichFlags adjustRingCountFlags;
+  boost::uint32_t adjustRingCountFlags;
 
   bool makeDummiesQueries; /**< convert dummy atoms without isotope labels to
                               any-atom queries */
+  bool aromatizeIfPossible;
+  bool makeBondsGeneric; /**< convert bonds to generic queries (any bonds) */
+  boost::uint32_t makeBondsGenericFlags;
+  bool makeAtomsGeneric; /**< convert atoms to generic queries (any atoms) */
+  boost::uint32_t makeAtomsGenericFlags;
+  bool adjustHeavyDegree; /**< adjust the heavy-atom degree instead of overall
+                             degree */
+  boost::uint32_t adjustHeavyDegreeFlags;
+  bool adjustRingChain; /**< add ring-chain queries */
+  boost::uint32_t adjustRingChainFlags;
 
   AdjustQueryParameters()
       : adjustDegree(true),
-        adjustDegreeFlags(ADJUST_SETALL),
+        adjustDegreeFlags(ADJUST_IGNOREDUMMIES | ADJUST_IGNORECHAINS),
         adjustRingCount(false),
-        adjustRingCountFlags(ADJUST_SETALL),
-        makeDummiesQueries(true)
-
-  {}
+        adjustRingCountFlags(ADJUST_IGNOREDUMMIES | ADJUST_IGNORECHAINS),
+        makeDummiesQueries(true),
+        aromatizeIfPossible(true),
+        makeBondsGeneric(false),
+        makeBondsGenericFlags(ADJUST_IGNORENONE),
+        makeAtomsGeneric(false),
+        makeAtomsGenericFlags(ADJUST_IGNORENONE),
+        adjustHeavyDegree(false),
+        adjustHeavyDegreeFlags(ADJUST_IGNOREDUMMIES | ADJUST_IGNORECHAINS),
+        adjustRingChain(false),
+        adjustRingChainFlags(ADJUST_IGNORENONE) {}
 };
 //! returns a copy of a molecule with query properties adjusted
 /*!
@@ -272,12 +313,12 @@ struct AdjustQueryParameters {
 
   \return the new molecule
 */
-ROMol *adjustQueryProperties(const ROMol &mol,
-                             const AdjustQueryParameters *params = NULL);
+RDKIT_GRAPHMOL_EXPORT ROMol *adjustQueryProperties(
+    const ROMol &mol, const AdjustQueryParameters *params = NULL);
 //! \overload
 // modifies the molecule in place
-void adjustQueryProperties(RWMol &mol,
-                           const AdjustQueryParameters *params = NULL);
+RDKIT_GRAPHMOL_EXPORT void adjustQueryProperties(
+    RWMol &mol, const AdjustQueryParameters *params = NULL);
 
 //! returns a copy of a molecule with the atoms renumbered
 /*!
@@ -293,8 +334,8 @@ void adjustQueryProperties(RWMol &mol,
     - the caller is responsible for <tt>delete</tt>ing the pointer this returns.
 
 */
-ROMol *renumberAtoms(const ROMol &mol,
-                     const std::vector<unsigned int> &newOrder);
+RDKIT_GRAPHMOL_EXPORT ROMol *renumberAtoms(
+    const ROMol &mol, const std::vector<unsigned int> &newOrder);
 
 //@}
 
@@ -354,10 +395,29 @@ typedef enum {
       function to a ROMol, so that new atoms and bonds cannot be added to the
       molecule and screw up the sanitizing that has been done here
 */
-void sanitizeMol(RWMol &mol, unsigned int &operationThatFailed,
-                 unsigned int sanitizeOps = SANITIZE_ALL);
+RDKIT_GRAPHMOL_EXPORT void sanitizeMol(RWMol &mol,
+                                       unsigned int &operationThatFailed,
+                                       unsigned int sanitizeOps = SANITIZE_ALL);
 //! \overload
-void sanitizeMol(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT void sanitizeMol(RWMol &mol);
+
+//! Possible aromaticity models
+/*!
+- \c AROMATICITY_DEFAULT at the moment always uses \c AROMATICITY_RDKIT
+- \c AROMATICITY_RDKIT is the standard RDKit model (as documented in the RDKit
+Book)
+- \c AROMATICITY_SIMPLE only considers 5- and 6-membered simple rings (it
+does not consider the outer envelope of fused rings)
+- \c AROMATICITY_MDL
+- \c AROMATICITY_CUSTOM uses a caller-provided function
+*/
+typedef enum {
+  AROMATICITY_DEFAULT = 0x0,  ///< future proofing
+  AROMATICITY_RDKIT = 0x1,
+  AROMATICITY_SIMPLE = 0x2,
+  AROMATICITY_MDL = 0x4,
+  AROMATICITY_CUSTOM = 0xFFFFFFF  ///< use a function
+} AromaticityModel;
 
 //! Sets up the aromaticity for a molecule
 /*!
@@ -368,7 +428,7 @@ void sanitizeMol(RWMol &mol);
   candidates
         for aromaticity. A ring atom is a candidate if it can spare electrons
         to the ring and if it's from the first two rows of the periodic table.
-     -# ased on the candidate atoms, mark the rings to be either candidates
+     -# based on the candidate atoms, mark the rings to be either candidates
         or non-candidates. A ring is a candidate only if all its atoms are
   candidates
      -# apply Hueckel rule to each of the candidate rings to check if the ring
@@ -376,15 +436,20 @@ void sanitizeMol(RWMol &mol);
         aromatic
 
   \param mol the RWMol of interest
+  \param model the aromaticity model to use
+  \param func a custom function for assigning aromaticity (only used when
+  model=\c AROMATICITY_CUSTOM)
 
-  \return 1 on succes, 0 otherwise
+  \return >0 on success, <= 0 otherwise
 
   <b>Assumptions:</b>
     - Kekulization has been done (i.e. \c MolOps::Kekulize() has already
       been called)
 
 */
-int setAromaticity(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT int setAromaticity(
+    RWMol &mol, AromaticityModel model = AROMATICITY_DEFAULT,
+    int (*func)(RWMol &) = NULL);
 
 //! Designed to be called by the sanitizer to handle special cases before
 // anything is done.
@@ -407,10 +472,10 @@ int setAromaticity(RWMol &mol);
    \param mol    the molecule of interest
 
 */
-void cleanUp(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT void cleanUp(RWMol &mol);
 
 //! Called by the sanitizer to assign radical counts to atoms
-void assignRadicals(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT void assignRadicals(RWMol &mol);
 
 //! adjust the number of implicit and explicit Hs for special cases
 /*!
@@ -427,7 +492,7 @@ void assignRadicals(RWMol &mol);
          everything has been calculated.
 
 */
-void adjustHs(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT void adjustHs(RWMol &mol);
 
 //! Kekulizes the molecule
 /*!
@@ -453,14 +518,14 @@ void adjustHs(RWMol &mol);
        or RDKit::Bond::DOUBLE during Kekulization.
 
 */
-void Kekulize(RWMol &mol, bool markAtomsBonds = true,
-              unsigned int maxBackTracks = 100);
+RDKIT_GRAPHMOL_EXPORT void Kekulize(RWMol &mol, bool markAtomsBonds = true,
+                                    unsigned int maxBackTracks = 100);
 
 //! flags the molecule's conjugated bonds
-void setConjugation(ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT void setConjugation(ROMol &mol);
 
 //! calculates and sets the hybridization of all a molecule's Stoms
-void setHybridization(ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT void setHybridization(ROMol &mol);
 
 // @}
 
@@ -510,9 +575,11 @@ void setHybridization(ROMol &mol);
     - Since SSSR may not be unique, a post-SSSR step to symmetrize may be done.
       The extra rings this process adds can be quite useful.
 */
-int findSSSR(const ROMol &mol, std::vector<std::vector<int> > &res);
+RDKIT_GRAPHMOL_EXPORT int findSSSR(const ROMol &mol,
+                                   std::vector<std::vector<int>> &res);
 //! \overload
-int findSSSR(const ROMol &mol, std::vector<std::vector<int> > *res = 0);
+RDKIT_GRAPHMOL_EXPORT int findSSSR(const ROMol &mol,
+                                   std::vector<std::vector<int>> *res = 0);
 
 //! use a DFS algorithm to identify ring bonds and atoms in a molecule
 /*!
@@ -521,7 +588,7 @@ int findSSSR(const ROMol &mol, std::vector<std::vector<int> > *res = 0);
   mol.getRingInfo().numAtomRings(idx) or mol.getRingInfo().numBondRings(idx)
   return values >0
 */
-void fastFindRings(const ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT void fastFindRings(const ROMol &mol);
 
 //! symmetrize the molecule's Smallest Set of Smallest Rings
 /*!
@@ -548,9 +615,10 @@ void fastFindRings(const ROMol &mol);
    - if no SSSR rings are found on the molecule - MolOps::findSSSR() is called
   first
 */
-int symmetrizeSSSR(ROMol &mol, std::vector<std::vector<int> > &res);
+RDKIT_GRAPHMOL_EXPORT int symmetrizeSSSR(ROMol &mol,
+                                         std::vector<std::vector<int>> &res);
 //! \overload
-int symmetrizeSSSR(ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT int symmetrizeSSSR(ROMol &mol);
 
 //@}
 
@@ -574,10 +642,10 @@ int symmetrizeSSSR(ROMol &mol);
       this pointer.
 
 */
-double *getAdjacencyMatrix(const ROMol &mol, bool useBO = false,
-                           int emptyVal = 0, bool force = false,
-                           const char *propNamePrefix = 0,
-                           const boost::dynamic_bitset<> *bondsToUse = 0);
+RDKIT_GRAPHMOL_EXPORT double *getAdjacencyMatrix(
+    const ROMol &mol, bool useBO = false, int emptyVal = 0, bool force = false,
+    const char *propNamePrefix = 0,
+    const boost::dynamic_bitset<> *bondsToUse = 0);
 
 //! Computes the molecule's topological distance matrix
 /*!
@@ -601,9 +669,11 @@ double *getAdjacencyMatrix(const ROMol &mol, bool useBO = false,
 
 
 */
-double *getDistanceMat(const ROMol &mol, bool useBO = false,
-                       bool useAtomWts = false, bool force = false,
-                       const char *propNamePrefix = 0);
+RDKIT_GRAPHMOL_EXPORT double *getDistanceMat(const ROMol &mol,
+                                             bool useBO = false,
+                                             bool useAtomWts = false,
+                                             bool force = false,
+                                             const char *propNamePrefix = 0);
 
 //! Computes the molecule's topological distance matrix
 /*!
@@ -628,9 +698,10 @@ double *getDistanceMat(const ROMol &mol, bool useBO = false,
 
 
 */
-double *getDistanceMat(const ROMol &mol, const std::vector<int> &activeAtoms,
-                       const std::vector<const Bond *> &bonds,
-                       bool useBO = false, bool useAtomWts = false);
+RDKIT_GRAPHMOL_EXPORT double *getDistanceMat(
+    const ROMol &mol, const std::vector<int> &activeAtoms,
+    const std::vector<const Bond *> &bonds, bool useBO = false,
+    bool useAtomWts = false);
 
 //! Computes the molecule's 3D distance matrix
 /*!
@@ -648,14 +719,16 @@ double *getDistanceMat(const ROMol &mol, const std::vector<int> &activeAtoms,
   \return the distance matrix.
 
   <b>Notes</b>
-    - The result of this is cached in the molecule's local property dictionary,
-      which will handle deallocation. Do the caller should <b>not</b> \c delete
-      this pointer.
+    - If propNamePrefix is not empty the result of this is cached in the
+      molecule's local property dictionary, which will handle deallocation.
+      In other cases the caller is responsible for freeing the memory.
 
 */
-double *get3DDistanceMat(const ROMol &mol, int confId = -1,
-                         bool useAtomWts = false, bool force = false,
-                         const char *propNamePrefix = 0);
+RDKIT_GRAPHMOL_EXPORT double *get3DDistanceMat(const ROMol &mol,
+                                               int confId = -1,
+                                               bool useAtomWts = false,
+                                               bool force = false,
+                                               const char *propNamePrefix = 0);
 //! Find the shortest path between two atoms
 /*!
   Uses the Bellman-Ford algorithm
@@ -672,7 +745,8 @@ double *get3DDistanceMat(const ROMol &mol, int confId = -1,
    - if no path is found, an empty path is returned
 
 */
-std::list<int> getShortestPath(const ROMol &mol, int aid1, int aid2);
+RDKIT_GRAPHMOL_EXPORT std::list<int> getShortestPath(const ROMol &mol, int aid1,
+                                                     int aid2);
 
 //@}
 
@@ -701,7 +775,7 @@ std::list<int> getShortestPath(const ROMol &mol, int aid1, int aid2);
 	        individual atoms will be tracked.  The \c rankHistory pointer should be
 	        to a VECT_INT_VECT that has at least \c mol.getNumAtoms() elements.
     */
-    void rankAtoms(const ROMol &mol,std::vector<int> &ranks,
+    RDKIT_GRAPHMOL_EXPORT void rankAtoms(const ROMol &mol,std::vector<int> &ranks,
                    bool breakTies=true,
                    bool includeChirality=true,
                    bool includeIsotopes=true,
@@ -729,7 +803,7 @@ std::list<int> getShortestPath(const ROMol &mol, int aid1, int aid2);
 	        individual atoms will be tracked.  The \c rankHistory pointer should be
 	        to a VECT_INT_VECT that has at least \c mol.getNumAtoms() elements.
     */
-    void rankAtomsInFragment(const ROMol &mol,std::vector<int> &ranks,
+    RDKIT_GRAPHMOL_EXPORT void rankAtomsInFragment(const ROMol &mol,std::vector<int> &ranks,
                              const boost::dynamic_bitset<> &atomsToUse,
                              const boost::dynamic_bitset<> &bondsToUse,
                              const std::vector<std::string> *atomSymbols=0,
@@ -743,7 +817,7 @@ std::list<int> getShortestPath(const ROMol &mol, int aid1, int aid2);
 //@{
 
 //! removes bogus chirality markers (those on non-sp3 centers):
-void cleanupChirality(RWMol &mol);
+RDKIT_GRAPHMOL_EXPORT void cleanupChirality(RWMol &mol);
 
 //! \brief Uses a conformer to assign ChiralType to a molecule's atoms
 /*!
@@ -754,16 +828,50 @@ void cleanupChirality(RWMol &mol);
 
   If the conformer provided is not a 3D conformer, nothing will be done.
 */
-void assignChiralTypesFrom3D(ROMol &mol, int confId = -1,
-                             bool replaceExistingTags = true);
+RDKIT_GRAPHMOL_EXPORT void assignChiralTypesFrom3D(
+    ROMol &mol, int confId = -1, bool replaceExistingTags = true);
+
+//! \brief Uses a conformer to assign ChiralTypes to a molecule's atoms and
+//! stereo flags to its bonds
+/*!
+
+  \param mol                  the molecule of interest
+  \param confId               the conformer to use
+  \param replaceExistingTags  if this flag is true, any existing info about
+                              stereochemistry will be replaced
+
+  If the conformer provided is not a 3D conformer, nothing will be done.
+*/
+RDKIT_GRAPHMOL_EXPORT void assignStereochemistryFrom3D(
+    ROMol &mol, int confId = -1, bool replaceExistingTags = true);
+
+//! \brief Uses a conformer to assign directionality to the single bonds
+//!   around double bonds
+/*!
+
+  \param mol                  the molecule of interest
+  \param confId               the conformer to use
+*/
+RDKIT_GRAPHMOL_EXPORT void detectBondStereochemistry(ROMol &mol,
+                                                     int confId = -1);
+RDKIT_GRAPHMOL_EXPORT void setDoubleBondNeighborDirections(
+    ROMol &mol, const Conformer *conf = NULL);
 
 //! Assign stereochemistry tags to atoms (i.e. R/S) and bonds (i.e. Z/E)
 /*!
+  Does the CIP stereochemistry assignment for the molecule's atoms
+  (R/S) and double bond (Z/E). Chiral atoms will have a property
+  '_CIPCode' indicating their chiral code.
 
-  \param mol     the molecule of interest
-  \param cleanIt toggles removal of stereo flags from double bonds that can
-                 not have stereochemistry
-  \param force   forces the calculation to be repeated even if it has
+  \param mol     the molecule to use
+  \param cleanIt if true, atoms with a chiral specifier that aren't
+                 actually chiral (e.g. atoms with duplicate
+                 substituents or only 2 substituents, etc.) will have
+                 their chiral code set to CHI_UNSPECIFIED. Bonds with
+                 STEREOCIS/STEREOTRANS specified that have duplicate
+                 substituents based upon the CIP atom ranks will be
+                 marked STEREONONE.
+  \param force   causes the calculation to be repeated even if it has
                  already been done
   \param flagPossibleStereoCenters   set the _ChiralityPossible property on
                                      atoms that are possible stereocenters
@@ -773,36 +881,46 @@ void assignChiralTypesFrom3D(ROMol &mol, int confId = -1,
       graph.
 
 */
-void assignStereochemistry(ROMol &mol, bool cleanIt = false, bool force = false,
-                           bool flagPossibleStereoCenters = false);
+RDKIT_GRAPHMOL_EXPORT void assignStereochemistry(
+    ROMol &mol, bool cleanIt = false, bool force = false,
+    bool flagPossibleStereoCenters = false);
 //! Removes all stereochemistry information from atoms (i.e. R/S) and bonds
 //(i.e. Z/E)
 /*!
 
   \param mol     the molecule of interest
 */
-void removeStereochemistry(ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT void removeStereochemistry(ROMol &mol);
 
 //! \brief finds bonds that could be cis/trans in a molecule and mark them as
-//!  Bond::STEREONONE
+//!  Bond::STEREOANY.
 /*!
   \param mol     the molecule of interest
   \param cleanIt toggles removal of stereo flags from double bonds that can
                  not have stereochemistry
 
-  This function is usefuly in two situations
+  This function finds any double bonds that can potentially be part of
+  a cis/trans system. No attempt is made here to mark them cis or
+  trans. No attempt is made to detect double bond stereo in ring systems.
+
+  This function is useful in the following situations:
     - when parsing a mol file; for the bonds marked here, coordinate
-  informations
-      on the neighbors can be used to indentify cis or trans states
+      information on the neighbors can be used to indentify cis or trans states
     - when writing a mol file; bonds that can be cis/trans but not marked as
-  either
-      need to be specially marked in the mol file
+      either need to be specially marked in the mol file
+    - finding double bonds with unspecified stereochemistry so they
+      can be enumerated for downstream 3D tools
+
+  The CIPranks on the neighboring atoms are checked in this function. The
+  _CIPCode property if set to any on the double bond.
 */
-void findPotentialStereoBonds(ROMol &mol, bool cleanIt = false);
+RDKIT_GRAPHMOL_EXPORT void findPotentialStereoBonds(ROMol &mol,
+                                                    bool cleanIt = false);
 //@}
 
 //! returns the number of atoms which have a particular property set
-unsigned getNumAtomsWithDistinctProperty(const ROMol &mol, std::string prop);
+RDKIT_GRAPHMOL_EXPORT unsigned getNumAtomsWithDistinctProperty(
+    const ROMol &mol, std::string prop);
 
 };  // end of namespace MolOps
 };  // end of namespace RDKit
