@@ -7,6 +7,7 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef _RD_GASTEIGERCHARGES_H
 #define _RD_GASTEIGERCHARGES_H
 
@@ -14,11 +15,11 @@
 
 namespace RDKit {
 class ROMol;
-void computeGasteigerCharges(const ROMol *mol, int nIter = 12,
+RDKIT_PARTIALCHARGES_EXPORT void computeGasteigerCharges(const ROMol *mol, int nIter = 12,
                              bool throwOnParamFailure = false);
-void computeGasteigerCharges(const ROMol &mol, int nIter = 12,
+RDKIT_PARTIALCHARGES_EXPORT void computeGasteigerCharges(const ROMol &mol, int nIter = 12,
                              bool throwOnParamFailure = false);
-void computeGasteigerCharges(const ROMol &mol, std::vector<double> &charges,
+RDKIT_PARTIALCHARGES_EXPORT void computeGasteigerCharges(const ROMol &mol, std::vector<double> &charges,
                              int nIter = 12, bool throwOnParamFailure = false);
 }
 
