@@ -14,6 +14,7 @@
 
 */
 
+#include <RDGeneral/export.h>
 #ifndef __RD_ROMOL_H__
 #define __RD_ROMOL_H__
 
@@ -29,20 +30,18 @@
 
 // our stuff
 #include <RDGeneral/types.h>
+#include <RDGeneral/RDProps.h>
 #include "Atom.h"
 #include "Bond.h"
-
 #include "Conformer.h"
+#include "StereoGroup.h"
 
 namespace RDKit {
 class Atom;
 class Bond;
-typedef boost::shared_ptr<Atom> ATOM_SPTR;
-typedef boost::shared_ptr<Bond> BOND_SPTR;
-
 //! This is the BGL type used to store the topology:
 typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS,
-                              ATOM_SPTR, BOND_SPTR>
+                              Atom *, Bond *>
     MolGraph;
 class MolPickler;
 class RWMol;
@@ -64,9 +63,9 @@ class QueryAtomIterator_;
 template <class T1, class T2>
 class MatchingAtomIterator_;
 
-extern const int ci_RIGHTMOST_ATOM;
-extern const int ci_LEADING_BOND;
-extern const int ci_ATOM_HOLDER;
+RDKIT_GRAPHMOL_EXPORT extern const int ci_RIGHTMOST_ATOM;
+RDKIT_GRAPHMOL_EXPORT extern const int ci_LEADING_BOND;
+RDKIT_GRAPHMOL_EXPORT extern const int ci_ATOM_HOLDER;
 
 //! ROMol is a molecule class that is intended to have a fixed topology
 /*!
@@ -99,7 +98,75 @@ extern const int ci_ATOM_HOLDER;
 
  */
 
-class ROMol {
+//! \name C++11 Iterators
+
+template <class Graph, class Vertex>
+struct CXXAtomIterator {
+  Graph *graph;
+  typename Graph::vertex_iterator vstart, vend;
+
+  struct CXXAtomIter {
+    Graph *graph;
+    typename Graph::vertex_iterator pos;
+    Atom *current;
+
+    CXXAtomIter(Graph *graph, typename Graph::vertex_iterator pos)
+        : graph(graph), pos(pos), current(nullptr) {}
+
+    Vertex &operator*() {
+      current = (*graph)[*pos];
+      return current;
+    }
+    CXXAtomIter &operator++() {
+      ++pos;
+      return *this;
+    }
+    bool operator!=(const CXXAtomIter &it) const { return pos != it.pos; }
+  };
+
+  CXXAtomIterator(Graph *graph) : graph(graph) {
+    auto vs = boost::vertices(*graph);
+    vstart = vs.first;
+    vend = vs.second;
+  }
+  CXXAtomIter begin() { return {graph, vstart}; }
+  CXXAtomIter end() { return {graph, vend}; }
+};
+
+template <class Graph, class Edge>
+struct CXXBondIterator {
+  Graph *graph;
+  typename Graph::edge_iterator vstart, vend;
+
+  struct CXXBondIter {
+    Graph *graph;
+    typename Graph::edge_iterator pos;
+    Bond *current;
+
+    CXXBondIter(Graph *graph, typename Graph::edge_iterator pos)
+        : graph(graph), pos(pos), current(nullptr) {}
+
+    Edge &operator*() {
+      current = (*graph)[*pos];
+      return current;
+    }
+    CXXBondIter &operator++() {
+      ++pos;
+      return *this;
+    }
+    bool operator!=(const CXXBondIter &it) const { return pos != it.pos; }
+  };
+
+  CXXBondIterator(Graph *graph) : graph(graph) {
+    auto vs = boost::edges(*graph);
+    vstart = vs.first;
+    vend = vs.second;
+  }
+  CXXBondIter begin() { return {graph, vstart}; }
+  CXXBondIter end() { return {graph, vend}; }
+};
+
+class RDKIT_GRAPHMOL_EXPORT ROMol : public RDProps {
  public:
   friend class MolPickler;
   friend class RWMol;
@@ -120,13 +187,6 @@ class ROMol {
   typedef std::pair<VERTEX_ITER, VERTEX_ITER> ATOM_ITER_PAIR;
   typedef std::pair<ADJ_ITER, ADJ_ITER> ADJ_ITER_PAIR;
 
-  typedef std::vector<ATOM_SPTR> ATOM_SPTR_VECT;
-  typedef ATOM_SPTR_VECT::iterator ATOM_SPTR_VECT_I;
-  typedef ATOM_SPTR_VECT::const_iterator ATOM_SPTR_VECT_CI;
-  typedef std::vector<BOND_SPTR> BOND_SPTR_VECT;
-  typedef BOND_SPTR_VECT::iterator BOND_SPTR_VECT_I;
-  typedef BOND_SPTR_VECT::const_iterator BOND_SPTR_VECT_CI;
-
   typedef std::vector<Atom *> ATOM_PTR_VECT;
   typedef ATOM_PTR_VECT::iterator ATOM_PTR_VECT_I;
   typedef ATOM_PTR_VECT::const_iterator ATOM_PTR_VECT_CI;
@@ -174,7 +234,38 @@ class ROMol {
   //@}
   //! \endcond
 
-  ROMol() { initMol(); }
+  //! C++11 Range iterator
+  /*!
+    <b>Usage</b>
+    \code
+      for(auto atom : mol.atoms()) {
+         atom->getIdx();
+      };
+    \endcode
+   */
+
+  CXXAtomIterator<MolGraph, Atom *> atoms() { return {&d_graph}; }
+
+  CXXAtomIterator<const MolGraph, Atom *const> atoms() const {
+    return {&d_graph};
+  }
+
+  /*!
+  <b>Usage</b>
+  \code
+    for(auto bond : mol.bonds()) {
+       bond->getIdx();
+    };
+  \endcode
+ */
+
+  CXXBondIterator<MolGraph, Bond *> bonds() { return {&d_graph}; }
+
+  CXXBondIterator<const MolGraph, Bond *const> bonds() const {
+    return {&d_graph};
+  }
+
+  ROMol() : RDProps(), numBonds(0) { initMol(); }
 
   //! copy constructor with a twist
   /*!
@@ -187,16 +278,18 @@ class ROMol {
     only
          the specified conformer from \c other.
   */
-  ROMol(const ROMol &other, bool quickCopy = false, int confId = -1) {
-    dp_props = 0;
+  ROMol(const ROMol &other, bool quickCopy = false, int confId = -1)
+      : RDProps() {
     dp_ringInfo = 0;
     initFromOther(other, quickCopy, confId);
+    numBonds = rdcast<unsigned int>(boost::num_edges(d_graph));
   };
   //! construct a molecule from a pickle string
   ROMol(const std::string &binStr);
 
   virtual ~ROMol() { destroy(); };
 
+  //@}
   //! \name Atoms
   //@{
 
@@ -220,8 +313,6 @@ class ROMol {
   }
   //! returns the degree (number of neighbors) of an Atom in the graph
   unsigned int getAtomDegree(const Atom *at) const;
-  //! \overload
-  unsigned int getAtomDegree(ATOM_SPTR at) const;
   //@}
 
   //! \name Bonds
@@ -266,19 +357,10 @@ class ROMol {
   //@{
 
   //! associates an Atom pointer with a bookmark
-  void setAtomBookmark(ATOM_SPTR at, int mark) {
-    d_atomBookmarks[mark].push_back(at.get());
-  };
-  //! \overload
   void setAtomBookmark(Atom *at, int mark) {
     d_atomBookmarks[mark].push_back(at);
   };
   //! associates an Atom pointer with a bookmark
-  void replaceAtomBookmark(ATOM_SPTR at, int mark) {
-    d_atomBookmarks[mark].clear();
-    d_atomBookmarks[mark].push_back(at.get());
-  };
-  //! \overload
   void replaceAtomBookmark(Atom *at, int mark) {
     d_atomBookmarks[mark].clear();
     d_atomBookmarks[mark].push_back(at);
@@ -291,10 +373,7 @@ class ROMol {
   void clearAtomBookmark(const int mark);
   //! removes a particular Atom from the list associated with the \c bookmark
   void clearAtomBookmark(const int mark, const Atom *atom);
-  //! \overload
-  void clearAtomBookmark(const int mark, ATOM_SPTR atom) {
-    clearAtomBookmark(mark, atom.get());
-  };
+
   //! blows out all atomic \c bookmarks
   void clearAllAtomBookmarks() { d_atomBookmarks.clear(); };
   //! queries whether or not any atoms are associated with a \c bookmark
@@ -303,10 +382,6 @@ class ROMol {
   ATOM_BOOKMARK_MAP *getAtomBookmarks() { return &d_atomBookmarks; };
 
   //! associates a Bond pointer with a bookmark
-  void setBondBookmark(BOND_SPTR bond, int mark) {
-    d_bondBookmarks[mark].push_back(bond.get());
-  };
-  //! \overload
   void setBondBookmark(Bond *bond, int mark) {
     d_bondBookmarks[mark].push_back(bond);
   };
@@ -318,10 +393,7 @@ class ROMol {
   void clearBondBookmark(int mark);
   //! removes a particular Bond from the list associated with the \c bookmark
   void clearBondBookmark(int mark, const Bond *bond);
-  //! \overload
-  void clearBondBookmark(int mark, BOND_SPTR bond) {
-    clearBondBookmark(mark, bond.get());
-  };
+
   //! blows out all bond \c bookmarks
   void clearAllBondBookmarks() { d_bondBookmarks.clear(); };
   //! queries whether or not any bonds are associated with a \c bookmark
@@ -379,21 +451,19 @@ class ROMol {
 
     <b>Usage</b>
     \code
-      ... molPtr is a const ROMol & ...
+      ... mol is a const ROMol & ...
       ... atomPtr is a const Atom * ...
-      ROMol::ADJ_ITER nbrIdx,endNbrs;
-      boost::tie(nbrIdx,endNbrs) = molPtr.getAtomNeighbors(atomPtr);
-      while(nbrIdx!=endNbrs){
-        const ATOM_SPTR at=molPtr[*nbrIdx];
-        ... do something with the Atom ...
-        ++nbrIdx;
+      ... requires #include <boost/range/iterator_range.hpp>
+      for (const auto &nbri :
+           boost::make_iterator_range(m.getAtomNeighbors(atomPtr))) {
+        const auto &nbr = (*m)[nbri];
+        // nbr is an atom pointer
       }
+
     \endcode
 
   */
   ADJ_ITER_PAIR getAtomNeighbors(Atom const *at) const;
-  //! \overload
-  ADJ_ITER_PAIR getAtomNeighbors(ATOM_SPTR at) const;
 
   //! provides access to all Bond objects connected to an Atom
   /*!
@@ -401,26 +471,24 @@ class ROMol {
 
     <b>Usage</b>
     \code
-      ... molPtr is a const ROMol * ...
+      ... mol is a const ROMol & ...
       ... atomPtr is a const Atom * ...
-      ROMol::OEDGE_ITER beg,end;
-      boost::tie(beg,end) = molPtr->getAtomBonds(atomPtr);
-      while(beg!=end){
-        const BOND_SPTR bond=(*molPtr)[*beg];
-        ... do something with the Bond ...
-        ++beg;
+      ... requires #include <boost/range/iterator_range.hpp>
+      for (const auto &nbri :
+           boost::make_iterator_range(m.getAtomBonds(atomPtr))) {
+        const auto &nbr = (*m)[nbri];
+        // nbr is a bond pointer
       }
     \endcode
     or, if you need a non-const Bond *:
     \code
-      ... molPtr is a ROMol * ...
+      ... mol is a const ROMol & ...
       ... atomPtr is a const Atom * ...
-      ROMol::OEDGE_ITER beg,end;
-      boost::tie(beg,end) = molPtr->getAtomBonds(atomPtr);
-      while(beg!=end){
-        BOND_SPTR bond=(*molPtr)[*beg];
-        ... do something with the Bond ...
-        ++beg;
+      ... requires #include <boost/range/iterator_range.hpp>
+      for (const auto &nbri :
+           boost::make_iterator_range(m.getAtomBonds(atomPtr))) {
+        auto nbr = (*m)[nbri];
+        // nbr is a bond pointer
       }
     \endcode
 
@@ -553,140 +621,6 @@ class ROMol {
   //! \name Properties
   //@{
 
-  //! returns a list with the names of our \c properties
-  STR_VECT getPropList(bool includePrivate = true,
-                       bool includeComputed = true) const {
-    const STR_VECT &tmp = dp_props->keys();
-    STR_VECT res, computed;
-    if (!includeComputed &&
-        getPropIfPresent(detail::computedPropName, computed)) {
-      computed.push_back(detail::computedPropName);
-    }
-
-    STR_VECT::const_iterator pos = tmp.begin();
-    while (pos != tmp.end()) {
-      if ((includePrivate || (*pos)[0] != '_') &&
-          std::find(computed.begin(), computed.end(), *pos) == computed.end()) {
-        res.push_back(*pos);
-      }
-      pos++;
-    }
-    return res;
-  }
-
-  //! sets a \c property value
-  /*!
-     \param key the name under which the \c property should be stored.
-         If a \c property is already stored under this name, it will be
-         replaced.
-     \param val the value to be stored
-     \param computed (optional) allows the \c property to be flagged
-         \c computed.
-   */
-  template <typename T>
-  void setProp(const char *key, T val, bool computed = false) const {
-    std::string what(key);
-    setProp(what, val, computed);
-  }
-  //! \overload
-  template <typename T>
-  void setProp(const std::string &key, T val, bool computed = false) const {
-    if (computed) {
-      STR_VECT compLst;
-      dp_props->getVal(detail::computedPropName, compLst);
-      if (std::find(compLst.begin(), compLst.end(), key) == compLst.end()) {
-        compLst.push_back(key);
-        dp_props->setVal(detail::computedPropName, compLst);
-      }
-    }
-    dp_props->setVal(key, val);
-  }
-
-  //! allows retrieval of a particular property value
-  /*!
-
-     \param key the name under which the \c property should be stored.
-         If a \c property is already stored under this name, it will be
-         replaced.
-     \param res a reference to the storage location for the value.
-
-     <b>Notes:</b>
-       - if no \c property with name \c key exists, a KeyErrorException will be
-     thrown.
-       - the \c boost::lexical_cast machinery is used to attempt type
-     conversions.
-         If this fails, a \c boost::bad_lexical_cast exception will be thrown.
-
-  */
-  template <typename T>
-  void getProp(const char *key, T &res) const {
-    dp_props->getVal(key, res);
-  }
-  //! \overload
-  template <typename T>
-  void getProp(const std::string &key, T &res) const {
-    dp_props->getVal(key, res);
-  }
-  //! \overload
-  template <typename T>
-  T getProp(const char *key) const {
-    return dp_props->getVal<T>(key);
-  }
-  //! \overload
-  template <typename T>
-  T getProp(const std::string &key) const {
-    return dp_props->getVal<T>(key);
-  }
-
-  //! returns whether or not we have a \c property with name \c key
-  //!  and assigns the value if we do
-  template <typename T>
-  bool getPropIfPresent(const char *key, T &res) const {
-    return dp_props->getValIfPresent(key, res);
-  }
-  //! \overload
-  template <typename T>
-  bool getPropIfPresent(const std::string &key, T &res) const {
-    return dp_props->getValIfPresent(key, res);
-  }
-
-  //! returns whether or not we have a \c property with name \c key
-  bool hasProp(const char *key) const {
-    if (!dp_props) return false;
-    return dp_props->hasVal(key);
-  }
-  //! \overload
-  bool hasProp(const std::string &key) const {
-    if (!dp_props) return false;
-    return dp_props->hasVal(key);
-    // return hasProp(key.c_str());
-  }
-
-  //! clears the value of a \c property
-  /*!
-     <b>Notes:</b>
-       - if no \c property with name \c key exists, a KeyErrorException
-         will be thrown.
-       - if the \c property is marked as \c computed, it will also be removed
-         from our list of \c computedProperties
-  */
-  void clearProp(const char *key) const {
-    std::string what(key);
-    clearProp(what);
-  };
-  //! \overload
-  void clearProp(const std::string &key) const {
-    STR_VECT compLst;
-    getProp(detail::computedPropName, compLst);
-    STR_VECT_I svi = std::find(compLst.begin(), compLst.end(), key);
-    if (svi != compLst.end()) {
-      compLst.erase(svi);
-      dp_props->setVal(detail::computedPropName, compLst);
-    }
-
-    dp_props->clearVal(key);
-  };
-
   //! clears all of our \c computed \c properties
   void clearComputedProps(bool includeRings = true) const;
   //! calculates any of our lazy \c properties
@@ -706,28 +640,38 @@ class ROMol {
   void debugMol(std::ostream &str) const;
   //@}
 
-  ATOM_SPTR operator[](const vertex_descriptor &v) { return d_graph[v]; };
-  const ATOM_SPTR operator[](const vertex_descriptor &v) const {
+  Atom *operator[](const vertex_descriptor &v) { return d_graph[v]; };
+  const Atom *operator[](const vertex_descriptor &v) const {
     return d_graph[v];
   };
 
-  BOND_SPTR operator[](const edge_descriptor &e) { return d_graph[e]; };
-  const BOND_SPTR operator[](const edge_descriptor &e) const {
-    return d_graph[e];
-  };
+  Bond *operator[](const edge_descriptor &e) { return d_graph[e]; };
+  const Bond *operator[](const edge_descriptor &e) const { return d_graph[e]; };
+
+  //! Gets a reference to the groups of atoms with relative stereochemistry
+  /*!
+    Stereo groups are also called enhanced stereochemistry in the SDF/Mol3000
+    file format.
+  */
+  const std::vector<StereoGroup> &getStereoGroups() const {
+    return d_stereo_groups;
+  }
 
  private:
   MolGraph d_graph;
   ATOM_BOOKMARK_MAP d_atomBookmarks;
   BOND_BOOKMARK_MAP d_bondBookmarks;
-  Dict *dp_props;
   RingInfo *dp_ringInfo;
   CONF_SPTR_LIST d_confs;
+  std::vector<StereoGroup> d_stereo_groups;
+
   ROMol &operator=(
       const ROMol &);  // disable assignment, RWMol's support assignment
 
-#ifdef WIN32
  protected:
+  unsigned int numBonds;
+#ifndef WIN32
+ private:
 #endif
   void initMol();
   virtual void destroy();
@@ -744,21 +688,6 @@ class ROMol {
   */
   unsigned int addAtom(Atom *atom, bool updateLabel = true,
                        bool takeOwnership = false);
-  //! adds an Atom to our collection
-  /*!
-    \param atom          pointer to the Atom to add
-    \param updateLabel   (optional) if this is true, the new Atom will be
-                         our \c activeAtom
-
-
-    \return the new number of atoms
-
-    <b>Note:</b> since this is using a smart pointer, we don't need to worry
-    about
-    issues of ownership.
-
-  */
-  unsigned int addAtom(ATOM_SPTR, bool updateLabel = true);
   //! adds a Bond to our collection
   /*!
     \param bond          pointer to the Bond to add
@@ -769,6 +698,15 @@ class ROMol {
     \return the new number of bonds
   */
   unsigned int addBond(Bond *bond, bool takeOwnership = false);
+
+  //! Sets groups of atoms with relative stereochemistry
+  /*!
+    \param stereo_groups the new set of stereo groups. All will be replaced.
+
+    Stereo groups are also called enhanced stereochemistry in the SDF/Mol3000
+    file format. stereo_groups should be std::move()ed into this function.
+  */
+  void setStereoGroups(std::vector<StereoGroup> stereo_groups);
   //! adds a Bond to our collection
   /*!
     \param bond          pointer to the Bond to add
@@ -779,18 +717,6 @@ class ROMol {
     about
     issues of ownership.
   */
-  unsigned int addBond(BOND_SPTR bsp);
-
-  //! initializes from the contents of another molecule
-  /*!
-    \param other     the molecule to be copied
-    \param quickCopy if this is true, we will not
-         copy any of the properties or bookmarks and conformers from \c other.
-    This can
-         make the copy substantially faster (thus the name).
-    \param confId if this is >=0, the resulting ROMol will contain only
-         the specified conformer from \c other.
-  */
   void initFromOther(const ROMol &other, bool quickCopy, int confId);
 };
 
@@ -802,5 +728,5 @@ typedef std::vector<ROMOL_SPTR> MOL_SPTR_VECT;
 typedef MOL_PTR_VECT::const_iterator MOL_PTR_VECT_CI;
 typedef MOL_PTR_VECT::iterator MOL_PTR_VECT_I;
 
-};  // end of RDKit namespace
+};  // namespace RDKit
 #endif
