@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2003-2010 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2003-2016 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -31,6 +30,7 @@ RWMol &RWMol::operator=(const RWMol &other) {
   if (this != &other) {
     this->clear();
     d_partialBonds.clear();
+    numBonds = 0;
     initFromOther(other, false, -1);
   }
   return *this;
@@ -79,9 +79,9 @@ void RWMol::insertMol(const ROMol &other) {
 
   // add atom to any conformers as well, if we have any
   if (other.getNumConformers() && !getNumConformers()) {
-    for (ConstConformerIterator cfi = other.beginConformers();
-         cfi != other.endConformers(); ++cfi) {
-      Conformer *nconf = new Conformer(getNumAtoms());
+    for (auto cfi = other.beginConformers(); cfi != other.endConformers();
+         ++cfi) {
+      auto *nconf = new Conformer(getNumAtoms());
       nconf->set3D((*cfi)->is3D());
       nconf->setId((*cfi)->getId());
       for (unsigned int i = 0; i < newAtomIds.size(); ++i)
@@ -99,43 +99,70 @@ void RWMol::insertMol(const ROMol &other) {
           (*cfi)->setAtomPos(newAtomIds[i], (*ocfi)->getAtomPos(i));
       }
     } else {
-      for (ConformerIterator cfi = this->beginConformers();
-           cfi != this->endConformers(); ++cfi) {
+      for (auto cfi = this->beginConformers(); cfi != this->endConformers();
+           ++cfi) {
         (*cfi)->resize(getNumAtoms());
-        for (unsigned int i = 0; i < newAtomIds.size(); ++i)
-          (*cfi)->setAtomPos(newAtomIds[i], RDGeom::Point3D(0.0, 0.0, 0.0));
+        for (unsigned int newAtomId : newAtomIds)
+          (*cfi)->setAtomPos(newAtomId, RDGeom::Point3D(0.0, 0.0, 0.0));
       }
     }
   }
 }
 
 unsigned int RWMol::addAtom(bool updateLabel) {
-  Atom *atom_p = new Atom();
+  auto *atom_p = new Atom();
   atom_p->setOwningMol(this);
   MolGraph::vertex_descriptor which = boost::add_vertex(d_graph);
-  d_graph[which].reset(atom_p);
+  d_graph[which] = atom_p;
   atom_p->setIdx(which);
   if (updateLabel) {
     clearAtomBookmark(ci_RIGHTMOST_ATOM);
     setAtomBookmark(atom_p, ci_RIGHTMOST_ATOM);
   }
   // add atom to any conformers as well, if we have any
-  for (ConformerIterator cfi = this->beginConformers();
-       cfi != this->endConformers(); ++cfi) {
+  for (auto cfi = this->beginConformers(); cfi != this->endConformers();
+       ++cfi) {
     (*cfi)->setAtomPos(which, RDGeom::Point3D(0.0, 0.0, 0.0));
   }
   return rdcast<unsigned int>(which);
 }
 
-void RWMol::replaceAtom(unsigned int idx, Atom *atom_pin, bool updateLabel) {
+void RWMol::replaceAtom(unsigned int idx, Atom *atom_pin, bool updateLabel,
+                        bool preserveProps) {
   RDUNUSED_PARAM(updateLabel);
   PRECONDITION(atom_pin, "bad atom passed to replaceAtom");
-  URANGE_CHECK(idx, getNumAtoms() - 1);
+  URANGE_CHECK(idx, getNumAtoms());
   Atom *atom_p = atom_pin->copy();
   atom_p->setOwningMol(this);
   atom_p->setIdx(idx);
   MolGraph::vertex_descriptor vd = boost::vertex(idx, d_graph);
-  d_graph[vd].reset(atom_p);
+  if (preserveProps) {
+    const bool replaceExistingData = false;
+    atom_p->updateProps(*d_graph[vd], replaceExistingData);
+  }
+  delete d_graph[vd];
+  d_graph[vd] = atom_p;
+  // FIX: do something about bookmarks
+};
+
+void RWMol::replaceBond(unsigned int idx, Bond *bond_pin, bool preserveProps) {
+  PRECONDITION(bond_pin, "bad bond passed to replaceBond");
+  URANGE_CHECK(idx, getNumBonds());
+  BOND_ITER_PAIR bIter = getEdges();
+  for (unsigned int i = 0; i < idx; i++) ++bIter.first;
+  Bond *obond = d_graph[*(bIter.first)];
+  Bond *bond_p = bond_pin->copy();
+  bond_p->setOwningMol(this);
+  bond_p->setIdx(idx);
+  bond_p->setBeginAtomIdx(obond->getBeginAtomIdx());
+  bond_p->setEndAtomIdx(obond->getEndAtomIdx());
+  if (preserveProps) {
+    const bool replaceExistingData = false;
+    bond_p->updateProps(*d_graph[*(bIter.first)], replaceExistingData);
+  }
+
+  delete d_graph[*(bIter.first)];
+  d_graph[*(bIter.first)] = bond_p;
   // FIX: do something about bookmarks
 };
 
@@ -164,13 +191,13 @@ void RWMol::removeAtom(Atom *atom) {
 
   // remove any bookmarks which point to this atom:
   ATOM_BOOKMARK_MAP *marks = getAtomBookmarks();
-  ATOM_BOOKMARK_MAP::iterator markI = marks->begin();
+  auto markI = marks->begin();
   while (markI != marks->end()) {
     const ATOM_PTR_LIST &atoms = markI->second;
     // we need to copy the iterator then increment it, because the
     // deletion we're going to do in clearAtomBookmark will invalidate
     // it.
-    ATOM_BOOKMARK_MAP::iterator tmpI = markI;
+    auto tmpI = markI;
     ++markI;
     if (std::find(atoms.begin(), atoms.end(), atom) != atoms.end()) {
       clearAtomBookmark(tmpI->first, atom);
@@ -178,27 +205,27 @@ void RWMol::removeAtom(Atom *atom) {
   }
 
   // remove bonds attached to the atom
-  std::vector<std::pair<unsigned int, unsigned int> > nbrs;
+  std::vector<std::pair<unsigned int, unsigned int>> nbrs;
   ADJ_ITER b1, b2;
   boost::tie(b1, b2) = getAtomNeighbors(atom);
   while (b1 != b2) {
     nbrs.push_back(std::make_pair(atom->getIdx(), rdcast<unsigned int>(*b1)));
     ++b1;
   }
-  for (unsigned int i = 0; i < nbrs.size(); ++i) {
-    removeBond(nbrs[i].first, nbrs[i].second);
+  for (auto &nbr : nbrs) {
+    removeBond(nbr.first, nbr.second);
   }
 
   // loop over all atoms with higher indices and update their indices
   for (unsigned int i = idx + 1; i < getNumAtoms(); i++) {
-    Atom *atom = getAtomWithIdx(i);
-    atom->setIdx(i - 1);
+    Atom *higher_index_atom = getAtomWithIdx(i);
+    higher_index_atom->setIdx(i - 1);
   }
 
   // do the same with the coordinates in the conformations
   BOOST_FOREACH (CONFORMER_SPTR conf, d_confs) {
     RDGeom::POINT3D_VECT &positions = conf->getPositions();
-    RDGeom::POINT3D_VECT_I pi = positions.begin();
+    auto pi = positions.begin();
     for (unsigned int i = 0; i < getNumAtoms() - 1; i++) {
       ++pi;
       if (i >= idx) {
@@ -214,13 +241,13 @@ void RWMol::removeAtom(Atom *atom) {
   EDGE_ITER beg, end;
   boost::tie(beg, end) = getEdges();
   while (beg != end) {
-    BOND_SPTR bond = d_graph[*beg++];
+    Bond *bond = d_graph[*beg++];
     unsigned int tmpIdx = bond->getBeginAtomIdx();
     if (tmpIdx > idx) bond->setBeginAtomIdx(tmpIdx - 1);
     tmpIdx = bond->getEndAtomIdx();
     if (tmpIdx > idx) bond->setEndAtomIdx(tmpIdx - 1);
     bond->setIdx(nBonds++);
-    for (INT_VECT::iterator bsi = bond->getStereoAtoms().begin();
+    for (auto bsi = bond->getStereoAtoms().begin();
          bsi != bond->getStereoAtoms().end(); ++bsi) {
       if ((*bsi) == rdcast<int>(idx)) {
         bond->getStereoAtoms().clear();
@@ -231,28 +258,32 @@ void RWMol::removeAtom(Atom *atom) {
     }
   }
 
+  // Remove any stereo group which includes the atom being deleted
+  removeGroupsWithAtom(atom, d_stereo_groups);
+
   // clear computed properties and reset our ring info structure
   // they are pretty likely to be wrong now:
   clearComputedProps(true);
 
-  atom->setOwningMol(NULL);
+  atom->setOwningMol(nullptr);
 
   // remove all connections to the atom:
   MolGraph::vertex_descriptor vd = boost::vertex(idx, d_graph);
   boost::clear_vertex(vd, d_graph);
   // finally remove the vertex itself
   boost::remove_vertex(vd, d_graph);
+  delete atom;
 }
 
 unsigned int RWMol::addBond(unsigned int atomIdx1, unsigned int atomIdx2,
                             Bond::BondType bondType) {
-  URANGE_CHECK(atomIdx1, getNumAtoms() - 1);
-  URANGE_CHECK(atomIdx2, getNumAtoms() - 1);
+  URANGE_CHECK(atomIdx1, getNumAtoms());
+  URANGE_CHECK(atomIdx2, getNumAtoms());
   PRECONDITION(atomIdx1 != atomIdx2, "attempt to add self-bond");
   PRECONDITION(!(boost::edge(atomIdx1, atomIdx2, d_graph).second),
                "bond already exists");
 
-  Bond *b = new Bond(bondType);
+  auto *b = new Bond(bondType);
   b->setOwningMol(this);
   if (bondType == Bond::AROMATIC) {
     b->setIsAromatic(1);
@@ -268,9 +299,10 @@ unsigned int RWMol::addBond(unsigned int atomIdx1, unsigned int atomIdx2,
   bool ok;
   MolGraph::edge_descriptor which;
   boost::tie(which, ok) = boost::add_edge(atomIdx1, atomIdx2, d_graph);
-  d_graph[which].reset(b);
-  unsigned int res = rdcast<unsigned int>(boost::num_edges(d_graph));
-  b->setIdx(res - 1);
+  d_graph[which] = b;
+  // unsigned int res = rdcast<unsigned int>(boost::num_edges(d_graph));
+  ++numBonds;
+  b->setIdx(numBonds - 1);
   b->setBeginAtomIdx(atomIdx1);
   b->setEndAtomIdx(atomIdx2);
 
@@ -282,7 +314,7 @@ unsigned int RWMol::addBond(unsigned int atomIdx1, unsigned int atomIdx2,
     dp_ringInfo->reset();
   }
 
-  return res;
+  return numBonds;  // res;
 }
 
 unsigned int RWMol::addBond(Atom *atom1, Atom *atom2, Bond::BondType bondType) {
@@ -290,27 +322,22 @@ unsigned int RWMol::addBond(Atom *atom1, Atom *atom2, Bond::BondType bondType) {
   return addBond(atom1->getIdx(), atom2->getIdx(), bondType);
 }
 
-unsigned int RWMol::addBond(Atom::ATOM_SPTR atom1, Atom::ATOM_SPTR atom2,
-                            Bond::BondType bondType) {
-  return addBond(atom1->getIdx(), atom2->getIdx(), bondType);
-}
-
 void RWMol::removeBond(unsigned int aid1, unsigned int aid2) {
-  URANGE_CHECK(aid1, getNumAtoms() - 1);
-  URANGE_CHECK(aid2, getNumAtoms() - 1);
+  URANGE_CHECK(aid1, getNumAtoms());
+  URANGE_CHECK(aid2, getNumAtoms());
   Bond *bnd = getBondBetweenAtoms(aid1, aid2);
   if (!bnd) return;
   unsigned int idx = bnd->getIdx();
 
   // remove any bookmarks which point to this bond:
   BOND_BOOKMARK_MAP *marks = getBondBookmarks();
-  BOND_BOOKMARK_MAP::iterator markI = marks->begin();
+  auto markI = marks->begin();
   while (markI != marks->end()) {
     BOND_PTR_LIST &bonds = markI->second;
     // we need to copy the iterator then increment it, because the
     // deletion we're going to do in clearBondBookmark will invalidate
     // it.
-    BOND_BOOKMARK_MAP::iterator tmpI = markI;
+    auto tmpI = markI;
     ++markI;
     if (std::find(bonds.begin(), bonds.end(), bnd) != bonds.end()) {
       clearBondBookmark(tmpI->first, bnd);
@@ -327,16 +354,20 @@ void RWMol::removeBond(unsigned int aid1, unsigned int aid2) {
     if (oIdx == aid2) continue;
     Bond *obnd = getBondBetweenAtoms(aid1, oIdx);
     if (!obnd) continue;
-    obnd->getStereoAtoms().clear();
+    if (std::find(obnd->getStereoAtoms().begin(), obnd->getStereoAtoms().end(),
+                  aid2) != obnd->getStereoAtoms().end())
+      obnd->getStereoAtoms().clear();
   }
-  boost::tie(a1, a2) = boost::adjacent_vertices(aid1, d_graph);
+  boost::tie(a1, a2) = boost::adjacent_vertices(aid2, d_graph);
   while (a1 != a2) {
     unsigned int oIdx = rdcast<unsigned int>(*a1);
     ++a1;
     if (oIdx == aid1) continue;
     Bond *obnd = getBondBetweenAtoms(aid2, oIdx);
     if (!obnd) continue;
-    obnd->getStereoAtoms().clear();
+    if (std::find(obnd->getStereoAtoms().begin(), obnd->getStereoAtoms().end(),
+                  aid1) != obnd->getStereoAtoms().end())
+      obnd->getStereoAtoms().clear();
   }
 
   // reset our ring info structure, because it is pretty likely
@@ -344,21 +375,28 @@ void RWMol::removeBond(unsigned int aid1, unsigned int aid2) {
   dp_ringInfo->reset();
 
   // loop over all bonds with higher indices and update their indices
-  for (unsigned int i = idx + 1; i < getNumBonds(); ++i) {
-    getBondWithIdx(i)->setIdx(i - 1);
+  ROMol::EDGE_ITER firstB, lastB;
+  boost::tie(firstB, lastB) = this->getEdges();
+  while (firstB != lastB) {
+    Bond *bond = (*this)[*firstB];
+    if (bond->getIdx() > idx) {
+      bond->setIdx(bond->getIdx() - 1);
+    }
+    ++firstB;
   }
-
-  bnd->setOwningMol(NULL);
+  bnd->setOwningMol(nullptr);
 
   MolGraph::vertex_descriptor vd1 = boost::vertex(aid1, d_graph);
   MolGraph::vertex_descriptor vd2 = boost::vertex(aid2, d_graph);
   boost::remove_edge(vd1, vd2, d_graph);
+  delete bnd;
+  --numBonds;
 }
 
 Bond *RWMol::createPartialBond(unsigned int atomIdx1, Bond::BondType bondType) {
-  URANGE_CHECK(atomIdx1, getNumAtoms() - 1);
+  URANGE_CHECK(atomIdx1, getNumAtoms());
 
-  Bond *b = new Bond(bondType);
+  auto *b = new Bond(bondType);
   b->setOwningMol(this);
   b->setBeginAtomIdx(atomIdx1);
   return b;
@@ -367,7 +405,7 @@ Bond *RWMol::createPartialBond(unsigned int atomIdx1, Bond::BondType bondType) {
 unsigned int RWMol::finishPartialBond(unsigned int atomIdx2, int bondBookmark,
                                       Bond::BondType bondType) {
   PRECONDITION(hasBondBookmark(bondBookmark), "no such partial bond");
-  URANGE_CHECK(atomIdx2, getNumAtoms() - 1);
+  URANGE_CHECK(atomIdx2, getNumAtoms());
 
   Bond *bsp = getBondWithBookmark(bondBookmark);
   if (bondType == Bond::UNSPECIFIED) {
@@ -376,4 +414,4 @@ unsigned int RWMol::finishPartialBond(unsigned int atomIdx2, int bondBookmark,
   return addBond(bsp->getBeginAtomIdx(), atomIdx2, bondType);
 }
 
-}  // end o' namespace
+}  // namespace RDKit
