@@ -12,6 +12,10 @@
 #include <GraphMol/Resonance.h>
 #include <RDGeneral/hash/hash.hpp>
 #include <RDGeneral/RDThreads.h>
+#ifdef RDK_THREADSAFE_SSS
+#include <thread>
+#include <future>
+#endif
 
 namespace RDKit {
 // class definitions that do not need being exposed in Resonance.h
@@ -46,7 +50,9 @@ class CEMetrics {
   unsigned int d_fcSameSignDist;
   unsigned int d_fcOppSignDist;
   unsigned int d_nbMissing;
-  unsigned int d_wtdFormalCharges;
+  int d_wtdFormalCharges;
+  unsigned int d_sumFormalChargeIdxs;
+  unsigned int d_sumMultipleBondIdxs;
 };
 
 class ConjElectrons {
@@ -69,8 +75,6 @@ class ConjElectrons {
   void pushToBeginStack(unsigned int ai);
   bool popFromBeginStack(unsigned int &ai);
   bool isBeginStackEmpty() { return d_beginAIStack.empty(); };
-  unsigned int lowestFcIndex() const;
-  unsigned int lowestMultipleBondIndex() const;
   int allowedChgLeftOfN() const { return d_allowedChgLeftOfN; };
   void decrAllowedChgLeftOfN(int d) { d_allowedChgLeftOfN -= d; };
   int totalFormalCharge() const { return d_totalFormalCharge; };
@@ -86,10 +90,14 @@ class ConjElectrons {
   unsigned int fcSameSignDist() const { return d_ceMetrics.d_fcSameSignDist; };
   unsigned int fcOppSignDist() const { return d_ceMetrics.d_fcOppSignDist; };
   unsigned int nbMissing() const { return d_ceMetrics.d_nbMissing; }
+  unsigned int sumFormalChargeIdxs() const {
+    return d_ceMetrics.d_sumFormalChargeIdxs;
+  }
+  unsigned int sumMultipleBondIdxs() const {
+    return d_ceMetrics.d_sumMultipleBondIdxs;
+  }
   CEMetrics &metrics() { return d_ceMetrics; }
-  unsigned int wtdFormalCharges() const {
-    return d_ceMetrics.d_wtdFormalCharges;
-  };
+  int wtdFormalCharges() const { return d_ceMetrics.d_wtdFormalCharges; };
   void enumerateNonBonded(CEMap &ceMap);
   void initCeFromMol();
   void assignNonBonded();
@@ -117,6 +125,8 @@ class ConjElectrons {
   ConjElectrons &operator=(const ConjElectrons &);
   unsigned int countTotalElectrons();
   void computeDistFormalCharges();
+  void computeSumFormalChargeIdxs();
+  void computeSumMultipleBondIdxs();
   void checkOctets();
 };
 
@@ -232,9 +242,8 @@ void updateV(unsigned int &v) {
 // sanitize the resonance structure which has been assembled
 void sanitizeMol(RWMol &mol) {
   unsigned int opFailed;
-  MolOps::sanitizeMol(mol, opFailed, MolOps::SANITIZE_FINDRADICALS |
-                                         MolOps::SANITIZE_SETAROMATICITY |
-                                         MolOps::SANITIZE_ADJUSTHS);
+  MolOps::sanitizeMol(
+      mol, opFailed, MolOps::SANITIZE_FINDRADICALS | MolOps::SANITIZE_ADJUSTHS);
 }
 
 // fix the number of explicit and implicit Hs in the
@@ -307,7 +316,7 @@ bool AtomElectrons::isNbrCharged(unsigned int bo, unsigned int oeConstraint) {
   ROMol::OEDGE_ITER nbrIdx, endNbrs;
   boost::tie(nbrIdx, endNbrs) = mol.getAtomBonds(d_atom);
   for (; !res && (nbrIdx != endNbrs); ++nbrIdx) {
-    const Bond *bondNbr = mol[*nbrIdx].get();
+    const Bond *bondNbr = mol[*nbrIdx];
     unsigned int biNbr = bondNbr->getIdx();
     if (d_parent->parent()->getBondConjGrpIdx(biNbr) != conjGrpIdx()) continue;
     BondElectrons *beNbr = d_parent->getBondElectronsWithIdx(biNbr);
@@ -369,7 +378,7 @@ void AtomElectrons::allConjBondsDefinitiveBut(unsigned int bi) {
   ROMol::OEDGE_ITER nbrIdx, endNbrs;
   boost::tie(nbrIdx, endNbrs) = mol.getAtomBonds(d_atom);
   for (; allDefinitive && (nbrIdx != endNbrs); ++nbrIdx) {
-    unsigned int nbi = mol[*nbrIdx].get()->getIdx();
+    unsigned int nbi = mol[*nbrIdx]->getIdx();
     if ((nbi != bi) &&
         (d_parent->parent()->getBondConjGrpIdx(nbi) == conjGrpIdx()))
       allDefinitive = d_parent->getBondElectronsWithIdx(nbi)->isDefinitive();
@@ -445,7 +454,9 @@ CEMetrics::CEMetrics()
       d_fcSameSignDist(0),
       d_fcOppSignDist(0),
       d_nbMissing(0),
-      d_wtdFormalCharges(0){};
+      d_wtdFormalCharges(0),
+      d_sumFormalChargeIdxs(0),
+      d_sumMultipleBondIdxs(0){};
 
 bool CEMetrics::operator==(const CEMetrics &other) {
   return ((d_absFormalCharges == other.d_absFormalCharges) &&
@@ -479,10 +490,10 @@ ConjElectrons::ConjElectrons(ResonanceMolSupplier *parent,
     d_conjBondMap[bi] = new BondElectrons(this, bond);
     // store the pointers to AtomElectrons objects in a map
     const Atom *atom[2] = {bond->getBeginAtom(), bond->getEndAtom()};
-    for (unsigned int i = 0; i < 2; ++i) {
-      unsigned int ai = atom[i]->getIdx();
+    for (auto &i : atom) {
+      unsigned int ai = i->getIdx();
       if (d_conjAtomMap.find(ai) == d_conjAtomMap.end())
-        d_conjAtomMap[ai] = new AtomElectrons(this, atom[i]);
+        d_conjAtomMap[ai] = new AtomElectrons(this, i);
     }
   }
   // count total number of valence electrons in conjugated group
@@ -501,12 +512,10 @@ ConjElectrons::ConjElectrons(const ConjElectrons &ce)
       d_ceMetrics(ce.d_ceMetrics),
       d_beginAIStack(ce.d_beginAIStack),
       d_parent(ce.d_parent) {
-  for (ConjAtomMap::const_iterator it = ce.d_conjAtomMap.begin();
-       it != ce.d_conjAtomMap.end(); ++it)
-    d_conjAtomMap[it->first] = new AtomElectrons(this, *(it->second));
-  for (ConjBondMap::const_iterator it = ce.d_conjBondMap.begin();
-       it != ce.d_conjBondMap.end(); ++it)
-    d_conjBondMap[it->first] = new BondElectrons(this, *(it->second));
+  for (const auto &it : ce.d_conjAtomMap)
+    d_conjAtomMap[it.first] = new AtomElectrons(this, *(it.second));
+  for (const auto &it : ce.d_conjBondMap)
+    d_conjBondMap[it.first] = new BondElectrons(this, *(it.second));
 }
 
 // object destructor
@@ -527,7 +536,8 @@ bool ConjElectrons::storeFP(CEMap &ceMap, unsigned int flags) {
   ConjFP fp;
   unsigned int fpSize = 0;
   if (flags & FP_ATOMS) fpSize += rdcast<unsigned int>(d_conjAtomMap.size());
-  if (flags & FP_BONDS) fpSize += rdcast<unsigned int>((d_conjBondMap.size() - 1) / 4 + 1);
+  if (flags & FP_BONDS)
+    fpSize += rdcast<unsigned int>((d_conjBondMap.size() - 1) / 4 + 1);
   fp.reserve(fpSize);
   if (flags & FP_ATOMS) {
     // for each atom, we push a byte to the FP vector whose
@@ -736,16 +746,17 @@ void ConjElectrons::enumerateNonBonded(CEMap &ceMap) {
     // we compute the number of permutations (numComb) and a
     // binary code (v) which indicates which of the atom indices in
     // aiVec will be octet-unsatisfied for each permutation
-    ResonanceUtils::getNumCombStartV(numCand, rdcast<unsigned int>(aiVec.size()), numComb, v);
+    ResonanceUtils::getNumCombStartV(
+        numCand, rdcast<unsigned int>(aiVec.size()), numComb, v);
     // if there are multiple permutations, make a copy of the original
     // ConjElectrons object, since the latter will be modified
-    ConjElectrons *ceCopy = ((numComb > 1) ? new ConjElectrons(*ce) : NULL);
+    ConjElectrons *ceCopy = ((numComb > 1) ? new ConjElectrons(*ce) : nullptr);
     // enumerate all permutations
     for (unsigned int c = 0; c < numComb; ++c) {
       if (c) ce = new ConjElectrons(*ceCopy);
       unsigned int vc = v;
-      for (unsigned int i = 0; i < aiVec.size(); ++i) {
-        AtomElectrons *ae = ce->getAtomElectronsWithIdx(aiVec[i]);
+      for (unsigned int i : aiVec) {
+        AtomElectrons *ae = ce->getAtomElectronsWithIdx(i);
         unsigned int e = ae->neededNbForOctet();
         // if this atom was chosen to be octet-unsatisfied in
         // this permutation, give it one electron pair less than
@@ -797,6 +808,8 @@ void ConjElectrons::computeMetrics() {
     d_ceMetrics.d_nbMissing += it->second->neededNbForOctet();
   }
   computeDistFormalCharges();
+  computeSumFormalChargeIdxs();
+  computeSumMultipleBondIdxs();
 }
 
 // compute sum of shortest path distances between all pairs of
@@ -805,12 +818,11 @@ void ConjElectrons::computeDistFormalCharges() {
   for (ConjAtomMap::const_iterator it1 = d_conjAtomMap.begin();
        it1 != d_conjAtomMap.end(); ++it1) {
     if (!it1->second->fc()) continue;
-    for (ConjAtomMap::const_iterator it2 = it1; it2 != d_conjAtomMap.end();
-         ++it2) {
+    for (auto it2 = it1; it2 != d_conjAtomMap.end(); ++it2) {
       if ((it1 == it2) || !it2->second->fc()) continue;
       unsigned int dist = rdcast<unsigned int>(
-          MolOps::getShortestPath(d_parent->mol(), it1->first,
-                                                   it2->first).size());
+          MolOps::getShortestPath(d_parent->mol(), it1->first, it2->first)
+              .size());
       if ((it1->second->fc() * it2->second->fc()) > 0)
         d_ceMetrics.d_fcSameSignDist += dist;
       else
@@ -819,26 +831,20 @@ void ConjElectrons::computeDistFormalCharges() {
   }
 }
 
-// return the lowest index of atom bearing a formal charge
-unsigned int ConjElectrons::lowestFcIndex() const {
-  unsigned int na = d_parent->mol().getNumAtoms();
-  unsigned int ai = na;
+// compute the sum of indices of atoms bearing a formal charge
+void ConjElectrons::computeSumFormalChargeIdxs() {
   for (ConjAtomMap::const_iterator it = d_conjAtomMap.begin();
-       (ai == na) && (it != d_conjAtomMap.end()); ++it) {
-    if (it->second->fc()) ai = it->first;
+       it != d_conjAtomMap.end(); ++it) {
+    if (it->second->fc()) d_ceMetrics.d_sumFormalChargeIdxs += it->first;
   }
-  return ai;
 }
 
-// return the lowest index of a multiple bond
-unsigned int ConjElectrons::lowestMultipleBondIndex() const {
-  unsigned int nb = d_parent->mol().getNumBonds();
-  unsigned int bi = nb;
+// compute the sum of indices of multiple bonds
+void ConjElectrons::computeSumMultipleBondIdxs() {
   for (ConjBondMap::const_iterator it = d_conjBondMap.begin();
-       (bi == nb) && (it != d_conjBondMap.end()); ++it) {
-    if (it->second->order() > 1) bi = it->first;
+       it != d_conjBondMap.end(); ++it) {
+    if (it->second->order() > 1) d_ceMetrics.d_sumMultipleBondIdxs += it->first;
   }
-  return bi;
 }
 
 // decrement the count of current electrons by d
@@ -880,8 +886,8 @@ bool ConjElectrons::popFromBeginStack(unsigned int &ai) {
 // 3) Number of formal charges weighted by atom electronegativity
 // 4) Distance between formal charges with the same sign
 // 5) Distance between formal charges with opposite signs
-// 6) Index of the first atom bearing a formal charge
-// 7) Index of the first multiple bond
+// 6) Sum of the indices of atoms bearing a formal charge
+// 7) Sum of the indices of multiple bonds
 bool CEVect2::resonanceStructureCompare(const ConjElectrons *a,
                                         const ConjElectrons *b) {
   return (
@@ -895,11 +901,12 @@ bool CEVect2::resonanceStructureCompare(const ConjElectrons *a,
                             ? (a->fcSameSignDist() > b->fcSameSignDist())
                             : (a->fcOppSignDist() != b->fcOppSignDist())
                                   ? (a->fcOppSignDist() > b->fcOppSignDist())
-                                  : (a->lowestFcIndex() != b->lowestFcIndex())
-                                        ? (a->lowestFcIndex() <
-                                           b->lowestFcIndex())
-                                        : (a->lowestMultipleBondIndex() <
-                                           b->lowestMultipleBondIndex()));
+                                  : (a->sumFormalChargeIdxs() !=
+                                     b->sumFormalChargeIdxs())
+                                        ? (a->sumFormalChargeIdxs() <
+                                           b->sumFormalChargeIdxs())
+                                        : (a->sumMultipleBondIdxs() <
+                                           b->sumMultipleBondIdxs()));
 }
 
 CEVect2::CEVect2(CEMap &ceMap) {
@@ -912,11 +919,11 @@ CEVect2::CEVect2(CEMap &ceMap) {
   for (CEVect::const_iterator it = d_ceVect.begin(); it != d_ceVect.end();
        ++it) {
     if (first || ((*it)->metrics() != metricsPrev)) {
+      first = false;
       metricsPrev = (*it)->metrics();
       d_degVect.push_back(1);
     } else
       ++d_degVect.back();
-    first = false;
   }
 }
 
@@ -1147,11 +1154,17 @@ void ResonanceMolSupplier::enumerate() {
   if (d_numThreads == 1) mainLoop(0, 1);
 #ifdef RDK_THREADSAFE_SSS
   else {
-    boost::thread_group tg;
-    for (unsigned int ti = 0; ti < d_numThreads; ++ti)
-      tg.add_thread(new boost::thread(boost::bind(
-          &ResonanceMolSupplier::mainLoop, this, ti, d_numThreads)));
-    tg.join_all();
+    std::vector<std::future<void>> tg;
+    auto functor = [this](unsigned int ti, unsigned int d_numThreads) -> void {
+      mainLoop(ti, d_numThreads);
+    };
+    for (unsigned int ti = 0; ti < d_numThreads; ++ti) {
+      tg.emplace_back(
+          std::async(std::launch::async, functor, ti, d_numThreads));
+    }
+    for (auto &fut : tg) {
+      fut.get();
+    }
   }
 #endif
   setResonanceMolSupplierLength();
@@ -1214,8 +1227,7 @@ void ResonanceMolSupplier::assignConjGrpIdx() {
 // objects are collected in ceMap
 void ResonanceMolSupplier::enumerateNbArrangements(CEMap &ceMap,
                                                    CEMap &ceMapTmp) {
-  for (CEMap::iterator it = ceMapTmp.begin(); it != ceMapTmp.end(); ++it)
-    it->second->enumerateNonBonded(ceMap);
+  for (auto &it : ceMapTmp) it.second->enumerateNonBonded(ceMap);
 }
 
 void ResonanceMolSupplier::pruneStructures(CEMap &ceMap) {
@@ -1290,8 +1302,8 @@ void ResonanceMolSupplier::buildCEMap(CEMap &ceMap, unsigned int conjGrpIdx) {
   //    start from to complete the bond arrangement for each
   //    ConjElectrons object
   std::stack<ConjElectrons *> ceStack;
-  ConjElectrons *ce = new ConjElectrons(this, conjGrpIdx);
-  ConjElectrons *ceCopy = new ConjElectrons(*ce);
+  auto *ce = new ConjElectrons(this, conjGrpIdx);
+  auto *ceCopy = new ConjElectrons(*ce);
   // the first ConjElectrons object has the user-supplied bond
   // and formal charge arrangement and is stored as such
   ce->initCeFromMol();
@@ -1325,16 +1337,17 @@ void ResonanceMolSupplier::buildCEMap(CEMap &ceMap, unsigned int conjGrpIdx) {
     while (ce && ce->popFromBeginStack(aiBegin)) {
       // aiBegin holds the atom index just popped from stack
       unsigned int ai[2] = {aiBegin, na};
-      AtomElectrons *ae[2] = {ce->getAtomElectronsWithIdx(ai[BEGIN_POS]), NULL};
+      AtomElectrons *ae[2] = {ce->getAtomElectronsWithIdx(ai[BEGIN_POS]),
+                              nullptr};
       unsigned int bi = nb;
-      BondElectrons *be = NULL;
+      BondElectrons *be = nullptr;
       // loop over neighbors of the atom popped from the
       // atom index stack
       ROMol::ADJ_ITER nbrIdx, endNbrs;
       boost::tie(nbrIdx, endNbrs) =
           d_mol->getAtomNeighbors(ae[BEGIN_POS]->atom());
       for (; nbrIdx != endNbrs; ++nbrIdx) {
-        unsigned int aiNbr = (*d_mol)[*nbrIdx].get()->getIdx();
+        unsigned int aiNbr = (*d_mol)[*nbrIdx]->getIdx();
         // if this neighbor is not part of the conjugated group,
         // ignore it
         if (ce->parent()->getAtomConjGrpIdx(aiNbr) !=
@@ -1387,7 +1400,7 @@ void ResonanceMolSupplier::buildCEMap(CEMap &ceMap, unsigned int conjGrpIdx) {
       // consider single, double and triple bond alternatives in turn
       for (unsigned int i = 0; i < 3; ++i) {
         unsigned int t = i * 2;
-        ConjElectrons *ceToSet = NULL;
+        ConjElectrons *ceToSet = nullptr;
         boost::uint8_t orderMask = (1 << t);
         boost::uint8_t chgMask = (1 << (t + 1));
         unsigned int bo = i + 1;
@@ -1407,7 +1420,7 @@ void ResonanceMolSupplier::buildCEMap(CEMap &ceMap, unsigned int conjGrpIdx) {
             needToFork = true;
             ceToSet = ce;
           } else {
-            ConjElectrons *ceFork = new ConjElectrons(*ceCopy);
+            auto *ceFork = new ConjElectrons(*ceCopy);
             ceStack.push(ceFork);
             ceToSet = ceFork;
           }
@@ -1425,7 +1438,7 @@ void ResonanceMolSupplier::buildCEMap(CEMap &ceMap, unsigned int conjGrpIdx) {
       // if a dead end was hit, discard this ConjElectrons object
       if (!isAnyBondAllowed) {
         delete ce;
-        ce = NULL;
+        ce = nullptr;
       }
     }
     if (ce) {
@@ -1467,7 +1480,7 @@ int ResonanceMolSupplier::getAtomConjGrpIdx(unsigned int ai) const {
 
 // resizes d_ceVect3 vector
 inline void ResonanceMolSupplier::resizeCeVect() {
-  d_ceVect3.resize(d_nConjGrp, NULL);
+  d_ceVect3.resize(d_nConjGrp, nullptr);
 }
 
 // stores the ConjElectrons pointers currently stored in ceMap
@@ -1511,7 +1524,7 @@ bool ResonanceMolSupplier::atEnd() {
 // the pointer
 ROMol *ResonanceMolSupplier::next() {
   enumerate();
-  return (atEnd() ? NULL : (*this)[d_idx++]);
+  return (atEnd() ? nullptr : (*this)[d_idx++]);
 }
 
 // sets the ResonanceMolSupplier index to idx
@@ -1558,7 +1571,7 @@ void ResonanceMolSupplier::assignBondsFormalChargesHelper(
 // with the c vector
 ROMol *ResonanceMolSupplier::assignBondsFormalCharges(
     std::vector<unsigned int> &c) const {
-  ROMol *mol = new ROMol(this->mol());
+  auto *mol = new ROMol(this->mol());
   assignBondsFormalChargesHelper(*mol, c);
   ResonanceUtils::fixExplicitImplicitHs(*mol);
   ResonanceUtils::sanitizeMol((RWMol &)*mol);
