@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2002-2012 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2002-2016 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -13,9 +12,12 @@
 #include <RDGeneral/types.h>
 #include <GraphMol/Canon.h>
 #include <GraphMol/new_canon.h>
+#include <RDGeneral/BoostStartInclude.h>
 #include <boost/lexical_cast.hpp>
 #include <boost/foreach.hpp>
 #include <boost/dynamic_bitset.hpp>
+#include <RDGeneral/BoostEndInclude.h>
+
 #include <sstream>
 #include <map>
 #include <list>
@@ -25,7 +27,7 @@
 namespace RDKit {
 
 namespace SmilesWrite {
-const int atomicSmiles[] = {5, 6, 7, 8, 9, 15, 16, 17, 35, 53, -1};
+const int atomicSmiles[] = {0, 5, 6, 7, 8, 9, 15, 16, 17, 35, 53, -1};
 bool inOrganicSubset(int atomicNumber) {
   unsigned int idx = 0;
   while (atomicSmiles[idx] < atomicNumber && atomicSmiles[idx] != -1) {
@@ -38,7 +40,7 @@ bool inOrganicSubset(int atomicNumber) {
 }
 
 std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
-                          bool allHsExplicit) {
+                          bool allHsExplicit, bool isomericSmiles) {
   RDUNUSED_PARAM(bondIn);
   PRECONDITION(atom, "bad atom");
   INT_VECT atomicSmilesVect(
@@ -51,15 +53,33 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
 
   bool needsBracket = false;
   std::string symb;
-  if (!atom->getPropIfPresent(common_properties::smilesSymbol, symb)) {
+  bool hasCustomSymbol =
+      atom->getPropIfPresent(common_properties::smilesSymbol, symb);
+  if (!hasCustomSymbol) {
     symb = PeriodicTable::getTable()->getElementSymbol(num);
   }
-  // symb = atom->getSymbol();
+
+  // check for atomic stereochemistry
   std::string atString = "";
+  if (isomericSmiles ||
+      atom->getOwningMol().hasProp(common_properties::_doIsoSmiles)) {
+    if (atom->getChiralTag() != Atom::CHI_UNSPECIFIED &&
+        !atom->hasProp(common_properties::_brokenChirality)) {
+      switch (atom->getChiralTag()) {
+        case Atom::CHI_TETRAHEDRAL_CW:
+          atString = "@@";
+          break;
+        case Atom::CHI_TETRAHEDRAL_CCW:
+          atString = "@";
+          break;
+        default:
+          break;
+      }
+    }
+  }
+
   if (!allHsExplicit && inOrganicSubset(num)) {
     // it's a member of the organic subset
-    // if(!doKekule && atom->getIsAromatic() && symb[0] < 'a') symb[0] -=
-    // ('A'-'a');
 
     // -----
     // figure out if we need to put a bracket around the atom,
@@ -73,7 +93,7 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
     int totalValence = atom->getTotalValence();
     bool nonStandard = false;
 
-    if (atom->getNumRadicalElectrons()) {
+    if (hasCustomSymbol || atom->getNumRadicalElectrons()) {
       nonStandard = true;
     } else if ((num == 7 || num == 15) && atom->getIsAromatic() &&
                atom->getNumExplicitHs()) {
@@ -85,28 +105,12 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
           (totalValence != defaultVs.front() && atom->getTotalNumHs());
     }
 
-    if (fc || nonStandard) {
+    if (fc || nonStandard ||
+        atom->hasProp(common_properties::molAtomMapNumber)) {
       needsBracket = true;
-    }
-    if (atom->getOwningMol().hasProp(common_properties::_doIsoSmiles)) {
-      if (atom->getChiralTag() != Atom::CHI_UNSPECIFIED &&
-          !atom->hasProp(common_properties::_brokenChirality)) {
-        switch (atom->getChiralTag()) {
-          case Atom::CHI_TETRAHEDRAL_CW:
-            atString = "@@";
-            break;
-          case Atom::CHI_TETRAHEDRAL_CCW:
-            atString = "@";
-            break;
-          default:
-            break;
-        }
-        needsBracket = true;
-      } else if (isotope) {
-        needsBracket = true;
-      }
-    }
-    if (atom->hasProp(common_properties::molAtomMapNumber)) {
+    } else if ((isomericSmiles || atom->getOwningMol().hasProp(
+                                      common_properties::_doIsoSmiles)) &&
+               (isotope || atString != "")) {
       needsBracket = true;
     }
   } else {
@@ -114,9 +118,9 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
   }
   if (needsBracket) res += "[";
 
-  if (isotope &&
-      atom->getOwningMol().hasProp(common_properties::_doIsoSmiles)) {
-    res += boost::lexical_cast<std::string>(isotope);
+  if (isotope && (isomericSmiles || atom->getOwningMol().hasProp(
+                                        common_properties::_doIsoSmiles))) {
+    res += std::to_string(isotope);
   }
   // this was originally only done for the organic subset,
   // applying it to other atom-types is a fix for Issue 3152751:
@@ -131,14 +135,14 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
     unsigned int totNumHs = atom->getTotalNumHs();
     if (totNumHs > 0) {
       res += "H";
-      if (totNumHs > 1) res += boost::lexical_cast<std::string>(totNumHs);
+      if (totNumHs > 1) res += std::to_string(totNumHs);
     }
     if (fc > 0) {
       res += "+";
-      if (fc > 1) res += boost::lexical_cast<std::string>(fc);
+      if (fc > 1) res += std::to_string(fc);
     } else if (fc < 0) {
       if (fc < -1)
-        res += boost::lexical_cast<std::string>(fc);
+        res += std::to_string(fc);
       else
         res += "-";
     }
@@ -146,7 +150,7 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule, const Bond *bondIn,
     int mapNum;
     if (atom->getPropIfPresent(common_properties::molAtomMapNumber, mapNum)) {
       res += ":";
-      res += boost::lexical_cast<std::string>(mapNum);
+      res += std::to_string(mapNum);
     }
     res += "]";
   }
@@ -196,14 +200,17 @@ std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx, bool doKekule,
       if (dir != Bond::NONE && dir != Bond::UNKNOWN) {
         switch (dir) {
           case Bond::ENDDOWNRIGHT:
-            if (bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
+            if (allBondsExplicit ||
+                bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
               res = "\\";
             break;
           case Bond::ENDUPRIGHT:
-            if (bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
+            if (allBondsExplicit ||
+                bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
               res = "/";
             break;
           default:
+            if (allBondsExplicit) res = "-";
             break;
         }
       } else {
@@ -221,7 +228,7 @@ std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx, bool doKekule,
       break;
     case Bond::DOUBLE:
       // see note above
-      if (!aromatic || !bond->getIsAromatic()) res = "=";
+      if (!aromatic || !bond->getIsAromatic() || allBondsExplicit) res = "=";
       break;
     case Bond::TRIPLE:
       res = "#";
@@ -230,14 +237,17 @@ std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx, bool doKekule,
       if (dir != Bond::NONE && dir != Bond::UNKNOWN) {
         switch (dir) {
           case Bond::ENDDOWNRIGHT:
-            if (bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
+            if (allBondsExplicit ||
+                bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
               res = "\\";
             break;
           case Bond::ENDUPRIGHT:
-            if (bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
+            if (allBondsExplicit ||
+                bond->getOwningMol().hasProp(common_properties::_doIsoSmiles))
               res = "/";
             break;
           default:
+            if (allBondsExplicit || !aromatic) res = ":";
             break;
         }
       } else if (allBondsExplicit || !aromatic) {
@@ -247,9 +257,9 @@ std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx, bool doKekule,
     case Bond::DATIVE:
       if (atomToLeftIdx >= 0 &&
           bond->getBeginAtomIdx() == static_cast<unsigned int>(atomToLeftIdx))
-        res = ">";
+        res = "->";
       else
-        res = "<";
+        res = "<-";
       break;
     default:
       res = "~";
@@ -261,10 +271,10 @@ std::string FragmentSmilesConstruct(
     ROMol &mol, int atomIdx, std::vector<Canon::AtomColors> &colors,
     const UINT_VECT &ranks, bool doKekule, bool canonical,
     bool doIsomericSmiles, bool allBondsExplicit, bool allHsExplicit,
-    std::vector<unsigned int> &atomOrdering,
-    const boost::dynamic_bitset<> *bondsInPlay = 0,
-    const std::vector<std::string> *atomSymbols = 0,
-    const std::vector<std::string> *bondSymbols = 0) {
+    bool doRandom, std::vector<unsigned int> &atomOrdering,
+    const boost::dynamic_bitset<> *bondsInPlay = nullptr,
+    const std::vector<std::string> *atomSymbols = nullptr,
+    const std::vector<std::string> *bondSymbols = nullptr) {
   PRECONDITION(!bondsInPlay || bondsInPlay->size() >= mol.getNumBonds(),
                "bad bondsInPlay");
   PRECONDITION(!atomSymbols || atomSymbols->size() >= mol.getNumAtoms(),
@@ -283,8 +293,8 @@ std::string FragmentSmilesConstruct(
   std::list<unsigned int> ringClosuresToErase;
 
   Canon::canonicalizeFragment(mol, atomIdx, colors, ranks, molStack,
-                              bondsInPlay, bondSymbols, doIsomericSmiles);
-  Bond *bond = 0;
+                              bondsInPlay, bondSymbols, doIsomericSmiles, doRandom);
+  Bond *bond = nullptr;
   BOOST_FOREACH (Canon::MolStackElem mSE, molStack) {
     switch (mSE.type) {
       case Canon::MOL_STACK_ATOM:
@@ -296,7 +306,8 @@ std::string FragmentSmilesConstruct(
         }
         // std::cout<<"\t\tAtom: "<<mSE.obj.atom->getIdx()<<std::endl;
         if (!atomSymbols) {
-          res << GetAtomSmiles(mSE.obj.atom, doKekule, bond, allHsExplicit);
+          res << GetAtomSmiles(mSE.obj.atom, doKekule, bond, allHsExplicit,
+                               doIsomericSmiles);
         } else {
           res << (*atomSymbols)[mSE.obj.atom->getIdx()];
         }
@@ -341,11 +352,12 @@ std::string FragmentSmilesConstruct(
           }
           ringClosureMap[ringIdx] = closureVal;
         }
-        if (closureVal >= 10) {
-          res << "%";
-        }
-        // std::cerr << " > " << closureVal <<std::endl;
-        res << closureVal;
+        if (closureVal < 10)
+          res << (char)(closureVal + '0');
+        else if (closureVal < 100)
+          res << '%' << closureVal;
+        else  // use extension to OpenSMILES
+          res << "%(" << closureVal << ')';
         break;
       case Canon::MOL_STACK_BRANCH_OPEN:
         res << "(";
@@ -363,22 +375,22 @@ std::string FragmentSmilesConstruct(
 }  // end of namespace SmilesWrite
 
 static bool SortBasedOnFirstElement(
-    const std::pair<std::string, std::vector<unsigned int> > &a,
-    const std::pair<std::string, std::vector<unsigned int> > &b) {
+    const std::pair<std::string, std::vector<unsigned int>> &a,
+    const std::pair<std::string, std::vector<unsigned int>> &b) {
   return a.first < b.first;
 }
 
 std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
                         int rootedAtAtom, bool canonical, bool allBondsExplicit,
-                        bool allHsExplicit) {
+                        bool allHsExplicit, bool doRandom) {
   if (!mol.getNumAtoms()) return "";
   PRECONDITION(rootedAtAtom < 0 ||
                    static_cast<unsigned int>(rootedAtAtom) < mol.getNumAtoms(),
                "rootedAtomAtom must be less than the number of atoms");
 
-  std::vector<std::vector<int> > fragsMolAtomMapping;
+  std::vector<std::vector<int>> fragsMolAtomMapping;
   std::vector<ROMOL_SPTR> mols =
-      MolOps::getMolFrags(mol, false, NULL, &fragsMolAtomMapping, false);
+      MolOps::getMolFrags(mol, false, nullptr, &fragsMolAtomMapping, false);
   std::vector<std::string> vfragsmi;
 
   //    for(unsigned i=0; i<fragsMolAtomMapping.size(); i++){
@@ -389,7 +401,7 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
   //      std::cout << std::endl;
   //    }
 
-  std::vector<std::vector<RDKit::UINT> > allAtomOrdering;
+  std::vector<std::vector<RDKit::UINT>> allAtomOrdering;
   for (unsigned i = 0; i < mols.size(); i++) {
     ROMol *tmol = mols[i].get();
 
@@ -413,6 +425,18 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
       tmol->debugMol(std::cout);
       std::cout << "----------------------------" << std::endl;
 #endif
+    
+    // adding randomness without setting the rootedAtAtom
+    if (doRandom ) {
+      if (rootedAtAtom == -1) {
+        rootedAtAtom = std::rand() % mol.getNumAtoms(); 
+        // need to find an atom id between 0 and mol.getNumAtoms() exclusively
+        PRECONDITION(rootedAtAtom < 0 ||
+                   static_cast<unsigned int>(rootedAtAtom) < mol.getNumAtoms(),
+               "rootedAtomAtom must be less than the number of atoms");
+      }
+    }
+
 
     std::string res;
     unsigned int nAtoms = tmol->getNumAtoms();
@@ -437,7 +461,7 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
 #ifdef VERBOSE_CANON
     for (unsigned int tmpI = 0; tmpI < ranks.size(); tmpI++) {
       std::cout << tmpI << " " << ranks[tmpI] << " "
-                << *(tmol.getAtomWithIdx(tmpI)) << std::endl;
+                << *(tmol->getAtomWithIdx(tmpI)) << std::endl;
     }
 #endif
 
@@ -467,7 +491,7 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
 
       subSmi = SmilesWrite::FragmentSmilesConstruct(
           *tmol, nextAtomIdx, colors, ranks, doKekule, canonical,
-          doIsomericSmiles, allBondsExplicit, allHsExplicit, atomOrdering);
+          doIsomericSmiles, allBondsExplicit, allHsExplicit, doRandom ,  atomOrdering);
 
       res += subSmi;
       colorIt = std::find(colors.begin(), colors.end(), Canon::WHITE_NODE);
@@ -477,10 +501,9 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
     }
     vfragsmi.push_back(res);
 
-    for (std::vector<RDKit::UINT>::iterator vit = atomOrdering.begin();
-         vit != atomOrdering.end(); ++vit) {
-      *vit = fragsMolAtomMapping[i][*vit];  // Lookup the Id in the original
-                                            // molecule
+    for (unsigned int &vit : atomOrdering) {
+      vit = fragsMolAtomMapping[i][vit];  // Lookup the Id in the original
+                                          // molecule
     }
     allAtomOrdering.push_back(atomOrdering);
   }
@@ -490,7 +513,7 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
   if (canonical) {
     // Sort the vfragsmi, but also sort the atom order vectors into the same
     // order
-    typedef std::pair<std::string, std::vector<unsigned int> > PairStrAndVec;
+    typedef std::pair<std::string, std::vector<unsigned int>> PairStrAndVec;
     std::vector<PairStrAndVec> tmp(vfragsmi.size());
     for (unsigned int ti = 0; ti < vfragsmi.size(); ++ti)
       tmp[ti] = PairStrAndVec(vfragsmi[ti], allAtomOrdering[ti]);
@@ -505,10 +528,9 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles, bool doKekule,
                                    tmp[ti].second.end());
     }
   } else {  // Not canonical
-    for (unsigned int i = 0; i < allAtomOrdering.size(); ++i)
-      flattenedAtomOrdering.insert(flattenedAtomOrdering.end(),
-                                   allAtomOrdering[i].begin(),
-                                   allAtomOrdering[i].end());
+    for (auto &i : allAtomOrdering)
+      flattenedAtomOrdering.insert(flattenedAtomOrdering.end(), i.begin(),
+                                   i.end());
     for (unsigned i = 0; i < vfragsmi.size(); ++i) {
       result += vfragsmi[i];
       if (i < vfragsmi.size() - 1) {
@@ -533,10 +555,10 @@ std::string MolFragmentToSmiles(const ROMol &mol,
   PRECONDITION(rootedAtAtom < 0 ||
                    static_cast<unsigned int>(rootedAtAtom) < mol.getNumAtoms(),
                "rootedAtomAtom must be less than the number of atoms");
-  PRECONDITION(rootedAtAtom < 0 ||
-                   std::find(atomsToUse.begin(), atomsToUse.end(),
-                             rootedAtAtom) != atomsToUse.end(),
-               "rootedAtomAtom not found in atomsToUse");
+  PRECONDITION(
+      rootedAtAtom < 0 || std::find(atomsToUse.begin(), atomsToUse.end(),
+                                    rootedAtAtom) != atomsToUse.end(),
+      "rootedAtomAtom not found in atomsToUse");
   PRECONDITION(!atomSymbols || atomSymbols->size() >= mol.getNumAtoms(),
                "bad atomSymbols vector");
   PRECONDITION(!bondSymbols || bondSymbols->size() >= mol.getNumBonds(),
@@ -560,7 +582,7 @@ std::string MolFragmentToSmiles(const ROMol &mol,
       ROMol::OEDGE_ITER beg, end;
       boost::tie(beg, end) = mol.getAtomBonds(mol.getAtomWithIdx(aidx));
       while (beg != end) {
-        const BOND_SPTR bond = mol[*beg];
+        const Bond *bond = mol[*beg];
         if (atomsInPlay[bond->getOtherAtomIdx(aidx)])
           bondsInPlay.set(bond->getIdx());
         ++beg;
@@ -627,6 +649,10 @@ std::string MolFragmentToSmiles(const ROMol &mol,
   if (canonical) {
     Canon::rankFragmentAtoms(tmol, ranks, atomsInPlay, bondsInPlay, atomSymbols,
                              true, doIsomericSmiles, doIsomericSmiles);
+    // std::cerr << "RANKS: ";
+    // std::copy(ranks.begin(), ranks.end(),
+    //           std::ostream_iterator<int>(std::cerr, " "));
+    // std::cerr << std::endl;
     // MolOps::rankAtomsInFragment(tmol,ranks,atomsInPlay,bondsInPlay,atomSymbols,bondSymbols);
   } else {
     for (unsigned int i = 0; i < tmol.getNumAtoms(); ++i) ranks[i] = i;
@@ -665,7 +691,7 @@ std::string MolFragmentToSmiles(const ROMol &mol,
 
     subSmi = SmilesWrite::FragmentSmilesConstruct(
         tmol, nextAtomIdx, colors, ranks, doKekule, canonical, doIsomericSmiles,
-        allBondsExplicit, allHsExplicit, atomOrdering, &bondsInPlay,
+        allBondsExplicit, allHsExplicit, false, atomOrdering, &bondsInPlay,
         atomSymbols, bondSymbols);
 
     res += subSmi;
@@ -677,4 +703,4 @@ std::string MolFragmentToSmiles(const ROMol &mol,
   mol.setProp(common_properties::_smilesAtomOutputOrder, atomOrdering, true);
   return res;
 }  // end of MolFragmentToSmiles()
-}
+}  // namespace RDKit
