@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2002-2012 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2002-2017 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -7,11 +7,13 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
+#include <RDGeneral/export.h>
 #ifndef _RD_SMILESWRITE_H
 #define _RD_SMILESWRITE_H
 
 #include <string>
 #include <vector>
+#include <memory>
 
 namespace RDKit {
 class Atom;
@@ -19,7 +21,7 @@ class Bond;
 class ROMol;
 namespace SmilesWrite {
 //! \brief returns true if the atom number is in the SMILES organic subset
-bool inOrganicSubset(int atomicNumber);
+RDKIT_SMILESPARSE_EXPORT bool inOrganicSubset(int atomicNumber);
 
 //! \brief returns the SMILES for an atom
 /*!
@@ -30,9 +32,13 @@ bool inOrganicSubset(int atomicNumber);
     chirality calculation
   \param allHsExplicit : if true, hydrogen counts will be provided for every
   atom.
+  \param isomericSmiles : if true, isomeric SMILES will be generated
 */
-std::string GetAtomSmiles(const Atom *atom, bool doKekule = false,
-                          const Bond *bondIn = 0, bool allHsExplicit = false);
+RDKIT_SMILESPARSE_EXPORT std::string GetAtomSmiles(const Atom *atom,
+                                                   bool doKekule = false,
+                                                   const Bond *bondIn = 0,
+                                                   bool allHsExplicit = false,
+                                                   bool isomericSmiles = true);
 
 //! \brief returns the SMILES for a bond
 /*!
@@ -43,9 +49,10 @@ std::string GetAtomSmiles(const Atom *atom, bool doKekule = false,
     bond orders for aromatic bonds)
   \param allBondsExplicit : if true, symbols will be included for all bonds.
 */
-std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx = -1,
-                          bool doKekule = false, bool allBondsExplicit = false);
-}
+RDKIT_SMILESPARSE_EXPORT std::string GetBondSmiles(
+    const Bond *bond, int atomToLeftIdx = -1, bool doKekule = false,
+    bool allBondsExplicit = false);
+}  // namespace SmilesWrite
 
 //! \brief returns canonical SMILES for a molecule
 /*!
@@ -61,10 +68,10 @@ std::string GetBondSmiles(const Bond *bond, int atomToLeftIdx = -1,
   \param allHsExplicit : if true, hydrogen counts will be provided for every
   atom.
  */
-std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles = false,
-                        bool doKekule = false, int rootedAtAtom = -1,
-                        bool canonical = true, bool allBondsExplicit = false,
-                        bool allHsExplicit = false);
+RDKIT_SMILESPARSE_EXPORT std::string MolToSmiles(
+    const ROMol &mol, bool doIsomericSmiles = true, bool doKekule = false,
+    int rootedAtAtom = -1, bool canonical = true, bool allBondsExplicit = false,
+    bool allHsExplicit = false, bool doRandom = false);
 
 //! \brief returns canonical SMILES for part of a molecule
 /*!
@@ -89,15 +96,13 @@ std::string MolToSmiles(const ROMol &mol, bool doIsomericSmiles = false,
   \b NOTE: the bondSymbols are *not* currently used in the canonicalization.
 
  */
-std::string MolFragmentToSmiles(const ROMol &mol,
-                                const std::vector<int> &atomsToUse,
-                                const std::vector<int> *bondsToUse = 0,
-                                const std::vector<std::string> *atomSymbols = 0,
-                                const std::vector<std::string> *bondSymbols = 0,
-                                bool doIsomericSmiles = false,
-                                bool doKekule = false, int rootedAtAtom = -1,
-                                bool canonical = true,
-                                bool allBondsExplicit = false,
-                                bool allHsExplicit = false);
-}
+RDKIT_SMILESPARSE_EXPORT std::string MolFragmentToSmiles(
+    const ROMol &mol, const std::vector<int> &atomsToUse,
+    const std::vector<int> *bondsToUse = 0,
+    const std::vector<std::string> *atomSymbols = 0,
+    const std::vector<std::string> *bondSymbols = 0,
+    bool doIsomericSmiles = true, bool doKekule = false, int rootedAtAtom = -1,
+    bool canonical = true, bool allBondsExplicit = false,
+    bool allHsExplicit = false);
+}  // namespace RDKit
 #endif
