@@ -1,6 +1,5 @@
-// $Id$
 //
-//  Copyright (C) 2003-2013 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2003-2017 Greg Landrum and Rational Discovery LLC
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -28,27 +27,6 @@
 
 namespace python = boost::python;
 namespace RDKit {
-namespace {
-std::string qhelper(Atom::QUERYATOM_QUERY *q, unsigned int depth) {
-  std::string res = "";
-  if (q) {
-    for (unsigned int i = 0; i < depth; ++i) res += "  ";
-    res += q->getFullDescription() + "\n";
-    for (Atom::QUERYATOM_QUERY::CHILD_VECT_CI ci = q->beginChildren();
-         ci != q->endChildren(); ++ci) {
-      res += qhelper((*ci).get(), depth + 1);
-    }
-  }
-  return res;
-}
-}  // end of local namespace
-std::string describeQuery(const Atom *atom) {
-  std::string res = "";
-  if (atom->hasQuery()) {
-    res = qhelper(atom->getQuery(), 0);
-  }
-  return res;
-}
 void expandQuery(QueryAtom *self, const QueryAtom *other,
                  Queries::CompositeQueryType how, bool maintainOrder) {
   if (other->hasQuery()) {
@@ -57,6 +35,12 @@ void expandQuery(QueryAtom *self, const QueryAtom *other,
   }
 }
 
+void setQuery(QueryAtom *self, const QueryAtom *other) {
+  if (other->hasQuery()) {
+    self->setQuery(other->getQuery()->copy());
+  }
+}
+
 template <class T>
 void AtomSetProp(const Atom *atom, const char *key, const T &val) {
   // std::cerr<<"asp: "<<atom<<" " << key<<" - " << val << std::endl;
@@ -94,7 +78,7 @@ python::tuple AtomGetBonds(Atom *atom) {
   ROMol::OEDGE_ITER begin, end;
   boost::tie(begin, end) = parent->getAtomBonds(atom);
   while (begin != end) {
-    Bond *tmpB = (*parent)[*begin].get();
+    const Bond *tmpB = (*parent)[*begin];
     res.append(python::ptr(tmpB));
     begin++;
   }
@@ -115,12 +99,15 @@ bool AtomIsInRingSize(const Atom *atom, int size) {
                                                                 size);
 }
 
-std::string AtomGetSmarts(const Atom *atom) {
+std::string AtomGetSmarts(const Atom *atom, bool doKekule, bool allHsExplicit,
+                          bool isomericSmiles) {
   std::string res;
   if (atom->hasQuery()) {
     res = SmartsWrite::GetAtomSmarts(static_cast<const QueryAtom *>(atom));
   } else {
-    res = SmilesWrite::GetAtomSmiles(atom);
+    // FIX: this should not be necessary
+    res = SmilesWrite::GetAtomSmiles(atom, doKekule, nullptr, allHsExplicit,
+                                     isomericSmiles);
   }
   return res;
 }
@@ -134,13 +121,16 @@ AtomMonomerInfo *AtomGetMonomerInfo(Atom *atom) {
 }
 AtomPDBResidueInfo *AtomGetPDBResidueInfo(Atom *atom) {
   AtomMonomerInfo *res = atom->getMonomerInfo();
-  if (!res) return NULL;
+  if (!res) return nullptr;
   if (res->getMonomerType() != AtomMonomerInfo::PDBRESIDUE) {
     throw_value_error("MonomerInfo is not a PDB Residue");
   }
   return (AtomPDBResidueInfo *)res;
 }
 
+struct MDLDummy {};
+struct DaylightDummy {};
+
 // FIX: is there any reason at all to not just prevent the construction of
 // Atoms?
 std::string atomClassDoc =
@@ -191,7 +181,7 @@ struct atom_wrapper {
              "Returns the total number of implicit Hs on the atom.\n")
 
         .def("GetExplicitValence", &Atom::getExplicitValence,
-             "Returns the number of explicit Hs on the atom.\n")
+             "Returns the explicit valence of the atom.\n")
         .def("GetImplicitValence", &Atom::getImplicitValence,
              "Returns the number of implicit Hs on the atom.\n")
         .def("GetTotalValence", &Atom::getTotalValence,
@@ -230,7 +220,7 @@ struct atom_wrapper {
 
         .def("GetOwningMol", &Atom::getOwningMol,
              "Returns the Mol that owns this atom.\n",
-             python::return_value_policy<python::reference_existing_object>())
+             python::return_internal_reference<>())
 
         .def("GetNeighbors", AtomGetNeighbors,
              "Returns a read-only sequence of the atom's neighbors\n")
@@ -262,6 +252,9 @@ struct atom_wrapper {
              "debugging purposes.\n\n")
 
         .def("GetSmarts", AtomGetSmarts,
+             (python::arg("self"), python::arg("doKekule") = false,
+              python::arg("allHsExplicit") = false,
+              python::arg("isomericSmiles") = true),
              "returns the SMARTS (or SMILES) string for an Atom\n\n")
 
         // properties
@@ -292,9 +285,10 @@ struct atom_wrapper {
              (python::arg("self"), python::arg("key"), python::arg("val")),
              "Sets an atomic property\n\n"
              "  ARGUMENTS:\n"
-             "    - key: the name of the property to be set (an unsigned integer).\n"
+             "    - key: the name of the property to be set (an unsigned "
+             "integer).\n"
              "    - value: the property value (a int >= 0).\n\n")
-        
+
         .def("GetIntProp", GetProp<Atom, int>,
              "Returns the value of the property.\n\n"
              "  ARGUMENTS:\n"
@@ -307,7 +301,8 @@ struct atom_wrapper {
         .def("GetUnsignedProp", GetProp<Atom, unsigned>,
              "Returns the value of the property.\n\n"
              "  ARGUMENTS:\n"
-             "    - key: the name of the property to return (an unsigned integer).\n\n"
+             "    - key: the name of the property to return (an unsigned "
+             "integer).\n\n"
              "  RETURNS: an integer (Python has no unsigned type)\n\n"
              "  NOTE:\n"
              "    - If the property has not been set, a KeyError exception "
@@ -357,13 +352,17 @@ struct atom_wrapper {
              "  ARGUMENTS:\n"
              "    - key: the name of the property to be removed.\n")
 
-        .def("GetPropNames", &Atom::getPropList, (python::arg("self")),
+        .def("GetPropNames", &Atom::getPropList,
+             (python::arg("self"), python::arg("includePrivate") = false,
+              python::arg("includeComputed") = false),
              "Returns a list of the properties set on the Atom.\n\n")
 
-        .def("GetPropsAsDict", GetPropsAsDict<Atom>, (python::arg("self")),
+        .def("GetPropsAsDict", GetPropsAsDict<Atom>,
+             (python::arg("self"), python::arg("includePrivate") = true,
+              python::arg("includeComputed") = true),
              "Returns a dictionary of the properties set on the Atom.\n"
              " n.b. some properties cannot be converted to python types.\n")
-        
+
         .def("UpdatePropertyCache", &Atom::updatePropertyCache,
              (python::arg("self"), python::arg("strict") = true),
              "Regenerates computed properties like implicit valence and ring "
@@ -376,17 +375,24 @@ struct atom_wrapper {
 
         .def("GetMonomerInfo", AtomGetMonomerInfo,
              python::return_internal_reference<
-                 1, python::with_custodian_and_ward_postcall<0, 1> >(),
+                 1, python::with_custodian_and_ward_postcall<0, 1>>(),
              "Returns the atom's MonomerInfo object, if there is one.\n\n")
         .def("GetPDBResidueInfo", AtomGetPDBResidueInfo,
              python::return_internal_reference<
-                 1, python::with_custodian_and_ward_postcall<0, 1> >(),
+                 1, python::with_custodian_and_ward_postcall<0, 1>>(),
              "Returns the atom's MonomerInfo object, if there is one.\n\n")
         .def("SetMonomerInfo", SetAtomMonomerInfo,
-             "Sets the atom's MonomerInfo object.\n\n");
+             "Sets the atom's MonomerInfo object.\n\n")
+        .def("GetAtomMapNum", &Atom::getAtomMapNum,
+             "Gets the atoms map number, returns 0 if not set")
+        .def("SetAtomMapNum", &Atom::setAtomMapNum,
+             (python::arg("self"), python::arg("mapno"),
+              python::arg("strict") = false),
+             "Sets the atoms map number, a value of 0 clears the atom map");
 
     python::enum_<Atom::HybridizationType>("HybridizationType")
         .value("UNSPECIFIED", Atom::UNSPECIFIED)
+        .value("S", Atom::S)
         .value("SP", Atom::SP)
         .value("SP2", Atom::SP2)
         .value("SP3", Atom::SP3)
@@ -411,14 +417,52 @@ struct atom_wrapper {
     atomClassDoc =
         "The class to store QueryAtoms.\n\
 These cannot currently be constructed directly from Python\n";
-    python::class_<QueryAtom, python::bases<Atom> >(
+    python::class_<QueryAtom, python::bases<Atom>>(
         "QueryAtom", atomClassDoc.c_str(), python::no_init)
         .def("ExpandQuery", expandQuery,
              (python::arg("self"), python::arg("other"),
               python::arg("how") = Queries::COMPOSITE_AND,
               python::arg("maintainOrder") = true),
-             "combines the query from other with ours");
-  };
+             "combines the query from other with ours")
+        .def("SetQuery", setQuery,
+             (python::arg("self"), python::arg("other")),
+             "Replace our query with a copy of the other query");
+
+    python::def(
+        "GetAtomRLabel", getAtomRLabel, (python::arg("atom")),
+        "Returns the atom's MDL AtomRLabel (this is an integer from 0 to 99)");
+    python::def("SetAtomRLabel", setAtomRLabel,
+                (python::arg("atom"), python::arg("rlabel")),
+                "Sets the atom's MDL RLabel (this is an integer from 0 to "
+                "99).\nSetting to 0 clears the rlabel.");
+
+    python::def("GetAtomAlias", getAtomAlias, (python::arg("atom")),
+                "Returns the atom's MDL alias text");
+    python::def("SetAtomAlias", setAtomAlias,
+                (python::arg("atom"), python::arg("rlabel")),
+                "Sets the atom's MDL alias text.\nSetting to an empty string "
+                "clears the alias.");
+    python::def("GetAtomValue", getAtomValue, (python::arg("atom")),
+                "Returns the atom's MDL alias text");
+    python::def("SetAtomValue", setAtomValue,
+                (python::arg("atom"), python::arg("rlabel")),
+                "Sets the atom's MDL alias text.\nSetting to an empty string "
+                "clears the alias.");
+
+    python::def("GetSupplementalSmilesLabel", getSupplementalSmilesLabel,
+                (python::arg("atom")),
+                "Gets the supplemental smiles label on an atom, returns an "
+                "empty string if not present.");
+    python::def(
+        "SetSupplementalSmilesLabel", setSupplementalSmilesLabel,
+        (python::arg("atom"), python::arg("label")),
+        "Sets a supplemental label on an atom that is written to the smiles "
+        "string.\n"
+        ">>> m = Chem.MolFromSmiles(\"C\")\n"
+        ">>> Chem.SetSupplementalSmilesLabel(m.GetAtomWithIdx(0), '<xxx>')\n"
+        ">>> Chem.MolToSmiles(m)\n"
+        "'C<xxx>'\n");
+  }
 };
-}  // end of namespace
+}  // namespace RDKit
 void wrap_atom() { RDKit::atom_wrapper::wrap(); }
