@@ -18,10 +18,11 @@
 
 #ifdef RDK_THREADSAFE_SSS
 // Thread local storage for output buffer for RDKit Logging
-#include <boost/thread/tss.hpp>
+#include <thread>
 #endif
 
 #include <sstream>
+#include <utility>
 
 #include "seqs.hpp"
 namespace python = boost::python;
@@ -31,12 +32,12 @@ namespace RDKit {
 void tossit() { throw IndexErrorException(1); }
 }
 
-void rdExceptionTranslator(RDKit::ConformerException const& x) {
+void rdExceptionTranslator(RDKit::ConformerException const &x) {
   RDUNUSED_PARAM(x);
   PyErr_SetString(PyExc_ValueError, "Bad Conformer Id");
 }
 
-void rdSanitExceptionTranslator(RDKit::MolSanitizeException const& x) {
+void rdSanitExceptionTranslator(RDKit::MolSanitizeException const &x) {
   std::ostringstream ss;
   ss << "Sanitization error: " << x.message();
   PyErr_SetString(PyExc_ValueError, ss.str().c_str());
@@ -51,35 +52,34 @@ void wrap_ringinfo();
 void wrap_EditableMol();
 void wrap_monomerinfo();
 void wrap_resmolsupplier();
+void wrap_molbundle();
 
-struct PySysErrWrite : std::ostream, std::streambuf
-{
+struct PySysErrWrite : std::ostream, std::streambuf {
   std::string prefix;
-  
-  PySysErrWrite(const std::string &prefix) :
-      std::ostream(this), prefix(prefix) {}
 
-  int overflow(int c) { write(c); return 0;}
-  
+  PySysErrWrite(std::string prefix)
+      : std::ostream(this), prefix(std::move(prefix)) {}
+
+  int overflow(int c) override {
+    write(c);
+    return 0;
+  }
+
 #ifdef RDK_THREADSAFE_SSS
-  void write(char c) { // enable thread safe logging
-    static boost::thread_specific_ptr< std::string > buffer;
-    if( !buffer.get() ) {
-      buffer.reset( new std::string );
-    }
-    (*buffer.get()) += c;
+  void write(char c) {  // enable thread safe logging
+    static thread_local std::string buffer = "";
+    buffer += c;
     if (c == '\n') {
       // Python IO is not thread safe, so grab the GIL
       PyGILState_STATE gstate;
       gstate = PyGILState_Ensure();
-      PySys_WriteStderr("%s", (prefix + (*buffer.get())).c_str());
+      PySys_WriteStderr("%s", (prefix + buffer).c_str());
       PyGILState_Release(gstate);
-      buffer->clear();
+      buffer.clear();
     }
   }
-  
 #else
-  std::string buffer; // unlimited! flushes in endl
+  std::string buffer;  // unlimited! flushes in endl
   void write(char c) {
     buffer += c;
     if (c == '\n') {
@@ -88,8 +88,6 @@ struct PySysErrWrite : std::ostream, std::streambuf
     }
   }
 #endif
-
-  
 };
 
 void RDLogError(const std::string &msg) {
@@ -103,24 +101,25 @@ void RDLogWarning(const std::string &msg) {
 }
 
 void WrapLogs() {
-  static PySysErrWrite debug  ("RDKit DEBUG: ");
-  static PySysErrWrite error  ("RDKit ERROR: ");
-  static PySysErrWrite info   ("RDKit INFO: ");
+  static PySysErrWrite debug("RDKit DEBUG: ");
+  static PySysErrWrite error("RDKit ERROR: ");
+  static PySysErrWrite info("RDKit INFO: ");
   static PySysErrWrite warning("RDKit WARNING: ");
-  if( rdDebugLog == NULL || rdInfoLog == NULL || rdErrorLog == NULL || rdWarningLog == NULL ){
+  if (rdDebugLog == nullptr || rdInfoLog == nullptr || rdErrorLog == nullptr ||
+      rdWarningLog == nullptr) {
     RDLog::InitLogs();
   }
-  if( rdDebugLog != NULL ) rdDebugLog->AddTee(debug);
-  if( rdInfoLog != NULL ) rdInfoLog->AddTee(info);
-  if( rdErrorLog != NULL ) rdErrorLog->AddTee(error);
-  if( rdWarningLog != NULL ) rdWarningLog->AddTee(warning);
+  if (rdDebugLog != nullptr) rdDebugLog->SetTee(debug);
+  if (rdInfoLog != nullptr) rdInfoLog->SetTee(info);
+  if (rdErrorLog != nullptr) rdErrorLog->SetTee(error);
+  if (rdWarningLog != nullptr) rdWarningLog->SetTee(warning);
 }
 
 BOOST_PYTHON_MODULE(rdchem) {
   python::scope().attr("__doc__") =
       "Module containing the core chemistry functionality of the RDKit";
-  RegisterListConverter<RDKit::Atom*>();
-  RegisterListConverter<RDKit::Bond*>();
+  RegisterListConverter<RDKit::Atom *>();
+  RegisterListConverter<RDKit::Bond *>();
   rdkit_import_array();
   python::register_exception_translator<RDKit::MolSanitizeException>(
       &rdSanitExceptionTranslator);
@@ -144,7 +143,7 @@ BOOST_PYTHON_MODULE(rdchem) {
       python::no_init)
       .def("__iter__", &AtomIterSeq::__iter__,
            python::return_internal_reference<
-               1, python::with_custodian_and_ward_postcall<0, 1> >())
+               1, python::with_custodian_and_ward_postcall<0, 1>>())
       .def(NEXT_METHOD, &AtomIterSeq::next,
            python::return_value_policy<python::reference_existing_object>())
 
@@ -157,7 +156,7 @@ BOOST_PYTHON_MODULE(rdchem) {
                                    python::no_init)
       .def("__iter__", &QueryAtomIterSeq::__iter__,
            python::return_internal_reference<
-               1, python::with_custodian_and_ward_postcall<0, 1> >())
+               1, python::with_custodian_and_ward_postcall<0, 1>>())
       .def(NEXT_METHOD, &QueryAtomIterSeq::next,
            python::return_value_policy<python::reference_existing_object>())
       .def("__len__", &QueryAtomIterSeq::len)
@@ -186,6 +185,7 @@ BOOST_PYTHON_MODULE(rdchem) {
   wrap_ringinfo();
   wrap_monomerinfo();
   wrap_resmolsupplier();
+  wrap_molbundle();
 
   //*********************************************
   //
