@@ -15,7 +15,8 @@
 //       with the distribution.
 //     * Neither the name of Novartis Institutes for BioMedical Research Inc. 
 //       nor the names of its contributors may be used to endorse or promote 
-//       products derived from this software without specific prior written permission.
+//       products derived from this software without specific prior written
+//       permission.
 //
 // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 // "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
@@ -29,12 +30,13 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
+#include <postgres.h>
+#include <fmgr.h>
+#include <utils/bytea.h>
+#include <utils/builtins.h>
+
 #include "rdkit.h"
-#include "fmgr.h"
-#if PG_VERSION_NUM>=90000
-#include "utils/bytea.h"
-#endif
-#include "utils/builtins.h"
+#include "cache.h"
 
 PG_MODULE_MAGIC;
 
@@ -84,7 +86,7 @@ mol_out(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -103,7 +105,7 @@ mol_send(PG_FUNCTION_ARGS) {
   char *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -205,13 +207,14 @@ mol_to_ctab(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  bool createDepictionIfMissing = PG_GETARG_BOOL(1);
+
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
                                             NULL, &mol, NULL);
 
-  bool createDepictionIfMissing = PG_GETARG_BOOL(1);
   str = makeCtabText(mol, &len, createDepictionIfMissing);
 
   PG_RETURN_CSTRING( pnstrdup(str, len) );
@@ -225,7 +228,7 @@ mol_to_smiles(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -243,7 +246,7 @@ mol_to_smarts(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -279,7 +282,7 @@ mol_to_pkl(PG_FUNCTION_ARGS) {
   char *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -320,7 +323,7 @@ qmol_out(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchMolCache(
+  fcinfo->flinfo->fn_extra = searchMolCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -335,17 +338,14 @@ PGDLLEXPORT Datum           bfp_in(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(bfp_in);
 Datum
 bfp_in(PG_FUNCTION_ARGS) {
-  MolBitmapFingerPrint    fp;
-  BitmapFingerPrint       *b = DatumGetBitmapFingerPrintP(DirectFunctionCall1(
-                                                                              byteain,
-                                                                              PG_GETARG_DATUM(0)
-                                                                              ));
+  CBfp fp;
+  Bfp *b = DatumGetBfpP( DirectFunctionCall1( byteain, PG_GETARG_DATUM(0) ) );
 
   /* check correctness */
-  fp = constructMolBitmapFingerPrint(b);
-  freeMolBitmapFingerPrint(fp);
+  fp = constructCBfp(b);
+  freeCBfp(fp);
 
-  PG_RETURN_BITMAPFINGERPRINT_P(b);
+  PG_RETURN_BFP_P(b);
 }
 
 PGDLLEXPORT Datum           bfp_out(PG_FUNCTION_ARGS);
@@ -360,28 +360,27 @@ PGDLLEXPORT Datum           bfp_from_binary_text(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(bfp_from_binary_text);
 Datum
 bfp_from_binary_text(PG_FUNCTION_ARGS) {
-  MolBitmapFingerPrint    fp;
-  BitmapFingerPrint       *b =PG_GETARG_BYTEA_P(0);
+  CBfp fp;
+  Bfp *b =PG_GETARG_BYTEA_P(0);
 
-  fp = constructMolBitmapFingerPrint(b);
-  freeMolBitmapFingerPrint(fp);
+  fp = constructCBfp(b);
+  freeCBfp(fp);
 
-  PG_RETURN_BITMAPFINGERPRINT_P(b);
+  PG_RETURN_BFP_P(b);
 }
 
 PGDLLEXPORT Datum           bfp_to_binary_text(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(bfp_to_binary_text);
 Datum
 bfp_to_binary_text(PG_FUNCTION_ARGS) {
-  MolBitmapFingerPrint    abfp;
-  fcinfo->flinfo->fn_extra = SearchBitmapFPCache(
-                                                 fcinfo->flinfo->fn_extra,
-                                                 fcinfo->flinfo->fn_mcxt,
-                                                 PG_GETARG_DATUM(0), 
-                                                 NULL, &abfp, NULL);
+  CBfp abfp;
+  fcinfo->flinfo->fn_extra = searchBfpCache(
+					    fcinfo->flinfo->fn_extra,
+					    fcinfo->flinfo->fn_mcxt,
+					    PG_GETARG_DATUM(0), 
+					    NULL, &abfp, NULL);
   
-  BitmapFingerPrint *b=deconstructMolBitmapFingerPrint(abfp);
-  PG_RETURN_BYTEA_P( b );
+  PG_RETURN_BYTEA_P( deconstructCBfp(abfp) );
 }
 
 
@@ -389,17 +388,14 @@ PGDLLEXPORT Datum           sfp_in(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(sfp_in);
 Datum
 sfp_in(PG_FUNCTION_ARGS) {
-  MolSparseFingerPrint    fp;
-  SparseFingerPrint       *b = DatumGetSparseFingerPrintP(DirectFunctionCall1(
-                                                                              byteain,
-                                                                              PG_GETARG_DATUM(0)
-                                                                              ));
+  CSfp fp;
+  Sfp *b = DatumGetSfpP(DirectFunctionCall1( byteain, PG_GETARG_DATUM(0) ));
 
   /* check correctness */
-  fp = constructMolSparseFingerPrint(b);
-  freeMolSparseFingerPrint(fp);
+  fp = constructCSfp(b);
+  freeCSfp(fp);
 
-  PG_RETURN_SPARSEFINGERPRINT_P(b);
+  PG_RETURN_SFP_P(b);
 }
 
 PGDLLEXPORT Datum           sfp_out(PG_FUNCTION_ARGS);
@@ -432,20 +428,20 @@ PG_FUNCTION_INFO_V1(reaction_in);
 Datum
 reaction_in(PG_FUNCTION_ARGS) {
   char    *data = PG_GETARG_CSTRING(0);
-  CChemicalReaction  rxn;
-  ChemReactionBA     *rxnBA;
+  CChemicalReaction crxn;
+  Reaction *rxn;
 
-  rxn = parseChemReactText(data,false,false);
+  crxn = parseChemReactText(data,false,false);
 
-  if(!rxn){
+  if(!crxn){
     ereport(ERROR,
             (errcode(ERRCODE_DATA_EXCEPTION),
              errmsg("could not construct chemical reaction")));
   }
-  rxnBA = deconstructChemReact(rxn);
-  freeChemReaction(rxn);
+  rxn = deconstructChemReact(crxn);
+  freeChemReaction(crxn);
 
-  PG_RETURN_CHEMREACTION_P(rxnBA);           
+  PG_RETURN_REACTION_P(rxn);           
 }
 
 PGDLLEXPORT Datum           reaction_recv(PG_FUNCTION_ARGS);
@@ -454,17 +450,17 @@ Datum
 reaction_recv(PG_FUNCTION_ARGS) {
   bytea    *data = PG_GETARG_BYTEA_P(0);
   int len=VARSIZE(data)-VARHDRSZ;
-  CChemicalReaction  rxn;
-  ChemReactionBA     *rxnBA;
+  CChemicalReaction crxn;
+  Reaction *rxn;
 
-  rxn = parseChemReactBlob(VARDATA(data),len);
+  crxn = parseChemReactBlob(VARDATA(data),len);
 
-  rxnBA = deconstructChemReact(rxn);
-  freeChemReaction(rxn);
+  rxn = deconstructChemReact(crxn);
+  freeChemReaction(crxn);
 
   PG_FREE_IF_COPY(data, 0);
 
-  PG_RETURN_CHEMREACTION_P(rxnBA);           
+  PG_RETURN_REACTION_P(rxn);           
 }
 
 
@@ -476,7 +472,7 @@ reaction_out(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchChemReactionCache(
+  fcinfo->flinfo->fn_extra = searchReactionCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -495,7 +491,7 @@ reaction_send(PG_FUNCTION_ARGS) {
   char *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchChemReactionCache(
+  fcinfo->flinfo->fn_extra = searchReactionCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -513,15 +509,15 @@ PG_FUNCTION_INFO_V1(reaction_from_ctab);
 Datum
 reaction_from_ctab(PG_FUNCTION_ARGS) {
   char    *data = PG_GETARG_CSTRING(0);
-  CChemicalReaction  rxn;
-  ChemReactionBA     *rxnBA;
+  CChemicalReaction  crxn;
+  Reaction *rxn;
 
-  rxn = parseChemReactCTAB(data,true);
-  if(!rxn) PG_RETURN_NULL();
-  rxnBA = deconstructChemReact(rxn);
-  freeChemReaction(rxn);
+  crxn = parseChemReactCTAB(data,true);
+  if(!crxn) PG_RETURN_NULL();
+  rxn = deconstructChemReact(crxn);
+  freeChemReaction(crxn);
 
-  PG_RETURN_CHEMREACTION_P(rxnBA);
+  PG_RETURN_REACTION_P(rxn);
 }
 
 PGDLLEXPORT Datum           reaction_from_smarts(PG_FUNCTION_ARGS);
@@ -529,15 +525,15 @@ PG_FUNCTION_INFO_V1(reaction_from_smarts);
 Datum
 reaction_from_smarts(PG_FUNCTION_ARGS) {
   char    *data = PG_GETARG_CSTRING(0);
-  CChemicalReaction  rxn;
-  ChemReactionBA     *rxnBA;
+  CChemicalReaction crxn;
+  Reaction *rxn;
 
-  rxn = parseChemReactText(data,true,true);
-  if(!rxn) PG_RETURN_NULL();
-  rxnBA = deconstructChemReact(rxn);
-  freeChemReaction(rxn);
+  crxn = parseChemReactText(data,true,true);
+  if(!crxn) PG_RETURN_NULL();
+  rxn = deconstructChemReact(crxn);
+  freeChemReaction(crxn);
 
-  PG_RETURN_CHEMREACTION_P(rxnBA);
+  PG_RETURN_REACTION_P(rxn);
 }
 
 PGDLLEXPORT Datum           reaction_from_smiles(PG_FUNCTION_ARGS);
@@ -545,15 +541,15 @@ PG_FUNCTION_INFO_V1(reaction_from_smiles);
 Datum
 reaction_from_smiles(PG_FUNCTION_ARGS) {
   char    *data = PG_GETARG_CSTRING(0);
-  CChemicalReaction  rxn;
-  ChemReactionBA     *rxnBA;
+  CChemicalReaction crxn;
+  Reaction *rxn;
 
-  rxn = parseChemReactText(data,false,true);
-  if(!rxn) PG_RETURN_NULL();
-  rxnBA = deconstructChemReact(rxn);
-  freeChemReaction(rxn);
+  crxn = parseChemReactText(data,false,true);
+  if(!crxn) PG_RETURN_NULL();
+  rxn = deconstructChemReact(crxn);
+  freeChemReaction(crxn);
 
-  PG_RETURN_CHEMREACTION_P(rxnBA);
+  PG_RETURN_REACTION_P(rxn);
 }
 
 PGDLLEXPORT Datum           reaction_to_ctab(PG_FUNCTION_ARGS);
@@ -564,7 +560,7 @@ reaction_to_ctab(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchChemReactionCache(
+  fcinfo->flinfo->fn_extra = searchReactionCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -583,7 +579,7 @@ reaction_to_smiles(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchChemReactionCache(
+  fcinfo->flinfo->fn_extra = searchReactionCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
@@ -602,7 +598,7 @@ reaction_to_smarts(PG_FUNCTION_ARGS) {
   char    *str;
   int     len;
 
-  fcinfo->flinfo->fn_extra = SearchChemReactionCache(
+  fcinfo->flinfo->fn_extra = searchReactionCache(
                                             fcinfo->flinfo->fn_extra,
                                             fcinfo->flinfo->fn_mcxt,
                                             PG_GETARG_DATUM(0),
