@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2015 Greg Landrum
+// Copyright (C) 2015-2018 Greg Landrum
 //
 //  @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -8,21 +8,21 @@
 //  of the RDKit source tree.
 //
 
+#include <RDGeneral/export.h>
 #ifndef RDTHREADS_H_2015
 #define RDTHREADS_H_2015
 
+#include <RDGeneral/Invariant.h>
+
 #ifdef RDK_THREADSAFE_SSS
-#include "RDGeneral/Invariant.h"
-#include <RDGeneral/BoostStartInclude.h>
-#include <boost/thread.hpp>
-#include <RDGeneral/BoostEndInclude.h>
+#include <thread>
 
 namespace RDKit {
 inline unsigned int getNumThreadsToUse(int target) {
   if (target >= 1) {
     return static_cast<unsigned int>(target);
   }
-  unsigned int res = boost::thread::hardware_concurrency();
+  unsigned int res = std::thread::hardware_concurrency();
   if (res > rdcast<unsigned int>(-target)) {
     return res + target;
   } else {
