@@ -1,5 +1,6 @@
 //
 //  Copyright (C) 2003-2007 Greg Landrum and Rational Discovery LLC
+//  Copyright (C) 2017 Greg Landrum and NextMove Software
 //
 //   @@ All Rights Reserved @@
 //  This file is part of the RDKit.
@@ -7,8 +8,9 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
-#ifndef __RD_MAXMINPICKER_H__
-#define __RD_MAXMINPICKER_H__
+#include <RDGeneral/export.h>
+#ifndef RD_MAXMINPICKER_H
+#define RD_MAXMINPICKER_H
 
 #include <RDGeneral/types.h>
 #include <RDGeneral/utils.h>
@@ -22,7 +24,7 @@
 namespace RDPickers {
 
 namespace {
-class distmatFunctor {
+class RDKIT_SIMDIVPICKERS_EXPORT distmatFunctor {
  public:
   distmatFunctor(const double *distMat) : dp_distMat(distMat){};
   double operator()(unsigned int i, unsigned int j) {
@@ -42,7 +44,7 @@ class distmatFunctor {
  *  aimed at diversity. See documentation for "pick()" member function for the
  *algorithm details
  */
-class MaxMinPicker : public DistPicker {
+class RDKIT_SIMDIVPICKERS_EXPORT MaxMinPicker : public DistPicker {
  public:
   /*! \brief Default Constructor
    *
@@ -67,11 +69,21 @@ class MaxMinPicker : public DistPicker {
    *   \param seed - (optional) seed for the random number generator
    */
   template <typename T>
+  RDKit::INT_VECT lazyPick(T &func, unsigned int poolSize,
+                           unsigned int pickSize) const;
+
+  template <typename T>
   RDKit::INT_VECT lazyPick(T &func, unsigned int poolSize,
                            unsigned int pickSize,
-                           RDKit::INT_VECT firstPicks = RDKit::INT_VECT(),
+                           const RDKit::INT_VECT &firstPicks,
                            int seed = -1) const;
 
+  template <typename T>
+  RDKit::INT_VECT lazyPick(T &func, unsigned int poolSize,
+                           unsigned int pickSize,
+                           const RDKit::INT_VECT &firstPicks, int seed,
+                           double &threshold) const;
+
   /*! \brief Contains the implementation for the MaxMin diversity picker
    *
    * Here is how the picking algorithm works, refer to
@@ -110,6 +122,7 @@ class MaxMinPicker : public DistPicker {
                        unsigned int pickSize, RDKit::INT_VECT firstPicks,
                        int seed = -1) const {
     CHECK_INVARIANT(distMat, "Invalid Distance Matrix");
+    if (!poolSize) throw ValueErrorException("empty pool to pick from");
     if (poolSize < pickSize)
       throw ValueErrorException("pickSize cannot be larger than the poolSize");
     distmatFunctor functor(distMat);
@@ -123,83 +136,165 @@ class MaxMinPicker : public DistPicker {
     return pick(distMat, poolSize, pickSize, iv);
   }
 };
+
+struct MaxMinPickInfo {
+  double dist_bound;   // distance to closest reference
+  unsigned int picks;  // number of references considered
+  unsigned int next;   // singly linked list of candidates
+};
+
 // we implement this here in order to allow arbitrary functors without link
 // errors
 template <typename T>
 RDKit::INT_VECT MaxMinPicker::lazyPick(T &func, unsigned int poolSize,
                                        unsigned int pickSize,
-                                       RDKit::INT_VECT firstPicks,
-                                       int seed) const {
+                                       const RDKit::INT_VECT &firstPicks,
+                                       int seed, double &threshold) const {
+  if (!poolSize) throw ValueErrorException("empty pool to pick from");
+
   if (poolSize < pickSize)
     throw ValueErrorException("pickSize cannot be larger than the poolSize");
 
-  RDKit::INT_LIST pool;
-
   RDKit::INT_VECT picks;
+
+  unsigned int memsize = (unsigned int)(poolSize * sizeof(MaxMinPickInfo));
+  MaxMinPickInfo *pinfo = new MaxMinPickInfo[memsize];
+  if (!pinfo) {
+    threshold = -1.0;
+    return picks;
+  }
+  memset(pinfo, 0, memsize);
+
   picks.reserve(pickSize);
+  unsigned int picked = 0;  // picks.size()
   unsigned int pick = 0;
 
-  // enter the pool into a list so that we can pick out of it easily
-  for (unsigned int i = 0; i < poolSize; i++) {
-    pool.push_back(i);
-  }
-
-  // get a seeded random number generator:
-  typedef boost::mt19937 rng_type;
-  typedef boost::uniform_int<> distrib_type;
-  typedef boost::variate_generator<rng_type &, distrib_type> source_type;
-  rng_type generator(42u);
-  distrib_type dist(0, poolSize);
-  source_type randomSource(generator, dist);
-  if (seed > 0) generator.seed(static_cast<rng_type::result_type>(seed));
-
   // pick the first entry
-  if (!firstPicks.size()) {
+  if (firstPicks.empty()) {
+    // get a seeded random number generator:
+    typedef boost::mt19937 rng_type;
+    typedef boost::uniform_int<> distrib_type;
+    typedef boost::variate_generator<rng_type &, distrib_type> source_type;
+    rng_type generator(42u);
+    distrib_type dist(0, poolSize - 1);
+    source_type randomSource(generator, dist);
+    if (seed > 0) generator.seed(static_cast<rng_type::result_type>(seed));
+
     pick = randomSource();
     // add the pick to the picks
     picks.push_back(pick);
     // and remove it from the pool
-    pool.remove(pick);
+    pinfo[pick].picks = 1;
+    picked = 1;
+
   } else {
     for (RDKit::INT_VECT::const_iterator pIdx = firstPicks.begin();
          pIdx != firstPicks.end(); ++pIdx) {
       pick = static_cast<unsigned int>(*pIdx);
-      if (pick >= poolSize)
+      if (pick >= poolSize) {
+        delete[] pinfo;
         throw ValueErrorException("pick index was larger than the poolSize");
+      }
       picks.push_back(pick);
-      pool.remove(pick);
+      pinfo[pick].picks = 1;
+      picked++;
     }
   }
+
+  if (picked >= pickSize) {
+    threshold = -1.0;
+    delete[] pinfo;
+    return picks;
+  }
+
+  unsigned int pool_list = 0;
+  unsigned int *prev = &pool_list;
+  // enter the pool into a list so that we can pick out of it easily
+  for (unsigned int i = 0; i < poolSize; i++)
+    if (pinfo[i].picks == 0) {
+      *prev = i;
+      prev = &pinfo[i].next;
+    }
+  *prev = 0;
+
+  unsigned int poolIdx;
+  unsigned int pickIdx;
+
+  // First pass initialize dist_bound
+  prev = &pool_list;
+  pickIdx = picks[0];
+  do {
+    poolIdx = *prev;
+    pinfo[poolIdx].dist_bound = func(poolIdx, pickIdx);
+    pinfo[poolIdx].picks = 1;
+    prev = &pinfo[poolIdx].next;
+  } while (*prev != 0);
+
   // now pick 1 compound at a time
-  while (picks.size() < pickSize) {
-    double maxOFmin = -1.0;
-    RDKit::INT_LIST_I plri = pool.end();
-    for (RDKit::INT_LIST_I pli = pool.begin(); pli != pool.end(); ++pli) {
-      unsigned int poolIdx = (*pli);
-      double minTOi = RDKit::MAX_DOUBLE;
-      for (RDKit::INT_VECT_CI pi = picks.begin(); pi != picks.end(); ++pi) {
-        unsigned int pickIdx = (*pi);
-        CHECK_INVARIANT(poolIdx != pickIdx, "");
-        double dist = func(poolIdx, pickIdx);
-        if (dist <= minTOi) {
-          minTOi = dist;
+  double maxOFmin = -1.0;
+  double tmpThreshold = -1.0;
+  while (picked < pickSize) {
+    unsigned int *pick_prev = 0;
+    maxOFmin = -1.0;
+    prev = &pool_list;
+    do {
+      poolIdx = *prev;
+      double minTOi = pinfo[poolIdx].dist_bound;
+      if (minTOi > maxOFmin) {
+        unsigned int pi = pinfo[poolIdx].picks;
+        while (pi < picked) {
+          unsigned int picki = picks[pi];
+          CHECK_INVARIANT(poolIdx != picki, "pool index != pick index");
+          double dist = func(poolIdx, picki);
+          pi++;
+          if (dist <= minTOi) {
+            minTOi = dist;
+            if (minTOi <= maxOFmin) break;
+          }
+        }
+        pinfo[poolIdx].dist_bound = minTOi;
+        pinfo[poolIdx].picks = pi;
+        if (minTOi > maxOFmin) {
+          maxOFmin = minTOi;
+          pick_prev = prev;
+          pick = poolIdx;
         }
       }
-      if (minTOi > maxOFmin ||
-          (RDKit::feq(minTOi, maxOFmin) && poolIdx < pick)) {
-        maxOFmin = minTOi;
-        pick = poolIdx;
-        plri = pli;
-      }
-    }
+      prev = &pinfo[poolIdx].next;
+    } while (*prev != 0);
 
-    // now add the new pick to  picks and remove it from the pool
+    // if the current distance is closer then threshold, we're done
+    if (threshold >= 0.0 && maxOFmin < threshold) break;
+    tmpThreshold = maxOFmin;
+    // now add the new pick to picks and remove it from the pool
+    *pick_prev = pinfo[pick].next;
     picks.push_back(pick);
-    CHECK_INVARIANT(plri != pool.end(), "");
-    pool.erase(plri);
+    picked++;
   }
+
+  threshold = tmpThreshold;
+  delete[] pinfo;
   return picks;
 }
+
+template <typename T>
+RDKit::INT_VECT MaxMinPicker::lazyPick(T &func, unsigned int poolSize,
+                                       unsigned int pickSize,
+                                       const RDKit::INT_VECT &firstPicks,
+                                       int seed) const {
+  double threshold = -1.0;
+  return MaxMinPicker::lazyPick(func, poolSize, pickSize, firstPicks, seed,
+                                threshold);
+}
+
+template <typename T>
+RDKit::INT_VECT MaxMinPicker::lazyPick(T &func, unsigned int poolSize,
+                                       unsigned int pickSize) const {
+  RDKit::INT_LIST firstPicks;
+  double threshold = -1.0;
+  return MaxMinPicker::lazyPick(func, poolSize, pickSize, firstPicks, -1,
+                                threshold);
+}
 };
 
 #endif
