@@ -1,20 +1,19 @@
-# $Id$
 #
 #  Copyright (c) 2009, Novartis Institutes for BioMedical Research Inc.
 #  All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
-# met: 
+# met:
 #
-#     * Redistributions of source code must retain the above copyright 
+#     * Redistributions of source code must retain the above copyright
 #       notice, this list of conditions and the following disclaimer.
 #     * Redistributions in binary form must reproduce the above
-#       copyright notice, this list of conditions and the following 
-#       disclaimer in the documentation and/or other materials provided 
+#       copyright notice, this list of conditions and the following
+#       disclaimer in the documentation and/or other materials provided
 #       with the distribution.
-#     * Neither the name of Novartis Institutes for BioMedical Research Inc. 
-#       nor the names of its contributors may be used to endorse or promote 
+#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
+#       nor the names of its contributors may be used to endorse or promote
 #       products derived from this software without specific prior written permission.
 #
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -30,12 +29,11 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 # Created by Greg Landrum, Nov 2008
-
 """ Implementation of the BRICS algorithm from Degen et al. ChemMedChem *3* 1503-7 (2008)
 
 """
 from __future__ import print_function
-import sys,re,random
+import sys, re, random
 from rdkit import Chem
 from rdkit.Chem import rdChemReactions as Reactions
 from rdkit.six import iteritems, iterkeys, next
@@ -43,7 +41,7 @@ from rdkit.six.moves import range
 
 # These are the definitions that will be applied to fragment molecules:
 environs = {
-  'L1':'[C;D3]([#0,#6,#7,#8])(=O)',
+  'L1': '[C;D3]([#0,#6,#7,#8])(=O)',
   #
   # After some discussion, the L2 definitions ("N.pl3" in the original
   # paper) have been removed and incorporated into a (almost) general
@@ -63,180 +61,180 @@ environs = {
   # this one turned out to be too tricky to define above, so we set it off
   # in its own definition:
   #'L2a':'[N;D3;R;$(N(@[C;!$(C=*)])@[C;!$(C=*)])]',
-  'L3':'[O;D2]-;!@[#0,#6,#1]',
-  'L4':'[C;!D1;!$(C=*)]-;!@[#6]',
+  'L3': '[O;D2]-;!@[#0,#6,#1]',
+  'L4': '[C;!D1;!$(C=*)]-;!@[#6]',
   #'L5':'[N;!D1;!$(N*!-*);!$(N=*);!$(N-[!C;!#0])]-[#0,C]',
-  'L5':'[N;!D1;!$(N=*);!$(N-[!#6;!#16;!#0;!#1]);!$([N;R]@[C;R]=O)]',
-  'L6':'[C;D3;!R](=O)-;!@[#0,#6,#7,#8]',
-  'L7a':'[C;D2,D3]-[#6]',
-  'L7b':'[C;D2,D3]-[#6]',
-  '#L8':'[C;!R;!D1]-;!@[#6]',
-  'L8':'[C;!R;!D1;!$(C!-*)]',
-  'L9':'[n;+0;$(n(:[c,n,o,s]):[c,n,o,s])]',
-  'L10':'[N;R;$(N(@C(=O))@[C,N,O,S])]',
-  'L11':'[S;D2](-;!@[#0,#6])',
-  'L12':'[S;D4]([#6,#0])(=O)(=O)',
-  'L13':'[C;$(C(-;@[C,N,O,S])-;@[N,O,S])]',
-  'L14':'[c;$(c(:[c,n,o,s]):[n,o,s])]',
-  'L14b':'[c;$(c(:[c,n,o,s]):[n,o,s])]',
-  'L15':'[C;$(C(-;@C)-;@C)]',
-  'L16':'[c;$(c(:c):c)]',
-  'L16b':'[c;$(c(:c):c)]',
-  }
+  'L5': '[N;!D1;!$(N=*);!$(N-[!#6;!#16;!#0;!#1]);!$([N;R]@[C;R]=O)]',
+  'L6': '[C;D3;!R](=O)-;!@[#0,#6,#7,#8]',
+  'L7a': '[C;D2,D3]-[#6]',
+  'L7b': '[C;D2,D3]-[#6]',
+  '#L8': '[C;!R;!D1]-;!@[#6]',
+  'L8': '[C;!R;!D1;!$(C!-*)]',
+  'L9': '[n;+0;$(n(:[c,n,o,s]):[c,n,o,s])]',
+  'L10': '[N;R;$(N(@C(=O))@[C,N,O,S])]',
+  'L11': '[S;D2](-;!@[#0,#6])',
+  'L12': '[S;D4]([#6,#0])(=O)(=O)',
+  'L13': '[C;$(C(-;@[C,N,O,S])-;@[N,O,S])]',
+  'L14': '[c;$(c(:[c,n,o,s]):[n,o,s])]',
+  'L14b': '[c;$(c(:[c,n,o,s]):[n,o,s])]',
+  'L15': '[C;$(C(-;@C)-;@C)]',
+  'L16': '[c;$(c(:c):c)]',
+  'L16b': '[c;$(c(:c):c)]',
+}
 reactionDefs = (
   # L1
   [
-    ('1','3','-'),
-    ('1','5','-'),
-    ('1','10','-'),
+    ('1', '3', '-'),
+    ('1', '5', '-'),
+    ('1', '10', '-'),
   ],
 
-   # L3 
-   [
-    ('3','4','-'),
-    ('3','13','-'),
-    ('3','14','-'),
-    ('3','15','-'),
-    ('3','16','-'),
-   ],
-  
-   # L4
-   [
-    ('4','5','-'),
-    ('4','11','-'),
-   ],
+  # L3
+  [
+    ('3', '4', '-'),
+    ('3', '13', '-'),
+    ('3', '14', '-'),
+    ('3', '15', '-'),
+    ('3', '16', '-'),
+  ],
 
-   # L5
-   [
-    ('5','12','-'),
-    ('5','14','-'),
-    ('5','16','-'),
-    ('5','13','-'),
-    ('5','15','-'),
-   ],
-  
-    # L6
-    [
-    ('6','13','-'),
-    ('6','14','-'),
-    ('6','15','-'),
-    ('6','16','-'),
-    ],
-  
-    # L7
-    [
-    ('7a','7b','='),
-    ],
+  # L4
+  [
+    ('4', '5', '-'),
+    ('4', '11', '-'),
+  ],
 
-    # L8
-    [
-    ('8','9','-'),
-    ('8','10','-'),
-    ('8','13','-'),
-    ('8','14','-'),
-    ('8','15','-'),
-    ('8','16','-'),
-    ],
-  
-    # L9
-    [
-    ('9','13','-'),# not in original paper
-    ('9','14','-'),# not in original paper
-    ('9','15','-'),
-    ('9','16','-'),
-    ],
-  
-    # L10
-    [
-    ('10','13','-'),
-    ('10','14','-'),
-    ('10','15','-'),
-    ('10','16','-'),
-    ],
-  
-    # L11
-    [
-    ('11','13','-'),
-    ('11','14','-'),
-    ('11','15','-'),
-    ('11','16','-'),
-    ],
-  
-    # L12
-    # none left
+  # L5
+  [
+    ('5', '12', '-'),
+    ('5', '14', '-'),
+    ('5', '16', '-'),
+    ('5', '13', '-'),
+    ('5', '15', '-'),
+  ],
 
-    # L13
-    [
-    ('13','14','-'),
-    ('13','15','-'),
-    ('13','16','-'),
-    ],
-  
-    # L14
-    [
-    ('14','14','-'),# not in original paper
-    ('14','15','-'),
-    ('14','16','-'),
-    ],
-  
-    # L15
-    [
-    ('15','16','-'),
-    ],
+  # L6
+  [
+    ('6', '13', '-'),
+    ('6', '14', '-'),
+    ('6', '15', '-'),
+    ('6', '16', '-'),
+  ],
 
-    # L16
-    [
-    ('16','16','-'), # not in original paper
-    ],
-  )
+  # L7
+  [
+    ('7a', '7b', '='),
+  ],
+
+  # L8
+  [
+    ('8', '9', '-'),
+    ('8', '10', '-'),
+    ('8', '13', '-'),
+    ('8', '14', '-'),
+    ('8', '15', '-'),
+    ('8', '16', '-'),
+  ],
+
+  # L9
+  [
+    ('9', '13', '-'),  # not in original paper
+    ('9', '14', '-'),  # not in original paper
+    ('9', '15', '-'),
+    ('9', '16', '-'),
+  ],
+
+  # L10
+  [
+    ('10', '13', '-'),
+    ('10', '14', '-'),
+    ('10', '15', '-'),
+    ('10', '16', '-'),
+  ],
+
+  # L11
+  [
+    ('11', '13', '-'),
+    ('11', '14', '-'),
+    ('11', '15', '-'),
+    ('11', '16', '-'),
+  ],
+
+  # L12
+  # none left
+
+  # L13
+  [
+    ('13', '14', '-'),
+    ('13', '15', '-'),
+    ('13', '16', '-'),
+  ],
+
+  # L14
+  [
+    ('14', '14', '-'),  # not in original paper
+    ('14', '15', '-'),
+    ('14', '16', '-'),
+  ],
+
+  # L15
+  [
+    ('15', '16', '-'),
+  ],
+
+  # L16
+  [
+    ('16', '16', '-'),  # not in original paper
+  ], )
 import copy
-smartsGps=copy.deepcopy(reactionDefs)
+smartsGps = copy.deepcopy(reactionDefs)
 for gp in smartsGps:
-  for j,defn in enumerate(gp):
-    g1,g2,bnd = defn
-    r1=environs['L'+g1]
-    r2=environs['L'+g2]
-    g1 = re.sub('[a-z,A-Z]','',g1)
-    g2 = re.sub('[a-z,A-Z]','',g2)
-    sma='[$(%s):1]%s;!@[$(%s):2]>>[%s*]-[*:1].[%s*]-[*:2]'%(r1,bnd,r2,g1,g2)
-    gp[j] =sma
+  for j, defn in enumerate(gp):
+    g1, g2, bnd = defn
+    r1 = environs['L' + g1]
+    r2 = environs['L' + g2]
+    g1 = re.sub('[a-z,A-Z]', '', g1)
+    g2 = re.sub('[a-z,A-Z]', '', g2)
+    sma = '[$(%s):1]%s;!@[$(%s):2]>>[%s*]-[*:1].[%s*]-[*:2]' % (r1, bnd, r2, g1, g2)
+    gp[j] = sma
 
 for gp in smartsGps:
   for defn in gp:
     try:
-      t=Reactions.ReactionFromSmarts(defn)
+      t = Reactions.ReactionFromSmarts(defn)
       t.Initialize()
     except Exception:
       print(defn)
       raise
 
-environMatchers={}
-for env,sma in iteritems(environs):
-  environMatchers[env]=Chem.MolFromSmarts(sma)
-  
-bondMatchers=[]
-for i,compats in enumerate(reactionDefs):
-  tmp=[]
-  for i1,i2,bType in compats:
-      e1 = environs['L%s'%i1]
-      e2 = environs['L%s'%i2]
-      patt = '[$(%s)]%s;!@[$(%s)]'%(e1,bType,e2)
-      patt = Chem.MolFromSmarts(patt)
-      tmp.append((i1,i2,bType,patt))
+environMatchers = {}
+for env, sma in iteritems(environs):
+  environMatchers[env] = Chem.MolFromSmarts(sma)
+
+bondMatchers = []
+for i, compats in enumerate(reactionDefs):
+  tmp = []
+  for i1, i2, bType in compats:
+    e1 = environs['L%s' % i1]
+    e2 = environs['L%s' % i2]
+    patt = '[$(%s)]%s;!@[$(%s)]' % (e1, bType, e2)
+    patt = Chem.MolFromSmarts(patt)
+    tmp.append((i1, i2, bType, patt))
   bondMatchers.append(tmp)
-    
+
 reactions = tuple([[Reactions.ReactionFromSmarts(y) for y in x] for x in smartsGps])
 reverseReactions = []
-for i,rxnSet in enumerate(smartsGps):
-  for j,sma in enumerate(rxnSet):
-    rs,ps = sma.split('>>')
-    sma = '%s>>%s'%(ps,rs)
+for i, rxnSet in enumerate(smartsGps):
+  for j, sma in enumerate(rxnSet):
+    rs, ps = sma.split('>>')
+    sma = '%s>>%s' % (ps, rs)
     rxn = Reactions.ReactionFromSmarts(sma)
-    labels = re.findall(r'\[([0-9]+?)\*\]',ps)
-    rxn._matchers=[Chem.MolFromSmiles('[%s*]'%x) for x in labels]
+    labels = re.findall(r'\[([0-9]+?)\*\]', ps)
+    rxn._matchers = [Chem.MolFromSmiles('[%s*]' % x) for x in labels]
     reverseReactions.append(rxn)
 
-def FindBRICSBonds(mol,randomizeOrder=False,silent=True):
+
+def FindBRICSBonds(mol, randomizeOrder=False, silent=True):
   """ returns the bonds in a molecule that BRICS would cleave
 
   >>> from rdkit import Chem
@@ -268,44 +266,47 @@ def FindBRICSBonds(mol,randomizeOrder=False,silent=True):
   >>> res = list(FindBRICSBonds(m))
   >>> sorted(res)
   [((1, 2), ('7', '7'))]
-  
+
   make sure we don't match ring bonds:
   >>> m = Chem.MolFromSmiles('O=C1NCCC1')
   >>> list(FindBRICSBonds(m))
   []
-  
+
   another nice one, make sure environment 8 doesn't match something connected
   to a ring atom:
   >>> m = Chem.MolFromSmiles('CC1(C)CCCCC1')
   >>> list(FindBRICSBonds(m))
   []
-  
+
   """
   letter = re.compile('[a-z,A-Z]')
   indices = list(range(len(bondMatchers)))
-  bondsDone=set()
-  if randomizeOrder: random.shuffle(indices,random=random.random)
+  bondsDone = set()
+  if randomizeOrder:
+    random.shuffle(indices, random=random.random)
 
-  envMatches={}
-  for env,patt in iteritems(environMatchers):
-    envMatches[env]=mol.HasSubstructMatch(patt)
+  envMatches = {}
+  for env, patt in iteritems(environMatchers):
+    envMatches[env] = mol.HasSubstructMatch(patt)
   for gpIdx in indices:
     if randomizeOrder:
-      compats =bondMatchers[gpIdx][:]
-      random.shuffle(compats,random=random.random)
+      compats = bondMatchers[gpIdx][:]
+      random.shuffle(compats, random=random.random)
     else:
       compats = bondMatchers[gpIdx]
-    for i1,i2,bType,patt in compats:
-      if not envMatches['L'+i1] or not envMatches['L'+i2]: continue
+    for i1, i2, bType, patt in compats:
+      if not envMatches['L' + i1] or not envMatches['L' + i2]:
+        continue
       matches = mol.GetSubstructMatches(patt)
-      i1 = letter.sub('',i1)
-      i2 = letter.sub('',i2)
+      i1 = letter.sub('', i1)
+      i2 = letter.sub('', i2)
       for match in matches:
-        if match not in bondsDone and (match[1],match[0]) not in bondsDone:
+        if match not in bondsDone and (match[1], match[0]) not in bondsDone:
           bondsDone.add(match)
-          yield(((match[0],match[1]),(i1,i2)))
+          yield (((match[0], match[1]), (i1, i2)))
 
-def BreakBRICSBonds(mol,bonds=None,sanitize=True,silent=True):
+
+def BreakBRICSBonds(mol, bonds=None, sanitize=True, silent=True):
   """ breaks the BRICS bonds in a molecule and returns the results
 
   >>> from rdkit import Chem
@@ -326,7 +327,7 @@ def BreakBRICSBonds(mol,bonds=None,sanitize=True,silent=True):
   >>> m2 = BreakBRICSBonds(m,[((3, 2), ('3', '4'))])
   >>> Chem.MolToSmiles(m2,True)
   '[3*]OCC.[4*]CCC'
-  
+
   this can be used as an alternate approach for doing a BRICS decomposition by
   following BreakBRICSBonds with a call to Chem.GetMolFrags:
   >>> m = Chem.MolFromSmiles('CCCOCC')
@@ -340,49 +341,50 @@ def BreakBRICSBonds(mol,bonds=None,sanitize=True,silent=True):
     #bonds = FindBRICSBonds(mol)
     res = Chem.FragmentOnBRICSBonds(mol)
     if sanitize:
-       Chem.SanitizeMol(res)
+      Chem.SanitizeMol(res)
     return res
   eMol = Chem.EditableMol(mol)
   nAts = mol.GetNumAtoms()
 
-  dummyPositions=[]
-  for indices,dummyTypes in bonds:
-    ia,ib = indices
-    obond = mol.GetBondBetweenAtoms(ia,ib)
-    bondType=obond.GetBondType()
-    eMol.RemoveBond(ia,ib)
+  dummyPositions = []
+  for indices, dummyTypes in bonds:
+    ia, ib = indices
+    obond = mol.GetBondBetweenAtoms(ia, ib)
+    bondType = obond.GetBondType()
+    eMol.RemoveBond(ia, ib)
 
-    da,db = dummyTypes
+    da, db = dummyTypes
     atoma = Chem.Atom(0)
     atoma.SetIsotope(int(da))
     atoma.SetNoImplicit(True)
     idxa = nAts
-    nAts+=1
+    nAts += 1
     eMol.AddAtom(atoma)
-    eMol.AddBond(ia,idxa,bondType)
-    
+    eMol.AddBond(ia, idxa, bondType)
+
     atomb = Chem.Atom(0)
     atomb.SetIsotope(int(db))
     atomb.SetNoImplicit(True)
     idxb = nAts
-    nAts+=1
+    nAts += 1
     eMol.AddAtom(atomb)
-    eMol.AddBond(ib,idxb,bondType)
+    eMol.AddBond(ib, idxb, bondType)
     if mol.GetNumConformers():
-      dummyPositions.append((idxa,ib))
-      dummyPositions.append((idxb,ia))
+      dummyPositions.append((idxa, ib))
+      dummyPositions.append((idxb, ia))
   res = eMol.GetMol()
   if sanitize:
     Chem.SanitizeMol(res)
   if mol.GetNumConformers():
     for conf in mol.GetConformers():
       resConf = res.GetConformer(conf.GetId())
-      for ia,pa in dummyPositions:
-        resConf.SetAtomPosition(ia,conf.GetAtomPosition(pa))
+      for ia, pa in dummyPositions:
+        resConf.SetAtomPosition(ia, conf.GetAtomPosition(pa))
   return res
 
-def BRICSDecompose(mol,allNodes=None,minFragmentSize=1,onlyUseReactions=None,
-                   silent=True,keepNonLeafNodes=False,singlePass=False,returnMols=False):
+
+def BRICSDecompose(mol, allNodes=None, minFragmentSize=1, onlyUseReactions=None, silent=True,
+                   keepNonLeafNodes=False, singlePass=False, returnMols=False):
   """ returns the BRICS decomposition for a molecule
 
   >>> from rdkit import Chem
@@ -437,64 +439,65 @@ def BRICSDecompose(mol,allNodes=None,minFragmentSize=1,onlyUseReactions=None,
 
   """
   global reactions
-  mSmi = Chem.MolToSmiles(mol,1)
-  
+  mSmi = Chem.MolToSmiles(mol, 1)
+
   if allNodes is None:
-    allNodes=set()
+    allNodes = set()
 
   if mSmi in allNodes:
     return set()
 
-  activePool={mSmi:mol}
+  activePool = {mSmi: mol}
   allNodes.add(mSmi)
-  foundMols={mSmi:mol}
-  for gpIdx,reactionGp in enumerate(reactions):
+  foundMols = {mSmi: mol}
+  for gpIdx, reactionGp in enumerate(reactions):
     newPool = {}
     while activePool:
-      matched=False
+      matched = False
       nSmi = next(iterkeys(activePool))
       mol = activePool.pop(nSmi)
-      for rxnIdx,reaction in enumerate(reactionGp):
-        if onlyUseReactions and (gpIdx,rxnIdx) not in onlyUseReactions:
+      for rxnIdx, reaction in enumerate(reactionGp):
+        if onlyUseReactions and (gpIdx, rxnIdx) not in onlyUseReactions:
           continue
         if not silent:
           print('--------')
           print(smartsGps[gpIdx][rxnIdx])
-        ps = reaction.RunReactants((mol,))
+        ps = reaction.RunReactants((mol, ))
         if ps:
-          if not silent: print(nSmi,'->',len(ps),'products')
+          if not silent:
+            print(nSmi, '->', len(ps), 'products')
           for prodSeq in ps:
-            seqOk=True
+            seqOk = True
             # we want to disqualify small fragments, so sort the product sequence by size
-            tSeq = [(prod.GetNumAtoms(onlyExplicit=True),idx) for idx,prod in enumerate(prodSeq)]
+            tSeq = [(prod.GetNumAtoms(onlyExplicit=True), idx) for idx, prod in enumerate(prodSeq)]
             tSeq.sort()
-            for nats,idx in tSeq:
+            for nats, idx in tSeq:
               prod = prodSeq[idx]
               try:
                 Chem.SanitizeMol(prod)
               except Exception:
                 continue
-              pSmi = Chem.MolToSmiles(prod,1)
-              if minFragmentSize>0:
+              pSmi = Chem.MolToSmiles(prod, 1)
+              if minFragmentSize > 0:
                 nDummies = pSmi.count('*')
-                if nats-nDummies<minFragmentSize:
-                  seqOk=False
+                if nats - nDummies < minFragmentSize:
+                  seqOk = False
                   break
               prod.pSmi = pSmi
-            ts = [(x,prodSeq[y]) for x,y in tSeq]
-            prodSeq=ts
+            ts = [(x, prodSeq[y]) for x, y in tSeq]
+            prodSeq = ts
             if seqOk:
-              matched=True
-              for nats,prod in prodSeq:
+              matched = True
+              for nats, prod in prodSeq:
                 pSmi = prod.pSmi
                 #print('\t',nats,pSmi)
                 if pSmi not in allNodes:
                   if not singlePass:
                     activePool[pSmi] = prod
                   allNodes.add(pSmi)
-                  foundMols[pSmi]=prod
+                  foundMols[pSmi] = prod
       if singlePass or keepNonLeafNodes or not matched:
-        newPool[nSmi]=mol
+        newPool[nSmi] = mol
     activePool = newPool
   if not (singlePass or keepNonLeafNodes):
     if not returnMols:
@@ -510,41 +513,43 @@ def BRICSDecompose(mol,allNodes=None,minFragmentSize=1,onlyUseReactions=None,
 
 
 import random
-dummyPattern=Chem.MolFromSmiles('[*]')
-def BRICSBuild(fragments,onlyCompleteMols=True,seeds=None,uniquify=True,
-               scrambleReagents=True,maxDepth=3):
+dummyPattern = Chem.MolFromSmiles('[*]')
+
+
+def BRICSBuild(fragments, onlyCompleteMols=True, seeds=None, uniquify=True, scrambleReagents=True,
+               maxDepth=3):
   seen = set()
   if not seeds:
     seeds = list(fragments)
   if scrambleReagents:
     seeds = list(seeds)
-    random.shuffle(seeds,random=random.random)
+    random.shuffle(seeds, random=random.random)
   if scrambleReagents:
     tempReactions = list(reverseReactions)
-    random.shuffle(tempReactions,random=random.random)
+    random.shuffle(tempReactions, random=random.random)
   else:
-    tempReactions=reverseReactions
+    tempReactions = reverseReactions
   for seed in seeds:
-    seedIsR1=False
-    seedIsR2=False
-    nextSteps=[]
+    seedIsR1 = False
+    seedIsR2 = False
+    nextSteps = []
     for rxn in tempReactions:
       if seed.HasSubstructMatch(rxn._matchers[0]):
-        seedIsR1=True
+        seedIsR1 = True
       if seed.HasSubstructMatch(rxn._matchers[1]):
-        seedIsR2=True
+        seedIsR2 = True
       for fragment in fragments:
         ps = None
         if fragment.HasSubstructMatch(rxn._matchers[0]):
           if seedIsR2:
-            ps = rxn.RunReactants((fragment,seed))
+            ps = rxn.RunReactants((fragment, seed))
         if fragment.HasSubstructMatch(rxn._matchers[1]):
           if seedIsR1:
-            ps = rxn.RunReactants((seed,fragment))
+            ps = rxn.RunReactants((seed, fragment))
         if ps:
           for p in ps:
             if uniquify:
-              pSmi =Chem.MolToSmiles(p[0],True)
+              pSmi = Chem.MolToSmiles(p[0], True)
               if pSmi in seen:
                 continue
               else:
@@ -555,132 +560,130 @@ def BRICSBuild(fragments,onlyCompleteMols=True,seeds=None,uniquify=True,
                 yield p[0]
             else:
               yield p[0]
-    if nextSteps and maxDepth>0:
-      for p in BRICSBuild(fragments,onlyCompleteMols=onlyCompleteMols,
-                          seeds=nextSteps,uniquify=uniquify,
-                          maxDepth=maxDepth-1):
+    if nextSteps and maxDepth > 0:
+      for p in BRICSBuild(fragments, onlyCompleteMols=onlyCompleteMols, seeds=nextSteps,
+                          uniquify=uniquify, maxDepth=maxDepth - 1):
         if uniquify:
-          pSmi =Chem.MolToSmiles(p,True)
+          pSmi = Chem.MolToSmiles(p, True)
           if pSmi in seen:
             continue
           else:
             seen.add(pSmi)
         yield p
-  
+
+  # ------- ------- ------- ------- ------- ------- ------- -------
+  # Begin testing code
 
 
-# ------- ------- ------- ------- ------- ------- ------- -------
-# Begin testing code
-
-
-#------------------------------------
-#
-#  doctest boilerplate
-#
+  #------------------------------------
+  #
+  #  doctest boilerplate
+  #
 def _test():
-  import doctest,sys
+  import doctest, sys
   return doctest.testmod(sys.modules["__main__"],
-                         optionflags=doctest.ELLIPSIS+doctest.NORMALIZE_WHITESPACE)
+                         optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
 
-if __name__=='__main__':
+
+if __name__ == '__main__':
   import unittest
+
   class TestCase(unittest.TestCase):
+
     def test1(self):
       m = Chem.MolFromSmiles('CC(=O)OC')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2)
+      self.assertTrue(len(res) == 2)
 
       m = Chem.MolFromSmiles('CC(=O)N1CCC1=O')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2,res)
+      self.assertTrue(len(res) == 2, res)
 
       m = Chem.MolFromSmiles('c1ccccc1N(C)C')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2,res)
+      self.assertTrue(len(res) == 2, res)
 
       m = Chem.MolFromSmiles('c1cccnc1N(C)C')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2,res)
+      self.assertTrue(len(res) == 2, res)
 
       m = Chem.MolFromSmiles('o1ccnc1N(C)C')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2)
+      self.assertTrue(len(res) == 2)
 
       m = Chem.MolFromSmiles('c1ccccc1OC')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2)
+      self.assertTrue(len(res) == 2)
 
       m = Chem.MolFromSmiles('o1ccnc1OC')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2)
+      self.assertTrue(len(res) == 2)
 
       m = Chem.MolFromSmiles('O1CCNC1OC')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==2)
+      self.assertTrue(len(res) == 2)
 
       m = Chem.MolFromSmiles('CCCSCC')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==3,res)
-      self.assertTrue('[11*]S[11*]' in res,res)
+      self.assertTrue(len(res) == 3, res)
+      self.assertTrue('[11*]S[11*]' in res, res)
 
       m = Chem.MolFromSmiles('CCNC(=O)C1CC1')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==4,res)
-      self.assertTrue('[5*]N[5*]' in res,res)
+      self.assertTrue(len(res) == 4, res)
+      self.assertTrue('[5*]N[5*]' in res, res)
 
     def test2(self):
-      # example from the paper, nexavar: 
+      # example from the paper, nexavar:
       m = Chem.MolFromSmiles('CNC(=O)C1=NC=CC(OC2=CC=C(NC(=O)NC3=CC(=C(Cl)C=C3)C(F)(F)F)C=C2)=C1')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==9,res)
+      self.assertTrue(len(res) == 9, res)
 
     def test3(self):
       m = Chem.MolFromSmiles('FC(F)(F)C1=C(Cl)C=CC(NC(=O)NC2=CC=CC=C2)=C1')
       res = BRICSDecompose(m)
       self.assertTrue(res)
-      self.assertTrue(len(res)==5,res)
-      self.assertTrue('[5*]N[5*]' in res,res)
-      self.assertTrue('[16*]c1ccccc1' in res,res)
-      self.assertTrue('[8*]C(F)(F)F' in res,res)
-
+      self.assertTrue(len(res) == 5, res)
+      self.assertTrue('[5*]N[5*]' in res, res)
+      self.assertTrue('[16*]c1ccccc1' in res, res)
+      self.assertTrue('[8*]C(F)(F)F' in res, res)
 
     def test4(self):
       allNodes = set()
       m = Chem.MolFromSmiles('c1ccccc1OCCC')
-      res = BRICSDecompose(m,allNodes=allNodes)
+      res = BRICSDecompose(m, allNodes=allNodes)
       self.assertTrue(res)
-      leaves=res
-      self.assertTrue(len(leaves)==3,leaves)
-      self.assertTrue(len(allNodes)==6,allNodes)
-      res = BRICSDecompose(m,allNodes=allNodes)
+      leaves = res
+      self.assertTrue(len(leaves) == 3, leaves)
+      self.assertTrue(len(allNodes) == 6, allNodes)
+      res = BRICSDecompose(m, allNodes=allNodes)
       self.assertFalse(res)
-      self.assertTrue(len(allNodes)==6,allNodes)
+      self.assertTrue(len(allNodes) == 6, allNodes)
 
       m = Chem.MolFromSmiles('c1ccccc1OCCCC')
-      res = BRICSDecompose(m,allNodes=allNodes)
+      res = BRICSDecompose(m, allNodes=allNodes)
       self.assertTrue(res)
       leaves.update(res)
-      self.assertTrue(len(allNodes)==9,allNodes)
-      self.assertTrue(len(leaves)==4,leaves)
-      
-      
+      self.assertTrue(len(allNodes) == 9, allNodes)
+      self.assertTrue(len(leaves) == 4, leaves)
+
       m = Chem.MolFromSmiles('c1cc(C(=O)NCC)ccc1OCCC')
-      res = BRICSDecompose(m,allNodes=allNodes)
+      res = BRICSDecompose(m, allNodes=allNodes)
       self.assertTrue(res)
       leaves.update(res)
-      self.assertTrue(len(leaves)==8,leaves)
-      self.assertTrue(len(allNodes)==18,allNodes)
+      self.assertTrue(len(leaves) == 8, leaves)
+      self.assertTrue(len(allNodes) == 18, allNodes)
 
     def test5(self):
       allNodes = set()
@@ -688,13 +691,13 @@ if __name__=='__main__':
         '[14*]c1ncncn1',
         '[16*]c1ccccc1',
         '[14*]c1ncccc1',
-        ]
+      ]
       frags = [Chem.MolFromSmiles(x) for x in frags]
       res = BRICSBuild(frags)
       self.assertTrue(res)
-      res= list(res)
-      self.assertTrue(len(res)==6)
-      smis = [Chem.MolToSmiles(x,True) for x in res]
+      res = list(res)
+      self.assertTrue(len(res) == 6)
+      smis = [Chem.MolToSmiles(x, True) for x in res]
       self.assertTrue('c1ccc(-c2ccccc2)cc1' in smis)
       self.assertTrue('c1ccc(-c2ccccn2)cc1' in smis)
 
@@ -703,34 +706,32 @@ if __name__=='__main__':
       frags = [
         '[3*]O[3*]',
         '[16*]c1ccccc1',
-        ]
+      ]
       frags = [Chem.MolFromSmiles(x) for x in frags]
       res = BRICSBuild(frags)
       self.assertTrue(res)
-      res=list(res)
-      smis = [Chem.MolToSmiles(x,True) for x in res]
-      self.assertTrue(len(smis)==2,smis)
+      res = list(res)
+      smis = [Chem.MolToSmiles(x, True) for x in res]
+      self.assertTrue(len(smis) == 2, smis)
       self.assertTrue('c1ccc(Oc2ccccc2)cc1' in smis)
       self.assertTrue('c1ccc(-c2ccccc2)cc1' in smis)
 
-      
-      
     def test6(self):
       allNodes = set()
       frags = [
         '[16*]c1ccccc1',
         '[3*]OC',
         '[9*]n1cccc1',
-        ]
+      ]
       frags = [Chem.MolFromSmiles(x) for x in frags]
       res = BRICSBuild(frags)
       self.assertTrue(res)
-      res= list(res)
-      self.assertTrue(len(res)==3)
-      smis = [Chem.MolToSmiles(x,True) for x in res]
+      res = list(res)
+      self.assertTrue(len(res) == 3)
+      smis = [Chem.MolToSmiles(x, True) for x in res]
       self.assertTrue('c1ccc(-c2ccccc2)cc1' in smis)
       self.assertTrue('COc1ccccc1' in smis)
-      self.assertTrue('c1ccc(-n2cccc2)cc1' in smis,smis)
+      self.assertTrue('c1ccc(-n2cccc2)cc1' in smis, smis)
 
     def test7(self):
       allNodes = set()
@@ -738,13 +739,13 @@ if __name__=='__main__':
         '[16*]c1ccccc1',
         '[3*]OC',
         '[3*]OCC(=O)[6*]',
-        ]
+      ]
       frags = [Chem.MolFromSmiles(x) for x in frags]
       res = BRICSBuild(frags)
       self.assertTrue(res)
-      res= list(res)
-      smis = [Chem.MolToSmiles(x,True) for x in res]
-      self.assertTrue(len(res)==3)
+      res = list(res)
+      smis = [Chem.MolToSmiles(x, True) for x in res]
+      self.assertTrue(len(res) == 3)
       self.assertTrue('c1ccc(-c2ccccc2)cc1' in smis)
       self.assertTrue('COc1ccccc1' in smis)
       self.assertTrue('O=C(COc1ccccc1)c1ccccc1' in smis)
@@ -753,39 +754,39 @@ if __name__=='__main__':
       random.seed(23)
       base = Chem.MolFromSmiles("n1cncnc1OCC(C1CC1)OC1CNC1")
       catalog = BRICSDecompose(base)
-      self.assertTrue(len(catalog)==5,catalog)
+      self.assertTrue(len(catalog) == 5, catalog)
       catalog = [Chem.MolFromSmiles(x) for x in catalog]
-      ms = list(BRICSBuild(catalog,maxDepth=4))
+      ms = list(BRICSBuild(catalog, maxDepth=4))
       for m in ms:
         Chem.SanitizeMol(m)
       ms = [Chem.MolToSmiles(x) for x in ms]
-      self.assertEqual(len(ms),36)
+      self.assertEqual(len(ms), 36)
 
-      ts = ['n1cnc(C2CNC2)nc1','n1cnc(-c2ncncn2)nc1','C(OC1CNC1)C(C1CC1)OC1CNC1',
-            'n1cnc(OC(COC2CNC2)C2CC2)nc1','n1cnc(OCC(OC2CNC2)C2CNC2)nc1']
-      ts = [Chem.MolToSmiles(Chem.MolFromSmiles(x),True) for x in ts]
+      ts = ['n1cnc(C2CNC2)nc1', 'n1cnc(-c2ncncn2)nc1', 'C(OC1CNC1)C(C1CC1)OC1CNC1',
+            'n1cnc(OC(COC2CNC2)C2CC2)nc1', 'n1cnc(OCC(OC2CNC2)C2CNC2)nc1']
+      ts = [Chem.MolToSmiles(Chem.MolFromSmiles(x), True) for x in ts]
       for t in ts:
-        self.assertTrue(t in ms,(t,ms))
-        
+        self.assertTrue(t in ms, (t, ms))
+
     def test9(self):
       m = Chem.MolFromSmiles('CCOc1ccccc1c1ncc(c2nc(NCCCC)ncn2)cc1')
-      res=BRICSDecompose(m)
-      self.assertEqual(len(res),7)
+      res = BRICSDecompose(m)
+      self.assertEqual(len(res), 7)
       self.assertTrue('[3*]O[3*]' in res)
       self.assertFalse('[14*]c1ncnc(NCCCC)n1' in res)
-      res = BRICSDecompose(m,singlePass=True)
-      self.assertEqual(len(res),13)
+      res = BRICSDecompose(m, singlePass=True)
+      self.assertEqual(len(res), 13)
       self.assertTrue('[3*]OCC' in res)
       self.assertTrue('[14*]c1ncnc(NCCCC)n1' in res)
 
     def test10(self):
       m = Chem.MolFromSmiles('C1CCCCN1c1ccccc1')
-      res=BRICSDecompose(m)
-      self.assertEqual(len(res),2,res)
+      res = BRICSDecompose(m)
+      self.assertEqual(len(res), 2, res)
 
     def test11(self):
       # test coordinate preservation:
-      molblock="""
+      molblock = """
      RDKit          3D
 
  13 14  0  0  0  0  0  0  0  0999 V2000
@@ -821,46 +822,45 @@ M  END
       m = Chem.MolFromMolBlock(molblock)
       pieces = BreakBRICSBonds(m)
 
-      frags = Chem.GetMolFrags(pieces,asMols=True)
-      self.assertEqual(len(frags),3)
-      self.assertEqual(frags[0].GetNumAtoms(),7)
-      self.assertEqual(frags[1].GetNumAtoms(),3)
-      self.assertEqual(frags[2].GetNumAtoms(),7)
+      frags = Chem.GetMolFrags(pieces, asMols=True)
+      self.assertEqual(len(frags), 3)
+      self.assertEqual(frags[0].GetNumAtoms(), 7)
+      self.assertEqual(frags[1].GetNumAtoms(), 3)
+      self.assertEqual(frags[2].GetNumAtoms(), 7)
 
       c1 = m.GetConformer()
       c2 = frags[0].GetConformer()
       for i in range(6):
         p1 = c1.GetAtomPosition(i)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[2].GetConformer()
       for i in range(6):
-        p1 = c1.GetAtomPosition(i+7)
+        p1 = c1.GetAtomPosition(i + 7)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[1].GetConformer()
       for i in range(1):
-        p1 = c1.GetAtomPosition(i+6)
+        p1 = c1.GetAtomPosition(i + 6)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(5)
       p2 = c2.GetAtomPosition(1)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(0)
-      self.assertEqual((p1-p2).Length(),0.0)
-
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       # make sure multiple conformations (include 2D) also work:
-      molblock="""
+      molblock = """
      RDKit          2D
 
  13 14  0  0  0  0  0  0  0  0999 V2000
@@ -894,96 +894,102 @@ M  END
 M  END
 """
       m2 = Chem.MolFromMolBlock(molblock)
-      m.AddConformer(m2.GetConformer(),assignId=True)
-      self.assertEqual(m.GetNumConformers(),2)
+      m.AddConformer(m2.GetConformer(), assignId=True)
+      self.assertEqual(m.GetNumConformers(), 2)
 
       pieces = BreakBRICSBonds(m)
-      frags = Chem.GetMolFrags(pieces,asMols=True)
-      self.assertEqual(len(frags),3)
-      self.assertEqual(frags[0].GetNumAtoms(),7)
-      self.assertEqual(frags[1].GetNumAtoms(),3)
-      self.assertEqual(frags[2].GetNumAtoms(),7)
-      self.assertEqual(frags[0].GetNumConformers(),2)
-      self.assertEqual(frags[1].GetNumConformers(),2)
-      self.assertEqual(frags[2].GetNumConformers(),2)
+      frags = Chem.GetMolFrags(pieces, asMols=True)
+      self.assertEqual(len(frags), 3)
+      self.assertEqual(frags[0].GetNumAtoms(), 7)
+      self.assertEqual(frags[1].GetNumAtoms(), 3)
+      self.assertEqual(frags[2].GetNumAtoms(), 7)
+      self.assertEqual(frags[0].GetNumConformers(), 2)
+      self.assertEqual(frags[1].GetNumConformers(), 2)
+      self.assertEqual(frags[2].GetNumConformers(), 2)
 
       c1 = m.GetConformer(0)
       c2 = frags[0].GetConformer(0)
       for i in range(6):
         p1 = c1.GetAtomPosition(i)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[2].GetConformer(0)
       for i in range(6):
-        p1 = c1.GetAtomPosition(i+7)
+        p1 = c1.GetAtomPosition(i + 7)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[1].GetConformer(0)
       for i in range(1):
-        p1 = c1.GetAtomPosition(i+6)
+        p1 = c1.GetAtomPosition(i + 6)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(5)
       p2 = c2.GetAtomPosition(1)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(0)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c1 = m.GetConformer(1)
       c2 = frags[0].GetConformer(1)
       for i in range(6):
         p1 = c1.GetAtomPosition(i)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[2].GetConformer(1)
       for i in range(6):
-        p1 = c1.GetAtomPosition(i+7)
+        p1 = c1.GetAtomPosition(i + 7)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(6)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
       c2 = frags[1].GetConformer(1)
       for i in range(1):
-        p1 = c1.GetAtomPosition(i+6)
+        p1 = c1.GetAtomPosition(i + 6)
         p2 = c2.GetAtomPosition(i)
-        self.assertEqual((p1-p2).Length(),0.0)
+        self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(5)
       p2 = c2.GetAtomPosition(1)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
       p1 = c1.GetAtomPosition(6)
       p2 = c2.GetAtomPosition(0)
-      self.assertEqual((p1-p2).Length(),0.0)
+      self.assertEqual((p1 - p2).Length(), 0.0)
 
     def test12(self):
       m = Chem.MolFromSmiles('CCS(=O)(=O)NCC')
-      res=list(FindBRICSBonds(m))
-      self.assertEqual(len(res),2,res)
+      res = list(FindBRICSBonds(m))
+      self.assertEqual(len(res), 2, res)
       atIds = [x[0] for x in res]
       atIds.sort()
-      self.assertEqual(atIds,[(5,2), (6,5)])
-      
-      
+      self.assertEqual(atIds, [(5, 2), (6, 5)])
+
+    def testGithub1734(self):
+      m = Chem.MolFromSmiles('c1ccccc1[C@H](C)NC')
+      res = BRICSDecompose(m)
+      self.assertEqual(len(res), 3)
+      self.assertTrue('[4*][C@H]([8*])C' in res)
+      res = BreakBRICSBonds(m)
+      self.assertEqual(Chem.MolToSmiles(res,isomericSmiles=True),
+                       '[16*]c1ccccc1.[4*][C@H]([8*])C.[5*]NC')
 
 
-      
-  failed,tried = _test()
+
+  failed, tried = _test()
   if failed:
     sys.exit(failed)
 
   unittest.main()
-
