@@ -3,17 +3,16 @@
 #  Copyright (C) 2003 Rational Discovery LLC
 #     All Rights Reserved
 #
-import sys
 from rdkit import six
-
 from rdkit.VLib.Node import VLibNode
 
+
 class SupplyNode(VLibNode):
   """ base class for nodes which supply things
 
   Assumptions:
     1) no parents
-  
+
   Usage Example:
     >>> supplier = SupplyNode(contents=[1,2,3])
     >>> supplier.next()
@@ -31,11 +30,12 @@ class SupplyNode(VLibNode):
     1
     >>> [x for x in supplier]
     [1, 2, 3]
-    
+
 
   """
-  def __init__(self,contents=None,**kwargs):
-    VLibNode.__init__(self,**kwargs)
+
+  def __init__(self, contents=None, **kwargs):
+    VLibNode.__init__(self, **kwargs)
     if contents is not None:
       self._contents = contents
     else:
@@ -45,31 +45,33 @@ class SupplyNode(VLibNode):
   def reset(self):
     VLibNode.reset(self)
     self._pos = 0
+
   def next(self):
     if self._pos == len(self._contents):
       raise StopIteration
 
-    res=self._contents[self._pos]
+    res = self._contents[self._pos]
     self._pos += 1
     return res
-  def AddParent(self,parent,notify=1):
+
+  def AddParent(self, parent, notify=1):
     raise ValueError('SupplyNodes do not have parents')
 
+
 if six.PY3:
-    SupplyNode.__next__ = SupplyNode.next
-  
-#------------------------------------
+  SupplyNode.__next__ = SupplyNode.next
+
+
+# ------------------------------------
 #
 #  doctest boilerplate
 #
-def _test():
-  import doctest,sys
-  return doctest.testmod(sys.modules["__main__"])
-
-if __name__ == '__main__':
+def _runDoctests(verbose=None):  # pragma: nocover
   import sys
-  failed,tried = _test()
+  import doctest
+  failed, _ = doctest.testmod(optionflags=doctest.ELLIPSIS, verbose=verbose)
   sys.exit(failed)
 
 
-  
+if __name__ == '__main__':  # pragma: nocover
+  _runDoctests()
